
declare var process: {
  // noinspection TsLint
  env: {
    // noinspection TsLint
    NODE_ENV: string
  }
}

/**
 * changes url origin, api origin, disables logging
 * @type {boolean}
 */
export const isProduction = (process.env.NODE_ENV === 'production')

if (!isProduction) {
  console.warn("production is disabled, see debug.ts!")
  console.warn("production is disabled, see debug.ts!")
  console.warn("production is disabled, see debug.ts!")
}

/**
 * delays the given promise (the resolve NOT THE EXECUTION)
 *
 *
 * usage for redux
 *
 * export function getSystemRolesAsync(): GET_systemRolesAction {
 *   return {
 *     type: ActionType.GET_systemRoles,
 *     payload: delay(getSystemRoles()) //the function is already called but the resolve/reject will be delayed
 *   }
 * }
 *
 * @param {Promise<T>} promise
 * @param timeout
 * @returns {Promise<T>}
 */
export async function delay<T>(promise: Promise<T>, timeout = 100): Promise<T> {

  //somehow throws 2 times...
  return new Promise<T>((resolve, reject) => {

    setTimeout(async () => {
      try {
        const result = await promise
          resolve(result)
      } catch (err) {
        reject(err)
      }

    }, timeout)

  })
}

export async function wait(timeout = 100): Promise<void> {
  return new Promise<void>((resolve, reject) => {
    setTimeout(async () => {
      resolve()
    }, timeout)
  })
}

//for links but we use webpack dev server which runs in 8080
//the backend runs on 5000...
/**
 * this is used for debugging so that the links are still working when running webpack dev server
 * @type {string}
 */
export const debugOriginUrl = isProduction ? '' : 'http://localhost:5000'

export const debugApiHost = isProduction ? '/' :'http://localhost:5000/'
