//some sources
/*
https://medium.com/@jackub/writing-custom-ace-editor-mode-5a7aa83dbe50
https://ace.c9.io/#nav=higlighter
https://ace.c9.io/tool/mode_creator.html

and look into the already defined modes e.g.
/node_modules/ace-builds/src/mode-java.js

for possible tokens see https://github.com/ajaxorg/ace/wiki/Creating-or-Extending-an-Edit-Mode
 */

ace.define('ace/mode/blackbox_test_highlight_rules', ['require', 'exports', 'ace/lib/oop'], function (acequire, exports) {

  var oop = acequire("ace/lib/oop");
  var TextHighlightRules = acequire("ace/mode/text_highlight_rules").TextHighlightRules;

  var BlackBoxTestHighlightRules = function () {
    this.$rules = {
      "start": [{
        token: "keyword.operator",
        regex: "^\\$"
      },
        {
          token: 'keyword.operator',
          regex: /^>\*$/, //arbitrary output
        },
        {
          token: 'keyword.operator',
          regex: "^>",
        },
        {
          token: 'keyword.operator',
          regex: "^<",
        },
        {
          token: 'keyword.operator',
          regex: "^r",
          next: "regex"
        },
        {
          token: 'keyword.operator',
          regex: "^e[0-9]+$",
        },
        {
          token: "comment.line.number-sign",
          regex: /^#.*$/,
        },
      ],

      "regex": [
        {
          token: "string.regexp",
          regex: "\\/",
          next: "start"
        },
        {
          // escapes
          token: "regexp.keyword.operator",
          regex: "\\\\(?:u[\\da-fA-F]{4}|x[\\da-fA-F]{2}|.)"
        }, {
          // flag
          token: "string.regexp",
          regex: "/(im|mi)",
          next: "start"
        },
        {
          // invalid operators
          token: "invalid",
          regex: /\{\d+\b,?\d*\}[+*]|[+*$^?][+*]|[$^][?]|\?{3,}/
        }, {
          // operators
          token: "constant.language.escape",
          regex: /\(\?[:=!]|\)|\{\d+\b,?\d*\}|[+*]\?|[()$^+*?.]/
        }, {
          token: "constant.language.delimiter",
          regex: /\|/
        }, {
          token: "constant.language.escape",
          regex: /\[\^?/,
          next: "regex_character_class"
        }, {
          token: "empty",
          regex: "$",
          next: "start"
        },
        {
          defaultToken: "string.regexp"
        }
      ],

      "regex_character_class": [
        {
          token: "regexp.charclass.keyword.operator",
          regex: "\\\\(?:u[\\da-fA-F]{4}|x[\\da-fA-F]{2}|.)"
        }, {
          token: "constant.language.escape",
          regex: "]",
          next: "regex"
        }, {
          token: "constant.language.escape",
          regex: "-"
        }, {
          token: "empty",
          regex: "$",
          next: "start"
        }, {
          defaultToken: "string.regexp.charachterclass"
        }
      ],

    };
  };

  oop.inherits(BlackBoxTestHighlightRules, TextHighlightRules);

  exports.BlackBoxTestHighlightRules = BlackBoxTestHighlightRules;

})


ace.define('ace/mode/blackbox_test', ['require', 'exports', 'ace/lib/oop', 'ace/mode/text', 'ace/mode/blackbox_test_highlight_rules'], function (acequire, exports) {

  var oop = acequire("ace/lib/oop");
  var TextMode = acequire("ace/mode/text").Mode;
  var BlackBoxTestHighlightRules = acequire("ace/mode/blackbox_test_highlight_rules").BlackBoxTestHighlightRules;

  var Mode = function () {
    this.HighlightRules = BlackBoxTestHighlightRules;
  };

  oop.inherits(Mode, TextMode);

  (function () {
    this.$id = "ace/mode/blackbox_test";
  }).call(Mode.prototype);

  exports.Mode = Mode;
});
