import * as genericLayer from './genericLayer'
import {
  ClientServerAppSettings,
  ClientServerConstants,
  HardwareUsageStatistic,
  LogicalUsageStatistic,
  TestServerPrivateSettings
} from "../types/dashboard";
import {SystemSettingsBase, SystemSettingsFromBackend} from '../types/systemSettings'


const callingName = 'dashboardLayer'
const controllerPrefix = 'dashboard'


export async function getHardwareUsage(): Promise<HardwareUsageStatistic> {
  return genericLayer.get<HardwareUsageStatistic>(callingName,
                                                  `${controllerPrefix}/hardware/usage`
  )
}

export async function getLogicalUsage(): Promise<LogicalUsageStatistic> {
  return genericLayer.get<LogicalUsageStatistic>(callingName,
                                                 `${controllerPrefix}/logical/usage`
  )
}

export async function getClientServerConstants(): Promise<ClientServerConstants> {
  return genericLayer.get<ClientServerConstants>(callingName,
                                                 `${controllerPrefix}/server/constants`
  )
}


export async function getClientServerAppSettings(): Promise<ClientServerAppSettings> {
  return genericLayer.get<ClientServerAppSettings>(callingName,
                                                   `${controllerPrefix}/server/appsettings`
  )
}

export async function getClientServerSystemSettings(): Promise<SystemSettingsFromBackend> {
  return genericLayer.get<SystemSettingsFromBackend>(callingName,
                                              `${controllerPrefix}/server/systemsettings`
  )
}


export async function getTestServerSettings(isSubmitTestServerSettings: boolean): Promise<TestServerPrivateSettings> {
  return genericLayer.get<TestServerPrivateSettings>(callingName,
                                                     `${controllerPrefix}/testserver/settings/private/${isSubmitTestServerSettings}`
  )
}

