import * as genericLayer from './genericLayer'
import {AfterSolutionDoExerciseFullBase, AfterSolutionTupleFromBackend} from '../types/exerciseAfterSolution'
import {
  AssessedSubmitTestFromBackend,
  StoredTestResultFromBackend,
  StoredTestResultTupleFromBackend
} from '../types/testResults'

const callingName = 'doExerciseAfterSolutionLayer'
const controllerPrefix = 'exercise/do/aftersolution'


export async function getDoExerciseAfterSolution(releaseCode: string, plangId: number): Promise<AfterSolutionDoExerciseFullBase> {
  return genericLayer.get<AfterSolutionDoExerciseFullBase>(callingName,
                                                 `${controllerPrefix}/solution/${releaseCode}/${plangId}`)
}



export async function createOrUpdateAfterSolution(releaseCode: string, plangId: number, shouldClearAllTestResults: boolean, solution: AfterSolutionDoExerciseFullBase): Promise<AfterSolutionTupleFromBackend> {
  return genericLayer.update<AfterSolutionDoExerciseFullBase, AfterSolutionTupleFromBackend>(callingName,
                                                           `${controllerPrefix}/save/${releaseCode}/${plangId}/${shouldClearAllTestResults}`, solution)
}


/**
 * returns all test results but submit tests
 * @param releaseCode
 * @param plangId
 */
export async function getAllAfterTestResults(releaseCode: string, plangId: number): Promise<StoredTestResultTupleFromBackend> {
  return genericLayer.get<StoredTestResultTupleFromBackend>(callingName,
                                                            `${controllerPrefix}/testresults/${releaseCode}/${plangId}`)
}


export async function getSubmitTestsAndResults(releaseCode: string, pLangId: number): Promise<ReadonlyArray<AssessedSubmitTestFromBackend>> {
  return genericLayer.get<ReadonlyArray<AssessedSubmitTestFromBackend>>(callingName,
                                                            `${controllerPrefix}/testresults/submit/${releaseCode}/${pLangId}`)
}

/**
 * * returns a single after normal test result
 * @param releaseCode
 * @param plangId
 * @param testId
 */
export async function getSingleAfterNormalTestResult(releaseCode: string, plangId: number, testId: number): Promise<StoredTestResultFromBackend> {
  return genericLayer.get<StoredTestResultFromBackend>(callingName,
    `${controllerPrefix}/testresult/normal/${releaseCode}/${plangId}/${testId}`)
}

/**
 * * returns a single after custom test result
 * @param releaseCode
 * @param plangId
 * @param testId
 */
export async function getSingleAfterCustomTestResult(releaseCode: string, plangId: number, testId: number): Promise<StoredTestResultFromBackend> {
  return genericLayer.get<StoredTestResultFromBackend>(callingName,
    `${controllerPrefix}/testresult/custom/${releaseCode}/${plangId}/${testId}`)
}

/**
 * returns a single after submit test result
 * user must have access to it e.g. release must be closed...
 * @param releaseCode
 * @param plangId
 * @param testId
 */
export async function getSingleAfterSubmitTestResult(releaseCode: string, plangId: number, testId: number): Promise<StoredTestResultFromBackend> {
  return genericLayer.get<StoredTestResultFromBackend>(callingName,
    `${controllerPrefix}/testresult/submit/${releaseCode}/${plangId}/${testId}`)
}
