/**
 * Created by janis dähne.
 */

import * as genericLayer from './genericLayer'
import {
  EditableExercisePreviewFromBackend,
  ExercisePreviewFromBackend,
  SingleGroupPreviewExerciseTupleFromBackend
} from "../types/exercisePreview";
import {PaginatedData, PaginationPostData, PaginationPostDataWithTagsFilter} from '../types/pagination'
import {UserGroupFromBackend, UserGroupBase} from '../types/group'

const callingName = 'exercisePreviewLayer'
const controllerPrefix = 'exercise/overview'

//TODO maybe rename to accessible exercises preview??
// better rename accessible exercises to exercises preview everywhere!!

// export async function getAccessibleExercisePreviews(): Promise<ReadonlyArray<ExercisePreviewFromBackend>> {
//   return genericLayer.get<ReadonlyArray<ExercisePreviewFromBackend>>(
//     callingName, `${controllerPrefix}/all`
//   )
// }

/**
 * TODO make separate api path!!
 * @returns {Promise<ReadonlyArray<ExercisePreviewFromBackend>>}
 */
export async function getOpenExercisePreviews(): Promise<ReadonlyArray<ExercisePreviewFromBackend>> {

  const list = await genericLayer.get<ReadonlyArray<ExercisePreviewFromBackend>>(
    callingName, `${controllerPrefix}/all`
  )

  return list.filter(p => p.isReleased === true)
}

/**
 * only the releases where the user entered a code and not isVisibleToAllAfterRelease
 */
export async function getOpenExercisePreviewsViaCode(paginationPostData: PaginationPostDataWithTagsFilter): Promise<PaginatedData<ExercisePreviewFromBackend>> {

  return genericLayer.update<PaginationPostDataWithTagsFilter, PaginatedData<ExercisePreviewFromBackend>>(
    callingName, `${controllerPrefix}/all/open/${true}`,paginationPostData
  )
}

export async function getOpenExercisePreviewsViaVisibility(paginationPostData: PaginationPostDataWithTagsFilter): Promise<PaginatedData<ExercisePreviewFromBackend>> {

  return await genericLayer.update<PaginationPostDataWithTagsFilter, PaginatedData<ExercisePreviewFromBackend>>(
    callingName, `${controllerPrefix}/all/open/${false}`, paginationPostData
  )

}


export async function getClosedExercisePreviewsViaCode(paginationPostData: PaginationPostDataWithTagsFilter): Promise<PaginatedData<ExercisePreviewFromBackend>> {

  return genericLayer.update<PaginationPostDataWithTagsFilter, PaginatedData<ExercisePreviewFromBackend>>(
    callingName, `${controllerPrefix}/all/closed/${true}`,paginationPostData
  )
}

/**
 * @returns {Promise<ReadonlyArray<ExercisePreviewFromBackend>>}
 */
export async function getClosedExercisePreviewsViaVisibility(paginationPostData: PaginationPostDataWithTagsFilter): Promise<PaginatedData<ExercisePreviewFromBackend>> {

  return genericLayer.update<PaginationPostDataWithTagsFilter, PaginatedData<ExercisePreviewFromBackend>>(
    callingName, `${controllerPrefix}/all/closed/${false}`, paginationPostData
  )
}


export async function getOwnExercisePreviews(paginationPostData: PaginationPostDataWithTagsFilter): Promise<PaginatedData<EditableExercisePreviewFromBackend>> {
  return genericLayer.update<PaginationPostDataWithTagsFilter, PaginatedData<EditableExercisePreviewFromBackend>>(
    callingName, `${controllerPrefix}/own`, paginationPostData
  )
}

export async function getGroupsWithAccessibleExercises(): Promise<ReadonlyArray<UserGroupFromBackend>> {
  return genericLayer.get<ReadonlyArray<UserGroupFromBackend>>(
    callingName, `${controllerPrefix}/groups`
  )
}


export async function getAccessibleExercisePreviewsByGroup(groupId: number, paginationPostData: PaginationPostDataWithTagsFilter): Promise<PaginatedData<EditableExercisePreviewFromBackend>> {
  return genericLayer.update<PaginationPostDataWithTagsFilter, PaginatedData<EditableExercisePreviewFromBackend>>(
    callingName, `${controllerPrefix}/groups/${groupId}/exercises`, paginationPostData
  )
}


export async function getSingleOwnExercisePreview(exerciseId: number): Promise<EditableExercisePreviewFromBackend> {
  return genericLayer.get<EditableExercisePreviewFromBackend>(
    callingName, `${controllerPrefix}/own/exercise/${exerciseId}`,
  )
}

export async function getSingleGroupExercisePreview(exerciseId: number): Promise<SingleGroupPreviewExerciseTupleFromBackend> {
  return genericLayer.get<SingleGroupPreviewExerciseTupleFromBackend>(
    callingName, `${controllerPrefix}/group/exercise/${exerciseId}`,
  )
}

/**
 * deletes the given exercise by id
 * @param exerciseId the exercise id
 * @returns {Promise<void>}
 */
export async function deleteExercise(exerciseId:number):Promise<void> {
  return genericLayer.del<void>(callingName,
    `${controllerPrefix}/delete/${exerciseId}`)
}
