/**
 * Created by janis dähne.
 */


import * as genericLayer from './genericLayer'
import {SystemSettingsForBackend, SystemSettingsFromBackend, TestServerPublicSettings} from "../types/systemSettings";
import {TestServerPrivateSettings} from '../types/dashboard'
import {GroupPreviewFromBackend} from '../types/group'
import {GroupRoleFullBase} from '../types/groupRoles'

const callingName = 'systemSettingsLayer'
const controllerPrefix = 'systemsettings'

export const getServerMessageUrl = `${controllerPrefix}/servermessage`


/**
 * gets the system settings
 */
export async function getSystemSettingsFromBackend(): Promise<SystemSettingsFromBackend> {
  return genericLayer.get<SystemSettingsFromBackend>(callingName,
    controllerPrefix)
}


export async function getInitialSystemSettingsFromBackend(): Promise<SystemSettingsFromBackend> {
  return genericLayer.get<SystemSettingsFromBackend>(callingName,
                                                     `${controllerPrefix}/initial`)
}

export async function geServerMessageFromBackend(): Promise<string> {
  return genericLayer.get<string>(callingName,
                                                     `${controllerPrefix}/servermessage`)
}


/**
 * updates the system settings
 */
export async function changeSystemSettingsFromBackend(systemSettings: SystemSettingsForBackend): Promise<SystemSettingsFromBackend> {
  return genericLayer.update<SystemSettingsForBackend, SystemSettingsFromBackend>(callingName,
                                                     controllerPrefix, systemSettings)
}


/**
 * if the test server is down we get back a response from backend with initial values -1 which should signal that there is some error/problem
 */
export async function getTestPublicServerSettings(isSubmitTestServerSettings: boolean): Promise<TestServerPublicSettings> {
  return genericLayer.get<TestServerPublicSettings>(callingName,
                                                     `${controllerPrefix}/testserver/settings/public/${isSubmitTestServerSettings}`
  )
}


/**
 * returns all groups
 */
export async function getAllDefaultGroups(): Promise<ReadonlyArray<GroupPreviewFromBackend>> {
  return genericLayer.get<ReadonlyArray<GroupPreviewFromBackend>>(callingName,
                                                    `${controllerPrefix}/defaultgroups`
  )
}

/**
 * returns all roles
 */
export async function getAllDefaultRoles(): Promise<ReadonlyArray<GroupRoleFullBase>> {
  return genericLayer.get<ReadonlyArray<GroupRoleFullBase>>(callingName,
                                                   `${controllerPrefix}/defaultroles`
  )
}
