/**
 * Created by janis dähne.
 */

import * as genericLayer from './genericLayer'
import {
  AssessedSubmitTestFromBackend, AssessmentForUserWithTestResults, StoredTestResultFromBackend,
  StoredTestResultTupleFromBackend
} from "../types/testResults";
import {SolutionTupleFromBackend} from "../types/exerciseSolution";
import {DoExerciseFromBackend} from "../types/doExercise";
import {AssessmentFullBase} from "../types/submissions";
import {AfterSolutionDoExerciseFullBase, AfterSolutionTupleFromBackend} from '../types/exerciseAfterSolution'


const callingName = 'tutorViewAssessmentLayer'
const controllerPrefix = 'tutor/view/assessment'


export function getExerciseAsTutor(releaseId: number, userId: number, pLangId: number): Promise<DoExerciseFromBackend> {
  return genericLayer.get<DoExerciseFromBackend>(callingName,
    `${controllerPrefix}/exercise/${releaseId}/${userId}/${pLangId}`)
}

//--- user solution

export function getUserSolutionAsTutor(releaseId: number, userId: number, pLangId: number): Promise<SolutionTupleFromBackend> {
  return genericLayer.get<SolutionTupleFromBackend>(callingName,
                                                    `${controllerPrefix}/solution/${releaseId}/${userId}/${pLangId}`)
}

export function getAllTestResultsAsTutor(releaseId: number,
                                         userId: number,
                                         pLangId: number): Promise<StoredTestResultTupleFromBackend> {
  return genericLayer.get<StoredTestResultTupleFromBackend>(callingName,
    `${controllerPrefix}/testresults/${releaseId}/${userId}/${pLangId}`)
}

export function getSubmissionTestResultsAsTutor(releaseId: number, userId: number, pLangId: number): Promise<AssessmentForUserWithTestResults> {
  return genericLayer.get<AssessmentForUserWithTestResults>(callingName,
    `${controllerPrefix}/testresults/submit/${releaseId}/${userId}/${pLangId}`)
}


export function getSingleNormalTestResultAsTutor(releaseId: number, userId: number, pLangId: number, testId: number): Promise<StoredTestResultFromBackend> {
  return genericLayer.get<StoredTestResultFromBackend>(callingName,
    `${controllerPrefix}/testresult/normal/${releaseId}/${userId}/${pLangId}/${testId}`)
}

export function getSingleCustomTestResultAsTutor(releaseId: number, userId: number, pLangId: number, testId: number): Promise<StoredTestResultFromBackend> {
  return genericLayer.get<StoredTestResultFromBackend>(callingName,
    `${controllerPrefix}/testresult/custom/${releaseId}/${userId}/${pLangId}/${testId}`)
}

export function getSingleSubmitTestResultAsTutor(releaseId: number, userId: number, pLangId: number, testId: number): Promise<StoredTestResultFromBackend> {
  return genericLayer.get<StoredTestResultFromBackend>(callingName,
    `${controllerPrefix}/testresult/submit/${releaseId}/${userId}/${pLangId}/${testId}`)
}

//--- after solution

export function getUserAfterSolutionAsTutor(releaseId: number, userId: number, pLangId: number): Promise<AfterSolutionDoExerciseFullBase> {
  return genericLayer.get<AfterSolutionDoExerciseFullBase>(callingName,
                                                    `${controllerPrefix}/after/solution/${releaseId}/${userId}/${pLangId}`)
}


export function getAllAfterTestResultsAsTutor(releaseId: number,
                                         userId: number,
                                         pLangId: number): Promise<StoredTestResultTupleFromBackend> {
  return genericLayer.get<StoredTestResultTupleFromBackend>(callingName,
                                                            `${controllerPrefix}/after/testresults/${releaseId}/${userId}/${pLangId}`)
}

export function getAfterSubmissionTestResultsAsTutor(releaseId: number, userId: number, pLangId: number): Promise<ReadonlyArray<AssessedSubmitTestFromBackend>> {
  return genericLayer.get<ReadonlyArray<AssessedSubmitTestFromBackend>>(callingName,
                                                            `${controllerPrefix}/after/testresults/submit/${releaseId}/${userId}/${pLangId}`)
}


export function getSingleAfterNormalTestResultAsTutor(releaseId: number, userId: number, pLangId: number, testId: number): Promise<StoredTestResultFromBackend> {
  return genericLayer.get<StoredTestResultFromBackend>(callingName,
    `${controllerPrefix}/after/testresult/normal/${releaseId}/${userId}/${pLangId}/${testId}`)
}

export function getSingleAfterCustomTestResultAsTutor(releaseId: number, userId: number, pLangId: number, testId: number): Promise<StoredTestResultFromBackend> {
  return genericLayer.get<StoredTestResultFromBackend>(callingName,
    `${controllerPrefix}/after/testresult/custom/${releaseId}/${userId}/${pLangId}/${testId}`)
}

export function getSingleAfterSubmitTestResultAsTutor(releaseId: number, userId: number, pLangId: number, testId: number): Promise<StoredTestResultFromBackend> {
  return genericLayer.get<StoredTestResultFromBackend>(callingName,
    `${controllerPrefix}/after/testresult/submit/${releaseId}/${userId}/${pLangId}/${testId}`)
}

//--- assessment

export async function getSingleAssessment(releaseId: number, userId: number, pLangId: number): Promise<AssessmentFullBase> {
  return genericLayer.get<AssessmentFullBase>(
    callingName, `${controllerPrefix}/assessment/${releaseId}/${userId}/${pLangId}`
  )
}


export async function updateSingleAssessment(assessment: AssessmentFullBase): Promise<void> {
  return genericLayer.update<AssessmentFullBase, void>(
    callingName, `${controllerPrefix}/assessment/update`, assessment
  )
}
