/**
 * Created by janis dähne.
 */

import * as React from "react";
import {connect} from "react-redux";
import {bindActionCreators, Dispatch} from "redux";
import {returntypeof} from 'react-redux-typescript';
import {RootState} from "../../../state/reducers";
import {Button, Icon} from "semantic-ui-react";
import {leftIconPopupOffset, tempPrintDivId} from "../../../constants";
import {joinWith} from "../../../helpers/typeHelpers";
import {SimpleVDivider} from "../../helpers/simpleVDivider";
import {HelpPopup} from "../../helpers/helpPopup";
import {initial as initialCodeEditorSettings} from "../../../state/reducers/editorSettingsDialog/editorSettingsReducer"

import ExplorerView from '../multiTabEditorsCommon/explorerView'

import ReactResizeDetector from "react-resize-detector";

import {Editor, IEditSession} from 'brace';

import {EditorTab, MultiTabCodeEditorProps} from "../multiTabCodeEditorProps";
import {CodeEditorSettingsFullBase} from "../../../types/userData";
import {getEditorInternalThemeName} from "../../../helpers/themes";
import 'brace/ext/searchbox';
import {throttle} from 'lodash-es'
import {MyPopup} from "../../helpers/myPopup";
import {getI18n} from "../../../../i18n/i18nRoot";
import {AceEditorsStorageHelper} from "./aceEditorsStorageHelper";
import Logger from "../../../helpers/logger";

import {getNewActiveTabId, getNewTabDisplayIndices} from '../../../helpers/editorHelper'
import AceEditor from 'react-ace'
import {setQuickCodeEditorFontSizeInPx} from '../../../state/actions/globalUiActions'
import {debounce, divide} from 'lodash-es'
import {PrintHelper} from '../../../helpers/printHelper'
import markdown from '../../../helpers/markdownHelper'

//included language highlighters are included inside simple ace editor e.g. import 'brace/mode/text';

import 'brace/ext/static_highlight'
// @ts-ignore
const ace_static_highlighter = ace.acequire('ace/ext/static_highlight')

//const css = require('../styles.styl');

const _initialCodeEditorSettings = initialCodeEditorSettings

const mapStateToProps = (rootState: RootState, props: MultiTabCodeEditorProps) => {
  return {
    //test0: rootState...
    //test: props.test
    ...props,
    editorSettings: (rootState.userDataSettingsSate.userData.settings && rootState.userDataSettingsSate.userData.settings.codeEditorSettings
                     ? rootState.userDataSettingsSate.userData.settings.codeEditorSettings
                     : initialCodeEditorSettings) as CodeEditorSettingsFullBase,

    codeEditorFontSizeInPx: rootState.globalUiState.quickCodeEditorFontSizeInPx,
    langId: rootState.i18nState.langId
  }
}

const mapDispatchToProps = (dispatch: Dispatch) => bindActionCreators({
                                                                        //imported reducer funcs here
                                                                        setQuickCodeEditorFontSizeInPx,
                                                                      }, dispatch)


const stateProps = returntypeof(mapStateToProps);
const dispatchProps = returntypeof(mapDispatchToProps);
type Props = typeof stateProps & typeof dispatchProps;


interface AceWrapper {
  readonly editor: Editor
}

//only one multi editor is allowed that is rendered (because we use html id as resize anchor and html id needs to be unique)


class MultiFileAceEditor extends React.Component<Props, any> {


  aceEditorWrapper: AceWrapper

  /**
   * should be called when the editor gets resized
   * @type {any}
   */
  forceRerenderThrottled: () => void = null
  debouncedOnChange: () => void = null


  /**
   * apply the settings to all tabs (used when we change the settings)
   * when we reload the page the tabs has no sessions yet so new sessions with editor settings are created
   * and attached to the tabs
   *
   * but when we change the editor settings after the initial tab session set we always take the tab.editSession
   * to overwrite the this.editSessions options/settings
   * @param {ReadonlyArray<EditorTab>} editorTabs
   */
  applyEditorSettingsToTabs(editorTabs: ReadonlyArray<EditorTab>): void {
    // noinspection TsLint
    for (let i = 0; i < editorTabs.length; i++) {
      const tab: EditorTab = editorTabs[i]

      const storedTabs = AceEditorsStorageHelper.getEditor(this.props.id)

      const storedTab = storedTabs.find(p => p.tab.id === tab.id)

      storedTab.session.setUseWrapMode(this.props.editorSettings.useWrapping)
      storedTab.session.setTabSize(this.props.editorSettings.tabSize)
    }
  }

  componentWillMount(): void {

    // this.rebuildSessionStorage(this.props.editorTabs)
    // this.applySavedSessionStates(this.props.editorTabs)
    // noinspection TsLint
    this.forceRerenderThrottled = throttle(this.forceRerender, 200)

    if (this.props.debounceTimeInMs !== undefined) {
      this.debouncedOnChange = debounce(this.onChangeDebounce, this.props.debounceTimeInMs)
    }

  }

  componentDidMount(): void {
    if (!AceEditorsStorageHelper.getEditor(this.props.id)) {
      //create initial tabs
      // noinspection TsLint
      AceEditorsStorageHelper.addEditorWithTabs(this.props.id, this.props.editorTabs as Array<EditorTab>)
    }
    this.setActiveTabSession(this.props.selectedTabId)

  }

  zoomEditorIn(e: ExtendedKeyboardEvent | null): void {

    if (e) {
      e.preventDefault()
    }

    if (this.props.codeEditorFontSizeInPx === null) {
      this.props.setQuickCodeEditorFontSizeInPx(this.props.editorSettings.fontSize + 1)
    } else {
      this.props.setQuickCodeEditorFontSizeInPx(this.props.codeEditorFontSizeInPx + 1)
    }
  }

  zoomEditorOut(e: ExtendedKeyboardEvent | null): void {

    if (e) {
      e.preventDefault()
    }

    if (this.props.codeEditorFontSizeInPx === null) {
      this.props.setQuickCodeEditorFontSizeInPx(this.props.editorSettings.fontSize - 1)
    } else {
      this.props.setQuickCodeEditorFontSizeInPx(this.props.codeEditorFontSizeInPx - 1)
    }
  }

  zoomEditorReset(e: ExtendedKeyboardEvent | null): void {

    if (e) {
      e.preventDefault()
    }

    this.props.setQuickCodeEditorFontSizeInPx(null)
  }

  setActiveTabSession(newActiveTabId: number): void {

    if (newActiveTabId === -1) return

    if (this.aceEditorWrapper && this.aceEditorWrapper.editor) {

      const editorTabs = AceEditorsStorageHelper.getEditor(this.props.id)

      const activeTab = this.props.editorTabs.find(p => p.id === newActiveTabId)

      if (activeTab) {
        const wrapperTab = editorTabs.find(p => p.tab.id === activeTab.id)

        if (!wrapperTab) {
          Logger.error('could not find editor tab!')
          return
        }

        this.aceEditorWrapper.editor.setSession(wrapperTab.session)
      }
    }
    this.applyEditorSettingsToTabs(this.props.editorTabs)
  }


  componentDidUpdate(): void {

    if (this.aceEditorWrapper && this.aceEditorWrapper.editor) {
      this.aceEditorWrapper.editor.$blockScrolling = Infinity
      const editorTabs = AceEditorsStorageHelper.getEditor(this.props.id)
      const activeTab = this.getActiveEditorTab()

      if (activeTab) {
        const wrapperTab = editorTabs.find(p => p.tab.id === activeTab.id)

        if (!wrapperTab) {
          //this can happen in the time when we update files (get new ids)
          //and we already got the new ids and update this component but not yet refreshed the ace storage
          //the warning gets then generated between the pending and fulfilled actions
          Logger.warn('maybe could not find editor tab but could be on update!')
          return
        }

        if (wrapperTab.session === this.aceEditorWrapper.editor.getSession()) {
          return
        }

        this.aceEditorWrapper.editor.setSession(wrapperTab.session)
        this.applyEditorSettingsToTabs(this.props.editorTabs)
      }
    }
  }


  componentWillUnmount(): void {
  }

  unsetSession(): void {
    if (this.aceEditorWrapper && this.aceEditorWrapper.editor) {

      const initialSession = AceEditorsStorageHelper.createNewSession('', `text` as any)
      //setting a new session will clear the handlers of the old session
      this.aceEditorWrapper.editor.setSession(initialSession)
    }
  }


  /**
   * updates the editor ref and tries to set the active session
   * @param {AceWrapper} wrapper
   */
  updateEditorRef(wrapper: AceWrapper): void {
    this.aceEditorWrapper = wrapper

    if (this.aceEditorWrapper.editor['zoomIn']) {
      //we already added the commands
      return
    }

    /*
    we now use a custom version of brace (ace editor) where we fixed the shortcut ctrl++/cmd++
     see https://github.com/ajaxorg/ace/blob/51f53efa96ca2507bafb1c980d022953df6f35c6/lib/ace/lib/keys.js
     and
     https://github.com/ajaxorg/ace/issues/2566
    */

    const self = this

    this.aceEditorWrapper.editor.commands.addCommand(
      {
        name: 'zoomIn',
        bindKey: {
          win: 'Ctrl-+',
          mac: "Command-+"
        },
        exec: function (): void {
          self.zoomEditorIn(null)
        },
        readOnly: true //also working on readonly files
      }
    )

    this.aceEditorWrapper.editor.commands.addCommand(
      {
        name: 'zoomOut',
        bindKey: {
          win: 'Ctrl--',
          mac: "Command--"
        },
        exec: function (): void {
          console.log(arguments)
          self.zoomEditorOut(null)
        },
        readOnly: true
      }
    )

    this.aceEditorWrapper.editor.commands.addCommand(
      {
        name: 'zoomReset',
        bindKey: {
          win: 'Ctrl-0',
          mac: "Command-0"
        },
        exec: function (): void {
          self.zoomEditorReset(null)
        },
        readOnly: true
      }
    )

  }

  forceRerender(): void {
    if (this.aceEditorWrapper && this.aceEditorWrapper.editor) {
      //console.log("resized")
      this.aceEditorWrapper.editor.resize(false)
    }
  }

  getActiveEditorTab(): EditorTab | null {

    if (this.props.editorTabs.length === 0) return null
    const tab = this.props.editorTabs.find(p => p.id === this.props.selectedTabId)
    if (!tab) return null

    return tab
  }

  onBlur(): void {

    if (!this.aceEditorWrapper || !this.aceEditorWrapper.editor) return

    //make sure the editor storage is in sync
    const editorTabs = AceEditorsStorageHelper.getEditor(this.props.id)

    const activeTab = this.getActiveEditorTab()

    if (!activeTab) {
      Logger.error('could not find active editor tab!')
      return
    }

    const wrapperTab = editorTabs.find(p => p.tab.id === activeTab.id)
    if (!wrapperTab) {
      Logger.error('could not find editor tab!')
      return
    }

    const newVal = this.aceEditorWrapper.editor.getValue()

    if (newVal === wrapperTab.tab.content) {
      return //lost focus but nothing to update
    }

    // @ts-ignore
    wrapperTab.tab.content = newVal //this NOT updates the redux store!
    //this only updated the AceEditorStorage content we then need to call update solution files
    //and apply the new content to the real solution files

    this.props.onUpdateHasFileChangedMap(this.getActiveEditorTab(), this.props.editorTabs)

    if (this.props.onBlur) {
      this.props.onBlur(newVal, activeTab)
    }
    this.props.onHasUnsavedChangesChanged(true)
  }

  onChangeDebounce(): void {

    if (!this.aceEditorWrapper || !this.aceEditorWrapper.editor) return

    //make sure the editor storage is in sync
    const editorTabs = AceEditorsStorageHelper.getEditor(this.props.id)

    const activeTab = this.getActiveEditorTab()

    if (!activeTab) {
      Logger.error('could not find active editor tab!')
      return
    }

    const wrapperTab = editorTabs.find(p => p.tab.id === activeTab.id)
    if (!wrapperTab) {
      Logger.error('could not find editor tab!')
      return
    }

    const newVal = this.aceEditorWrapper.editor.getValue()

    if (newVal === wrapperTab.tab.content) {
      return //lost focus but nothing to update
    }

    // @ts-ignore
    wrapperTab.tab.content = newVal //this NOT updates the redux store!
    //this only updated the AceEditorStorage content we then need to call update solution files
    //and apply the new content to the real solution files

    this.props.onUpdateHasFileChangedMap(this.getActiveEditorTab(), this.props.editorTabs)


    if (!this.debouncedOnChange || !this.props.onBlur) return
    this.props.onBlur(newVal, activeTab)

    this.props.onHasUnsavedChangesChanged(true)
  }

  /**
   * @param onlyCurrentFile true: print only the active file (if any)
   * @param onePagePerFile (only if @see onlyCurrentFile is false) true: prints every file on a separate page, false: continuous text
   */
  onPrintFiles(onlyCurrentFile: boolean, onePagePerFile: boolean): void {


    // tslint:disable-next-line
    const tabsToPrint: Array<EditorTab & {session: IEditSession}> = []

    if (onlyCurrentFile) {
      const activeTab = this.getActiveEditorTab()
      if (!activeTab) {
        Logger.warn('could not find active tab for printing')
        return
      }

      const aceEditorTabs = AceEditorsStorageHelper.getEditor(this.props.id)
      const aceTab = aceEditorTabs.find(p => p.tab.id === activeTab.id)
      tabsToPrint.push(
        {
          ...activeTab,
          session: aceTab
                   ? aceTab.session
                   : null
        }
      )
    } else {
      //all tabs

      //because the print button we onBlur the editor first updating the redux storage
      const editorTabs = this.props.editorTabs

      if (editorTabs.length === 0) {
        Logger.warn('could not find any tab for printing')
        return
      }


      const aceEditorTabs = AceEditorsStorageHelper.getEditor(this.props.id)

      // tslint:disable-next-line
      for (let i = 0; i < editorTabs.length; i++) {
        const tab = editorTabs[i]
        const aceTab = aceEditorTabs.find(p => p.tab.id === tab.id)
        tabsToPrint.push(
          {
            ...tab,
            session: aceTab
                     ? aceTab.session
                     : null
          }
        )
      }
    }

    if (tabsToPrint.length === 0) {
      Logger.log('no files to print...')
      return
    }

    //see https://github.com/ajaxorg/ace/issues/1480#issuecomment-20201314

    // tslint:disable-next-line
    let innerHtml = ``
    // tslint:disable-next-line
    let editorCss = ''

    //the theme is the same for all tabs... but we could not have a tab open...
    const aceThemeName = getEditorInternalThemeName(this.props.editorSettings.theme)

    // @ts-ignore
    // tslint:disable-next-line
    let myAceThemeMode = ace.acequire(`ace/theme/${aceThemeName}`)

    if (!myAceThemeMode) {
      // @ts-ignore
      myAceThemeMode = ace.acequire(`ace/theme/monokai`)
    }

    // @ts-ignore
    const theme = (this.aceEditorWrapper && this.aceEditorWrapper.editor && this.aceEditorWrapper.editor.renderer)
      // @ts-ignore
                  ? this.aceEditorWrapper.editor.renderer.theme
                  : myAceThemeMode

    // tslint:disable-next-line
    for (let i = 0; i < tabsToPrint.length; i++) {
      const tab = tabsToPrint[i]

      // tslint:disable-next-line
      let markdownContent = ''

      markdownContent += `**${tab.fileNameWithExtensions}**\n`

      const aceRenderedObj = ace_static_highlighter.renderSync(tab.content, tab.session.getMode(), theme)

      //markdown is bad because it cannot break code lines properly
      //console.log(aceRenderedObj.css)
      //console.log(aceRenderedObj.html)
      // markdownContent += '```' + tab.language + '\n'
      // markdownContent += `${tab.content}`
      // markdownContent += `\n`
      // markdownContent += '```'

      if (!editorCss) {
        editorCss = aceRenderedObj.css
      }

      innerHtml += markdown.render(markdownContent)
      innerHtml += `<div class="font-sized">${aceRenderedObj.html}</div>`

      if (onePagePerFile === false) { //continuous text --> add new line after file
        innerHtml += `<br />`
      }

      if (onePagePerFile) {
        innerHtml += `<div class="page-break"></div>`
      }
    }

    //see https://davidwalsh.name/add-rules-stylesheets
    const editorStyle = document.createElement("style") as HTMLStyleElement
    editorStyle.appendChild(document.createTextNode(editorCss))
    document.head.appendChild(editorStyle)

    // tslint:disable-next-line
    let fontSize = this.props.codeEditorFontSizeInPx !== null
                   ? this.props.codeEditorFontSizeInPx
                   : this.props.editorSettings.fontSize


    //static_highlighter default is 12px... we only change the horizontal size of the lines not the height...
    //(because this is not that easy to calculate from the front size??)
    // tslint:disable-next-line
    fontSize = Math.min(fontSize, 17) //everything above 17 will not work properly (lines intersect, tested)
    // tslint:disable-next-line
    fontSize = Math.max(fontSize, 8) //8 would work too but 12 is normal...


    const editorStyleFontSize = document.createElement("style") as HTMLStyleElement
    editorStyleFontSize.appendChild(document.createTextNode(`
    #${tempPrintDivId} .font-sized .ace_static_highlight {
      font-size: ${fontSize}px;
    }
    `))
    document.head.appendChild(editorStyleFontSize)

    PrintHelper.printContent(innerHtml,
                             () => {
                             },
                             () => {
                               document.head.removeChild(editorStyle)
                               document.head.removeChild(editorStyleFontSize)
                             }
    )

  }

  render(): JSX.Element {

    //this sometimes does not re-evaluate
    // noinspection TsLint
    const activeEditorTab: EditorTab = this.props.editorTabs.find(p => p.id === this.props.selectedTabId)
    if (!activeEditorTab && this.props.editorTabs.length > 0) {
      //we don't have any file to display yet...
    }

    //.filter(p => p.displayIndex >= 0)

    return (<div className="fh flexed-h flex-stretch">
      <div style={{height: this.props.height}} className="flexed-h flex-stretch">

        <ReactResizeDetector handleHeight handleWidth onResize={(w, h) => {
          if (this.forceRerenderThrottled) {
            this.forceRerenderThrottled()
          }
        }}/>

        <div
          className="multi-editor-wrapper fh"
        >

          <div className="editor-side-menu two-flexed flexed-h">

            <div>

              <div className={this.props.isFileExplorerDisplayed
                              ? 'clickable-toggled mar-bot'
                              : 'clickable mar-bot'}
                   onClick={() => {
                     this.props.onChangeIsFileExplorerDisplayed(!this.props.isFileExplorerDisplayed)
                   }}
              >
                {!this.props.isFileExplorerDisplayed && <Icon.Group size="large">
                  <Icon name="copy" className="white-colored"/>
                  {this.props.hasUnsavedChanges && <MyPopup
                    trigger={<Icon corner name='exclamation triangle' className="warning-colored"/>}
                    content={getI18n(this.props.langId, 'At least one file has changed')}
                    offset={leftIconPopupOffset}
                  />}
                </Icon.Group>}

                {this.props.isFileExplorerDisplayed && <Icon.Group size="large">
                  <Icon name="copy" className="toggled-colored"/>
                  {this.props.hasUnsavedChanges && <MyPopup
                    trigger={<Icon corner name='exclamation triangle' className="warning-colored"/>}
                    content={getI18n(this.props.langId, 'At least one file has changed')}
                    offset={leftIconPopupOffset}
                  />}
                </Icon.Group>}
              </div>

              <MyPopup
                trigger={
                  <div className="clickable mar-bot" onClick={() => {
                    this.zoomEditorIn(null)
                  }}>
                    <Icon name="zoom in" className="white-colored" size="large"/>
                  </div>
                }
                mouseEnterDelay={1000}
                content={getI18n(this.props.langId, "Shortcut: ctrl++/cmd++")}
              />

              <MyPopup
                trigger={
                  <div className="clickable mar-bot" onClick={() => {
                    this.zoomEditorReset(null)
                  }}>
                    <Icon.Group size="large">
                      <Icon name="undo" className="white-colored"/>
                      <Icon corner name="font"/>
                    </Icon.Group>
                    {/*<Icon.Group size="large">*/}
                    {/*<Icon name="font" className="white-colored"/>*/}
                    {/*<Icon corner name="undo"/>*/}
                    {/*</Icon.Group>*/}

                  </div>
                }
                mouseEnterDelay={1000}
                content={getI18n(this.props.langId,
                                 "Shortcut: ctrl+0/cmd+0. Resets the quick-zoom to the zoom in the editor settings. Note that the quick-zoom is not persistent"
                )}
              />

              <MyPopup
                trigger={
                  <div className="clickable mar-bot" onClick={() => {
                    this.zoomEditorOut(null)
                  }}>
                    <Icon name="zoom out" className="white-colored" size="large"/>
                  </div>
                }
                mouseEnterDelay={1000}
                content={getI18n(this.props.langId, "Shortcut: ctrl+-/cmd+-")}
              />

            </div>


            <div>

              {
                this.props.canPrint &&
                <MyPopup
                  trigger={
                    <div className="clickable">
                      <Icon name="print" className="white-colored"/>
                    </div>
                  }
                  wide
                  on="click"
                  content={
                    <div className="flexed-h">
                      <Button className="mar-bot-half" labelPosition='left' disabled={!activeEditorTab} icon
                              onClick={() => {
                                this.onPrintFiles(true, false)
                              }}>
                        <Icon name="file"/>
                        {
                          getI18n(this.props.langId, "Print the current file only")
                        }
                      </Button>
                      <Button className="mar-bot-half" labelPosition='left' icon onClick={() => {
                        this.onPrintFiles(false, true)
                      }}>
                        <Icon name="copy"/>
                        {
                          getI18n(this.props.langId, "Print all files, one per page")
                        }
                      </Button>
                      <Button className="mar-bot-half" labelPosition='left' icon onClick={() => {
                        this.onPrintFiles(false, false)
                      }}>
                        <Icon name="file alternate"/>
                        {
                          getI18n(this.props.langId, "Print all files consecutively")
                        }
                      </Button>
                    </div>
                  }
                />
              }


            </div>

          </div>

          {//the file explorer
            this.props.isFileExplorerDisplayed && <ExplorerView
              hasFileChangedMap={this.props.hasFileChangedMap}
              onHasUnsavedChangesChanged={(hasChanges: boolean) => {

                this.props.onUpdateHasFileChangedMap(this.getActiveEditorTab(), this.props.editorTabs)
                this.props.onHasUnsavedChangesChanged(hasChanges)
              }}
              editorTabs={this.props.editorTabs}
              selectedTabId={this.props.selectedTabId}
              onOpenTabClicked={editorTab => {
                this.props.onOpenTabClicked(editorTab)
                this.setActiveTabSession(editorTab.id)
              }}
              userCanCreateNewFiles={this.props.userCanCreateNewFiles}
              newFileName={this.props.newFileName}
              onCreateFileClicked={(editorTab: EditorTab) => {

                this.props.onUpdateHasFileChangedMap(this.getActiveEditorTab(), this.props.editorTabs)
                this.props.onCreateFileClicked(editorTab)
                AceEditorsStorageHelper.addTabToEditor(this.props.id, editorTab)
              }}
              onNewFileNameChanged={this.props.onNewFileNameChanged}
              isRenamingFile={this.props.isRenamingFile}
              renamingFileName={this.props.renamingFileName}
              onFileRename={(editorTab: EditorTab, newName: string) => {

                this.props.onUpdateHasFileChangedMap(this.getActiveEditorTab(), this.props.editorTabs)
                this.props.onFileRename(editorTab, newName)
              }}
              onRenamingFileNameChanged={this.props.onRenamingFileNameChanged}
              renamingFileId={this.props.renamingFileId}
              onIsRenamingFileChanged={this.props.onIsRenamingFileChanged}
              onRenamingFileIdChanged={this.props.onRenamingFileIdChanged}
              onRemoveFile={(editorTab: EditorTab) => {
                this.props.onRemoveFile(editorTab)
                AceEditorsStorageHelper.removeTabFromEditor(this.props.id, editorTab)
              }}
            />}

          {//displayes the tab headers (line)
          }
          <div
            className="multi-editor-wrapper-right">

            <div className="editor-tabs-line">
              {joinWith(this.props.editorTabs.filter(p => p.displayIndex >= 0).map((tab: EditorTab) => {

                return (<div key={(tab.id)}
                             className={`editor-tab-line-item ${tab.id === this.props.selectedTabId
                                                                ? 'active'
                                                                : ''}`}

                >
                  {
                    tab.isReadonly && <div style={{marginRight: '0'}}>
                      <HelpPopup icon="key" defaultText={getI18n(this.props.langId, 'This file is readonly')}/>
                    </div>
                  }
                  {
                    tab.isContentVisibleForUser === false && <div style={{marginRight: '0'}}>
                      <HelpPopup icon="hide" defaultText={getI18n(this.props.langId, 'The file content is hidden')}/>
                    </div>
                  }
                  {
                    //not fully implemented??
                    this.props.hasFileChangedMap[tab.id.toString()] &&
                    <div className="changed-file-style" style={{marginRight: '0'}}>
                      <HelpPopup icon="circle"
                                 defaultText={getI18n(this.props.langId, 'This file was changed since last save')}/>
                    </div>}
                  <div className={tab.id === this.props.selectedTabId
                                  ? null
                                  : 'clickable'}
                       onClick={() => {
                         this.props.onOpenTabClicked(tab)
                         this.setActiveTabSession(tab.id)
                       }}
                  >
                    {tab.fileNameWithExtensions}
                  </div>
                  <div className="clickable"
                       onClick={() => {
                         this.props.onCloseTabClicked(tab)

                         //this would do nothing and it's a bug when we execute this because this would hide the file
                         // if (isDisplayed === true && targetFile.id === selectedFileId) return

                         const newTempFiles = getNewTabDisplayIndices(tab, false, this.props.editorTabs)
                         const nextActiveTabId = getNewActiveTabId(tab, false, tab.id, newTempFiles)

                         if (nextActiveTabId === null) {
                           //we closed not the activeg tab
                           return
                         }
                         //this must be the same as the editorHelper.ts > changeTabVisibility
                         this.setActiveTabSession(nextActiveTabId)

                       }}
                  >
                    <Icon name="remove"/>
                  </div>
                </div>)
              }), (i) => {
                return (<SimpleVDivider className="mar-null" key={i}/>)
              })}

            </div>

            {//the editor area
            }
            <div className={this.props.isFileExplorerDisplayed
                            ? 'editor-area'
                            : 'editor-area-expanded'}>

              {
                (!activeEditorTab && this.props.editorTabs.length > 0) && <div className="editor-no-file-selected">
                  {getI18n(this.props.langId, 'No file selected')}
                </div>
              }
              {
                (!activeEditorTab && this.props.editorTabs.length === 0) && <div className="editor-no-files">
                  {getI18n(this.props.langId, 'No files')}
                </div>
              }
              {activeEditorTab && <AceEditor
                width="100%"
                height="100%"
                focus={true}
                onBeforeUmount={() => {
                  this.unsetSession()
                }}
                ref={(editor: any) => {
                  if (editor) {
                    this.updateEditorRef(editor)
                  }
                }}
                name={this.props.id}
                mode={activeEditorTab.language}
                theme={getEditorInternalThemeName(this.props.editorSettings.theme)}
                onBlur={() => this.onBlur()}
                onChange={this.debouncedOnChange
                          ? () => this.debouncedOnChange()
                          : undefined}
                // onChange={(newValue: string, event: any) => {
                //
                //   this.props.onUpdateHasFileChangedMap(this.getActiveEditorTab(), this.props.editorTabs)
                //
                //   // const activeTab = this.getActiveEditorTab()
                //   //
                //   // const session = this.aceEditorWrapper.editor.getSession()
                //   //
                //   // this.props.onChange({
                //   //   ...activeTab,
                //   //   content: newValue,
                //   //   editSession: convertSession(session),
                //   // }, newValue, event)
                //
                //   this.props.onHasUnsavedChangesChanged(true)
                // }}
                value={activeEditorTab.content}
                editorProps={{$blockScrolling: true}}
                readOnly={!this.props.editorTabs || this.props.editorTabs.length === 0 || !activeEditorTab || activeEditorTab.isReadonly || activeEditorTab.isContentVisibleForUser === false}

                fontSize={this.props.codeEditorFontSizeInPx !== null
                          ? this.props.codeEditorFontSizeInPx
                          : this.props.editorSettings.fontSize}
                wrapEnabled={this.props.editorSettings.useWrapping}
                tabSize={this.props.editorSettings.tabSize}
                showPrintMargin={false}
                highlightActiveLine={true}

              />}
            </div>
          </div>

        </div>

      </div>

    </div>)
  }
}

export default connect(mapStateToProps, mapDispatchToProps)(MultiFileAceEditor)
