/**
 * Created by janis dähne.
 */

import * as React from "react";
import {connect} from "react-redux";
import {bindActionCreators, Dispatch} from "redux";
import {returntypeof} from 'react-redux-typescript';
import {RootState} from "../../state/reducers";
import FullSiteDialog from '../fullSiteDialog'
import EditorSettingsView from './editorSettingsView'
import {
  applyChangedEditorSettingsDialogAsync, cancelEditorSettingsDialog
} from "../../state/actions/editorSettingsDialog/editorSettingsDialogActions";
import {getI18n} from "../../../i18n/i18nRoot";
import {setQuickCodeEditorFontSizeInPx} from '../../state/actions/globalUiActions'

//const css = require('./styles.styl');

export interface MyProps {
  //readonly test: string
}

const mapStateToProps = (rootState: RootState /*, props: MyProps*/) => {
  return {
    //test0: rootState...
    //test: props.test
    userSettings: rootState.userDataSettingsSate.userData.settings
      ? rootState.userDataSettingsSate.userData.settings
      : null,
    editorSettings: rootState.editorSettingsDialogState.editorSettings,
    isValid: rootState.editorSettingsDialogState.isValid,
    isBusy: rootState.editorSettingsDialogState.isBusy || rootState.userDataSettingsSate.isLoadingMiscSettings,
    setIsDialogDisplayedActionCallback: rootState.editorSettingsDialogState.setIsDialogDisplayedActionCallback,
    langId: rootState.i18nState.langId
  }
}

const mapDispatchToProps = (dispatch: Dispatch) => bindActionCreators({
  //imported reducer funcs here
  cancelEditorSettingsDialog,
  applyChangedEditorSettingsDialogAsync,
                                                                        setQuickCodeEditorFontSizeInPx,
}, dispatch)


const stateProps = returntypeof(mapStateToProps);
const dispatchProps = returntypeof(mapDispatchToProps);
type Props = typeof stateProps & typeof dispatchProps;

class editorSettingsDialog extends React.Component<Props, any> {
  render(): JSX.Element {
    return (
      <FullSiteDialog
        isBusy={this.props.isBusy}
        isValid={this.props.isValid}
        caption={
          <h2>
            {getI18n(this.props.langId, 'Change Editor settings')}
          </h2>
        }

        onSave={() => {

          this.props.setQuickCodeEditorFontSizeInPx(this.props.editorSettings.fontSize) //make sure when we changed settings we use the most recent ones...
          this.props.applyChangedEditorSettingsDialogAsync(this.props.editorSettings, this.props.userSettings, this.props.setIsDialogDisplayedActionCallback)
        }}

        onCancel={() => {
          this.props.cancelEditorSettingsDialog(this.props.setIsDialogDisplayedActionCallback)
        }}
      >
        <EditorSettingsView/>
      </FullSiteDialog>
    )
  }
}

export default connect(mapStateToProps, mapDispatchToProps)(editorSettingsDialog)