import * as React from "react";
import {connect} from "react-redux";
import {bindActionCreators, Dispatch} from "redux";
import {returntypeof} from 'react-redux-typescript';
import {Form} from 'semantic-ui-react'
import MaterialSelectWrapper from '../material/materialSelectWrapper'
import {SelectOnChangeEventData} from '../../types/reactEvents'
import {paginationPageSizeOptions} from '../../constants'
import {getI18n, KnownLangs} from '../../../i18n/i18nRoot'
import {RootState} from '../../state/reducers'
import {SimpleVDivider} from './simpleVDivider'

export interface MyProps {
  readonly pageSize: number
  readonly totalItems: number
  readonly langId: KnownLangs
  readonly setPaginationPageSize: (pageSize: number) => void
}

const mapStateToProps = (rootState: RootState, props: MyProps) => {
  return {
    ...props
  }
}

const mapDispatchToProps = (dispatch: Dispatch<any>) => bindActionCreators({
                                                                             //imported reducer funcs here

                                                                           }, dispatch)


const stateProps = returntypeof(mapStateToProps);
const dispatchProps = returntypeof(mapDispatchToProps);
type Props = typeof stateProps & typeof dispatchProps;


class PaginationPageSizeSelect extends React.Component<Props, any> {
  render(): JSX.Element {
    return (
      <div className="flexed">


        <div className="ui form" style={{width: '45px'}}>
          <Form.Field>

            <MaterialSelectWrapper
              value={this.props.pageSize}
            >
              <select
                value={this.props.pageSize}
                onChange={(e: SelectOnChangeEventData) => {
                  const pageSize = parseInt(e.currentTarget.value, 10)

                  if (isNaN(pageSize)) return

                  this.props.setPaginationPageSize(pageSize)

                }}
              >
                {
                  paginationPageSizeOptions.map(pageSize => {
                    return (
                      <option key={pageSize} value={pageSize}>
                        {
                          pageSize === -1
                          ? getI18n(this.props.langId, "All")
                          : pageSize
                        }
                      </option>
                    )
                  })
                }
              </select>
            </MaterialSelectWrapper>
          </Form.Field>
        </div>

        <div style={{
          marginTop: '8px',
          marginRight: '3em'
        }}>
          {getI18n(this.props.langId, "Rows per page")}
        </div>

        <div style={{
          marginTop: '8px',
          marginRight: '2em'
        }}>
          {getI18n(this.props.langId, "Total items")}: {this.props.totalItems.toString()}
        </div>


      </div>
    )
  }
}

const _comp = connect(mapStateToProps, mapDispatchToProps)(PaginationPageSizeSelect)
export default _comp
