/**
 * Created by janis dähne.
 */

import * as React from "react";
import {connect} from "react-redux";
import {bindActionCreators, Dispatch} from "redux";
import {returntypeof} from 'react-redux-typescript';
import {RootState} from "../../state/reducers";
import {Button, Dimmer, Loader, Progress} from "semantic-ui-react";
import {getI18n} from "../../../i18n/i18nRoot";

//const css = require('./styles.styl');

export interface MyProps {
  //readonly test: string
  readonly opacity?: number


  /**
   * displays a loading bar if set
   * active will show 100% but with progress 'animation'
   */
  readonly percentage?: number | null | 'active'
  /**
   * if this is set a cancel button is displayed to cancel the operation
   */
  readonly onCancel?: (() => void) | null

}

const mapStateToProps = (rootState: RootState , props: MyProps) => {
  return {
    //test0: rootState...
    //test: props.test
    ...props,
    langId: rootState.i18nState.langId
  }
}

const mapDispatchToProps = (dispatch: Dispatch) => bindActionCreators({
  //imported reducer funcs here

}, dispatch)


const stateProps = returntypeof(mapStateToProps);
const dispatchProps = returntypeof(mapDispatchToProps);
type Props = typeof stateProps & typeof dispatchProps;

class Spinner extends React.Component<Props, any> {
  render(): JSX.Element {
    return (
      <div>
        <Dimmer active inverted style={{opacity: this.props.opacity}} >
          <Loader inverted content={getI18n(this.props.langId, 'Loading ...')} />

          {
            (this.props.percentage !== undefined && this.props.percentage !== null) &&
              <div style={{position: 'absolute', top: '60%', left: '10%', right: '10%'}}>

                {
                  typeof this.props.percentage === 'number'
                  ? <Progress progress="percent" percent={this.props.percentage}  />
                  : <Progress percent={100} active />
                }


                {
                  (this.props.onCancel !== undefined && this.props.onCancel !== null) &&
                  <div>
                    <Button
                      content={getI18n(this.props.langId, 'Cancel')}
                      onClick={() => this.props.onCancel()}
                    >
                    </Button>
                  </div>
                }

              </div>
          }



        </Dimmer>
      </div>
    )
  }
}

export default connect(mapStateToProps, mapDispatchToProps)(Spinner)
