import * as React from "react";
import {Button, ButtonProps, Icon, Table} from "semantic-ui-react";
import {RootState} from "../../../state/reducers";
import {bindActionCreators, Dispatch} from "redux";
import {connect} from "react-redux";
import {getI18n} from "../../../../i18n/i18nRoot";
import {CSSProperties} from 'react'
import {Link} from "react-router-dom"
import * as constants from '../../../constants'
import {HelpPopup} from '../helpPopup'
import {MyPopup} from '../myPopup'
import {returntypeof} from 'react-redux-typescript'


export interface MyProps {
}

const mapStateToProps = (rootState: RootState) => {
  return {

    testServerPublicSettings: rootState.globalSystemSettingsState.testServerPublicSettings,
    langId: rootState.i18nState.langId
  }
}

const mapDispatchToProps = (dispatch: Dispatch) => bindActionCreators({}, dispatch)

const stateProps = returntypeof(mapStateToProps);
const dispatchProps = returntypeof(mapDispatchToProps);
type Props = typeof stateProps & typeof dispatchProps;

//className={'ok-bg-colored white-colored ' + this.props.className}
class TestServerHints extends React.Component<Props & ButtonProps, any> {
  render(): JSX.Element {

    return (<div>

      <Link target="_blank" to={constants.markdownSiteTestSyntaxGuide}
            style={{marginTop: '0.5em'}}
      >
        <Icon className="mar-right-half" name="external share"/>
        <span>
                {getI18n(this.props.langId, 'Open help in new tab')}
              </span>
      </Link>

      <span className="mar-left">{getI18n(this.props.langId, "Test-Server Notes")}</span>

      <HelpPopup style={{marginTop: '0.5em'}} className="mar-left-half" defaultText={getI18n(this.props.langId,
                                                                                             "You can write to the stderr (with java System.err.println and use it as debug output) and it will be ignored by the test runner (the test will pass if all System.out.println (stdout) are correct). However if you write to stdout and then to stderr the output might print the stderr line before the stdout line, so the order of the lines is only correct for all stdout separately and all stderr separately. The test runner replaced every 0x00 char (e.g. when you print a (char)0 or an not initialized char-array entry) and replaces it with ☐. You can also ouput 3/4 bit ansi color codes via \\033[CODEm (see wikipedia ansi escape code, ubuntu colors are used). Colors are only printed on just run program or if the output was correct. Note that the colors are only set for one line (not like the terminal). After the line the style is reset to normal."
      )} wide="very"/>


      {
        //display all testServerPublicSettings
      }
      <span className="mar-left">{getI18n(this.props.langId, "Test-Server Limits")}</span>

      <MyPopup
        noHorizontalOffset={true}
        trigger={
          <Icon style={{marginTop: '0.5em'}} className="hoverable mar-left-half" name="question circle"/>
        }
        position="bottom center"
        content={
          <div>
            <Table style={{width: '600px'}} sortable selectable padded striped className="material-table">
              <Table.Header>
                <Table.Row>
                  <Table.HeaderCell style={{width: '200px'}}>
                    {getI18n(this.props.langId, "Setting")}
                  </Table.HeaderCell>
                  <Table.HeaderCell>
                    {getI18n(this.props.langId, "Value")}
                  </Table.HeaderCell>
                  <Table.HeaderCell>
                    {getI18n(this.props.langId, "Description")}
                  </Table.HeaderCell>
                </Table.Row>
              </Table.Header>

              <Table.Body>

                <Table.Row>
                  <Table.Cell>
                    {getI18n(this.props.langId, "Timeout in ms")}
                  </Table.Cell>
                  <Table.Cell>
                    {
                      this.props.testServerPublicSettings.hardTimeoutInMs
                    }
                  </Table.Cell>
                  <Table.Cell>
                     <span>
                    {
                      getI18n(this.props.langId, "The max time the user program is allowed to run in ms")
                    }
                  </span>
                  </Table.Cell>
                </Table.Row>

                <Table.Row>
                  <Table.Cell>
                    {getI18n(this.props.langId, "Max. RAM in kb")}
                  </Table.Cell>
                  <Table.Cell>
                    {
                      this.props.testServerPublicSettings.hardMemoryLimitInKb
                    }
                  </Table.Cell>
                  <Table.Cell>
                  </Table.Cell>
                </Table.Row>

                <Table.Row>
                  <Table.Cell>
                    {getI18n(this.props.langId, "Max disk space in kb")}
                  </Table.Cell>
                  <Table.Cell>
                    {
                      this.props.testServerPublicSettings.hardDiskSpaceLimitInKb
                    }
                  </Table.Cell>
                  <Table.Cell>
                  </Table.Cell>
                </Table.Row>

                <Table.Row>
                  <Table.Cell>
                    {getI18n(this.props.langId, "Max. tests with one request")}
                  </Table.Cell>
                  <Table.Cell>
                    {
                      this.props.testServerPublicSettings.maxNumberOfTestsWithOneRequest
                    }
                  </Table.Cell>
                  <Table.Cell>
                     <span>
                    {
                      getI18n(this.props.langId, "If one runs multiple tests this is the number of tests that are actually run with one requests. If there are tests left one must issue a new requests to run all tests")
                    }
                  </span>
                  </Table.Cell>
                </Table.Row>

              </Table.Body>
            </Table>

          </div>
        }
      />


    </div>)
  }
}

export default connect(mapStateToProps, mapDispatchToProps)(TestServerHints)
