/**
 * Created by janis dähne.
 */

import * as React from "react";
import {connect} from "react-redux";
import {bindActionCreators, Dispatch} from "redux";
import {returntypeof} from 'react-redux-typescript';
import {RootState} from "../../state/reducers";
import {Button, Icon, Divider, List, Label} from "semantic-ui-react";
import {default as Logger, SeverityLevel} from "../../helpers/logger";
import {setIsLoggerPanelOpen, setSelectedLogLevel} from "../../state/actions/loggerPanelActions";
import * as moment from "moment";
import {displayDateAndTime} from "../../helpers/convertersAndTransformers";
import reduxState from '../../state/state'
import {FrontendSettingsManager} from "../../helpers/frontendSettingsManager";
import {AceEditorsStorageHelper} from '../codeEditors/aceEditors/aceEditorsStorageHelper'
import {isProduction} from '../../../debug'

//const css = require('./styles.styl');

declare var require: any
const fileSaver = require('file-saver');

export interface MyProps {
  //readonly test: string
  readonly isCollapsed: boolean
}

const mapStateToProps = (rootState: RootState, props: MyProps) => {
  return {
    //test0: rootState...
    //test: props.test
    isCollapsed: props.isCollapsed,
    //log: rootState.loggerPanelState.log,
    logLevel: rootState.loggerPanelState.selectedLogLevel,
  }
}

const mapDispatchToProps = (dispatch: Dispatch) => bindActionCreators({
  //imported reducer funcs here
  setSelectedLogLevel,
  setIsLoggerPanelOpen,
}, dispatch)


const stateProps = returntypeof(mapStateToProps);
const dispatchProps = returntypeof(mapDispatchToProps);
type Props = typeof stateProps & typeof dispatchProps;

class LoggerPanel extends React.Component<Props, any> {

  componentDidMount(): void {
    // Logger.log('mounted')
    // Logger.warn('test')
    // Logger.error('test')
    // Logger.debug('logger', 'test')
  }

  render(): JSX.Element {


    const log = Logger.myLog

    const lastRenderedAt = moment()

    return (
      <div className="logger-panel" style={this.props.isCollapsed ? {display: 'none'} : null}>

        <div className="relative-positioned">
          <div className="log-level-buttons">

            <div className="mar-right">
              <Button icon onClick={() => {
                //force rerender
                this.forceUpdate()
              }}>
                <Icon size="large" name="refresh"
                />
              </Button>

              {
                //download the whole redux state + logger log as json
              }
              <Button icon onClick={() => {

                Logger.log('exporting redux state & log')

                //all times are utc... even log item times somehow...

                //not inlined in case we throw here and log another item...
                //i'm not sure if this also works inlined during obj creation
                const frontendSettings = FrontendSettingsManager.getFrontendSettings()
                const reduxStateWithLog = {
                  state: reduxState.getState(),
                  log: Logger.myLog,
                  frontendSettings: frontendSettings,
                  aceEditorsStorage: AceEditorsStorageHelper.exportAll()
                }

                const value = JSON.stringify(reduxStateWithLog, null, "\t")

                // noinspection TsLint
                let fileName = `debugYAPEX_${lastRenderedAt.format('YYYY-MM-DD_HH-mm-ss')}.json`

                const blob = new Blob([value], {type: "application/json;charset=utf-8"});
                fileSaver.saveAs(blob, fileName)

                //refresh to display the exporting log item
                this.setState(this.state)

              }}>
                <Icon size="large" name="download"
                />
              </Button>

            </div>

            <Button.Group>
              <Button className="log-filter-item" icon onClick={() => this.props.setSelectedLogLevel(null)}>
                <Icon size="large" name="asterisk"
                      className={this.props.logLevel === null ? 'accent-colored' : null}
                />
                <Label floating>
                  {
                    log.length
                  }
                </Label>
              </Button>
              <Button className="log-filter-item" icon
                      onClick={() => this.props.setSelectedLogLevel(SeverityLevel.info)}>
                <Icon size="large" name="align justify"
                      className={this.props.logLevel === SeverityLevel.info ? 'accent-colored' : null}
                />
                <Label floating>
                  {
                    log.filter(p => p.level === SeverityLevel.info).length
                  }
                </Label>
              </Button>
              <Button className="log-filter-item" icon
                      onClick={() => this.props.setSelectedLogLevel(SeverityLevel.warning)}>
                <Icon size="large" name="exclamation triangle"
                      className={this.props.logLevel === SeverityLevel.warning ? 'accent-colored' : null}
                />
                <Label className="error-colored" floating>
                  {
                    log.filter(p => p.level === SeverityLevel.warning).length
                  }
                </Label>
              </Button>
              <Button className="log-filter-item" icon
                      onClick={() => this.props.setSelectedLogLevel(SeverityLevel.error)}>
                <Icon size="large" name="remove circle"
                      className={this.props.logLevel === SeverityLevel.error ? 'accent-colored' : null}
                />
                <Label className="error-colored" floating>
                  {
                    log.filter(p => p.level === SeverityLevel.error).length
                  }
                </Label>
              </Button>
              <Button className="log-filter-item" icon
                      onClick={() => this.props.setSelectedLogLevel(SeverityLevel.debug)}>
                <Icon size="large" name="bug"
                      className={this.props.logLevel === SeverityLevel.debug ? 'accent-colored' : null}
                />
                <Label className="error-colored" floating>
                  {
                    log.filter(p => p.level === SeverityLevel.debug).length
                  }
                </Label>
              </Button>
            </Button.Group>


            <div className="log-timestamp mar-left">
              <span>
                {
                  displayDateAndTime(lastRenderedAt, -1)
                }
              </span>
            </div>

          </div>

          <div style={{position: 'absolute', right: '0.5em', top: '0.5em'}}>
            <Button circular icon onClick={() => this.props.setIsLoggerPanelOpen(false)}>
              <Icon name="close"/>
            </Button>
          </div>
        </div>

        <Divider/>

        <div className="log-list">
          <List divided relaxed>
            {
              //this.props.logLevel === null <--> this.props.logLevel ? should display all items
              log
                .filter(p => (this.props.logLevel === null ? true : false) || p.level === this.props.logLevel)
                .map((logItem, index) => {
                  return (
                    <List.Item key={logItem.id} className="log-item">
                      <List.Icon name={
                        logItem.level === SeverityLevel.debug ? 'bug' :
                          logItem.level === SeverityLevel.error ? 'remove circle' :
                            logItem.level === SeverityLevel.warning ? 'exclamation triangle' :
                              logItem.level === SeverityLevel.info ? null : null
                      }
                                 className={
                                   logItem.level === SeverityLevel.debug ? 'ok-colored' :
                                     logItem.level === SeverityLevel.error ? 'error-colored' :
                                       logItem.level === SeverityLevel.warning ? 'warning-colored' :
                                         logItem.level === SeverityLevel.info ? '' : ''
                                 }
                      >
                      </List.Icon>
                      <List.Content>
                        <span>{`${logItem.file ? `[${logItem.file}] ` : ''}${!logItem.isSilent || !isProduction ? logItem.message : '(hidden)'}`}</span>
                      </List.Content>
                    </List.Item>
                  )
                })
            }
          </List>
        </div>

      </div>
    )
  }
}

export default connect(mapStateToProps, mapDispatchToProps)(LoggerPanel)
