/**
 * Created by janis dähne.
 */

import * as React from "react";
import {connect} from "react-redux";
import {bindActionCreators, Dispatch} from "redux";
import {returntypeof} from 'react-redux-typescript';
import {RootState} from "../../state/reducers";

import LoginForm from './loginForm'
import {withRouter} from "react-router";
import RegisterFrom from './registerForm'
import {getCsrfTokenAndIsLoggedIn, getUserData} from "../../communicationLayer/loginLayer";
import {UserDataFullBase} from "../../types/userData";
import {setUserData} from "../../state/actions/settingsSite/userSettingsSiteActions";
import {reset, setIsCheckingAlreadyLoggedIn} from "../../state/actions/loginActions";
import {delay} from "../../../debug";
import {
  getGlobalSystemSettingsAsync,
  getTestServerPublicSettingsAsync
} from '../../state/actions/globalSystemSettings/globalSystemSettingsCrudActions'
import {getI18n} from '../../../i18n/i18nRoot'
import * as constants from '../../constants'
import {Icon} from 'semantic-ui-react'

//const css = require('./styles.styl');

export interface MyProps {
  //readonly test: string
}

const mapStateToProps = (rootState: RootState /*, props: MyProps*/) => {
  return {
    //test0: rootState...
    //test: props.test
    userId: rootState.userDataSettingsSate.userData.id,
    isCheckingAlreadyLoggedIn: rootState.loginDataState.isCheckingAlreadyLoggedIn,
    isRegisterFormDisplayed: rootState.loginDataState.isRegisterFormDisplayed,

    langId: rootState.i18nState.langId,
  }
}

const mapDispatchToProps = (dispatch: Dispatch) => bindActionCreators({
                                                                        //imported reducer funcs here
                                                                        setUserData,
                                                                        setIsCheckingAlreadyLoggedIn,
                                                                        getGlobalSystemSettingsAsync,
                                                                        getTestServerPublicSettingsAsync,

  reset,
                                                                      }, dispatch)


const stateProps = returntypeof(mapStateToProps);
const dispatchProps = returntypeof(mapDispatchToProps);
type Props = typeof stateProps & typeof dispatchProps;

class LoginOverlay extends React.Component<Props, any> {


  async componentWillMount(): Promise<void> {

    //try to get the user data...
    //if success the user is still logged in

    // noinspection TsLint
    let userData: UserDataFullBase = null
    // noinspection TsLint
    let isAlreadyLoggedIn = false

    this.props.setIsCheckingAlreadyLoggedIn(true)
    try {
      await delay(getCsrfTokenAndIsLoggedIn())
      isAlreadyLoggedIn = true
    } catch (err) {
      isAlreadyLoggedIn = false
    } finally {

    }


    if (isAlreadyLoggedIn) {
      try {
        userData = await delay(getUserData())

        //get the server message if we not log in but already logged in
        await this.props.getGlobalSystemSettingsAsync()

        await this.props.getTestServerPublicSettingsAsync()

      } catch (err) {

      }

      if (userData === null) {
        //some error...
      } else {

        //set user data --> login will disappear
        this.props.setUserData(userData)
      }
    }


    this.props.setIsCheckingAlreadyLoggedIn(false)

    this.props.reset()

  }

  render(): JSX.Element {
    return (
      <div className="fh">
        {

          (this.props.userId <= 0) &&
          <div className="fh login-screen-animated">
            {
              !this.props.isRegisterFormDisplayed &&
              <LoginForm/>
            }

            {
              this.props.isRegisterFormDisplayed &&
              <RegisterFrom/>
            }

            <div className="branding-logo">
              <img src="mluLogo.png" alt="mluLogo"/>
            </div>

            <div className="login-overlay-bottom-bar">
              <div className="frosted-glass">
                <a href={constants.privacyPolicyPageLinkPath} target="_blank">
                    <span className="mar-left-half">
                      {
                        getI18n(this.props.langId, 'Privacy Policy')
                      }
                    </span>
                  <Icon className="mar-left-half" name="external alternate"/>
                </a>

                <a href={constants.legalNoticePageLinkPath} target="_blank">
                    <span className="mar-left-half">
                      {
                        getI18n(this.props.langId, 'Legal Notice')
                      }
                    </span>
                  <Icon className="mar-left-half" name="external alternate"/>
                </a>

                <a href={constants.userGuideLinkPath} target="_blank">
                  <span className="mar-left-half">
                    {
                      getI18n(this.props.langId, 'Documentation')
                    }
                  </span>
                    <Icon className="mar-left-half" name="external alternate"/>
                </a>

                <a href={constants.aboutPageLinkPath} target="_blank">
                    <span className="mar-left-half">
                      {
                        getI18n(this.props.langId, 'About page')
                      }
                    </span>
                  <Icon className="mar-left-half" name="external alternate"/>
                </a>
              </div>


            </div>
          </div>
        }

        {
          //real content after login
          this.props.userId > 0 &&
          this.props.children
        }

      </div>
    )
  }
}

export default withRouter(connect(mapStateToProps, mapDispatchToProps)(LoginOverlay))
