/**
 * Created by janis dähne.
 */

import * as React from "react";
import {SimpleDivider} from "../../helpers/simpleDivider";
import ScrollPositionManager from '../../helpers/scrollPosManager'

/**
 * props are propagated to the panel
 * @see Panel
 */
interface Props {
  readonly icon?: string
  readonly iconClass?: string
  readonly title: string
  readonly defaultHelpText?: string
  //cannot be checked at compile time to be of type tabActionsLine ...
  //see https://stackoverflow.com/questions/42955400/in-typescript-is-there-any-way-to-typecheck-passed-in-jsx-elements
  //see https://github.com/Microsoft/TypeScript/issues/13618
  readonly tabHeaderBar?: JSX.Element
  readonly tabFooterBar?: JSX.Element
  readonly applyDefaultPadding?: boolean
  readonly disabled?: boolean

}


export default class TabContentHost extends React.Component<Props, any> {

  render(): JSX.Element {

    const applyDefaultPadding = this.props.applyDefaultPadding === undefined || this.props.applyDefaultPadding === true

    return (
      <div
        className={`fh tab-content-host ${applyDefaultPadding ? null : 'tab-content-host-manual-padding'} ${this.props.disabled ? 'div-disabled' : ''}`}>
        {
          this.props.tabHeaderBar &&
          this.props.tabHeaderBar
        }

        {
          this.props.tabHeaderBar &&
          <SimpleDivider/>
        }


        <ScrollPositionManager scrollKey={this.props.title} scrolledChild={(connectScrollTarget) =>
          <div ref={connectScrollTarget} className="tab-content-host-content">
            {
              this.props.children
            }
          </div>}/>
        {
          this.props.tabFooterBar &&
          this.props.tabFooterBar
        }
      </div>
    )
  }
}
