/**
 * Created by janis dähne.
 */

import * as React from "react";
import {connect} from "react-redux";
import {bindActionCreators, Dispatch} from "redux";
import {returntypeof} from 'react-redux-typescript';
import {RootState} from "../../state/reducers";
import {MyProps} from './renderWrapper'
import {createAndRefreshAssetUrlsAndObjects, getObjectUrlForAsset} from "../../helpers/assetHelper";
import Logger from "../../helpers/logger";
import {assetPrefix} from '../../constants'

//const css = require('./styles.styl');

const mapStateToProps = (rootState: RootState , props: MyProps) => {
  return {
    //test0: rootState...
    //test: props.test
    contentType: props.contentType,
    content: props.content,
    assets: props.assets,
    dummy: props.dummy,
  }
}

const mapDispatchToProps = (dispatch: Dispatch) => bindActionCreators({
  //imported reducer funcs here

}, dispatch)


const stateProps = returntypeof(mapStateToProps);
const dispatchProps = returntypeof(mapDispatchToProps);
type Props = typeof stateProps & typeof dispatchProps;

class pdfRenderer extends React.Component<Props, any> {

  mapAssetUrlFilter(assetUrl:string):string {
    //this should be the displayContentControl

    //see https://html5sec.org/
    if (!assetUrl.startsWith(assetPrefix)) {
        Logger.warn('found invalid asset url... not starting with asset prefix')
        return ''
    }
    if (assetUrl.indexOf("data:") !== -1 || assetUrl.indexOf("base64") !== -1 || assetUrl.indexOf("javascript:") !== -1) {
      Logger.warn('found invalid asset url.... containing data or base64 or javascript')
      return ''
    }



    // noinspection TsLint
    for (let i = 0; i < this.props.assets.length; i++) {
      const asset = this.props.assets[i];

      if (`${assetPrefix}${asset.id}` === assetUrl) {
        const url = getObjectUrlForAsset(asset)

        if (!url) continue

        if (!url.startsWith("blob:")) {
          Logger.warn('found invalid translated asset url... not starting with blob')
          return ''
        }

        return url
      }
    }
    return ''
  }

  render(): JSX.Element {

    Logger.debug("pdfRenderer", "rendering...")

    return (
      //pdf cannot be printed... so not set printId!
      <div className="fh">
        <embed id="display-content-pdf" width="100%" height="100%" src={this.mapAssetUrlFilter(this.props.content)} />
      </div>
    )
  }
}

export default connect(mapStateToProps, mapDispatchToProps)(pdfRenderer)
