/**
 * Created by janis dähne.
 */

import * as React from "react";
import {connect} from "react-redux";
import {bindActionCreators, Dispatch} from "redux";
import {returntypeof} from 'react-redux-typescript';
import {RootState} from "../../state/reducers";
import MarkdownRenderer from './markdownRenderer'
import PdfRenderer from './pdfRenderer'
import TextRenderer from './textRenderer'
import {QuestionType} from "../../types/questions";
import {FileWithData} from "../../types/exerciseEditor";

//const css = require('./styles.styl');

export interface MyProps {
  //readonly test: string
  readonly contentType: QuestionType
  readonly content: string | null
  readonly assets: ReadonlyArray<FileWithData>

  /**
   * used to toggle re render
   * e.g. when assets are finished creating & math jax should be rendered
   */
  readonly dummy: boolean

  readonly fontSizeInPx: number

  /**
   * id used for printing
   */
  readonly printId: string

  /**
   * some (markdown) blocks might be only for specific languages,
   * use null to show them all
   */
  readonly plangId: number | null
}

const mapStateToProps = (rootState: RootState, props: MyProps) => {
  return {
    //test0: rootState...
    //test: props.test
    ...props
  }
}

const mapDispatchToProps = (dispatch: Dispatch) => bindActionCreators({
  //imported reducer funcs here

}, dispatch)


const stateProps = returntypeof(mapStateToProps);
const dispatchProps = returntypeof(mapDispatchToProps);
type Props = typeof stateProps & typeof dispatchProps;


//support mermaid https://stackoverflow.com/questions/28626329/marked-markdown-mermaid-flow-chart-diagrams

class RenderWrapper extends React.Component<Props, any> {
  render(): JSX.Element {
    return (
      <div className="fh full-w">

        {
          this.props.contentType === QuestionType.text &&
          <TextRenderer {...this.props} />
        }

        {
          this.props.contentType === QuestionType.markdown &&
          <MarkdownRenderer {...this.props} />
        }

        {
          this.props.contentType === QuestionType.pdf &&
          <PdfRenderer {...this.props} />
        }
      </div>
    )
  }
}

export default connect(mapStateToProps, mapDispatchToProps)(RenderWrapper)
