/**
 * Created by janis dähne.
 */

import * as React from "react";
import {connect} from "react-redux";
import {bindActionCreators, Dispatch} from "redux";
import {returntypeof} from 'react-redux-typescript';
import {RootState} from "../../../../state/reducers";
import EnterReleaseCodeView from './enterReleaseCodeView'
import {Button} from "semantic-ui-react";
import CancelButton from "../../../buttons/cancelButton";
import {setIsEnterReleaseCodeDialogDisplayed} from "../../../../state/actions/allAccessibleExercises/openExercisesSite/openExercisesSiteActions";
import {
  accessExerciseWithCodeAsync,
  resetEnterReleaseCodeDialog
} from "../../../../state/actions/allAccessibleExercises/openExercisesSite/enterReleaseCodeDialog/enterReleaseCodeDialogActions";
import {getI18n} from "../../../../../i18n/i18nRoot";
import history from '../../../../routerHistory'
import {getDoExerciseSiteToLink} from '../../../../constants'
import Spinner from '../../../helpers/spinner'

//const css = require('./styles.styl');

export interface MyProps {
  //readonly test: string
}

const mapStateToProps = (rootState: RootState /*, props: MyProps*/) => {
  return {
    //test0: rootState...
    //test: props.test
    langId: rootState.i18nState.langId,
    isBusy: rootState.enterReleaseCodeDialogState.isBusy,
    dialogState: rootState.enterReleaseCodeDialogState,
  }
}

const mapDispatchToProps = (dispatch: Dispatch) => bindActionCreators({
                                                                        //imported reducer funcs here
                                                                        setIsEnterReleaseCodeDialogDisplayed,
                                                                        resetEnterReleaseCodeDialog,
                                                                        accessExerciseWithCodeAsync
                                                                      }, dispatch)


const stateProps = returntypeof(mapStateToProps);
const dispatchProps = returntypeof(mapDispatchToProps);
type Props = typeof stateProps & typeof dispatchProps;

class enterReleaseCodeDialog extends React.Component<Props, any> {

  componentWillMount(): void {
    this.props.resetEnterReleaseCodeDialog()
  }

  render(): JSX.Element {
    return (
      <div>

        {
          this.props.isBusy &&
          <Spinner/>
        }

        <div className="view-padding">
          <div>
            <div className="view-caption">
              <h2>
                {
                  getI18n(this.props.langId, 'Enter release code')
                }
              </h2>
            </div>
          </div>

          <div className="view-content">
            <EnterReleaseCodeView/>

            <div className="flexed" style={{marginTop: '1em'}}>
              <Button icon='arrow right' labelPosition='left' color="green"
                      className={['mar-right', (this.props.dialogState.isValid ? null : 'div-disabled')].join(' ')}
                      onClick={async () => {

                        //we can just show the error message and display that the user already used the release code...without a redirect
                        //BUT if the release code the user can easily enter the url in the browser...
                        //we only need to take care if the exercise is only visible to the owner
                        //if the release is hidden in overviews the link still works so this is ok
                        try {
                          await this.props.accessExerciseWithCodeAsync(this.props.dialogState.releaseCode.trim())
                          history.push(getDoExerciseSiteToLink(this.props.dialogState.releaseCode.trim()))
                        } catch (err) {
                          //error message is not shown here but in accessExerciseWithCodeAsync
                        }

                      }}
                      content={
                        getI18n(this.props.langId, "Open")
                      }
              />

              <CancelButton onClick={() => {
                this.props.resetEnterReleaseCodeDialog()
                this.props.setIsEnterReleaseCodeDialogDisplayed(false)
              }}/>
            </div>

          </div>
        </div>
      </div>
    )
  }
}

export default connect(mapStateToProps, mapDispatchToProps)(enterReleaseCodeDialog)
