/**
 * Created by janis dähne.
 */

import * as React from "react";
import {connect} from "react-redux";
import {bindActionCreators, Dispatch} from "redux";
import {returntypeof} from 'react-redux-typescript';
import {RootState} from "../../../../state/reducers";
import {Checkbox, Icon, Table} from "semantic-ui-react";
import {ReleaseDurationType} from "../../../../types/ReleaseDurationType";
import {ReleaseStartType} from "../../../../types/ReleaseStartType";
import {ExercisePreviewFromBackend} from "../../../../types/exercisePreview";
import {isCurrentSortingColumnPaginated, rotateSorting} from "../../../../helpers/tablesHelper";
import {copyToClipboard} from "../../../../helpers/clipboard";
import {Link} from "react-router-dom";
import {getDoExerciseSiteToLink} from "../../../../constants";
import {ExercisePreviewsFrontendExclusive} from "./allOpenExercisesSite";
import {displayDateAndTime} from "../../../../helpers/convertersAndTransformers";
import Tag from '../../../tagComponents/tag'
import {HelpPopup} from "../../../helpers/helpPopup";
import {getI18n} from "../../../../../i18n/i18nRoot";
import {PaginationStateData} from '../../../../types/pagination'
import PaginationTableFooter from '../../../helpers/paginationTableFooter'

//const css = require('./styles.styl');

export interface MyProps {
  //readonly test: string
  readonly exercisePreviews: ReadonlyArray<ExercisePreviewsFrontendExclusive>

  readonly pagination: PaginationStateData<ExercisePreviewFromBackend>

  /**
   * true: display the releases as they were accessed via code,
   * false: display the releases as they are always accessible/accessed (release is visible to all setting)
   */
  readonly displayAsViaCode: boolean

  readonly sortByKey: keyof ExercisePreviewFromBackend
  readonly sortDirection: boolean | null
  readonly setSortByKey: (sortByKey: keyof ExercisePreviewFromBackend, sortDirection:  boolean | null, viaCode: boolean) => void

  readonly setPaginationPageSize: (pageSize: number, viaCode: boolean) => void
  readonly setActivePaginationPage: (activePage: number, viaCode: boolean) => void

  /**
   * for the open via visibility exercises we don't get this data (always false) because the way we do the query on the backend
   * (user might not have a solution yet)
   */
  readonly hideShouldNotCountColumn: boolean

}

const mapStateToProps = (rootState: RootState, props: MyProps) => {
  return {
    //test0: rootState...
    //test: props.test
    ...props,
    langId: rootState.i18nState.langId,
    pLangs: rootState.pLangsState.pLangs,
  }
}

const mapDispatchToProps = (dispatch: Dispatch) => bindActionCreators({
  //imported reducer funcs here
}, dispatch)

const stateProps = returntypeof(mapStateToProps);
const dispatchProps = returntypeof(mapDispatchToProps);
type Props = typeof stateProps & typeof dispatchProps;

class AllAccessibleExercisesList extends React.Component<Props, any> {


  render(): JSX.Element {

    const list: ReadonlyArray<ExercisePreviewsFrontendExclusive> = this.props.exercisePreviews

    return (
      <div>

        <Table sortable selectable padded striped singleLine className="material-table">
          <Table.Header>
            <Table.Row>
              <Table.HeaderCell>
                <span>
                {
                  getI18n(this.props.langId,'Open')
                }
                </span>
              </Table.HeaderCell>
              <Table.HeaderCell
                sorted={
                  isCurrentSortingColumnPaginated<ExercisePreviewFromBackend>(this.props.sortByKey, 'displayName',
                    this.props.sortDirection)
                }
              >
                <span className="clickable"
                      onClick={() => {
                        this.props.setSortByKey('displayName',
                                                rotateSorting<ExercisePreviewFromBackend>(this.props.sortDirection,
                                                                                          this.props.sortByKey,
                                                                                          'displayName',
                                                ),
                                                this.props.displayAsViaCode
                        )
                      }}
                >
                  {
                    getI18n(this.props.langId,'Name')
                  }
                </span>
              </Table.HeaderCell>
              <Table.HeaderCell
                sorted={
                  isCurrentSortingColumnPaginated<ExercisePreviewFromBackend>(this.props.sortByKey, 'shortDescription',
                    this.props.sortDirection)
                }
              >
                <span className="clickable"
                      onClick={() => {
                        this.props.setSortByKey('shortDescription',
                                                rotateSorting<ExercisePreviewFromBackend>(this.props.sortDirection,
                                                                                          this.props.sortByKey,
                                                                                          'shortDescription',
                                                ),
                                                this.props.displayAsViaCode
                        )
                      }}
                >
                  {
                    getI18n(this.props.langId,'Description')
                  }
                </span>
              </Table.HeaderCell>
              <Table.HeaderCell
                sorted={
                  isCurrentSortingColumnPaginated<ExercisePreviewFromBackend>(this.props.sortByKey, 'lastUpdatedAt',
                    this.props.sortDirection)
                }
              >
                <span className="clickable"
                      onClick={() => {
                        this.props.setSortByKey('lastUpdatedAt',
                          rotateSorting<ExercisePreviewFromBackend>(this.props.sortDirection,
                            this.props.sortByKey,
                            'lastUpdatedAt',
                          ),
                          this.props.displayAsViaCode
                        )
                      }}
                >
                  {
                    getI18n(this.props.langId,'Last updated at')
                  }
                </span>
              </Table.HeaderCell>
              {/*<Table.HeaderCell*/}
              {/*  sorted={*/}
              {/*    isCurrentSortingColumnPaginated<ExercisePreviewFromBackend>(this.props.sortByKey, 'firstAccessedAt',*/}
              {/*      this.props.sortDirection)*/}
              {/*  }*/}
              {/*>*/}
              {/*  <span className="clickable"*/}
              {/*        onClick={() => {*/}
              {/*          this.props.setSortByKey('firstAccessedAt',*/}
              {/*            rotateSorting<ExercisePreviewFromBackend>(this.props.sortDirection,*/}
              {/*              this.props.sortByKey,*/}
              {/*              'firstAccessedAt',*/}
              {/*            ),*/}
              {/*            this.props.displayAsViaCode*/}
              {/*          )*/}
              {/*        }}*/}
              {/*  >*/}
              {/*    {*/}
              {/*      getI18n(this.props.langId,'First accessed at')*/}
              {/*    }*/}
              {/*  </span>*/}
              {/*</Table.HeaderCell>*/}
              {
                //don't make this sortable (backend) because getting this data this is costly...
              }
              <Table.HeaderCell>
                <span>
                  <HelpPopup icon="flag checkered"
                             defaultText={getI18n(this.props.langId,'Passed normal tests')}/>
                </span>
              </Table.HeaderCell>
              {
                //consecutive editor is only available after the release finished...
              }
              {/*<Table.HeaderCell>*/}
              {/*  <span>*/}
              {/*    <HelpPopup icon="life ring"*/}
              {/*               defaultText={getI18n(this.props.langId,'Passed normal tests (subsequent editor)')}/>*/}
              {/*  </span>*/}
              {/*</Table.HeaderCell>*/}
              {
                this.props.hideShouldNotCountColumn === false &&
                <Table.HeaderCell>
                <span>
                  <HelpPopup icon="flag outline"
                             defaultText={getI18n(this.props.langId,'Should not count')}/>
                </span>
                </Table.HeaderCell>
              }
              <Table.HeaderCell>
                <span>
                  <HelpPopup icon="lock"
                             defaultText={getI18n(this.props.langId,'Locked permanently')}/>
                </span>
              </Table.HeaderCell>

              <Table.HeaderCell>
                <span>
                  {
                    getI18n(this.props.langId,'Tags')
                  }
                </span>
              </Table.HeaderCell>
              <Table.HeaderCell
                sorted={
                  isCurrentSortingColumnPaginated<ExercisePreviewFromBackend>(this.props.sortByKey, 'releasedForPLangId',
                    this.props.sortDirection)
                }
              >
                <span className="clickable"
                      onClick={() => {
                        this.props.setSortByKey('releasedForPLangId',
                          rotateSorting<ExercisePreviewFromBackend>(this.props.sortDirection,
                            this.props.sortByKey,
                            'releasedForPLangId',
                          ),
                          this.props.displayAsViaCode
                        )
                      }}
                >
                  {
                    <HelpPopup icon="puzzle piece"
                               defaultText={getI18n(this.props.langId,'Programming language')}/>
                  }
                </span>
              </Table.HeaderCell>
              {
                this.props.displayAsViaCode &&
                <Table.HeaderCell>
                  <span>
                    {
                      getI18n(this.props.langId,'Release code')
                    }
                  </span>
                </Table.HeaderCell>
              }
              <Table.HeaderCell>
                <span>
                  <HelpPopup icon="wait"
                             defaultText={getI18n(this.props.langId,'Working time in min')}/>
                </span>
              </Table.HeaderCell>
              <Table.HeaderCell>
                <span>
                  {
                    getI18n(this.props.langId,'Time left')
                  }
                </span>
                <HelpPopup className="mar-left-half" defaultText={getI18n(this.props.langId, "Note that the minimum of left working time and automatic end is used")}/>
              </Table.HeaderCell>
              <Table.HeaderCell>
                <span>
                  {
                    getI18n(this.props.langId,'End')
                  }
                </span>
              </Table.HeaderCell>

            </Table.Row>
          </Table.Header>

          <Table.Body>
            {
              list.map((exercisePreview, index) => {

                let programmingLanguageForRelease = this.props.pLangs.find(p => p.id === exercisePreview.releasedForPLangId)

                //we can't use the id for the key because one exercise can have multiple releases...
                return (
                  <Table.Row key={exercisePreview.generatedCode}>
                    <Table.Cell>
                      <Link to={getDoExerciseSiteToLink(exercisePreview.generatedCode)} className="clickable">
                        <Icon name="share"/>
                      </Link>
                    </Table.Cell>
                    <Table.Cell>
                      <div className="flexed">
                        <HelpPopup className="mar-right-half" defaultText={exercisePreview.displayName}/>
                        <span>{exercisePreview.displayName}</span>
                      </div>
                    </Table.Cell>
                    <Table.Cell>
                      {exercisePreview.shortDescription &&
                      <HelpPopup defaultText={exercisePreview.shortDescription}/>
                      }
                    </Table.Cell>
                    <Table.Cell>
                      <span>
                        {
                          displayDateAndTime(exercisePreview.lastUpdatedAt, -1)
                        }
                      </span>
                    </Table.Cell>
                    {/*<Table.Cell>*/}
                    {/*  <span>*/}
                    {/*    {*/}
                    {/*      displayDateAndTime(exercisePreview.firstAccessedAt, -1)*/}
                    {/*    }*/}
                    {/*  </span>*/}
                    {/*</Table.Cell>*/}
                    <Table.Cell>
                      {
                        exercisePreview.hasUserSolution &&
                        <span className={exercisePreview.passedNormalTestsCount === exercisePreview.maxNormalTestsCount ? 'ok-colored' : ''}>
                          {exercisePreview.passedNormalTestsCount} / {exercisePreview.maxNormalTestsCount}
                        </span>
                      }

                    </Table.Cell>
                    {
                      //consecutive editor is only available after the release finished...
                    }
                    {/*<Table.Cell>*/}
                    {/*  {*/}
                    {/*    (exercisePreview.hasUserSolution && exercisePreview.subsequentEditorPassedNormalTestsCount !== null) &&*/}
                    {/*    <span className={exercisePreview.subsequentEditorPassedNormalTestsCount === exercisePreview.maxNormalTestsCount ? 'ok-colored' : ''}>*/}
                    {/*      {exercisePreview.subsequentEditorPassedNormalTestsCount} / {exercisePreview.maxNormalTestsCount}*/}
                    {/*    </span>*/}
                    {/*  }*/}
                    {/*</Table.Cell>*/}
                    {
                      this.props.hideShouldNotCountColumn === false &&
                      <Table.Cell>
                        <div>
                          <Checkbox checked={exercisePreview.shouldNotCount} disabled/>
                        </div>
                      </Table.Cell>
                    }
                    <Table.Cell>
                      <div>
                        <Checkbox checked={exercisePreview.isPermanentlyLocked} disabled/>
                      </div>
                    </Table.Cell>
                    <Table.Cell>
                      {
                        (exercisePreview.tags && exercisePreview.tags.length > 0) &&
                        <HelpPopup
                          wide="very"
                          defaultText={
                            <div className="exercise-tag-wrapper">
                              {
                                exercisePreview.tags.map((tag, index2) => {
                                  return (
                                    <Tag key={tag.id} tag={tag}/>
                                  )
                                })
                              }
                            </div>
                          }
                        />
                      }
                    </Table.Cell>
                    <Table.Cell>
                      {
                        programmingLanguageForRelease &&
                        <span>
                          { programmingLanguageForRelease.displayName }
                        </span>
                      }
                    </Table.Cell>
                    {
                      this.props.displayAsViaCode &&
                      <Table.Cell>
                        <Icon name="copy" className="clickable mar-right-half"
                              onClick={() => {
                                copyToClipboard(exercisePreview.generatedCode)
                              }}
                        />
                        <span>{exercisePreview.generatedCode}</span>
                      </Table.Cell>
                    }

                    {
                      //available working time
                    }
                    <Table.Cell>
                      {
                        (exercisePreview.hasLimitedWorkingTime ?
                            <span>{exercisePreview.availableWorkingTimeInMinutes}</span> :
                            <span>&infin;</span>
                        )
                      }
                    </Table.Cell>

                    {
                      //left time
                    }
                    <Table.Cell>
                      {
                        (
                          (exercisePreview.hasLimitedWorkingTime)
                          || ( exercisePreview.releaseStartType === ReleaseStartType.Automatic && exercisePreview.automaticEndAt !== null && exercisePreview.releaseDurationType === ReleaseDurationType.Limited) ?
                            <span>{exercisePreview.leftTimeString}</span> :
                            <span>&infin;</span>
                        )
                      }
                    </Table.Cell>

                    <Table.Cell>
                      {
                        exercisePreview.releaseStartType === ReleaseStartType.Manual ?
                          <span>{getI18n(this.props.langId,'Manual')}</span> : null
                      }
                      {
                        exercisePreview.releaseStartType === ReleaseStartType.Automatic
                        && exercisePreview.releaseDurationType === ReleaseDurationType.Infinitely ?
                        <span>{getI18n(this.props.langId,'Manual')}</span> : null
                      }
                      {
                        exercisePreview.releaseStartType === ReleaseStartType.Automatic
                        && exercisePreview.releaseDurationType === ReleaseDurationType.Limited ?
                          <span>{displayDateAndTime(exercisePreview.automaticEndAt, -1)}</span> : null
                      }
                    </Table.Cell>

                  </Table.Row>
                )
              })
            }

          </Table.Body>
          <PaginationTableFooter
            pagination={this.props.pagination}
            setPaginationPageSize={(pageSize) => {
              this.props.setPaginationPageSize(pageSize, this.props.displayAsViaCode)
            }}
            setActivePaginationPage={(activePage) => {
              this.props.setActivePaginationPage(activePage, this.props.displayAsViaCode)
            }}
          />
        </Table>


      </div>
    )
  }
}

export default connect(mapStateToProps, mapDispatchToProps)(AllAccessibleExercisesList)
