/**
 * Created by janis dähne.
 */

import * as React from "react";
import {connect} from "react-redux";
import {bindActionCreators, Dispatch} from "redux";
import {returntypeof} from 'react-redux-typescript';
import {RootState} from "../../../state/reducers";
import {RouteComponentProps, withRouter} from "react-router";
import {parseId} from "../../../helpers/convertersAndTransformers";
import {loadDoExerciseSite} from "../../../state/actions/assessmentStatisticsSite/assessmentStatisticsSiteActions";
import ListView from './listView'
import ChartView from './chartView/chartView'
import {getI18n} from '../../../../i18n/i18nRoot'
import {errorDialog} from '../../../helpers/dialogHelper'
import Logger from '../../../helpers/logger'
import {ErrorHelper} from '../../../helpers/errorHelper'

//const css = require('./styles.styl');

export interface MyProps {
  //readonly test: string
}

const mapStateToProps = (rootState: RootState /*, props: MyProps*/) => {
  return {
    //test0: rootState...
    //test: props.test
    langId: rootState.i18nState.langId,
    currentUserId: rootState.userDataSettingsSate.userData
      ? rootState.userDataSettingsSate.userData.id
      : -1,
  }
}

const mapDispatchToProps = (dispatch: Dispatch) => bindActionCreators({
  //imported reducer funcs here
  loadDoExerciseSite
}, dispatch)


const stateProps = returntypeof(mapStateToProps);
const dispatchProps = returntypeof(mapDispatchToProps);
type Props = typeof stateProps & typeof dispatchProps;


interface MatchedProps {
  readonly targetUserId?: string
}

class assessmentStatisticsSite extends React.Component<Props & RouteComponentProps<MatchedProps>, any> {

  async componentWillMount(): Promise<void> {

    // noinspection TsLint
    let targetUserId = parseId(this.props.match.params.targetUserId)

    if (!targetUserId || targetUserId <= 0) {
      // errorDialog(
      //   getI18n(this.props.langId,'Error'),
      //   getI18n(this.props.langId,'Some url parameters were not found/set')
      // )
      // return

      targetUserId = this.props.currentUserId
    }

    try {
        await this.props.loadDoExerciseSite(targetUserId)
    } catch(err) {
      ErrorHelper.makeServerErrorDialog(err, this.props.langId, 'Error', 'Could not load site data')
    }

  }

  render(): JSX.Element {

    // if (!this.props.assessmentStatistics) {
    //   return null
    // }

    return (
      <div className="multiple-single-panels">

        <ChartView/>

        <ListView/>

      </div>
    )
  }
}

export default withRouter(connect(mapStateToProps, mapDispatchToProps)(assessmentStatisticsSite))