/**
 * Created by janis dähne.
 */

import * as React from "react";
import {TooltipProps} from "recharts";
import {AssessmentDiagramDataPoint, getAssessmentChartDataId} from "./chartView";
import {SingleAssessmentStatistic} from "../../../../types/assessmentStatistics";
import {displayDateAndTime} from "../../../../helpers/convertersAndTransformers";
import {Icon} from "semantic-ui-react";
import {getI18n, KnownLangs} from "../../../../../i18n/i18nRoot";

//const css = require('./styles.styl');

export interface MyProps {
  //readonly test: string
  readonly allRawData: ReadonlyArray<SingleAssessmentStatistic>
  readonly displaySubmitTestLine: boolean
  readonly displayNormalTestLine: boolean
  readonly displayManualPointsLine: boolean
  readonly langId: KnownLangs
}

interface PayloadObj {
  readonly payload: AssessmentDiagramDataPoint
  readonly color: string
}

export const AssessmentTooltip: React.SFC<MyProps & TooltipProps> = (props) => {

  const tooltipOriginalDataList = ((props.payload as ReadonlyArray<any>) as ReadonlyArray<PayloadObj>)
  // noinspection TsLint
  let tooltipTitle = ''
  // noinspection TsLint
  let submitTestData: SingleAssessmentStatistic = null
  // noinspection TsLint
  let submitTestPercentage = '0'
  // noinspection TsLint
  let normalTestData: SingleAssessmentStatistic = null
  // noinspection TsLint
  let normalTestPercentage = '0'

  // tslint:disable-next-line
  let manualPointsData: SingleAssessmentStatistic = null
  // noinspection TsLint
  let manualPointsPercentage: string | null = null

  // noinspection TsLint
  let submitPointsColor = 'black'
  // noinspection TsLint
  let normalPointsColor = 'black'

  // tslint:disable-next-line
  let manualPointsColor = 'black'

  // noinspection TsLint
  let exerciseDisplayName = ''

  // noinspection TsLint
  let lineIndex = 0

  // noinspection TsLint
  let shouldNotCount = false

  if (tooltipOriginalDataList.length > 0) {

    shouldNotCount = tooltipOriginalDataList[0].payload.shouldNotCount

    const firstRawData: SingleAssessmentStatistic = props.allRawData.find(
      p => getAssessmentChartDataId(p) === tooltipOriginalDataList[0].payload.id)

    tooltipTitle = firstRawData
                   ? displayDateAndTime(firstRawData.lastUpdatedAt, -1)
                   : props.label as any;

    //because we first draw normal lines the colors in tooltipOriginalDataList (for tooltipOriginalDataList[lineIndex])
    //are also in this order!!
    if (props.displayNormalTestLine) {
      normalTestData = props.allRawData.find(
        p => getAssessmentChartDataId(p) === tooltipOriginalDataList[lineIndex].payload.id)

      exerciseDisplayName = normalTestData.exerciseDisplayName

      // noinspection TsLint
      if (isNaN(tooltipOriginalDataList[lineIndex].payload.normalPointsPercentageValue)) {
        normalTestPercentage = ` (${getI18n(props.langId, 'Not assessed')})`
      } else {

        // noinspection TsLint
        if (shouldNotCount) {
          normalTestPercentage = ` (${tooltipOriginalDataList[lineIndex].payload.originalNormalPointsPercentageValue}%)`
        } else {
          normalTestPercentage = ` (${tooltipOriginalDataList[lineIndex].payload.normalPointsPercentageValue}%)`
        }
      }

      normalPointsColor = tooltipOriginalDataList[lineIndex].color
      lineIndex++
    }

    if (props.displaySubmitTestLine) {
      //normalTestData & submitTestData are the same objects
      submitTestData = props.allRawData.find(
        p => getAssessmentChartDataId(p) === tooltipOriginalDataList[lineIndex].payload.id)

      exerciseDisplayName = submitTestData.exerciseDisplayName

      // noinspection TsLint
      if (isNaN(tooltipOriginalDataList[lineIndex].payload.submitPointsPercentageValue)) {
        submitTestPercentage = ` (${getI18n(props.langId, 'Not assessed')})`
      } else {

        // noinspection TsLint
        if (shouldNotCount) {
          submitTestPercentage = ` (${tooltipOriginalDataList[lineIndex].payload.originalSubmitPointsPercentageValue}%)`
        } else {
          submitTestPercentage = ` (${tooltipOriginalDataList[lineIndex].payload.submitPointsPercentageValue}%)`
        }

      }

      submitPointsColor = tooltipOriginalDataList[lineIndex].color
      lineIndex++
    }

    if (props.displayManualPointsLine) {
      manualPointsData = props.allRawData.find(
        p => getAssessmentChartDataId(p) === tooltipOriginalDataList[lineIndex].payload.id)

      exerciseDisplayName = manualPointsData.exerciseDisplayName

      // const maxManualPoints =  //tooltipOriginalDataList[lineIndex].payload.maxManualPoints

      manualPointsPercentage = isNaN(tooltipOriginalDataList[lineIndex].payload.manualPointsPercentageValue)
                               ? null
                               : ` ${tooltipOriginalDataList[lineIndex].payload.manualPointsPercentageValue}%`

      if (manualPointsData.manualPoints === null && manualPointsData.maxManualPoints === null) {
        manualPointsPercentage = ` (${getI18n(props.langId, 'Not assessed')})`
      }

      manualPointsColor = tooltipOriginalDataList[lineIndex].color
      lineIndex++
    }


  }

  return (
    <div className="assessment-tooltip">
      <p className="recharts-tooltip-label">
        {
          tooltipTitle
        }
      </p>
      <ul className="recharts-tooltip-item-list assessment-tooltip-ul">

        {
          exerciseDisplayName &&
          <div className="assessment-tooltip-exercise-name">
            {
              exerciseDisplayName
            }
          </div>
        }
        {
          shouldNotCount &&
          <li className="recharts-tooltip-item assessment-tooltip-inline-item"
          >
            <div style={{fontWeight: 'bold'}}>
              <Icon className="warning-colored" name="exclamation triangle"/>
              <span>
              {
                getI18n(props.langId, 'Should not count')
              }
            </span>
            </div>
          </li>
        }


        {
          props.displaySubmitTestLine &&
          <li className="recharts-tooltip-item assessment-tooltip-inline-item"
              style={{color: submitPointsColor}}
          >
              <span className="recharts-tooltip-item-name">
                {
                  getI18n(props.langId, 'Submit test points')
                }
              </span>
            <span className="recharts-tooltip-item-separator assessment-tooltip-inline-item-separator">
            :
          </span>
            <span className="recharts-tooltip-item-value">
            {
              //dummy assessments only have submitTestData.submitTestPoints == null
              submitTestData &&
              submitTestData.submitTestPoints + '/' +
              (submitTestData.submitTestPoints === null
               ? 'null'
               : submitTestData.maxSubmitTestPoints) +
              submitTestPercentage
            }
          </span>
          </li>
        }

        {
          props.displayNormalTestLine &&
          <li className="recharts-tooltip-item assessment-tooltip-inline-item"
              style={{color: normalPointsColor}}
          >
              <span className="recharts-tooltip-item-name">
                {
                  getI18n(props.langId, 'Normal test points')
                }
              </span>
            <span className="recharts-tooltip-item-separator assessment-tooltip-inline-item-separator">
            :
          </span>
            <span className="recharts-tooltip-item-value">
            {
              //dummy assessments only have submitTestData.normalTestPoints == null
              normalTestData &&
              normalTestData.normalTestPoints + '/' +
              (normalTestData.normalTestPoints === null
               ? 'null'
               : normalTestData.maxNormalTestPoints) +
              normalTestPercentage

            }
          </span>
          </li>
        }

        {
          props.displayManualPointsLine &&
          <li className="recharts-tooltip-item assessment-tooltip-inline-item"
              style={{color: manualPointsColor}}
          >
              <span className="recharts-tooltip-item-name">
                {
                  getI18n(props.langId, 'Manual points')
                }
              </span>
            <span className="recharts-tooltip-item-separator assessment-tooltip-inline-item-separator">
            :
          </span>
            <span className="recharts-tooltip-item-value">
            {
              //dummy assessments only have submitTestData.normalTestPoints == null
              manualPointsData &&
              <span>
                {
                  manualPointsData.manualPoints === null
                  ? 'null'
                  : manualPointsData.manualPoints
                }
                {
                  manualPointsData.maxManualPoints !== null
                  ? `/${manualPointsData.maxManualPoints}`
                  : null
                }
                {
                  manualPointsData.maxManualPoints !== null && manualPointsData.manualPoints !== null && manualPointsPercentage !== null
                  ? ` (${manualPointsPercentage})`
                  : null
                }
              </span>
            }
          </span>
          </li>
        }

      </ul>
    </div>
  )
}
