/**
 * Created by janis dähne.
 */

import * as React from "react";
import {connect} from "react-redux";
import {bindActionCreators, Dispatch} from "redux";
import {returntypeof} from 'react-redux-typescript';
import {RootState} from "../../../state/reducers";
import {isCurrentSortingColumnPaginated, rotateSorting} from "../../../helpers/tablesHelper";
import {Table} from "semantic-ui-react";
import {SingleAssessmentStatistic} from "../../../types/assessmentStatistics";
import {HelpPopup} from "../../helpers/helpPopup";
import {
  setActivePaginationPage,
  setPaginationPageSize,
  setSearchText,
  setSortByKey
} from "../../../state/actions/assessmentStatisticsSite/assessmentStatisticsSiteActions";
import ListViewItem from './listViewItem'
import Spinner from "../../helpers/spinner";
import MaterialSearchInput from '../../material/materialSearchInput'
import SinglePanelSiteWrapper from '../../singlePanelSiteWrapper'
import {getI18n} from "../../../../i18n/i18nRoot";
import {InputOnChangeEvent} from '../../../types/reactEvents'
import PaginationTableFooter from '../../helpers/paginationTableFooter'

//const css = require('./styles.styl');

export interface MyProps {
  //readonly test: string
}

const mapStateToProps = (rootState: RootState, props: MyProps) => {
  return {
    //test0: rootState...
    //test: props.test
    assessmentStatistics: rootState.assessmentStatisticsSiteState.assessmentStatistics,
    assessmentStatisticsList: rootState.assessmentStatisticsSiteState.assessmentStatistics ?
      rootState.assessmentStatisticsSiteState.assessmentStatistics.assessmentStatistics : [],

    sortByKey: rootState.assessmentStatisticsSiteState.pagination.orderByProp,
    sortDirection: rootState.assessmentStatisticsSiteState.pagination.isOrderByDesc,
    pagination: rootState.assessmentStatisticsSiteState.pagination,

    isLoading: rootState.assessmentStatisticsSiteState.isLoading
      || rootState.pLangsState.isLoading,

    currentUserId: rootState.userDataSettingsSate.userData
                   ? rootState.userDataSettingsSate.userData.id
                   : -1,
    langId: rootState.i18nState.langId
  }
}

const mapDispatchToProps = (dispatch: Dispatch) => bindActionCreators({
                                                                        //imported reducer funcs here
                                                                        setSortByKey,
                                                                        setSearchText,
                                                                        setActivePaginationPage,
                                                                        setPaginationPageSize
                                                                      }, dispatch)

const stateProps = returntypeof(mapStateToProps);
const dispatchProps = returntypeof(mapDispatchToProps);
type Props = typeof stateProps & typeof dispatchProps;

class listView extends React.Component<Props, any> {
  render(): JSX.Element {

    const list: ReadonlyArray<SingleAssessmentStatistic> = this.props.assessmentStatisticsList

    return (
      <SinglePanelSiteWrapper>

        {
          this.props.isLoading &&
          <Spinner/>
        }

        <div className="view-padding">
          <div className="view-head-line">
            <div className="view-caption">
              <h1>
                <span className="mar-right-half">
                  {
                    getI18n(this.props.langId, 'Assessment statistic')
                  }
                </span>
                <span>
                  ({this.props.pagination.totalItems})
                </span>
              </h1>
            </div>
            <div className="view-options">
              <MaterialSearchInput
                value={this.props.pagination.searchText}
                onChange={(e: InputOnChangeEvent) => {
                  this.props.setSearchText(e.currentTarget.value)
                }}
              />
            </div>
          </div>

          <div className="view-content">

            <Table sortable padded striped singleLine className="material-table">
              <Table.Header>
                <Table.Row>

                  <Table.HeaderCell
                    sorted={
                      isCurrentSortingColumnPaginated<SingleAssessmentStatistic>(this.props.sortByKey, 'exerciseDisplayName',
                                                                        this.props.sortDirection
                      )
                    }
                  >
                <span
                  className="clickable"
                  onClick={() => {
                    this.props.setSortByKey('exerciseDisplayName',
                                            rotateSorting<SingleAssessmentStatistic>(this.props.sortDirection,
                                                                             this.props.sortByKey,
                                                                             'exerciseDisplayName'
                                            )
                    )
                  }}
                >
                  {
                    getI18n(this.props.langId, 'Exercise')
                  }
                </span>
                  </Table.HeaderCell>

                  <Table.HeaderCell
                    sorted={
                      isCurrentSortingColumnPaginated<SingleAssessmentStatistic>(this.props.sortByKey,
                                                                        'exerciseShortDescription',
                                                                        this.props.sortDirection
                      )
                    }
                  >
                <span
                  className="clickable"
                  onClick={() => {
                    this.props.setSortByKey('exerciseShortDescription',
                                            rotateSorting<SingleAssessmentStatistic>(this.props.sortDirection,
                                                                             this.props.sortByKey,
                                                                             'exerciseShortDescription'
                                            )
                    )
                  }}
                >
                  {
                    getI18n(this.props.langId, 'Description')
                  }
                </span>
                  </Table.HeaderCell>

                  <Table.HeaderCell
                    sorted={
                      isCurrentSortingColumnPaginated<SingleAssessmentStatistic>(this.props.sortByKey, 'pLangId',
                                                                        this.props.sortDirection
                      )
                    }
                  >
                    <span
                      className="clickable"
                      onClick={() => {
                        this.props.setSortByKey('pLangId',
                                                rotateSorting<SingleAssessmentStatistic>(this.props.sortDirection,
                                                                                 this.props.sortByKey,
                                                                                 'pLangId'
                                                )
                        )
                      }}
                    >
                      <HelpPopup icon="puzzle" defaultText={getI18n(this.props.langId, 'Programming language')}/>
                    </span>
                  </Table.HeaderCell>

                  <Table.HeaderCell
                    sorted={
                      isCurrentSortingColumnPaginated<SingleAssessmentStatistic>(this.props.sortByKey,
                                                                        'exerciseReleaseIsReleased',
                                                                        this.props.sortDirection
                      )
                    }
                  >
                    <span
                      className="clickable"
                      onClick={() => {
                        this.props.setSortByKey('exerciseReleaseIsReleased',
                                                rotateSorting<SingleAssessmentStatistic>(this.props.sortDirection,
                                                                                 this.props.sortByKey,
                                                                                 'exerciseReleaseIsReleased'
                                                )
                        )
                      }}
                    >
                     {
                       getI18n(this.props.langId, 'Is release open')
                     }
                    </span>
                  </Table.HeaderCell>

                  <Table.HeaderCell
                    sorted={
                      isCurrentSortingColumnPaginated<SingleAssessmentStatistic>(this.props.sortByKey, 'shouldNotCount',
                                                                        this.props.sortDirection
                      )
                    }
                  >
                      <span
                        className="clickable"
                        onClick={() => {
                          this.props.setSortByKey('shouldNotCount',
                                                  rotateSorting<SingleAssessmentStatistic>(this.props.sortDirection,
                                                                                   this.props.sortByKey,
                                                                                   'shouldNotCount'
                                                  )
                          )
                        }}
                      >
                        <HelpPopup icon="flag outline"
                                   defaultText={getI18n(this.props.langId, 'Should not count')}/>
                      </span>
                  </Table.HeaderCell>
                  <Table.HeaderCell
                    sorted={
                      isCurrentSortingColumnPaginated<SingleAssessmentStatistic>(this.props.sortByKey, 'participationCount',
                                                                        this.props.sortDirection
                      )
                    }
                  >
                <span
                  className="clickable"
                  onClick={() => {
                    this.props.setSortByKey('participationCount',
                                            rotateSorting<SingleAssessmentStatistic>(this.props.sortDirection,
                                                                             this.props.sortByKey,
                                                                             'participationCount'
                                            )
                    )
                  }}
                >
                  <HelpPopup icon="repeat" wide
                             defaultText={getI18n(this.props.langId,
                                                  'The times the user solved this exercise (e.g. in different releases). If a user solved the exercise (release) in different programming languages then two rows are shown but it\'s the same participation. The participation is counted as one because the restrictions (e.g. lef time, should not count) are stored in the participation.'
                             )}/>
                </span>
                  </Table.HeaderCell>
                  <Table.HeaderCell
                    sorted={
                      isCurrentSortingColumnPaginated<SingleAssessmentStatistic>(this.props.sortByKey, 'normalTestPoints',
                                                                        this.props.sortDirection
                      )
                    }
                  >

                <span
                  className="clickable mar-right-half"
                  onClick={() => {
                    this.props.setSortByKey('normalTestPoints',
                                            rotateSorting<SingleAssessmentStatistic>(this.props.sortDirection,
                                                                             this.props.sortByKey,
                                                                             'normalTestPoints'
                                            )
                    )
                  }}
                >
                  <HelpPopup wide icon="flag"
                             defaultText={getI18n(this.props.langId, 'Normal test points')}/>
                </span>
                  </Table.HeaderCell>
                  <Table.HeaderCell
                    sorted={
                      isCurrentSortingColumnPaginated<SingleAssessmentStatistic>(this.props.sortByKey, 'submitTestPoints',
                                                                        this.props.sortDirection
                      )
                    }
                  >
                <span className="clickable mar-right-half"
                      onClick={() => {
                        this.props.setSortByKey('submitTestPoints',
                                                rotateSorting<SingleAssessmentStatistic>(this.props.sortDirection,
                                                                                 this.props.sortByKey,
                                                                                 'submitTestPoints'
                                                )
                        )
                      }}
                >
                  <HelpPopup wide icon="flag checkered"
                             defaultText={getI18n(this.props.langId, 'Submit test points')}/>
                </span>
                  </Table.HeaderCell>
                  <Table.HeaderCell
                    sorted={
                      isCurrentSortingColumnPaginated<SingleAssessmentStatistic>(this.props.sortByKey, 'manualPoints',
                                                                        this.props.sortDirection
                      )
                    }
                  >
                <span className="clickable mar-right-half"
                      onClick={() => {
                        this.props.setSortByKey('manualPoints',
                                                rotateSorting<SingleAssessmentStatistic>(this.props.sortDirection,
                                                                                 this.props.sortByKey,
                                                                                 'manualPoints'
                                                )
                        )
                      }}
                >
                  {
                    getI18n(this.props.langId, 'Manual')
                  }
                </span>
                    <HelpPopup wide
                               defaultText={getI18n(this.props.langId, 'Points that a tutor has assigned')}/>
                  </Table.HeaderCell>

                  <Table.HeaderCell>
                    <HelpPopup icon="talk" defaultText={getI18n(this.props.langId, 'Feedback from tutor')}/>
                  </Table.HeaderCell>

                  <Table.HeaderCell
                    sorted={
                      isCurrentSortingColumnPaginated<SingleAssessmentStatistic>(this.props.sortByKey, 'lastUpdatedAt',
                                                                        this.props.sortDirection
                      )
                    }
                  >
                    <span className="clickable mar-right-half"
                          onClick={() => {
                            this.props.setSortByKey('lastUpdatedAt',
                                                    rotateSorting<SingleAssessmentStatistic>(this.props.sortDirection,
                                                                                     this.props.sortByKey,
                                                                                     'lastUpdatedAt'
                                                    )
                            )
                          }}
                    >
                      {
                        getI18n(this.props.langId, 'Last updated at')
                      }
                    </span>
                    <HelpPopup defaultText={getI18n(this.props.langId,
                                                    'The date & time when the solution (or the participation e.g. custom tests) got last updated'
                    )}
                               wide
                    />
                  </Table.HeaderCell>

                  {
                    //a user cannot go to his/her submission but another can (if he/she sees it,
                    //he/she has the permission)
                    (this.props.assessmentStatistics && this.props.currentUserId !== this.props.assessmentStatistics.userId) &&
                    <Table.HeaderCell>
                        <span className="mar-right-half">
                          {
                            getI18n(this.props.langId, 'Open as tutor')
                          }
                        </span>
                      <HelpPopup
                        wide
                        defaultText={getI18n(this.props.langId,
                                             'In the tutor view you can view the submission, the test results and manually assess the submission.'
                        )}/>
                    </Table.HeaderCell>
                  }

                  {
                    //if the target user is the current show a link to edit the exercise
                    //else he/she need so search through open/closed exercises for the link...
                    (this.props.assessmentStatistics && this.props.currentUserId === this.props.assessmentStatistics.userId) &&
                    <Table.HeaderCell>
                        <span className="mar-right-half">
                          {
                            getI18n(this.props.langId, 'Open')
                          }
                        </span>
                    </Table.HeaderCell>
                  }

                </Table.Row>
              </Table.Header>

              <Table.Body>
                {
                  list.map((singleAssessmentStatistic: SingleAssessmentStatistic, index) => {
                    return (
                      <ListViewItem
                        key={`${singleAssessmentStatistic.exerciseReleaseId}-${singleAssessmentStatistic.pLangId}`}
                        singleAssessmentStatistic={singleAssessmentStatistic}/>
                    )
                  })
                }
              </Table.Body>
              <PaginationTableFooter
                pagination={this.props.pagination}
                setActivePaginationPage={this.props.setActivePaginationPage}
                setPaginationPageSize={this.props.setPaginationPageSize}
                />
            </Table>

          </div>
        </div>

      </SinglePanelSiteWrapper>
    )
  }
}

export default connect(mapStateToProps, mapDispatchToProps)(listView)
