import * as React from 'react'
import {Table} from 'semantic-ui-react'
import {getI18n, KnownLangs} from '../../../../i18n/i18nRoot'


export interface SettingsObj {
  readonly [key: string]: any
}

export interface MyProps {
  readonly listObj: SettingsObj | null
  readonly langId: KnownLangs
}

interface AnySettingsPair {
  readonly property: string
  readonly value: any
}

export const AnySettingsTable: React.SFC<MyProps> = (props) => {

  const list: ReadonlyArray<AnySettingsPair> = props.listObj === null
                                               ? []
                                               : Object.keys(props.listObj).map(p => {

      const pair: AnySettingsPair = {
        property: p,
        value: props.listObj[p]
      }
      return pair

    })

  return (
    <Table sortable selectable padded striped singleLine className="material-table">
      <Table.Header>
        <Table.Row>
          <Table.HeaderCell>
                <span>
                  {
                    getI18n(props.langId, 'Property')
                  }
                </span>
          </Table.HeaderCell>

          <Table.HeaderCell>
                <span>
                  {
                    getI18n(props.langId, 'Value')
                  }
                </span>
          </Table.HeaderCell>
        </Table.Row>
      </Table.Header>

      <Table.Body>
        {
          list.map((pair, index) => {
            return (
              <Table.Row key={pair.property}>

                <Table.Cell>
                  <span>{pair.property}</span>
                </Table.Cell>

                <Table.Cell>
                          <span>
                             {
                               (pair.value !== null && pair.value !== undefined)
                               ? pair.value.toString()
                               : pair.value
                             }
                          </span>
                </Table.Cell>

              </Table.Row>
            )
          })
        }

      </Table.Body>

    </Table>
  )
}
