/**
 * Created by janis dähne.
 */

import * as React from "react";
import {connect} from "react-redux";
import {bindActionCreators, Dispatch} from "redux";
import {returntypeof} from 'react-redux-typescript';
import {RootState} from "../../../../../../state/reducers";
import {Form, Icon, Tab} from "semantic-ui-react";
import MaterialInput from '../../../../../material/materialInput'
import MaterialSelectWrapper from '../../../../../material/materialSelectWrapper'
import {validationRules} from "../../../../../../state/reducers/doExerciseSite/dialogs/changeCustomTestDialog/customTestReducer";
import {validationMessageKeys} from "../../../../../../state/reducers/doExerciseSite/dialogs/changeCustomTestDialog/customTestReducerValidation";
import {SelectOnChangeEventData} from "../../../../../../types/reactEvents";
import {convertTestProtocolFromString, parseId} from "../../../../../../helpers/convertersAndTransformers";
import {
  setCustomTestContent,
  setCustomTestDisplayName,
  setCustomTestFiles,
  setCustomTestTestTypeId
} from "../../../../../../state/actions/doExerciseSite/dialogs/changeCustomTestDialog/changeCustomTestActions";
import SimpleCodeEditorWrapper from '../../../../../codeEditors/simpleCodeEditorWrapper'

import ConsolePanelView from '../../../consolePanel/consolePanelView'
import {TestProtocolItem} from "../../../../../../types/testResults";


import TestFilesListView from '../../../../exerciseEditorSite/exerciseTestsPanel/dialogs/testFilesListView'
import {debugOriginUrl} from "../../../../../../../debug";
import Logger from "../../../../../../helpers/logger";
import {errorDialog} from "../../../../../../helpers/dialogHelper";
import {EditorTab} from "../../../../../codeEditors/multiTabCodeEditorProps";
import {
  aceCompareFilesTestLanguage,
  aceRegexLanguage,
  aceTestProtocolLanguage,
  aceTextLanguage
} from "../../../../../../helpers/editorHelper";
import {KnownInternalTestTypes} from "../../../../../../types/testTypes";
import {
  checkRegexs,
  convertCompareTestToObj,
  convertRegexTestObject,
  parseRegexTestContent,
  RegexTestObject,
  RegexTestObjectConverted
} from "../../../../../../types/testsAndTestProtocol";
import {isNotEmptyOrSpaces} from "../../../../../../validation/validations";
import {getI18n} from "../../../../../../../i18n/i18nRoot";
import {descriptionEditorDebounceInMs, TestTypeLimits} from '../../../../../../constants'
import CompareFilesTestSettingsView
  from '../../../../exerciseEditorSite/exerciseTestsPanel/dialogs/compareFilesTestSettingsView'
import TestSettingsView from '../../../../../helpers/testing/testSettingsView'
import {
  setAddCustomTestAssetFilesModalFilterText,
  setAddCustomTestAssetFilesModalUploadPercentage,
  setIsAddCustomTestAssetFilesModalDisplayed
} from '../../../../../../state/actions/doExerciseSite/doExerciseSiteActions'
import {
  attachExitingFileToCustomTestAsync,
  detachFileFromCustomTestAsync
} from '../../../../../../state/actions/doExerciseSite/dialogs/changeCustomTestDialog/customTestCrudActions'
import AddAssetFilesModal from '../../../../exerciseEditorSite/addAssetFilesModal/addAssetFilesModal'
import {getPossibleAllUserFilesAsync} from '../../../../../../state/actions/doExerciseSite/doExerciseCrudActions'
import axios, {CancelTokenSource} from 'axios'
import {uploadCustomTestAssetFile} from '../../../../../../communicationLayer/doExerciseLayer'
import {HelpPopup} from '../../../../../helpers/helpPopup'
import {setCustomTests} from '../../../../../../state/actions/doExerciseSite/sub/customTestActions'
import {CompareFileTestContentFrontendOnly} from '../../../../../../types/exerciseEditor'

//const css = require('./styles.styl');

export interface MyProps {
  readonly canUserEditCustomTests: boolean
}

const mapStateToProps = (rootState: RootState, props: MyProps) => {
  return {
    //test0: rootState...
    //test: props.test
    ...props,

    test: rootState.changeCustomTestDialogState.customTest,
    testTypes: rootState.testTypesState.testTypes,
    customTest: rootState.changeCustomTestDialogState.customTest,
    defaultCustomTestSettings: rootState.doExerciseSiteState.exercise.defaultCustomTestSettings,
    releaseCode: rootState.doExerciseSiteState.exercise.exerciseRelease.releaseCode,
    allCustomTests: rootState.doExerciseSiteState.exercise.customTests,

    isAddCustomTestAssetFilesModalDisplayed: rootState.doExerciseSiteState.isAddCustomTestAssetFilesModalDisplayed,
    isAddCustomTestAssetFilesModalBusy: rootState.doExerciseSiteState.isAddCustomTestAssetFilesModalBusy,
    addCustomTestAssetFilesModalFilterText: rootState.doExerciseSiteState.addCustomTestAssetFilesModalFilterText,
    addCustomTestAssetFilesModalUploadPercentage: rootState.doExerciseSiteState.addCustomTestAssetFilesModalUploadPercentage,
    allCustomTestAssetFilePreviews: rootState.doExerciseSiteState.allCustomTestAssetFilePreviews,

    langId: rootState.i18nState.langId
  }
}

const mapDispatchToProps = (dispatch: Dispatch) => bindActionCreators({
                                                                        //imported reducer funcs here
                                                                        setCustomTestTestTypeId,
                                                                        setCustomTestDisplayName,
                                                                        setCustomTestContent,
                                                                        setCustomTestFiles,

                                                                        setIsAddCustomTestAssetFilesModalDisplayed,
                                                                        setAddCustomTestAssetFilesModalFilterText,
                                                                        setAddCustomTestAssetFilesModalUploadPercentage,
                                                                        attachExitingFileToCustomTestAsync,
                                                                        detachFileFromCustomTestAsync,
                                                                        getPossibleAllUserFilesAsync,
                                                                        setCustomTests,
                                                                      }, dispatch)


const stateProps = returntypeof(mapStateToProps);
const dispatchProps = returntypeof(mapDispatchToProps);
type Props = typeof stateProps & typeof dispatchProps;


const customTestAssetInputId = 'customTestAssetInputId'

export const changeCustomTestViewEditorId = 'changeCustomTestViewEditorId'

const ChangeTestViewResizeId = 'ChangeTestViewResizeId'

// tslint:disable-next-line
let currentFileUploadCancelToken: CancelTokenSource | null = null

class ChangeCustomTestView extends React.Component<Props, any> {


  componentWillMount(): void {

    if (this.props.test.testTypeId <= 0 && this.props.testTypes.length > 0) {
      //not selected ... select the first one
      this.props.setCustomTestTestTypeId(this.props.testTypes[0].id)
    }
  }

  render(): JSX.Element {

    const testType = this.props.testTypes.find(p => p.id === this.props.test.testTypeId)

    if (this.props.test.testTypeId <= 0 || !testType) {
      //this happens when we render and we display the initial value (test where testTypeId is negative)
      //in componentWillMount we set the test type id and we should be finde on the second render
      return null
    }

    const isCompareFilesTest = testType.internalName === KnownInternalTestTypes.compareFilesTest

    const isRegexTest = testType.internalName === KnownInternalTestTypes.regexTest

    // noinspection TsLint
    let testParseError = '' //error when parsing the json object
    // noinspection TsLint
    let regexError = '' //error when a regex was invalid

    if (isRegexTest) {

      // noinspection TsLint
      let result: RegexTestObject | string = null
      // noinspection TsLint
      let convertedResult: RegexTestObjectConverted = null

      try {
        result = parseRegexTestContent(this.props.test.content)

        if (typeof result === 'string') {
          throw new Error(result)
        }

        convertedResult = convertRegexTestObject(result)
      } catch (err) {
        testParseError = err.message
      }

      if (convertedResult === null) {

        if (isNotEmptyOrSpaces(testParseError) === false) { //if testParseError is empty
          regexError = 'some regex are not properly separated, check that every regex is enclosed in /.../\n' + `also check that you don't use && or || inside a regex`
        }

      } else {

        const errors = checkRegexs(convertedResult)

        if (errors && errors.length > 0) {
          regexError = errors.join('\n')
        } else {


        }
      }
    }

    //we fake a test result to reuse the console view
    const visualTest: ReadonlyArray<TestProtocolItem> | null = convertTestProtocolFromString(this.props.test.content,
                                                                                             testType
    )

    const tempEditorTab: EditorTab = {
      id: this.props.customTest.id,
      content: this.props.customTest.content,
        language: testType.internalName === KnownInternalTestTypes.regexTest
                  ? aceRegexLanguage
                  : testType.internalName === KnownInternalTestTypes.blackBoxTest
                    ? aceTestProtocolLanguage
                    : testType.internalName === KnownInternalTestTypes.compareFilesTest
                      ? aceCompareFilesTestLanguage
                      : aceTextLanguage
      ,
      isReadonly: false,
      fileNameWithExtensions: this.props.customTest.displayName,
      displayIndex: this.props.customTest.displayIndex,
      isMainFile: true,
      isContentVisibleForUser: false,
    }

    // tslint:disable-next-line
    let fileCompares: CompareFileTestContentFrontendOnly[] = []

    try {
      if (isCompareFilesTest) {
        const temp = this.props.test.content
          .split('\n')
          .filter(p => p !== '')
          .map<CompareFileTestContentFrontendOnly | null>(convertCompareTestToObj)
          .filter(p => p !== null)
        fileCompares = temp
      }
    } catch (err) {
      //TODO show error??
    }

    return (<div className="fh flexed-h">
      <div className="ui form divided-checkboxes fh" style={{height: 'auto' /*firefox fix*/}}>

        <AddAssetFilesModal
          title={getI18n(this.props.langId, "Manage asset files")}
          isDisplayed={this.props.isAddCustomTestAssetFilesModalDisplayed}
          isBusy={this.props.isAddCustomTestAssetFilesModalBusy}
          filePreviewsListSourcesTooltip={"These are all files from custom projects and custom exercise tests."}

          uploadPercentage={this.props.addCustomTestAssetFilesModalUploadPercentage}
          uploadInputId={'do-exercise-custom-test-upload'}

          onUploadFile={async (file) => {

            this.props.setIsAddCustomTestAssetFilesModalDisplayed(true, true)
            this.props.setAddCustomTestAssetFilesModalUploadPercentage(0)

            currentFileUploadCancelToken = axios.CancelToken.source()

            try {
              const assetFilePreview = await uploadCustomTestAssetFile(this.props.releaseCode,
                                                                         this.props.test.id,
                                                                         file,
                                                                         currentFileUploadCancelToken.token,
                                                                         this.props.setAddCustomTestAssetFilesModalUploadPercentage
              )

              currentFileUploadCancelToken = null

              //hide progress bar
              this.props.setAddCustomTestAssetFilesModalUploadPercentage(null)
              Logger.debug('doExerciseSite...changeCustomTestView',
                           `upload finished (new id: ${assetFilePreview.id}), now downloading file content again...`
              )

              //this is a bit dirty... because it's not directly connected to the curd action... but ok for now (we call the raw crud operation and not a redux action)

              //set for the whole do exercise!
              this.props.setCustomTests(this.props.allCustomTests.map(p => p.id !== this.props.test.id
                                                                           ? p
                                                                           : {
                  ...p,
                  files: p.files.concat(assetFilePreview)
                }))

              //because this dialog edits a copy of the exercise we need to add the new preview there too
              this.props.setCustomTestFiles(this.props.test.files.concat(assetFilePreview))


              //renew all asset preview list (could have changed) this is easier than to manage all special cases
              //e.g. detach file... was it deleted or just detached??
              // this.props.finishAddAssetFilesModalUpload(assetFilePreview)
              await this.props.getPossibleAllUserFilesAsync()

            } catch (err) {
              Logger.error('could not upload file, err: ' + err.message)
            } finally {
              currentFileUploadCancelToken = null
              this.props.setIsAddCustomTestAssetFilesModalDisplayed(true, false)
              this.props.setAddCustomTestAssetFilesModalUploadPercentage(null)
            }

          }}

          onUploadCanceled={
            typeof this.props.addCustomTestAssetFilesModalUploadPercentage === 'number'
            ? () => {
              if (currentFileUploadCancelToken === null) return
              Logger.debug('doExerciseSite...changeCustomTestView', 'canceled file upload')
              currentFileUploadCancelToken.cancel('canceled')
            }
            : null
          }


          filterText={this.props.addCustomTestAssetFilesModalFilterText}
          onFilterTextChanged={(text) => {
            this.props.setAddCustomTestAssetFilesModalFilterText(text)
          }}

          notDetachableAssetFilePreviews={isCompareFilesTest
                                          ? this.props.test.files.filter(
              p => fileCompares.some(k => k.file1NameWithExtension === p.originalName))
                                          : []}
          notDetachableReason={getI18n(this.props.langId,
                                       "This file cannot be removed because it is part of the compare files test"
          )}

          attachedAssetFilePreviews={this.props.test.files}

          allAssetFilePreviews={this.props.allCustomTestAssetFilePreviews}

          onAttachAssetFilePreview={async (assetFilePreview) => {
            //we don't download the asset file so we don't need a progress bar (when adding an markdown file we need to download it after attaching)

            await this.props.attachExitingFileToCustomTestAsync(this.props.releaseCode,
                                                                this.props.test.id,
                                                                assetFilePreview.id,
                                                                // this.props.setAddAssetFilesModalUploadPercentage
            )
          }}

          onDetachAssetFilePreview={async(assetFilePreview) => {

            await this.props.detachFileFromCustomTestAsync(this.props.releaseCode, this.props.test.id, assetFilePreview.id)

            //refresh list
            await this.props.getPossibleAllUserFilesAsync()
          }}

          onClose={() => {
            this.props.setIsAddCustomTestAssetFilesModalDisplayed(false, false)
          }}
        />

        <Form.Group widths='equal'>
          <Form.Field required>
            <label>
              {
                getI18n(this.props.langId, 'Type')
              }
            </label>
            <MaterialSelectWrapper
              // readonly={!this.props.canUserEditCustomTests}
              value={this.props.test.testTypeId}
              validations={validationRules.testTypeId}
              validationMessageKey={validationMessageKeys.testTypeId}
            >
              <select
                disabled={!this.props.canUserEditCustomTests}
                value={this.props.test.testTypeId}
                onChange={(e: SelectOnChangeEventData) => {
                  const id = parseId(e.currentTarget.value)
                  if (id === null) {
                    errorDialog(getI18n(this.props.langId, 'Error'), getI18n(this.props.langId, 'Unknown test type'),
                                this.props.langId
                    )

                    return

                  } else {
                    this.props.setCustomTestTestTypeId(id)
                  }

                }}
              >
                {this.props.testTypes.map(testType => {
                  return (<option key={testType.id} value={testType.id}>
                    {testType.displayName}
                  </option>)
                })}
              </select>
            </MaterialSelectWrapper>

          </Form.Field>

          <Form.Field required>
            <label>
              {
                getI18n(this.props.langId, 'Name')
              }
            </label>
            <MaterialInput
              disabled={!this.props.canUserEditCustomTests}
              validations={validationRules.displayName}
              validationMessageKey={validationMessageKeys.displayName}
              value={this.props.test.displayName}
              onChange={(e) => {
                this.props.setCustomTestDisplayName(e.currentTarget.value)
              }}
            />
          </Form.Field>
        </Form.Group>

      </div>

      <Tab menu={{
        secondary: true,
        pointing: true
      }}
           className="fh flexed-h"
           panes={[{
             menuItem: `${getI18n(this.props.langId, 'Test content')}*`,
             render: () => {
               return (<div id={ChangeTestViewResizeId} className="flexed-h flex-stretch-0">

                 {//if we got a json parse error
                   (isRegexTest && isNotEmptyOrSpaces(testParseError)) && <div>
                     {getI18n(this.props.langId, 'Parse Error') + ':'}
                     <pre className="error-colored">
                          {testParseError}
                        </pre>
                   </div>}

                 {//if we got a regex error
                   (isRegexTest && isNotEmptyOrSpaces(regexError)) && <div>
                     {getI18n(this.props.langId, 'Regex Error') + ':'}
                     <pre className="error-colored">
                          {regexError}
                        </pre>
                   </div>}

                 {
                   isCompareFilesTest &&
                   <div>
                     <CompareFilesTestSettingsView files={this.props.test.files} testContent={this.props.test.content}
                                                   setTestContent={this.props.setCustomTestContent}
                                                   isReadonly={false}
                     />
                   </div>
                 }

                 {
                   //calc with because SimpleVDivider has 1em margin
                   //don't conditional render the editor because this probably (?) breaks ace editor storage
                 }
                 <div className="fh flexed-h flex-stretch">
                   <div className="flexed-h flex-stretch">
                     <SimpleCodeEditorWrapper
                       id={changeCustomTestViewEditorId}
                       editorTab={tempEditorTab}
                       onBlur={(value: string) => {
                         this.props.setCustomTestContent(value)
                       }}
                       debounceTimeInMs={descriptionEditorDebounceInMs}
                       // onChange={(editorTab: EditorTab) => {
                       //   this.props.setCustomTestContent(editorTab.content)
                       // }}
                       // onEditorSessionChanged={null}
                     />
                   </div>
                 </div>

               </div>)
             }
           }, {
             menuItem: getI18n(this.props.langId, 'Test content visually'),
             render: () => {
               return (<div className="mar-bot">
                 <ConsolePanelView test={null} protocolItems={visualTest} testTypes={this.props.testTypes}
                                   displayFilterConsoleItemsOptions={false}
                                   loadFullOutputFunc={null}
                 />
               </div>)
             }
           }, {
             menuItem: getI18n(
               this.props.langId,
               'Test settings & files'
             ), //DISABLED until we find a way to limit upload size (at backend)
             //div-disabled
             render: () => {
               return (<div className="ui form divided-checkboxes fh">

                 <TestSettingsView
                   testTypeLimits={TestTypeLimits.IsCustomTest}
                   timeoutInMs={this.props.defaultCustomTestSettings.timeoutInMs}
                   memoryLimitInKb={this.props.defaultCustomTestSettings.memoryLimitInKb}
                   maxDiskSpaceInKb={this.props.defaultCustomTestSettings.maxDiskSpaceInKb}
                   compileTimeoutInMs={this.props.defaultCustomTestSettings.compileTimeoutInMs}
                   compilerOptions={this.props.defaultCustomTestSettings.compilerOptions}

                   set_maxDiskSpaceInKb={null}
                   set_memoryLimitInKb={null}
                   set_timeoutInMs={null}
                   set_compileTimeoutInMs={null}
                   set_compilerOptions={null}
                   />

                 <h4 className="ui dividing header">
                   {getI18n(this.props.langId, 'Files')}
                 </h4>

                 {
                   this.props.canUserEditCustomTests &&
                   <div className="flexed">
                     <div className={this.props.test.id > 0
                                     ? 'clickable'
                                     : 'div-disabled'} style={{marginBottom: '0.5em'}}
                          onClick={() => {

                            if (this.props.test.id <= 0) return

                            this.props.setIsAddCustomTestAssetFilesModalDisplayed(true, false) //before loading because this will cancel isBusy
                            this.props.getPossibleAllUserFilesAsync()
                          }}
                     >
                       <Icon name="write"/>
                       <span>
                        {
                          getI18n(this.props.langId, 'Manage')
                        }
                      </span>
                     </div>

                     {
                       this.props.test.id <= 0 &&
                       <HelpPopup className="mar-left-half" defaultText={getI18n(this.props.langId,
                                                                                 "Managing test asset files is only allowed for saved tests (not for new tests), save the exercise first"
                       )}/>
                     }

                     <HelpPopup className="mar-left-half warning-colored" defaultText={getI18n(this.props.langId,
                                                                                               "All changes to assets are applied immediately in the backend (saving is not needed for this)!"
                     )}/>

                   </div>
                 }


                 <TestFilesListView files={this.props.test.files}
                                    getDownloadFileHRef={(file) => {
                                      return `${debugOriginUrl}/api/assets/custom/${this.props.releaseCode}/${this.props.test.id}/${file.id}`
                                    }}
                 />

               </div>)
             }
           },]

           }/>


    </div>)
  }
}

const _comp = connect(mapStateToProps, mapDispatchToProps)(ChangeCustomTestView)
export default _comp
