/**
 * Created by janis dähne.
 */

import * as React from "react";
import {connect} from "react-redux";
import {bindActionCreators, Dispatch} from "redux";
import {returntypeof} from 'react-redux-typescript';
import {RootState} from "../../../../state/reducers";
import {Icon} from "semantic-ui-react";
import {HelpPopup} from "../../../helpers/helpPopup";
import {SimpleVDivider} from "../../../helpers/simpleVDivider";
import {
  resetDoExerciseSolutionToTemplate, setIsDoCodeEditorSettingsDialogDisplayed
} from "../../../../state/actions/doExerciseSite/doExerciseSiteActions";
import {openEditEditorSettingsDialog} from "../../../../state/actions/editorSettingsDialog/editorSettingsDialogActions";
import {askDialog, errorDialog} from "../../../../helpers/dialogHelper";
import {displayDateAndTime} from "../../../../helpers/convertersAndTransformers";
import {MyPopup} from "../../../helpers/myPopup";
import {getI18n} from "../../../../../i18n/i18nRoot";
import {AceEditorsStorageHelper} from '../../../codeEditors/aceEditors/aceEditorsStorageHelper'
import {doEditorEditorPanelViewEditorId} from '../../../../constants'
import Logger from '../../../../helpers/logger'
import {ErrorHelper} from '../../../../helpers/errorHelper'

//const css = require('./styles.styl');

export interface MyProps {
  //readonly test: string
}

const mapStateToProps = (rootState: RootState /*, props: MyProps*/) => {
  return {
    //test0: rootState...
    //test: props.test
    codeEditorSettings: rootState.userDataSettingsSate.userData.settings
                        ?
                        rootState.userDataSettingsSate.userData.settings.codeEditorSettings
                        : null,

    pLangId: rootState.doExerciseSiteState.solution.pLangId,
    pLangs: rootState.pLangsState.pLangs,
    release: rootState.doExerciseSiteState.exercise.exerciseRelease,
    solutionCreatedAt: rootState.doExerciseSiteState.solution.lastUpdatedAt,

    canUserEditExercise: rootState.doExerciseSiteState.exercisePreviewFrontendTemp
      && rootState.doExerciseSiteState.exercisePreviewFrontendTemp.isStillOpen
      && rootState.doExerciseSiteState.exercisePreviewFrontendTemp.isPermanentlyLocked === false
      //isSomeTestRunning
      && !(rootState.doExerciseSiteState.exercise.tests.some(p => p.uiIsRunning)
        || rootState.doExerciseSiteState.exercise.customTests.some(p => p.uiIsRunning)
        || rootState.doExerciseSiteState.afterNormalTests.some(p => p.uiIsRunning)
        || rootState.doExerciseSiteState.afterSubmitTests.some(p => p.uiIsRunning)
        || rootState.doExerciseSiteState.afterCustomTestResults.some(p => p.uiIsRunning)),


    langId: rootState.i18nState.langId
  }
}

const mapDispatchToProps = (dispatch: Dispatch) => bindActionCreators({
                                                                        //imported reducer funcs here
                                                                        setIsDoCodeEditorSettingsDialogDisplayed,
                                                                        openEditEditorSettingsDialog,
                                                                        resetDoExerciseSolutionToTemplate,
                                                                      }, dispatch)


const stateProps = returntypeof(mapStateToProps);
const dispatchProps = returntypeof(mapDispatchToProps);
type Props = typeof stateProps & typeof dispatchProps;

class editorPanelHeaderBar extends React.Component<Props, any> {
  render(): JSX.Element {
    return (
      <div className="flexed">

        <div className="clickable"
             onClick={() => {
               this.props.openEditEditorSettingsDialog(this.props.codeEditorSettings,
                                                       this.props.setIsDoCodeEditorSettingsDialogDisplayed
               )
             }}
        >
          <Icon name="settings"/>
          <span>
            {
              getI18n(this.props.langId, 'Editor settings')
            }
          </span>
        </div>

        <SimpleVDivider/>

        <div className={`mar-right-half ${this.props.canUserEditExercise
                                          ? 'clickable'
                                          : 'div-disabled' }`}
             onClick={async () => {

               if (!this.props.canUserEditExercise) return

               const shouldReset = await askDialog(getI18n(this.props.langId, 'Reset code'),
                                                   getI18n(this.props.langId,
                                                           'Are you sure you want to reset your code to the template?'
                                                   ), this.props.langId
               )


               if (!shouldReset) return

               try {
                 await this.props.resetDoExerciseSolutionToTemplate(this.props.release.releaseCode, this.props.pLangId, this.props.pLangs)
               } catch (err) {

                 ErrorHelper.makeServerErrorDialog(err, this.props.langId, 'Error', 'Could not reset the code')
               }

             }}
        >
          <Icon name="undo"/>
          <span>
            {
              getI18n(this.props.langId, 'Reset code')
            }
          </span>
        </div>
        <HelpPopup
          defaultText={getI18n(this.props.langId,
                               'Resets your code to the template but does not saves the new code in the editor to the server.'
          )}
          wide
        />

        <SimpleVDivider/>

        <MyPopup
          trigger={
            <div className="hoverable">

              <Icon className="mar-right-half" name="database"/>
              <span className="mar-right-half">
              {
                getI18n(this.props.langId, 'Server version:')
              }
              </span>
              <span>
              {
                displayDateAndTime(this.props.solutionCreatedAt, -1)
              }
              </span>
            </div>
          }
          content={getI18n(this.props.langId, 'The time when the latest solution was saved/created on the server.')}
        />


      </div>
    )
  }
}

export default connect(mapStateToProps, mapDispatchToProps)(editorPanelHeaderBar)
