/**
 * Created by janis dähne.
 */

import * as React from "react";
import {connect} from "react-redux";
import {bindActionCreators, Dispatch} from "redux";
import {returntypeof} from 'react-redux-typescript';
import {RootState} from "../../../../../state/reducers";
import FullSiteDialog from '../../../../fullSiteDialog'
import ViewTestView, {viewTestViewEditorId} from './viewTestView'


import {getI18n} from "../../../../../../i18n/i18nRoot";
import {AceEditorsStorageHelper} from "../../../../codeEditors/aceEditors/aceEditorsStorageHelper";
import TestServerHints from '../../../../helpers/testing/testServerHints'

//const css = require('./styles.styl');

export interface MyProps {
  //readonly test: string
  readonly cancelViewTestDialog: () => void

  /**
   * returns the download link for a given file
   */
  readonly getDownloadTestAssetLink: (testId: number, fileId: number, isSubmitTest: boolean) => string
}

const mapStateToProps = (rootState: RootState , props: MyProps) => {
  return {
    //test0: rootState...
    //test: props.test
    isBusy: rootState.viewTestDialogState.isBusy,

    runnableTest: rootState.viewTestDialogState.runnableTest,
    cancelViewTestDialog: props.cancelViewTestDialog,
    getDownloadTestAssetLink: props.getDownloadTestAssetLink,
    langId: rootState.i18nState.langId
  }
}

const mapDispatchToProps = (dispatch: Dispatch) => bindActionCreators({
  //imported reducer funcs here
}, dispatch)


const stateProps = returntypeof(mapStateToProps);
const dispatchProps = returntypeof(mapDispatchToProps);
type Props = typeof stateProps & typeof dispatchProps;

class viewTestDialog extends React.Component<Props, any> {
  render(): JSX.Element {
    return (
      <FullSiteDialog
        isBusy={this.props.isBusy}
        isValid={true}
        caption={
          <div className="flexed">
            <h2 className="mar-right">
              {
                getI18n(this.props.langId,'Test content')
              }
            </h2>

            <TestServerHints/>

          </div>
        }
        onSave={null}
        onCancel={() => {
          this.props.cancelViewTestDialog()
          AceEditorsStorageHelper.removeEditor(viewTestViewEditorId)
        }}
      >
        <ViewTestView getDownloadTestAssetLink={this.props.getDownloadTestAssetLink}/>
      </FullSiteDialog>
    )
  }
}

export default connect(mapStateToProps, mapDispatchToProps)(viewTestDialog)
