/**
 * Created by janis dähne.
 */

import * as React from "react";
import {connect} from "react-redux";
import {bindActionCreators, Dispatch} from "redux";
import {returntypeof} from 'react-redux-typescript';
import ChangeTestView, {changeTestViewCustomProjectEditorId} from './changeTestView'
import FullSiteDialog from '../../../../fullSiteDialog'
import {RootState} from "../../../../../state/reducers";


import {
  cancelCustomProjectTestDialog,
  saveCustomProjectTestDialog
} from "../../../../../state/actions/editCustomProjectSite/dialogs/changeCustomProjectTestDialog/changeCustomProjectTestDialogActions";
import Logger from "../../../../../helpers/logger";
import {notifySuccess} from "../../../../../helpers/notificationHelper";
import {getI18n} from "../../../../../../i18n/i18nRoot";
import {AceEditorsStorageHelper} from "../../../../codeEditors/aceEditors/aceEditorsStorageHelper";
import {ErrorHelper} from '../../../../../helpers/errorHelper'
import TestServerHints from '../../../../helpers/testing/testServerHints'

//const css = require('./styles.styl');

export interface MyProps {
  //readonly test: string
}

const mapStateToProps = (rootState: RootState /*, props: MyProps*/) => {
  return {
    //test0: rootState...
    //test: props.test
    isBusy: rootState.changeCustomProjectTestDialogState.isBusy || rootState.testTypesState.isLoading,
    isCreating: rootState.changeCustomProjectTestDialogState.isCreating,
    isValid: rootState.changeCustomProjectTestDialogState.isValid,
    test: rootState.changeCustomProjectTestDialogState.test,
    allTests: rootState.editCustomProjectSiteState.customProject.tests,
    langId: rootState.i18nState.langId
  }
}

const mapDispatchToProps = (dispatch: Dispatch) => bindActionCreators({
  //imported reducer funcs here
  saveCustomProjectTestDialog,
  cancelCustomProjectTestDialog
}, dispatch)


const stateProps = returntypeof(mapStateToProps);
const dispatchProps = returntypeof(mapDispatchToProps);
type Props = typeof stateProps & typeof dispatchProps;

class changeTestDialog extends React.Component<Props, any> {
  render(): JSX.Element {
    return (
      <FullSiteDialog
        isBusy={this.props.isBusy}
        isValid={this.props.isValid}
        caption={
          <div className="flexed">
            <h2 className="mar-right">
              {
                this.props.isCreating &&
                getI18n(this.props.langId,'Create custom test')
              }
              {
                !this.props.isCreating &&
                getI18n(this.props.langId,'Change custom test')
              }
            </h2>

            <TestServerHints/>

          </div>
        }
        onSave={async () => {

          try {
            await this.props.saveCustomProjectTestDialog(this.props.test, this.props.isCreating, this.props.allTests)
          } catch (err) {

            if (this.props.isCreating) {
              ErrorHelper.makeServerErrorDialog(err, this.props.langId, 'Error', 'Could not create custom test')
            } else {
              ErrorHelper.makeServerErrorDialog(err, this.props.langId, 'Error', 'Could not update custom test')
            }
            Logger.error(`could not ${this.props.isCreating ? 'create' : 'update'} custom test, err: ${err.message}`)
            return
          }

          AceEditorsStorageHelper.removeEditor(changeTestViewCustomProjectEditorId)

          //notify because the user needs to know that he/she not need to save something e.g. solution or exercise
          //this was already done by the save button click
          if (this.props.isCreating) {
            notifySuccess(getI18n(this.props.langId,'Created custom test'))
          } else {
            notifySuccess(getI18n(this.props.langId,'Updated custom test'))
          }


        }}
        onCancel={() => {
          this.props.cancelCustomProjectTestDialog()
          AceEditorsStorageHelper.removeEditor(changeTestViewCustomProjectEditorId)
        }}
      >
        <ChangeTestView/>
      </FullSiteDialog>
    )
  }
}

export default connect(mapStateToProps, mapDispatchToProps)(changeTestDialog)
