/**
 * Created by janis dähne.
 */

import * as React from "react";
import {connect} from "react-redux";
import {bindActionCreators, Dispatch, Unsubscribe} from "redux";
import {returntypeof} from 'react-redux-typescript';
import {RootState} from "../../../state/reducers";
import {Icon, Popup} from "semantic-ui-react";
import CommonHeaderBarContentWrapper from '../../commonHeaderBarContentWrapper'
import {setEditorExercise} from "../../../state/actions/exerciseEditorSite/exerciseEditorSiteActions";
import state from '../../../state/state'
import history from '../../../routerHistory'
import {getCustomProjectToLinkPath,} from "../../../constants";
import {addCloseWindowEvent, clearCloseWindowEvent} from "../../../helpers/eventsHelper";
import {Prompt} from "react-router";
import {freeAssetUrlsAndObjects} from "../../../helpers/assetHelper";
import {notifyError, notifyInfo, notifySuccess} from "../../../helpers/notificationHelper";
import * as mousetrap from "mousetrap";


import {
  createCustomProjectAsync, justRunProgramCustomProjectAsync, runCompileSingleFileCustomProjectAsync,
  updateCustomProjectAsync
} from "../../../state/actions/editCustomProjectSite/editCustomProjectCrudActions";
import {HelpPopup} from "../../helpers/helpPopup";
import {
  JustRunProgramTestCustomProjectCommandForBackend,
  RunCompileSingleFileCommandCustomProjectForBackend, RunCompileSingleFileCommandForBackend,
  SolutionFileForTestServer
} from "../../../types/testsAndTestProtocol";
import {CustomProjectSolutionFileFullBase} from "../../../types/customProjects";
import {
  setIsJustRunProgramArgsInputDisplayedCustomProject,
  setJustRunProgramArgsCustomProject,
  setSelectedConsoleViewTest
} from "../../../state/actions/editCustomProjectSite/editCustomProjectActions";
import {SimpleVDivider} from "../../helpers/simpleVDivider";
import {justRunProgramArgsMaxLength} from "../../../state/reducers/doExerciseSite/doExerciseSiteReducer";
import {ProgramArgsInputView} from "../../helpers/programArgsInputView";
import {getI18n} from "../../../../i18n/i18nRoot";
import * as constants from '../../../constants'
import Logger from '../../../helpers/logger'

declare var require: any
const fileSaver = require('file-saver');

//const css = require('./styles.styl');

export interface MyProps {
  //readonly test: string
}

const mapStateToProps = (rootState: RootState /*, props: MyProps*/) => {
  return {
    //test0: rootState...
    //test: props.test

    customProject: rootState.editCustomProjectSiteState.customProject,
    testTypes: rootState.testTypesState.testTypes,
    selectedTabIdDoEditor: rootState.editCustomProjectSiteState.solutionEditorState.selectedTabIdDoEditor,
    editorHasUnsavedChanges: rootState.editCustomProjectSiteState.solutionEditorState.hasUnsavedChanges,

    isJustRunProgramArgsInputDisplayed: rootState.editCustomProjectSiteState.isJustRunProgramArgsInputDisplayed,
    justRunProgramArgs: rootState.editCustomProjectSiteState.justRunProgramArgs,

    savedNewCustomProject: rootState.editCustomProjectSiteState.savedNewCustomProject,

    isLoading: rootState.editCustomProjectSiteState.isLoading
    || rootState.testTypesState.isLoading
    || rootState.pLangsState.isLoading
  || rootState.systemSettingsSiteState.isLoading,

    isSomeTestRunning: rootState.editCustomProjectSiteState.customProject.tests.some(p => p.uiIsRunning),

    isNoDialogDisplayed: !rootState.editCustomProjectSiteState.isCustomProjectCodeEditorSettingsDialogDisplayed
      && !rootState.editCustomProjectSiteState.isChangeTestDialogDisplayed,

    isHeaderBarDisabled: rootState.editCustomProjectSiteState.isCustomProjectCodeEditorSettingsDialogDisplayed
    || rootState.editCustomProjectSiteState.isChangeTestDialogDisplayed,
    isLoggingOut: rootState.siteHeaderBarState.isLoggingOut,
    langId: rootState.i18nState.langId
  }
}

const mapDispatchToProps = (dispatch: Dispatch) => bindActionCreators({
  //imported reducer funcs here
  setEditorExercise,
  createCustomProjectAsync,
  updateCustomProjectAsync,
  setSelectedConsoleViewTest,
  runCompileSingleFileCustomProjectAsync,
  setJustRunProgramArgsCustomProject,
  setIsJustRunProgramArgsInputDisplayedCustomProject,

  justRunProgramCustomProjectAsync,
}, dispatch)


const stateProps = returntypeof(mapStateToProps);
const dispatchProps = returntypeof(mapDispatchToProps);
type Props = typeof stateProps & typeof dispatchProps;

class HeaderBar extends React.Component<Props, any> {

  htmlSaveButton: HTMLDivElement | undefined

  componentDidMount(): void {
    addCloseWindowEvent()
    mousetrap.bindGlobal(["ctrl+s", "meta+s"], this.simulateSaveButtonPress)
  }

  componentWillUnmount(): void {
    this.clearHandlers()
  }

  clearHandlers(): void {
    clearCloseWindowEvent()
    freeAssetUrlsAndObjects()
    mousetrap.unbind(["ctrl+s", "meta+s"])
  }

  simulateSaveButtonPress = async (e: ExtendedKeyboardEvent | null) => {

    if (!this.htmlSaveButton) {
      Logger.log('htmlSaveButton was undefined')
      return
    }

    const editorTextArea = document.querySelector(':focus') as HTMLTextAreaElement

    if (!editorTextArea) {
      //then the editor is probably up to date...
      Logger.log('we assume the editor has no focus and thus is up to date')
    }

    if (editorTextArea)
      editorTextArea.blur() //this should blur the editor before saving... (and update the store files)

    await this.onSaveKeyPress(e)

    if (editorTextArea)
      editorTextArea.focus()
  }

  onSaveKeyPress = async (e: ExtendedKeyboardEvent | null) => {

    if (e) {
      e.preventDefault()
    }

    if (this.props.isNoDialogDisplayed === false) {
        return
    }

    if (this.props.isSomeTestRunning) {
      return
    }

    //we need at least one file ... to be compatible with exercises
    //and backend would not recognize project when there is no solution (actually we need at least the main file id)
    if (this.props.customProject.solution.solutionFiles.length === 0) {
      return
    }

    if (this.props.customProject.id <= 0) {
      //creating (backend uses ids > 0

      // noinspection TsLint
      let created = false

      try {
        created = (await this.props.createCustomProjectAsync(this.props.customProject)) as any as boolean
      } catch (err) {
        //setup intercepts all now ok responses and displays a dialog...
        //when this happens created == false
        //when this trows then that's another error
        notifyError(getI18n(this.props.langId,'Could not create custom project'))
        return
      }

      if (!created) {
        return
      }

      //because we only get the new id back (intentionally)
      //navigate to the load exercise editor site

      //actually we don't need to get the state because we await in the method which re populates the props
      //but to be sure

      if (this.props.savedNewCustomProject) { //state.getState().editCustomProjectSiteState.savedNewCustomProject //works but wrong typing

        const newCustomProjectId = this.props.customProject.id //state.getState().editCustomProjectSiteState.customProject.id
        this.clearHandlers()

        history.push(getCustomProjectToLinkPath(newCustomProjectId))
      }


    } else {
      //updating
      try {
        await this.props.updateCustomProjectAsync(this.props.customProject, this.props.langId)
      } catch (err) {
        notifyError(getI18n(this.props.langId,'Could not update custom project'))
        return
      }

      notifySuccess(getI18n(this.props.langId,'Updated exercise'))

    }
  }


  render(): JSX.Element {

    const allFiles = this.props.customProject.solution.solutionFiles ? this.props.customProject.solution.solutionFiles : []

    //this is not the real main file (saved in the solution) but the CURRENT FILE
    const mainFile = allFiles.find(p => p.id === this.props.selectedTabIdDoEditor)

    const leftArea = (
      <div>
        <Prompt
          when={this.props.savedNewCustomProject === false && !this.props.isLoggingOut}
          message={getI18n(this.props.langId,'If you have unsaved changes then they will be discarded. Do you want to leave the page anyway?')}/>

        {
          //save all files
          !this.props.isLoading &&
          <div className="flexed">
            <div className={`mar-right-half ${allFiles.length === 0 && !this.props.isSomeTestRunning ? 'div-disabled' : 'clickable'}`}
                 ref={r => this.htmlSaveButton = r}
                 onClick={() => {
                   this.onSaveKeyPress(null)
                 }}
            >
              <Icon name={this.props.customProject.id <= 0 ? 'checkmark' : 'save'}/>
              <span>
            {
              this.props.customProject.id <= 0 &&
              getI18n(this.props.langId,'Create custom project')
            }
                {
                  this.props.customProject.id > 0 &&
                  getI18n(this.props.langId,'Save project')
                }
          </span>
            </div>
            <HelpPopup defaultText={getI18n(this.props.langId,'Shortcut: ctrl+s/cmd+s. This will also clear all test results if the code was changed (because they are then invalid)')}/>
            <SimpleVDivider/>
          </div>
        }

        {
          //compile single file
          !this.props.isLoading &&
          <div className="flexed">
            <div className={
              (allFiles.length > 0 && mainFile  && !this.props.isSomeTestRunning) ? 'clickable mar-right-half' : 'div-disabled mar-right-half'
            }
                 onClick={async () => {

                   if (allFiles.length === 0) {
                     return
                   }

                   if (!mainFile) {
                     //...maybe the exercise has no default files...
                     //or no file is selected...
                     return
                   }

                   const command: RunCompileSingleFileCommandCustomProjectForBackend = {
                     customProjectId: this.props.customProject.id,
                     plangId: this.props.customProject.solution.pLangId,
                     mainFileName: mainFile.fileNameWithExtension,
                     files: allFiles.map<SolutionFileForTestServer>((p: CustomProjectSolutionFileFullBase) => {
                       return {
                         fileName: p.fileNameWithExtension,
                         fileContent: p.content,
                         templateFileId: null,
                       }
                     })
                   }

                   if (this.props.editorHasUnsavedChanges) {
                     await this.onSaveKeyPress(null)
                   }

                   //else the old result could be displayed...
                   //actually new test should be displayed (because in runCompileSingleFileDoExercise we forge a temp test)
                   //but this don't happen somehow so clear
                   this.props.setSelectedConsoleViewTest(null)

                   this.props.runCompileSingleFileCustomProjectAsync(command, this.props.testTypes)

                 }}
            >
              <Icon name="bug"/>
              <span>
            {
              getI18n(this.props.langId,'Compile single file')
            }
          </span>
            </div>
            <HelpPopup
              defaultText={getI18n(this.props.langId,'Saves all files (ONLY if the editor has unsaved changes) and compiles the program with the current file as main file.')}
              wide
            />
            <SimpleVDivider/>
          </div>
        }


        {
          //just run program
          //behind compile single file in case we want to make cmd args accessible inline
          !this.props.isLoading &&
          <div className="flexed">
            <div className={
              (allFiles.length > 0  && !this.props.isSomeTestRunning) ? 'clickable mar-right-half' : 'div-disabled mar-right-half'
            }
                 onClick={async () => {

                   if (allFiles.length === 0) {
                     return
                   }

                   const command: JustRunProgramTestCustomProjectCommandForBackend = {
                     CmdArgs: this.props.justRunProgramArgs ? this.props.justRunProgramArgs : '',
                     customProjectId: this.props.customProject.id,
                     plangId: this.props.customProject.solution.pLangId,
                   }

                   if (this.props.editorHasUnsavedChanges) {
                     await this.onSaveKeyPress(null)
                   }

                   //clear old result
                   this.props.setSelectedConsoleViewTest(null)

                   this.props.justRunProgramCustomProjectAsync(command, this.props.testTypes)

                 }}
            >
              <Icon name="rocket"/>
              <span>
              {
                getI18n(this.props.langId,'Just Run Program')
              }
              </span>
            </div>
            <HelpPopup className="mar-right-half"
              defaultText={getI18n(this.props.langId,'Saves all files (ONLY if the editor has unsaved changes) and runs the program without any input (except the given args). If one arg should contain whitespaces enclose it in "" because args are separated by whitespaces. The max length for the program args is ') + justRunProgramArgsMaxLength}
              wide
            />


            <div style={{display: 'inline-block'}} className="clickable mar-right-half" onClick={() => {
              this.props.setIsJustRunProgramArgsInputDisplayedCustomProject(
                !this.props.isJustRunProgramArgsInputDisplayed)
            }}>
              <Icon className={this.props.justRunProgramArgs ? ' toggled-colored' : ''}
                    name="font"/>
            </div>

            {
              this.props.isJustRunProgramArgsInputDisplayed &&
              <div>
                <ProgramArgsInputView programArgs={this.props.justRunProgramArgs}
                                      setProgramArgs={this.props.setJustRunProgramArgsCustomProject}/>
              </div>
            }

            <SimpleVDivider/>

            <div  className="clickable" onClick={() => {
              history.push(constants.manageCustomProjectsSiteLinkMatchPath)
            }}>
              <Icon name="reply"/>
              <span>
                {getI18n(this.props.langId, 'Leave')}
              </span>
            </div>

          </div>
        }

      </div>
    )

    return (
      <CommonHeaderBarContentWrapper leftArea={leftArea}
                                     disabled={this.props.isLoading || this.props.isHeaderBarDisabled}/>
    )
  }
}

export default connect(mapStateToProps, mapDispatchToProps)(HeaderBar)
