/**
 * Created by janis dähne.
 */

import * as React from "react";
import {connect} from "react-redux";
import {bindActionCreators, Dispatch} from "redux";
import {returntypeof} from 'react-redux-typescript';
import {RootState} from "../../../../state/reducers";
import {Icon, Popup} from "semantic-ui-react";
import {openCreateCodeTemplateDialog} from "../../../../state/actions/exerciseEditorSite/dialogs/changeCodeTemplateDialog/changeCodeTemplateDialogActions";
import {HelpPopup} from "../../../helpers/helpPopup";


import {getI18n} from "../../../../../i18n/i18nRoot";

//const css = require('./styles.styl');

export interface MyProps {
  //readonly test: string
}

const mapStateToProps = (rootState: RootState /*, props: MyProps*/) => {
  return {
    //test0: rootState...
    //test: props.test
    allPLangs: rootState.pLangsState.pLangs,
    allCodeTemplates: rootState.exerciseEditorState.editorExercise.codeTemplates,
    numReleases: rootState.exerciseEditorState.editorExercise.numReleases,
    langId: rootState.i18nState.langId
  }
}

const mapDispatchToProps = (dispatch: Dispatch) => bindActionCreators({
  //imported reducer funcs here
  openCreateCodeTemplateDialog
}, dispatch)


const stateProps = returntypeof(mapStateToProps);
const dispatchProps = returntypeof(mapDispatchToProps);
type Props = typeof stateProps & typeof dispatchProps;

class TemplatesPanelHeaderBar extends React.Component<Props, any> {
  render(): JSX.Element {

    const possiblePLangs = this.props.allPLangs.filter(pLang => {
      const codeTemplatePLang = this.props.allCodeTemplates.find(codeTemplate => codeTemplate.pLangId === pLang.id)
      return !codeTemplatePLang
    })

    return (
      <div className="flexed">

        <div className={`mar-right-half ${possiblePLangs.length === 0 ? 'div-disabled' : 'clickable'}`}
             onClick={() => {
               this.props.openCreateCodeTemplateDialog()
             }}
        >
          <Icon name="add"/>
          <span>
            {
              getI18n(this.props.langId,'Add')
            }
          </span>
        </div>
        {
          possiblePLangs.length === 0 &&
          <HelpPopup defaultText={getI18n(this.props.langId,'All programming languages have already a code template.')}/>
        }
        {
          //we have no templates
          this.props.allCodeTemplates.length === 0 &&
          <HelpPopup wide className="warning-colored" defaultText={getI18n(this.props.langId,'You need at least one code template. Only programming languages with a code template will be choosable by the users.')}/>
        }
        {
          //we have some templates but not all
          possiblePLangs.length !== 0 && this.props.allCodeTemplates.length !== 0 &&
          <HelpPopup wide defaultText={getI18n(this.props.langId,'You need at least one code template. Only programming languages with a code template will be choosable by the users.')}/>
        }

        {
          (this.props.numReleases !== 0 && this.props.numReleases !== undefined) &&
          <HelpPopup wide className="warning-colored" defaultText={getI18n(this.props.langId,'This exercise has already releases so you are not able to delete templates, add/remove template files or change the main file to another file.')}/>
        }

      </div>
    )
  }
}

export default connect(mapStateToProps, mapDispatchToProps)(TemplatesPanelHeaderBar)
