/**
 * Created by janis dähne.
 */

import * as React from "react";
import {connect} from "react-redux";
import {bindActionCreators, Dispatch} from "redux";
import {returntypeof} from 'react-redux-typescript';
import {RootState} from "../../../../state/reducers";
import {Checkbox, Form, Radio} from "semantic-ui-react";
import MaterialInput from '../../../material/materialInput'
import {
  setCanUserCreateCustomTests,
  setCanUserCreateFiles,
  setDisplayName,
  setIsOnlyVisibleToMe,
  setShortDescription,
  setShowSubmitResults,
  setUserGroupId
} from "../../../../state/actions/exerciseEditorSite/subSets/editorExercisePropertiesAction";
import {CheckboxOnChangeEventData} from "../../../../types/reactEvents";
import {
  setCompilerOptions,
  setCompileTimeoutInMs,
  setMaxDiskSpaceInKb,
  setMemoryLimitInKb,
  setTimeoutInMs
} from "../../../../state/actions/exerciseEditorSite/subSets/editorExerciseDefaultTestSettingsActions";
import {getI18n} from "../../../../../i18n/i18nRoot";
import {HelpPopup} from '../../../helpers/helpPopup'
import TestSettingsView from '../../../helpers/testing/testSettingsView'
import {TestTypeLimits} from '../../../../constants'


//const css = require('./styles.styl');

export interface MyProps {
  //readonly test: string
}

const mapStateToProps = (rootState: RootState /*, props: MyProps*/) => {
  return {
    //test0: rootState...
    //test: props.test
    exerciseProperties: rootState.exerciseEditorState.editorExercise.exerciseProperties,
    oldIsOnlyVisibleToMe: rootState.exerciseEditorState.oldIsOnlyVisibleToMe,
    creatorId: rootState.exerciseEditorState.editorExercise.creatorId,
    numReleases: rootState.exerciseEditorState.editorExercise.numReleases,
    defaultTestSettings: rootState.exerciseEditorState.editorExercise.defaultTestSettings,
    userData: rootState.userDataSettingsSate.userData,
    groups: rootState.exerciseEditorState.groups,
    langId: rootState.i18nState.langId
  }
}

const mapDispatchToProps = (dispatch: Dispatch) => bindActionCreators({
                                                                        //imported reducer funcs here
                                                                        setDisplayName,
                                                                        setShortDescription,
                                                                        setIsOnlyVisibleToMe,
                                                                        setCanUserCreateCustomTests,
                                                                        setCanUserCreateFiles,
                                                                        setShowSubmitResults,
                                                                        setUserGroupId,

                                                                        setTimeoutInMs,
                                                                        setMemoryLimitInKb,
                                                                        setMaxDiskSpaceInKb,
                                                                        setCompileTimeoutInMs,
                                                                        setCompilerOptions,
                                                                      }, dispatch)


const stateProps = returntypeof(mapStateToProps);
const dispatchProps = returntypeof(mapDispatchToProps);
type Props = typeof stateProps & typeof dispatchProps;

class exerciseSettingsPanelView extends React.Component<Props, any> {

  componentWillMount(): void {

    //pre select managing group
    //done in reducer because the groups might not be loaded here
  }


  render(): JSX.Element {

    //allow if we this is a new exercise OR if we don't have any releases OR
    // (e.g. we have releases but we loaded the exercises where only the user could use releases) so all releases are connected the the user which is ok
    //when we had this enabled and the owner added releases and disables this he cannot immediately enable it again because numReleases > 0
    /** and it is also updated via {@link UPDATE_editorExercise_FULFILLEDAction} so the update should set oldIsOnlyVisibleToMe to false (via backend) */
    const isAllowed_Allowed_setIsOnlyVisibleToMe = this.props.numReleases === undefined
      || this.props.numReleases === 0 || this.props.oldIsOnlyVisibleToMe === true

    return (
      <div className="mar-right-half">

        <div className="ui form">

          <Form.Field required>
            <label>
              {
                getI18n(this.props.langId, 'Name')
              }
            </label>
            <MaterialInput
              value={this.props.exerciseProperties.displayName}
              onChange={(e) => this.props.setDisplayName(e.currentTarget.value)}
            />
          </Form.Field>

          <Form.Field>
            <label>
              {
                getI18n(this.props.langId, 'Description')
              }
            </label>
            <MaterialInput
              value={this.props.exerciseProperties.shortDescription}
              onChange={(e) => this.props.setShortDescription(e.currentTarget.value)}
            />
          </Form.Field>


          <div>
            <Form.Field>
              <Checkbox label={getI18n(this.props.langId, 'Users are allowed to create custom tests')}
                        checked={this.props.exerciseProperties.canUserCreateCustomTests}
                        onChange={(e, data: CheckboxOnChangeEventData) => {
                          this.props.setCanUserCreateCustomTests(data.checked)
                        }}
              />
            </Form.Field>

            <Form.Field>
              <Checkbox label={getI18n(this.props.langId, 'Users are allowed to rename files and create new ones')}
                        checked={this.props.exerciseProperties.canUserCreateFiles}
                        onChange={(e, data: CheckboxOnChangeEventData) => {
                          this.props.setCanUserCreateFiles(data.checked)
                        }}
              />
            </Form.Field>

            {
              //this.props.creatorId < 0 is for new exercises
              (this.props.creatorId === this.props.userData.id || this.props.creatorId < 0) &&
              <Form.Field>
                <Checkbox className="mar-right-half"
                          disabled={!isAllowed_Allowed_setIsOnlyVisibleToMe}
                          label={getI18n(this.props.langId,
                                         'Exercise is only visible to owner'
                          )}
                          checked={this.props.exerciseProperties.isOnlyVisibleToMe}
                          onChange={(e, data: CheckboxOnChangeEventData) => {
                            this.props.setIsOnlyVisibleToMe(data.checked)
                          }}
                />
                <HelpPopup defaultText={getI18n(this.props.langId,
                                                "This option can be checked if there are no releases for this exercise. If it's checked only you will see the exercise in the group (and only you can edit it). Releases for this exercise will only work for you."
                )} wide/>
              </Form.Field>
            }


          </div>


          <h4 className="ui dividing header">
            {
              getI18n(this.props.langId, 'Settings for custom tests')
            }
            <HelpPopup className="mar-left-half" defaultText={getI18n(this.props.langId,
                                                                      "These test settings are applied to custom tests (tests created by the user)"
            )}
            />
          </h4>

          <TestSettingsView
            testTypeLimits={TestTypeLimits.IsNormalTest}
            timeoutInMs={this.props.defaultTestSettings.timeoutInMs}
            memoryLimitInKb={this.props.defaultTestSettings.memoryLimitInKb}
            maxDiskSpaceInKb={this.props.defaultTestSettings.maxDiskSpaceInKb}
            compileTimeoutInMs={this.props.defaultTestSettings.compileTimeoutInMs}
            compilerOptions={this.props.defaultTestSettings.compilerOptions}

            set_timeoutInMs={this.props.setTimeoutInMs}
            set_memoryLimitInKb={this.props.setMemoryLimitInKb}
            set_maxDiskSpaceInKb={this.props.setMaxDiskSpaceInKb}
            set_compileTimeoutInMs={this.props.setCompileTimeoutInMs}
            set_compilerOptions={this.props.setCompilerOptions}
          />

          <h4 className="ui dividing header">
            {
              getI18n(this.props.langId, 'Managed by group')
            }
            <HelpPopup className="mar-left-half" defaultText={getI18n(this.props.langId,
                                                                      "The exercise will be connected to the selected group and members with permission will be able to see/edit/delete the exercise (as long as you not checked the 'is only visible to owner' option). If your account gets deleted then the exercise can still be managed through this group"
            )}
                       wide
            />
          </h4>


          <Form.Field>
            {
              this.props.groups.filter(p => p.groupRole && p.groupRole.canCreateExercises).map(group => {
                return (
                  <Form.Field key={group.id}>
                    <Radio label={group.displayName}
                           name="groupRadio"
                           checked={group.id === this.props.exerciseProperties.userGroupId}
                           value={group.id}
                           onChange={(e, data: CheckboxOnChangeEventData) => {
                             if (data.checked) {
                               this.props.setUserGroupId(group.id)
                             }
                           }}
                    />
                  </Form.Field>
                )
              })
            }
          </Form.Field>

        </div>

      </div>
    )
  }
}

export default connect(mapStateToProps, mapDispatchToProps)(exerciseSettingsPanelView)
