import * as React from "react";
import {bindActionCreators, Dispatch} from "redux";
import {returntypeof} from 'react-redux-typescript';
import {RootState} from '../../../../../state/reducers'
import {CompareFileTestContentFrontendOnly, FilePreviewFromBackend} from '../../../../../types/exerciseEditor'
import {connect} from 'react-redux'
import {getI18n} from '../../../../../../i18n/i18nRoot'
import {HelpPopup} from '../../../../helpers/helpPopup'
import {convertCompareTestToObj} from '../../../../../types/testsAndTestProtocol'
import {CompareTestOptions} from '../../../../../types/testResults'

export interface MyProps {
  readonly files: ReadonlyArray<FilePreviewFromBackend>
  readonly testContent: string

  readonly isReadonly: boolean

  /**
   * null is only allowed for readonly mode
   */
  readonly setTestContent: ((testContent: string) => void) | null
}

const mapStateToProps = (rootState: RootState, props: MyProps) => {
  return {
    langId: rootState.i18nState.langId,
    ...props
  }
}

const mapDispatchToProps = (dispatch: Dispatch) => bindActionCreators({
                                                                        //imported reducer funcs here

                                                                      }, dispatch)


const stateProps = returntypeof(mapStateToProps);
const dispatchProps = returntypeof(mapDispatchToProps);
type Props = typeof stateProps & typeof dispatchProps;


class CompareFilesTestSettingsView extends React.Component<Props, any> {


  render(): JSX.Element {


    // tslint:disable-next-line
    let fileCompares: CompareFileTestContentFrontendOnly[] = []

    try {
      const temp = this.props.testContent
        .split('\n')
        .filter(p => p !== '')
        .map<CompareFileTestContentFrontendOnly | null>(convertCompareTestToObj)
        .filter(p => p !== null)
      fileCompares = temp
    } catch (err) {
      //TODO show error??
    }

    const hasCompareTest = fileCompares.some(p => p !== null)

    // tslint:disable-next-line
    let errorMsg = ''

    if (hasCompareTest) {

      //compare tests must be the last lines (because they are executed after the user program finished)

      // tslint:disable-next-line
      let checkNext = false //if true the next test line must be a compare test too

      // tslint:disable-next-line
      for (let i = 0; i < fileCompares.length; i++) {
        const test = fileCompares[i]

        if (test !== null) {
          //next must be a compare test or end of test lines
          checkNext = true
          continue
        }

        if (checkNext) {

          if (test !== null) {
            //all ok
          } else {
            errorMsg = getI18n(this.props.langId, "A compare test line must be placed after all black box test lines")
            break
          }
        }
      }


      //check if the files exist
      // tslint:disable-next-line
      for (let i = 0; i < fileCompares.length; i++) {
        const test = fileCompares[i]
        if (test === null) continue

        const found = this.props.files.some(p => p.originalName === test.file1NameWithExtension)

        if (!found) {
          errorMsg = `${getI18n(this.props.langId, "Could not find file with name")} '${test.file1NameWithExtension}'`
          break
        }

      }

    }


    return (
      <div>

        <div className="flexed mar-bot">

          <span className="mar-right">{getI18n(this.props.langId, "Compare files options")}</span>

          <div>
            <span>{CompareTestOptions.ignoreCase.toString()}: {getI18n(this.props.langId, "Ignore case")}</span>
          </div>
          <div className="mar-left">
            <span className="mar-right-half">{CompareTestOptions.ignoreLeadingWhitespaces.toString()}: {getI18n(
              this.props.langId, "Ignore leading whitespaces")}</span>
            <HelpPopup defaultText={getI18n(this.props.langId,
                                            "This will ignore all leading whitespace characters (including tabs)"
            )}/>
          </div>
          <div className="mar-left">
            <span className="mar-right-half">{CompareTestOptions.ignoreTrailingWhitespaces.toString()}: {getI18n(
              this.props.langId, "Ignore trailing whitespaces")}</span>
            <HelpPopup defaultText={getI18n(this.props.langId,
                                            "This will ignore all trailing whitespace characters (including tabs)"
            )}/>
          </div>
          <div className="mar-left">
            <span>{CompareTestOptions.ignoreAllEmptyOrWhitespacesOnlyLines.toString()}: {getI18n(this.props.langId,
                                                                                                 "Ignore all additional empty lines in the user file"
            )}</span>
          </div>

        </div>

        {
          this.props.files.length === 0 &&
          <pre className="error-colored">
            {getI18n(this.props.langId, "For a compare files test you need to add files in the test files tab")}
          </pre>
        }

        {
          errorMsg !== '' &&
          <pre className="error-colored">
            {
              errorMsg
            }
          </pre>
        }


      </div>
    )
  }
}

export default connect(mapStateToProps, mapDispatchToProps)(CompareFilesTestSettingsView)


