/**
 * Created by janis dähne.
 */

import * as React from "react";
import {connect} from "react-redux";
import {bindActionCreators, Dispatch} from "redux";
import {returntypeof} from 'react-redux-typescript';
import {RootState} from "../../../../state/reducers";
import SurveyParser from "../../../../questionSystem/surveySyntaxParser";
import {QuestionParseResult} from "../../../../types/questions";
import RenderWrapper from '../../../renderContentComponents/renderWrapper'
import {List} from "semantic-ui-react";
import {FileWithData} from "../../../../types/exerciseEditor";

//const css = require('./styles.styl');

export interface MyProps {
  //readonly test: string
  readonly descriptionContent: string

  readonly assets: ReadonlyArray<FileWithData>

  /**
   * used to toggle re render
   */
  readonly dummy: boolean

  readonly fontSizeInPx: number

  readonly printId: string

  /**
   * some (markdown) blocks might be only for specific languages,
   * use null to show them all
   */
  readonly plangId: number | null
}

const mapStateToProps = (rootState: RootState, props: MyProps) => {
  return {
    //test0: rootState...
    //test: props.test
    ...props
  }
}

const mapDispatchToProps = (dispatch: Dispatch) => bindActionCreators({
                                                                        //imported reducer funcs here

                                                                      }, dispatch)


const stateProps = returntypeof(mapStateToProps);
const dispatchProps = returntypeof(mapDispatchToProps);
type Props = typeof stateProps & typeof dispatchProps;

class PreviewPanelView extends React.Component<Props, any> {
  render(): JSX.Element {

    const parsedQuestion: QuestionParseResult = SurveyParser.parseQuestion(this.props.descriptionContent)

    return (
      <div className="fh">

        {
          (parsedQuestion.errors && parsedQuestion.errors.length > 0) &&
          <div>
            <List selection divided relaxed>
              {
                parsedQuestion.errors.map((error, index) => {
                  return (
                    <List.Item key={index} className="error-colored-i default-cursor">
                      <List.Icon name='remove circle' size='large' verticalAlign='middle'/>
                      <List.Content verticalAlign='middle'>
                        {
                          error
                        }
                      </List.Content>
                    </List.Item>
                  )
                })
              }
            </List>
          </div>
        }

        {
          (!parsedQuestion.errors || parsedQuestion.errors.length === 0 || parsedQuestion.result) &&
          <RenderWrapper
            printId={this.props.printId}
            content={parsedQuestion.result.question} contentType={parsedQuestion.result.questionType}
            assets={this.props.assets}
            dummy={this.props.dummy}
            fontSizeInPx={this.props.fontSizeInPx}
            plangId={this.props.plangId}
          />
        }

      </div>
    )
  }
}

export default connect(mapStateToProps, mapDispatchToProps)(PreviewPanelView)
