/**
 * Created by janis dähne.
 */

import * as React from "react";
import {connect} from "react-redux";
import {bindActionCreators, Dispatch} from "redux";
import {returntypeof} from 'react-redux-typescript';
import {RootState} from "../../../../state/reducers";
import {Checkbox, Form} from "semantic-ui-react";
import MaterialInput from '../../../material/materialInput'
import {
  setCanAddUserToGroup,
  setCanAssessExercises,
  setCanChangeExercises, setCanChangeGroupData,
  setCanChangeOtherMembersRole,
  setCanCreateExercises,
  setCanDeleteExercises, setCanLockExercisesPermanently,
  setCanManageExerciseReleases,
  setCanRemoveMemberFromGroup,
  setCanSeeExercisesFromOthersInGroup,
  setCanSeeOtherMembers,
  setDisplayName,
  setId,
} from "../../../../state/actions/groupRolesSite/dialog/groupRoleSetAction";
import {
  validationRules
} from "../../../../state/reducers/groupRolesSite/groupRoleReducer/groupRoleReducer";
import {
  validationMessageKeys
} from "../../../../state/reducers/groupRolesSite/groupRoleReducer/groupRoleReducerValidation";
import {getI18n} from "../../../../../i18n/i18nRoot";

//const css = require('./styles.styl');

export interface MyProps {
  //readonly test: string
}

const mapStateToProps = (rootState: RootState /*, props: MyProps*/) => {
  return {
    //test0: rootState...
    //test: props.test
    groupRole: rootState.groupRolesChangeDialogState.groupRole,
    langId: rootState.i18nState.langId
  }
}

const mapDispatchToProps = (dispatch: Dispatch) => bindActionCreators({
                                                                        //imported reducer funcs here
                                                                        setId,
                                                                        setDisplayName,
                                                                        setCanCreateExercises,
                                                                        setCanChangeExercises,
                                                                        setCanDeleteExercises,
                                                                        setCanLockExercisesPermanently,
                                                                        setCanManageExerciseReleases,
                                                                        setCanAssessExercises,
                                                                        setCanSeeExercisesFromOthersInGroup,
                                                                        setCanAddUserToGroup,
                                                                        setCanRemoveMemberFromGroup,
                                                                        setCanChangeOtherMembersRole,
                                                                        setCanSeeOtherMembers,
                                                                        setCanChangeGroupData
                                                                      }, dispatch)


const stateProps = returntypeof(mapStateToProps);
const dispatchProps = returntypeof(mapDispatchToProps);
type Props = typeof stateProps & typeof dispatchProps;

class ChangeGroupRoleView extends React.Component<Props, any> {
  render(): JSX.Element {
    return (
      <div>
        <div className="ui form divided-checkboxes">
          <Form.Group widths='equal'>
            <Form.Field required>
              <label>
                {
                  getI18n(this.props.langId, 'Name')
                }
              </label>
              <MaterialInput
                value={this.props.groupRole.displayName}
                onChange={(e) => this.props.setDisplayName(e.currentTarget.value)}
                validations={validationRules.displayName}
                validationMessageKey={validationMessageKeys.displayName}
              />
            </Form.Field>
          </Form.Group>

          <h4 className="ui dividing header">
            {
              getI18n(this.props.langId, 'Other')
            }
          </h4>
          <Form.Group widths='equal'>

            <Form.Field>
              <Form.Field>
                <Checkbox label={getI18n(this.props.langId, 'Can create exercises')}
                          checked={this.props.groupRole.canCreateExercises}
                          onChange={(e, data) => {
                            this.props.setCanCreateExercises(data.checked)
                          }}
                />
              </Form.Field>
              <Form.Field>
                <Checkbox label={getI18n(this.props.langId, 'Can change exercises')}
                          checked={this.props.groupRole.canChangeExercises}
                          onChange={(e, data) => {
                            this.props.setCanChangeExercises(data.checked)
                          }}
                />
              </Form.Field>
              <Form.Field>
                <Checkbox label={getI18n(this.props.langId, 'Can delete exercises')}
                          checked={this.props.groupRole.canDeleteExercises}
                          onChange={(e, data) => {
                            this.props.setCanDeleteExercises(data.checked)
                          }}
                />
              </Form.Field>

              <Form.Field>
                <Checkbox label={getI18n(this.props.langId, 'Can lock exercises permanently')}
                          checked={this.props.groupRole.canLockExercisesPermanently}
                          onChange={(e, data) => {
                            this.props.setCanLockExercisesPermanently(data.checked)
                          }}
                />
              </Form.Field>
            </Form.Field>
            <Form.Field>
              <Form.Field>
                <Checkbox label={getI18n(this.props.langId, 'Can manage exercise releases')}
                          checked={this.props.groupRole.canManageExerciseReleases}
                          onChange={(e, data) => {
                            this.props.setCanManageExerciseReleases(data.checked)
                          }}
                />
              </Form.Field>
              <Form.Field>
                <Checkbox label={getI18n(this.props.langId, 'Can see exercises from other members')}
                          checked={this.props.groupRole.canSeeExercisesFromOthersInGroup}
                          onChange={(e, data) => {
                            this.props.setCanSeeExercisesFromOthersInGroup(data.checked)
                          }}
                />
              </Form.Field>


            </Form.Field>
          </Form.Group>


          <h4 className="ui dividing header">
            {
              getI18n(this.props.langId, 'Groups')
            }
          </h4>
          <Form.Group widths='equal'>
            <Form.Field>
              <Form.Field>
                <Checkbox label={getI18n(this.props.langId, 'Can add users to group')}
                          checked={this.props.groupRole.canAddUserToGroup}
                          onChange={(e, data) => {
                            this.props.setCanAddUserToGroup(data.checked)
                          }}
                />
              </Form.Field>
              <Form.Field>
                <Checkbox label={getI18n(this.props.langId, 'Can remove members from group')}
                          checked={this.props.groupRole.canRemoveMemberFromGroup}
                          onChange={(e, data) => {
                            this.props.setCanRemoveMemberFromGroup(data.checked)
                          }}
                />
              </Form.Field>
              <Form.Field>
                <Checkbox label={getI18n(this.props.langId, 'Can change other members group role')}
                          checked={this.props.groupRole.canChangeOtherMembersRole}
                          onChange={(e, data) => {
                            this.props.setCanChangeOtherMembersRole(data.checked)
                          }}
                />
              </Form.Field>
              <Form.Field>
                <Checkbox label={getI18n(this.props.langId, 'Can see other members')}
                          checked={this.props.groupRole.canSeeOtherMembers}
                          onChange={(e, data) => {
                            this.props.setCanSeeOtherMembers(data.checked)
                          }}
                />
              </Form.Field>
              <Form.Field>
                <Checkbox label={getI18n(this.props.langId, 'Can change group data')}
                          checked={this.props.groupRole.canChangeGroupData}
                          onChange={(e, data) => {
                            this.props.setCanChangeGroupData(data.checked)
                          }}
                />
              </Form.Field>
            </Form.Field>
          </Form.Group>

          <h4 className="ui dividing header">
            {
              getI18n(this.props.langId, 'Submissions')
            }
          </h4>
          <Form.Group widths='equal'>
            <Form.Field>
              <Form.Field>
                <Checkbox label={getI18n(this.props.langId,
                                         'Can assess exercises from group'
                )}
                          checked={this.props.groupRole.canAssessExercises}
                          onChange={(e, data) => {
                            this.props.setCanAssessExercises(data.checked)
                          }}
                />
              </Form.Field>
            </Form.Field>
          </Form.Group>


        </div>
      </div>
    )
  }
}

export default connect(mapStateToProps, mapDispatchToProps)(ChangeGroupRoleView)
