/**
 * Created by janis dähne.
 */

import * as React from "react";
import {connect} from "react-redux";
import {bindActionCreators, Dispatch} from "redux";
import {returntypeof} from 'react-redux-typescript';
import {RootState} from "../../../state/reducers";
import {Checkbox, Table} from "semantic-ui-react";
import {
  setActivatedUserUiIsSelected,
  setActivePaginationPage,
  setAllActivatedUsersUiIsSelected,
  setIsChangeUserDataDialogDisplayed,
  setPaginationPageSize,
  setSortByKey
} from "../../../state/actions/manageActivatedUsersSite/manageActivatedUsersSiteActions";
import {setTargetUser} from "../../../state/actions/manageActivatedUsersSite/dialogs/changeActivatedUserDataDialogActions";
import {CheckboxOnChangeEventData} from "../../../types/reactEvents";
import {deleteActivatedUserAsync} from "../../../state/actions/manageActivatedUsersSite/manageActivatedUsersSiteCrudActions";
import {SelectableUserFrontendExclusive} from "../../../types/users";
import {isCurrentSortingColumnPaginated, rotateSorting} from "../../../helpers/tablesHelper";
import ListViewItem from './listViewItem'
import PaginationTableFooter from '../../helpers/paginationTableFooter'


import {HelpPopup} from '../../helpers/helpPopup'
import {getI18n} from "../../../../i18n/i18nRoot";
import {MyPopup} from '../../helpers/myPopup'

//const css = require('./styles.styl');

export interface MyProps {
  //readonly test: string
}

const mapStateToProps = (rootState: RootState /*, props: MyProps*/) => {
  return {
    //test0: rootState...
    //test: props.test
    activatedUsers: rootState.manageActivatedUsersState.activatedUsers,
    sortByKey: rootState.manageActivatedUsersState.pagination.orderByProp,
    sortDirection: rootState.manageActivatedUsersState.pagination.isOrderByDesc,
    userLangId: rootState.userDataSettingsSate.userData.settings
                ? rootState.userDataSettingsSate.userData.settings.langId
                : -1,
    langId: rootState.i18nState.langId,
    pagination: rootState.manageActivatedUsersState.pagination
  }
}

const mapDispatchToProps = (dispatch: Dispatch) => bindActionCreators({
                                                                        //imported reducer funcs here
                                                                        setIsChangeUserDataDialogDisplayed,
                                                                        setTargetUser,
                                                                        setActivatedUserUiIsSelected,
                                                                        setAllActivatedUsersUiIsSelected,
                                                                        deleteActivatedUserAsync,
                                                                        setSortByKey,
                                                                        setActivePaginationPageAction: setActivePaginationPage,
                                                                        setPaginationPageSize
                                                                      }, dispatch)


const stateProps = returntypeof(mapStateToProps);
const dispatchProps = returntypeof(mapDispatchToProps);
type Props = typeof stateProps & typeof dispatchProps;


class ActivatedUsersListView extends React.Component<Props, any> {

  render(): JSX.Element {

    const list = this.props.activatedUsers

    return (
      <div>
        <Table sortable selectable padded striped singleLine className="material-table">
          <Table.Header>
            <Table.Row>
              <Table.HeaderCell>
                <MyPopup
                trigger={
                  <Checkbox
                    checked={
                      this.props.activatedUsers.every(p => p.uiIsSelected) && this.props.activatedUsers.length > 0
                    }
                    onChange={(e: any, data: CheckboxOnChangeEventData) => {
                      this.props.setAllActivatedUsersUiIsSelected(data.checked)
                    }}
                  />
                }
                on="hover"
                content={getI18n(this.props.langId, "Will only select all items on the current page")}
                />

              </Table.HeaderCell>
              <Table.HeaderCell
                sorted={
                  isCurrentSortingColumnPaginated<SelectableUserFrontendExclusive>(this.props.sortByKey, 'lastName',
                                                                                   this.props.sortDirection
                  )
                }
              >
                <span className="clickable"
                      onClick={() => {
                        this.props.setSortByKey('lastName',
                                                rotateSorting<SelectableUserFrontendExclusive>(this.props.sortDirection,
                                                                                               this.props.sortByKey,
                                                                                               'lastName'
                                                )
                        )
                      }}
                >
                  {
                    getI18n(this.props.langId, 'Last name')
                  }
                </span>
              </Table.HeaderCell>
              <Table.HeaderCell
                sorted={
                  isCurrentSortingColumnPaginated<SelectableUserFrontendExclusive>(this.props.sortByKey, 'firstName',
                                                                                   this.props.sortDirection
                  )
                }
              >
                <span className="clickable"
                      onClick={() => {
                        this.props.setSortByKey('firstName',
                                                rotateSorting<SelectableUserFrontendExclusive>(this.props.sortDirection,
                                                                                               this.props.sortByKey,
                                                                                               'firstName'
                                                )
                        )
                      }}
                >
                  {
                    getI18n(this.props.langId, 'First name')
                  }
                </span>
              </Table.HeaderCell>
              <Table.HeaderCell
                sorted={
                  isCurrentSortingColumnPaginated<SelectableUserFrontendExclusive>(this.props.sortByKey, 'token',
                                                                                   this.props.sortDirection
                  )
                }
              >
                <span className="clickable"
                      onClick={() => {
                        this.props.setSortByKey('token',
                                                rotateSorting<SelectableUserFrontendExclusive>(this.props.sortDirection,
                                                                                               this.props.sortByKey,
                                                                                               'token'
                                                )
                        )
                      }}
                >
                  {
                    getI18n(this.props.langId, 'Token')
                  }
              </span>
              </Table.HeaderCell>
              <Table.HeaderCell
                sorted={
                  isCurrentSortingColumnPaginated<SelectableUserFrontendExclusive>(this.props.sortByKey, 'email',
                                                                                   this.props.sortDirection
                  )
                }
              >
                <span
                  className="clickable"
                  onClick={() => {
                    this.props.setSortByKey('email',
                                            rotateSorting<SelectableUserFrontendExclusive>(this.props.sortDirection,
                                                                                           this.props.sortByKey, 'email'
                                            )
                    )
                  }}
                >
                  {
                    getI18n(this.props.langId, 'Email')
                  }
                </span>
              </Table.HeaderCell>
              <Table.HeaderCell
                sorted={
                  isCurrentSortingColumnPaginated<SelectableUserFrontendExclusive>(this.props.sortByKey, 'systemRole',
                                                                                   this.props.sortDirection
                  )
                }
              >
                <span
                  className="clickable"
                  onClick={() => {
                    this.props.setSortByKey('systemRole',
                                            rotateSorting<SelectableUserFrontendExclusive>(this.props.sortDirection,
                                                                                           this.props.sortByKey,
                                                                                           'systemRole'
                                            )
                    )
                  }}
                >
                  {
                    getI18n(this.props.langId, 'System role')
                  }
                </span>
              </Table.HeaderCell>
              <Table.HeaderCell
                sorted={
                  isCurrentSortingColumnPaginated<SelectableUserFrontendExclusive>(this.props.sortByKey, 'createdAt',
                                                                                   this.props.sortDirection
                  )
                }
              >
                <span
                  className="clickable"
                  onClick={() => {
                    this.props.setSortByKey('createdAt',
                                            rotateSorting<SelectableUserFrontendExclusive>(this.props.sortDirection,
                                                                                           this.props.sortByKey,
                                                                                           'createdAt'
                                            )
                    )
                  }}
                >
                  {
                    getI18n(this.props.langId, 'Created at')
                  }
                </span>
              </Table.HeaderCell>
              <Table.HeaderCell
                sorted={
                  isCurrentSortingColumnPaginated<SelectableUserFrontendExclusive>(this.props.sortByKey, 'lastLoginAt',
                                                                                   this.props.sortDirection
                  )
                }
              >
                <span
                  className="clickable"
                  onClick={() => {
                    this.props.setSortByKey('lastLoginAt',
                                            rotateSorting<SelectableUserFrontendExclusive>(this.props.sortDirection,
                                                                                           this.props.sortByKey,
                                                                                           'lastLoginAt'
                                            )
                    )
                  }}
                >
                  {
                    getI18n(this.props.langId, 'Last login at')
                  }
                </span>
              </Table.HeaderCell>
              <Table.HeaderCell>
                <HelpPopup icon="area graph" defaultText={getI18n(this.props.langId, 'Assessment statistic')}/>
              </Table.HeaderCell>
              <Table.HeaderCell>
                {
                  getI18n(this.props.langId, 'Change')
                }
              </Table.HeaderCell>
              <Table.HeaderCell>
                {
                  getI18n(this.props.langId, 'Delete')
                }
              </Table.HeaderCell>
            </Table.Row>
          </Table.Header>
          <Table.Body>
            {
              list
                .map((activatedUser, index) => {
                  return (
                    <ListViewItem key={activatedUser.id} activatedUser={activatedUser}/>
                  )
                })
            }

          </Table.Body>

          <PaginationTableFooter
            pagination={this.props.pagination}
            setActivePaginationPage={this.props.setActivePaginationPageAction}
            setPaginationPageSize={this.props.setPaginationPageSize}
          />
        </Table>

      </div>
    )
  }
}

export default connect(mapStateToProps, mapDispatchToProps)(ActivatedUsersListView)