/**
 * Created by janis dähne.
 */

import * as React from "react";
import {connect} from "react-redux";
import {bindActionCreators, Dispatch} from "redux";
import {returntypeof} from 'react-redux-typescript';
import {RootState} from "../../../state/reducers";
import {
  loadManageCustomProjectSite,
  setSearchText
} from "../../../state/actions/manageCustomProjectsSite/manageCustomProjectsSiteActions";
import {errorDialog} from "../../../helpers/dialogHelper";
import Logger from "../../../helpers/logger";


import SinglePanelSiteWrapper from '../../singlePanelSiteWrapper'
import MaterialSearchInput from '../../material/materialSearchInput'

import ListView from './listView'
import Spinner from "../../helpers/spinner";
import {getI18n} from "../../../../i18n/i18nRoot";
import {InputOnChangeEvent} from '../../../types/reactEvents'
import {ErrorHelper} from '../../../helpers/errorHelper'

//const css = require('./styles.styl');

export interface MyProps {
  //readonly test: string
}

const mapStateToProps = (rootState: RootState /*, props: MyProps*/) => {
  return {
    //test0: rootState...
    //test: props.test
    isLoading: rootState.manageCustomProjectsSiteState.isLoading
      || rootState.systemSettingsSiteState.isLoading
      || rootState.pLangsState.isLoading
      || rootState.globalSystemSettingsState.isLoading,

    langId: rootState.i18nState.langId,
    pagination: rootState.manageCustomProjectsSiteState.pagination
  }
}

const mapDispatchToProps = (dispatch: Dispatch) => bindActionCreators({
                                                                        //imported reducer funcs here
                                                                        loadManageCustomProjectSite,
                                                                        setSearchText
                                                                      }, dispatch)


const stateProps = returntypeof(mapStateToProps);
const dispatchProps = returntypeof(mapDispatchToProps);
type Props = typeof stateProps & typeof dispatchProps;

class ManageCustomProjectsSite extends React.Component<Props, any> {

  async componentWillMount(): Promise<void> {
    try {
      await this.props.loadManageCustomProjectSite()
    } catch (err) {
      ErrorHelper.makeServerErrorDialog(err, this.props.langId, 'Server error', 'Could not load custom projects')
      return null
    }
  }

  render(): JSX.Element {
    return (
      <SinglePanelSiteWrapper>

        {
          this.props.isLoading &&
          <Spinner/>
        }

        <div className="view-padding">
          <div className="view-head-line">
            <div className="view-caption">
              <h1>
                {
                  getI18n(this.props.langId, 'Manage custom projects')
                }
              </h1>
            </div>
            <div className="view-options">
              <MaterialSearchInput
                value={this.props.pagination.searchText}
                onChange={(e: InputOnChangeEvent) => {
                  this.props.setSearchText(e.currentTarget.value)
                }}
              />
            </div>
          </div>

          <div className="view-content">
            <ListView/>
          </div>
        </div>

      </SinglePanelSiteWrapper>
    )
  }
}

export default connect(mapStateToProps, mapDispatchToProps)(ManageCustomProjectsSite)
