/**
 * Created by janis dähne.
 */

import * as React from "react";
import {connect} from "react-redux";
import {bindActionCreators, Dispatch} from "redux";
import {returntypeof} from 'react-redux-typescript';
import {RootState} from "../../../state/reducers";
import {Checkbox, Icon, Table} from "semantic-ui-react";
import {EditableExercisePreviewFromBackend} from "../../../types/exercisePreview";
import {Link} from "react-router-dom";
import * as constants from '../../../constants'
import {getExerciseEditorEditToLink} from '../../../constants'
import {isCurrentSortingColumnPaginated, rotateSorting} from "../../../helpers/tablesHelper";
import {displayDateAndTime} from "../../../helpers/convertersAndTransformers";
import {HelpPopup} from "../../helpers/helpPopup";
import {askDialog, errorDialog} from "../../../helpers/dialogHelper";


import {GroupRoleWithPermissionFullBase} from "../../../types/groupRoles";
import Tag from '../../tagComponents/tag'
import {getI18n} from "../../../../i18n/i18nRoot";
import PaginationTableFooter from '../../helpers/paginationTableFooter'
import {PaginationStateData} from '../../../types/pagination'
import Spinner from '../../helpers/spinner'
import {copyExercise} from '../../../communicationLayer/exerciseEditorLayer'
import Logger from '../../../helpers/logger'
import history from '../../../routerHistory'
import {debugOriginUrl, isProduction} from '../../../../debug'


//const css = require('./styles.styl');

export interface MyProps {
  readonly exercisePreviews: ReadonlyArray<EditableExercisePreviewFromBackend>
  readonly comesFromOwnExercisesSite: boolean

  /**
   * for the groups exercise overview we need the group id
   */
  readonly groupId: number | null

  readonly sortByKey: keyof EditableExercisePreviewFromBackend
  readonly sortDirection: boolean | null
  readonly pagination: PaginationStateData<EditableExercisePreviewFromBackend>
  readonly setSortByKey: ((sortByKey: keyof EditableExercisePreviewFromBackend,
                           sortDirection: boolean | null
  ) => void) |
    ((sortByKey: keyof EditableExercisePreviewFromBackend, sortDirection: boolean | null, groupId: number) => void)

  readonly setPaginationPageSize: ((pageSize: number) => void) | ((pageSize: number, groupId: number) => void)
  readonly setActivePaginationPage: ((activePage: number) => void) | ((activePage: number, groupId: number) => void)


  readonly onDeleteExercise: (exercisePreview: EditableExercisePreviewFromBackend) => void

  /**
   * the group role, null = own exercises --> allow all
   */
  readonly currentUserGroupRole: GroupRoleWithPermissionFullBase | null

  readonly isLoading: boolean

  readonly setIsLoading: (isLoading: boolean) => void
}

const mapStateToProps = (rootState: RootState, props: MyProps) => {
  return {
    //test0: rootState...
    //test: props.test
    ...props,
    langId: rootState.i18nState.langId
  }
}

const mapDispatchToProps = (dispatch: Dispatch) => bindActionCreators({
                                                                        //imported reducer funcs here

                                                                      }, dispatch)


const stateProps = returntypeof(mapStateToProps);
const dispatchProps = returntypeof(mapDispatchToProps);
type Props = typeof stateProps & typeof dispatchProps;


class TableView extends React.Component<Props, any> {
  render(): JSX.Element {

    const list: ReadonlyArray<EditableExercisePreviewFromBackend> = this.props.exercisePreviews

    if (!list) return null

    return (
      <div>

        {
          this.props.isLoading &&
          <Spinner/>
        }

        <Table sortable selectable padded striped singleLine className="material-table">

          <Table.Header>
            <Table.Row>
              <Table.HeaderCell sorted={
                isCurrentSortingColumnPaginated<EditableExercisePreviewFromBackend>(this.props.sortByKey, 'displayName',
                                                                                    this.props.sortDirection
                )
              }
              >
                <span className="clickable"
                      onClick={() => {

                        if (this.props.groupId === null) {
                          const func = this.props.setSortByKey as (sortByKey: keyof EditableExercisePreviewFromBackend,
                                                                   sortDirection: boolean | null
                          ) => void
                          func('displayName',
                               rotateSorting<EditableExercisePreviewFromBackend>(
                                 this.props.sortDirection,
                                 this.props.sortByKey,
                                 'displayName'
                               )
                          )
                          return
                        }

                        const func = this.props.setSortByKey as (sortByKey: keyof EditableExercisePreviewFromBackend,
                                                                 sortDirection: boolean | null, groupId: number
                        ) => void
                        func('displayName',
                             rotateSorting<EditableExercisePreviewFromBackend>(
                               this.props.sortDirection,
                               this.props.sortByKey,
                               'displayName',
                             ),
                             this.props.groupId
                        )
                      }}
                >
                  {
                    getI18n(this.props.langId, 'Name')
                  }
                </span>
              </Table.HeaderCell>
              <Table.HeaderCell sorted={
                isCurrentSortingColumnPaginated<EditableExercisePreviewFromBackend>(this.props.sortByKey,
                                                                                    'shortDescription',
                                                                                    this.props.sortDirection
                )
              }
              >
                <span className="clickable"
                      onClick={() => {
                        if (this.props.groupId === null) {
                          const func = this.props.setSortByKey as (sortByKey: keyof EditableExercisePreviewFromBackend,
                                                                   sortDirection: boolean | null
                          ) => void
                          func('shortDescription',
                               rotateSorting<EditableExercisePreviewFromBackend>(
                                 this.props.sortDirection,
                                 this.props.sortByKey,
                                 'shortDescription'
                               )
                          )
                          return
                        }

                        const func = this.props.setSortByKey as (sortByKey: keyof EditableExercisePreviewFromBackend,
                                                                 sortDirection: boolean | null, groupId: number
                        ) => void
                        func('shortDescription',
                             rotateSorting<EditableExercisePreviewFromBackend>(
                               this.props.sortDirection,
                               this.props.sortByKey,
                               'shortDescription',
                             ),
                             this.props.groupId
                        )
                      }}
                >
                  {
                    getI18n(this.props.langId, 'Description')
                  }
                </span>
              </Table.HeaderCell>
              <Table.HeaderCell>
                <span>
                {
                  getI18n(this.props.langId, 'Tags')
                }
              </span>
              </Table.HeaderCell>
              <Table.HeaderCell sorted={
                isCurrentSortingColumnPaginated<EditableExercisePreviewFromBackend>(this.props.sortByKey,
                                                                                    'isOnlyVisibleToMe',
                                                                                    this.props.sortDirection
                )
              }
              >
                <span className="clickable"
                      onClick={() => {
                        if (this.props.groupId === null) {
                          const func = this.props.setSortByKey as (sortByKey: keyof EditableExercisePreviewFromBackend,
                                                                   sortDirection: boolean | null
                          ) => void
                          func('isOnlyVisibleToMe',
                               rotateSorting<EditableExercisePreviewFromBackend>(
                                 this.props.sortDirection,
                                 this.props.sortByKey,
                                 'isOnlyVisibleToMe'
                               )
                          )
                          return
                        }

                        const func = this.props.setSortByKey as (sortByKey: keyof EditableExercisePreviewFromBackend,
                                                                 sortDirection: boolean | null, groupId: number
                        ) => void
                        func('isOnlyVisibleToMe',
                             rotateSorting<EditableExercisePreviewFromBackend>(
                               this.props.sortDirection,
                               this.props.sortByKey,
                               'isOnlyVisibleToMe',
                             ),
                             this.props.groupId
                        )
                      }}
                >
                  {
                    getI18n(this.props.langId, 'Only visible to me')
                  }
                </span>
              </Table.HeaderCell>
              <Table.HeaderCell sorted={
                isCurrentSortingColumnPaginated<EditableExercisePreviewFromBackend>(this.props.sortByKey,
                                                                                    'lastUpdatedAt',
                                                                                    this.props.sortDirection
                )
              }
              >
                <span className="clickable"
                      onClick={() => {
                        if (this.props.groupId === null) {
                          const func = this.props.setSortByKey as (sortByKey: keyof EditableExercisePreviewFromBackend,
                                                                   sortDirection: boolean | null
                          ) => void
                          func('lastUpdatedAt',
                               rotateSorting<EditableExercisePreviewFromBackend>(
                                 this.props.sortDirection,
                                 this.props.sortByKey,
                                 'lastUpdatedAt'
                               )
                          )
                          return
                        }

                        const func = this.props.setSortByKey as (sortByKey: keyof EditableExercisePreviewFromBackend,
                                                                 sortDirection: boolean | null, groupId: number
                        ) => void
                        func('lastUpdatedAt',
                             rotateSorting<EditableExercisePreviewFromBackend>(
                               this.props.sortDirection,
                               this.props.sortByKey,
                               'lastUpdatedAt',
                             ),
                             this.props.groupId
                        )
                      }}
                >
                  {
                    getI18n(this.props.langId, 'Last updated at')
                  }
                </span>
              </Table.HeaderCell>
              <Table.HeaderCell sorted={
                isCurrentSortingColumnPaginated<EditableExercisePreviewFromBackend>(this.props.sortByKey,
                  'createdAt',
                  this.props.sortDirection
                )
              }
              >
                <span className="clickable"
                      onClick={() => {
                        if (this.props.groupId === null) {
                          const func = this.props.setSortByKey as (sortByKey: keyof EditableExercisePreviewFromBackend,
                            sortDirection: boolean | null
                          ) => void
                          func('createdAt',
                            rotateSorting<EditableExercisePreviewFromBackend>(
                              this.props.sortDirection,
                              this.props.sortByKey,
                              'createdAt'
                            )
                          )
                          return
                        }

                        const func = this.props.setSortByKey as (sortByKey: keyof EditableExercisePreviewFromBackend,
                          sortDirection: boolean | null, groupId: number
                        ) => void
                        func('createdAt',
                          rotateSorting<EditableExercisePreviewFromBackend>(
                            this.props.sortDirection,
                            this.props.sortByKey,
                            'createdAt',
                          ),
                          this.props.groupId
                        )
                      }}
                >
                  {
                    getI18n(this.props.langId, 'Created at')
                  }
                </span>
              </Table.HeaderCell>
              <Table.HeaderCell
                sorted={
                  isCurrentSortingColumnPaginated<EditableExercisePreviewFromBackend>(this.props.sortByKey, 'solutions',
                                                                                      this.props.sortDirection
                  )
                }
              >
                <div className="clickable"
                     onClick={() => {
                       if (this.props.groupId === null) {
                         const func = this.props.setSortByKey as (sortByKey: keyof EditableExercisePreviewFromBackend,
                                                                  sortDirection: boolean | null
                         ) => void
                         func('solutions',
                              rotateSorting<EditableExercisePreviewFromBackend>(
                                this.props.sortDirection,
                                this.props.sortByKey,
                                'solutions'
                              )
                         )
                         return
                       }

                       const func = this.props.setSortByKey as (sortByKey: keyof EditableExercisePreviewFromBackend,
                                                                sortDirection: boolean | null, groupId: number
                       ) => void
                       func('solutions',
                            rotateSorting<EditableExercisePreviewFromBackend>(
                              this.props.sortDirection,
                              this.props.sortByKey,
                              'solutions',
                            ),
                            this.props.groupId
                       )
                     }}
                >
                  <Icon name="flag" className="mar-right-half"/>
                  <span>
                    {
                      getI18n(this.props.langId, 'Solutions')
                    }
                  </span>
                </div>

              </Table.HeaderCell>

              <Table.HeaderCell
                sorted={
                  isCurrentSortingColumnPaginated<EditableExercisePreviewFromBackend>(this.props.sortByKey,
                                                                                      'isPermanentlyLocked',
                                                                                      this.props.sortDirection
                  )
                }
              >
                <div className="clickable"
                     onClick={() => {
                       if (this.props.groupId === null) {
                         const func = this.props.setSortByKey as (sortByKey: keyof EditableExercisePreviewFromBackend,
                                                                  sortDirection: boolean | null
                         ) => void
                         func('isPermanentlyLocked',
                              rotateSorting<EditableExercisePreviewFromBackend>(
                                this.props.sortDirection,
                                this.props.sortByKey,
                                'isPermanentlyLocked'
                              )
                         )
                         return
                       }

                       const func = this.props.setSortByKey as (sortByKey: keyof EditableExercisePreviewFromBackend,
                                                                sortDirection: boolean | null, groupId: number
                       ) => void
                       func('isPermanentlyLocked',
                            rotateSorting<EditableExercisePreviewFromBackend>(
                              this.props.sortDirection,
                              this.props.sortByKey,
                              'isPermanentlyLocked',
                            ),
                            this.props.groupId
                       )
                     }}
                >
                  <HelpPopup icon="lock"
                             defaultText={getI18n(this.props.langId, 'Locked permanently')}/>
                </div>

              </Table.HeaderCell>

              <Table.HeaderCell sorted={
                isCurrentSortingColumnPaginated<EditableExercisePreviewFromBackend>(this.props.sortByKey, 'releases',
                                                                                    this.props.sortDirection
                )
              }
              >
                <span className="clickable"
                      onClick={() => {
                        if (this.props.groupId === null) {
                          const func = this.props.setSortByKey as (sortByKey: keyof EditableExercisePreviewFromBackend,
                                                                   sortDirection: boolean | null
                          ) => void
                          func('releases',
                               rotateSorting<EditableExercisePreviewFromBackend>(
                                 this.props.sortDirection,
                                 this.props.sortByKey,
                                 'releases'
                               )
                          )
                          return
                        }

                        const func = this.props.setSortByKey as (sortByKey: keyof EditableExercisePreviewFromBackend,
                                                                 sortDirection: boolean | null, groupId: number
                        ) => void
                        func('releases',
                             rotateSorting<EditableExercisePreviewFromBackend>(
                               this.props.sortDirection,
                               this.props.sortByKey,
                               'releases',
                             ),
                             this.props.groupId
                        )
                      }}
                >
                  {
                    getI18n(this.props.langId, 'Releases')
                  }
                </span>
              </Table.HeaderCell>
              <Table.HeaderCell>
                <span>
                  {getI18n(this.props.langId, "Change")}
                </span>
              </Table.HeaderCell>
              <Table.HeaderCell>
                <span>
                {
                  getI18n(this.props.langId, 'Delete')
                }
              </span>
              </Table.HeaderCell>
              <Table.HeaderCell>
                <span className="mar-right-half">
                  {
                    getI18n(this.props.langId, 'Copy')
                  }
                </span>
                <HelpPopup defaultText={getI18n(this.props.langId,
                                                'This copies the exercise on the server and opens the edit exercise site.'
                )}/>
              </Table.HeaderCell>

              {
                //own exercises have all the same owner...
                this.props.groupId === null &&
                <Table.HeaderCell>
                  <span>
                  {
                    getI18n(this.props.langId, 'Owner')
                  }
                  </span>
                </Table.HeaderCell>
              }
              {
                this.props.groupId !== null &&
                <Table.HeaderCell
                  sorted={
                    isCurrentSortingColumnPaginated<EditableExercisePreviewFromBackend>(this.props.sortByKey,
                                                                                        'ownerId',
                                                                                        this.props.sortDirection
                    )
                  }
                >
                  <div className="clickable"
                       onClick={() => {
                         if (this.props.groupId === null) {
                           const func = this.props.setSortByKey as (sortByKey: keyof EditableExercisePreviewFromBackend,
                                                                    sortDirection: boolean | null
                           ) => void
                           func('ownerId',
                                rotateSorting<EditableExercisePreviewFromBackend>(
                                  this.props.sortDirection,
                                  this.props.sortByKey,
                                  'ownerId'
                                )
                           )
                           return
                         }

                         const func = this.props.setSortByKey as (sortByKey: keyof EditableExercisePreviewFromBackend,
                                                                  sortDirection: boolean | null, groupId: number
                         ) => void
                         func('ownerId',
                              rotateSorting<EditableExercisePreviewFromBackend>(
                                this.props.sortDirection,
                                this.props.sortByKey,
                                'ownerId',
                              ),
                              this.props.groupId
                         )
                       }}
                  >
                    <span>
                    {
                      getI18n(this.props.langId, 'Owner')
                    }
                  </span>
                  </div>

                </Table.HeaderCell>
              }

            </Table.Row>
          </Table.Header>

          <Table.Body>
            {
              list.map((exercisePreview, index) => {


                // tslint:disable-next-line
                let changeNode = null

                if (exercisePreview.isPermanentlyLocked) {

                  changeNode = (
                    <div>
                      <Link
                        to={getExerciseEditorEditToLink(exercisePreview.id, this.props.comesFromOwnExercisesSite)}>
                        <Icon name="eye"/>
                      </Link>
                    </div>
                  )

                } else if ((this.props.currentUserGroupRole === null || this.props.currentUserGroupRole.canChangeExercises)) {

                  changeNode = (

                    <div>
                      <Link
                        to={getExerciseEditorEditToLink(exercisePreview.id, this.props.comesFromOwnExercisesSite)}>
                        <Icon name="write"/>
                      </Link>
                    </div>
                  )

                }

                // tslint:disable-next-line
                let deleteNode = null

                if (exercisePreview.isPermanentlyLocked) {

                  deleteNode = (
                    <HelpPopup
                      defaultText={getI18n(this.props.langId,
                                           'Exercise is lock'
                      )}/>
                  )

                } else if (exercisePreview.releases > 0 && (this.props.currentUserGroupRole === null || this.props.currentUserGroupRole.canDeleteExercises)) {

                  deleteNode = (
                    <HelpPopup
                      defaultText={getI18n(this.props.langId,
                                           'The exercise can only be deleted if it has no releases.'
                      )}/>
                  )

                } else if (exercisePreview.releases === 0 && (this.props.currentUserGroupRole === null || this.props.currentUserGroupRole.canDeleteExercises)) {

                  deleteNode = (
                    <div className="clickable"
                         onClick={async () => {

                           if (exercisePreview.releases > 0) {
                             //only allowed if it has no exercises
                             //show never happen because we check above
                             return
                           }

                           const shouldRemove = await askDialog(
                             getI18n(this.props.langId, 'Delete exercise'),
                             `${getI18n(this.props.langId,
                                        'Are you sure you want to delete the exercise?'
                             )} (${exercisePreview.displayName})`,
                             this.props.langId
                           )

                           if (shouldRemove) {
                             this.props.onDeleteExercise(exercisePreview)
                           }

                         }}
                    >
                      <Icon name="trash"/>
                    </div>
                  )

                }

                return (
                  <Table.Row key={exercisePreview.id}>

                    <Table.Cell>
                      <span>{exercisePreview.displayName}</span>
                    </Table.Cell>

                    <Table.Cell>
                      {
                        exercisePreview.shortDescription &&
                        <HelpPopup defaultText={exercisePreview.shortDescription}/>
                      }
                    </Table.Cell>

                    <Table.Cell>
                      {
                        (exercisePreview.tags && exercisePreview.tags.length > 0) &&
                        <HelpPopup
                          wide="very"
                          defaultText={
                            <div className="exercise-tag-wrapper">
                              {
                                exercisePreview.tags.map((tag) => {
                                  return (
                                    <Tag key={tag.id} tag={tag}/>
                                  )
                                })
                              }
                            </div>
                          }
                        />
                      }
                    </Table.Cell>

                    <Table.Cell>
                      <Checkbox disabled checked={exercisePreview.isOnlyVisibleToMe}/>
                    </Table.Cell>

                    <Table.Cell>
                      <span>
                        {
                          displayDateAndTime(exercisePreview.lastUpdatedAt, -1)
                        }
                      </span>
                    </Table.Cell>
                    <Table.Cell>
                      <span>
                        {
                          displayDateAndTime(exercisePreview.createdAt, -1)
                        }
                      </span>
                    </Table.Cell>

                    <Table.Cell>
                      <span>{exercisePreview.solutions}</span>

                    </Table.Cell>

                    <Table.Cell>
                      <div>
                        <Checkbox checked={exercisePreview.isPermanentlyLocked} disabled/>
                      </div>
                    </Table.Cell>

                    <Table.Cell>
                      {
                        (this.props.currentUserGroupRole === null
                          || this.props.currentUserGroupRole.canManageExerciseReleases
                          || this.props.currentUserGroupRole.canAssessExercises) &&
                        <div>
                          <Link to={constants.getExerciseReleaseLinkPath(exercisePreview.id,
                                                                         this.props.comesFromOwnExercisesSite
                          )}>
                            <Icon name="privacy"/>
                            <span>{exercisePreview.releases}</span>
                          </Link>
                        </div>
                      }

                    </Table.Cell>

                    <Table.Cell>
                      {
                        changeNode
                      }
                    </Table.Cell>

                    <Table.Cell>
                      {
                        deleteNode
                      }
                    </Table.Cell>

                    <Table.Cell>
                      {
                        (this.props.currentUserGroupRole === null || this.props.currentUserGroupRole.canCreateExercises) &&
                        <div className="clickable" onClick={async () => {

                          this.props.setIsLoading(true)

                          // tslint:disable-next-line
                          let newExerciseId = -1

                          try {
                            newExerciseId = await copyExercise(exercisePreview.id)
                          } catch (err) {

                            if (!isProduction) {
                              Logger.warn(`could not create exercise copy: ${err.message}`, err)
                            }

                            Logger.warn('could not create exercise copy')
                            errorDialog(getI18n(this.props.langId, "Error"),
                                        getI18n(this.props.langId, "Could not copy exercise"), this.props.langId
                            )

                          } finally {
                            this.props.setIsLoading(false)
                          }


                          if (newExerciseId === -1) {
                            return
                          }

                          //then redirect...
                          const url = getExerciseEditorEditToLink(newExerciseId, this.props.comesFromOwnExercisesSite)
                          history.push(url)

                        }}>
                          <Icon name="copy"/>
                        </div>
                      }
                    </Table.Cell>

                    <Table.Cell>
                      {
                        exercisePreview.ownerId !== null && exercisePreview.owner === null &&
                        <HelpPopup defaultText={`${getI18n(this.props.langId,
                                                           "Owner was not loaded, id:"
                        )} ${exercisePreview.ownerId}`}/>
                      }

                      {
                        exercisePreview.ownerId !== null && exercisePreview.owner !== null &&
                        <HelpPopup icon="user"
                                   defaultText={`${exercisePreview.owner.lastName}, ${exercisePreview.owner.firstName} (${getI18n(
                                     this.props.langId, "Id")}: ${exercisePreview.owner.id})`}/>
                      }
                    </Table.Cell>

                  </Table.Row>
                )
              })
            }
          </Table.Body>
          <PaginationTableFooter
            pagination={this.props.pagination}
            setPaginationPageSize={(pageSize) => {

              if (this.props.groupId === null) {
                const func = this.props.setPaginationPageSize as (pageSize: number) => void
                func(pageSize)
                return
              }

              const func = this.props.setPaginationPageSize as (pageSize: number, groupId: number) => void
              func(pageSize, this.props.groupId)
            }}
            setActivePaginationPage={(activePage) => {

              if (this.props.groupId === null) {
                const func = this.props.setActivePaginationPage as (activePage: number) => void
                func(activePage)
                return
              }

              const func = this.props.setActivePaginationPage as (activePage: number, groupId: number) => void
              func(activePage, this.props.groupId)
            }}
          />
        </Table>

      </div>
    )
  }
}

export default connect(mapStateToProps, mapDispatchToProps)(TableView)
