/**
 * Created by janis dähne.
 */

import * as React from "react";
import {SyntheticEvent} from "react";
import {connect} from "react-redux";
import {bindActionCreators, Dispatch} from "redux";
import {returntypeof} from 'react-redux-typescript';
import {RootState} from "../../../../../../state/reducers";
import {Checkbox, Form, Icon, List, Radio} from "semantic-ui-react";
import MaterialInput from '../../../../../material/materialInput'
import MaterialSelectWrapper from '../../../../../material/materialSelectWrapper'
import {ReleaseStartType} from "../../../../../../types/ReleaseStartType";
import {CheckboxOnChangeEventData} from "../../../../../../types/reactEvents";
import {
  setAutomaticEndAt,
  setAutomaticStartAt,
  setAvailableWorkingTimeInMinutes,
  setHasLimitedWorkingTime, setHideExerciseLeaveActions,
  setHideInOverviews, setHidePrintOptions,
  setHideSiteHeaderBar,
  setIsReleased,
  setIsVisibleToAllAfterRelease, setMaxManualPoints,
  setNote,
  setPLangId,
  setReleaseDurationType,
  setReleaseStartType,
  setRunAlsoNormalTests,
  setShouldAutomaticAssessSubmissions,
  setShouldClearClipboard, setShowAdditionalLogButton,
} from "../../../../../../state/actions/ownOrGroupExerciseActions/releasesSiteActions/dialogs/changeExerciseReleaseDialog/setExerciseReleaseActions";
import {ReleaseDurationType} from "../../../../../../types/ReleaseDurationType";
import {selectAllTextEventHandler} from "../../../../../../helpers/commonEventHandlers";
import {parseId} from "../../../../../../helpers/convertersAndTransformers";
import {Moment} from "moment";
import MaterialDatetimeInput from '../../../../../material/materialDatetimeInput'
import MaterialTextArea from '../../../../../material/materialTextArea'
import {HelpPopup} from "../../../../../helpers/helpPopup";
import {
  validationRules
} from "../../../../../../state/reducers/ownOrGroupExerciseReducers/exerciseReleasesSite/dialogs/changeExerciseReleaseDialog/setExerciseReleaseReducer";
import {
  validationMessageKeys
} from "../../../../../../state/reducers/ownOrGroupExerciseReducers/exerciseReleasesSite/dialogs/changeExerciseReleaseDialog/setExerciseReleaseReducerValidation";
import {MyPopup} from "../../../../../helpers/myPopup";
import {getI18n} from "../../../../../../../i18n/i18nRoot";
import {PLangFullBase} from '../../../../../../types/pLangs'
import {FrontendSettingsManager} from '../../../../../../helpers/frontendSettingsManager'


//const css = require('./styles.styl');

export interface MyProps {
  //readonly test: string
}

const mapStateToProps = (rootState: RootState /*, props: MyProps*/) => {
  return {
    //test0: rootState...
    //test: props.test
    exercisePreview: rootState.changeExerciseReleaseDialogState.exercisePreview,
    exerciseRelease: rootState.changeExerciseReleaseDialogState.exerciseRelease,
    initialIsReleased: rootState.changeExerciseReleaseDialogState.initialIsReleased,
    exercises: rootState.releaseSiteState.exercise, //should be loaded at this point
    pLangs: rootState.pLangsState.pLangs,
    langId: rootState.i18nState.langId
  }
}

const mapDispatchToProps = (dispatch: Dispatch) => bindActionCreators({
                                                                        //imported reducer funcs here
                                                                        setIsReleased,
                                                                        setPLangId,
                                                                        setAutomaticStartAt,
                                                                        setAutomaticEndAt,
                                                                        setHasLimitedWorkingTime,
                                                                        setAvailableWorkingTimeInMinutes,
                                                                        setMaxManualPoints,
                                                                        setNote,
                                                                        setIsVisibleToAllAfterRelease,
                                                                        setShouldAutomaticAssessSubmissions,
                                                                        setRunAlsoNormalTests,
                                                                        setReleaseDurationType,
                                                                        setReleaseStartType,
                                                                        setHideInOverviews,
                                                                        setShouldClearClipboard,
                                                                        setHideSiteHeaderBar,
                                                                        setHidePrintOptions,
                                                                        setHideExerciseLeaveActions,
                                                                        setShowAdditionalLogButton,
                                                                      }, dispatch)


const stateProps = returntypeof(mapStateToProps);
const dispatchProps = returntypeof(mapDispatchToProps);
type Props = typeof stateProps & typeof dispatchProps;

class changeReleseView extends React.Component<Props, any> {

  componentWillMount(): void {

    const possiblePLangs = this.getPossiblePLangs()

    if ((this.props.exerciseRelease.pLangId === null || this.props.exerciseRelease.pLangId <= 0) && possiblePLangs.length > 0) {
      //select the first plang

      const lastReleasedPLangId = FrontendSettingsManager.getFrontendSettings().lastReleasedPLangId

      if (lastReleasedPLangId > 0 && possiblePLangs.some(p => p.id === lastReleasedPLangId)) {

        this.props.setPLangId(lastReleasedPLangId)

      } else {

        if (possiblePLangs.length > 0) {
          this.props.setPLangId(possiblePLangs[0].id)
        } else {
          //set to null... backend should throw
          this.props.setPLangId(null)
        }
      }

    }
  }

  getPossiblePLangs(): ReadonlyArray<PLangFullBase> {
    return this.props.exercisePreview
           ? this.props.pLangs.filter(
        p => this.props.exercisePreview.supportedPLangIds.indexOf(p.id) !== -1)
           : []
  }

  render(): JSX.Element {


    const possiblePLangs = this.getPossiblePLangs()

    return (
      <div>

        <div className="ui form divided-checkboxes">

          {
            this.props.exercises.isOnlyVisibleToMe &&
            <div style={{marginBottom: '1em'}}>
              <p>
                <Icon name="exclamation triangle"/>
                {
                  getI18n(this.props.langId,
                          "The exercise is only visible to the owner and thus all releases for this exercise will only work for the owner"
                  )
                }
              </p>
            </div>
          }

          <Form.Group widths='equal'>
            <Form.Field>
              <label>
                {
                  getI18n(this.props.langId, 'Start/End of the release')
                }
              </label>

              <div className="flexed">
                <Radio name="startReleaseMode" label={getI18n(this.props.langId, 'Manual')} className="mar-right"
                       checked={this.props.exerciseRelease.releaseStartType === ReleaseStartType.Manual}
                       value={ReleaseStartType.Manual}
                       onChange={(e, value: CheckboxOnChangeEventData) => {
                         this.props.setReleaseStartType(value.value)
                       }}
                />
                <Radio name="startReleaseMode" label={getI18n(this.props.langId, 'Automatic')}
                       checked={this.props.exerciseRelease.releaseStartType === ReleaseStartType.Automatic}
                       value={ReleaseStartType.Automatic}
                       onChange={(e, value: CheckboxOnChangeEventData) => {
                         this.props.setReleaseStartType(value.value)
                       }}

                />

              </div>
            </Form.Field>
          </Form.Group>

          {
            //one of the following manual start or automatic start
          }

          {
            this.props.exerciseRelease.releaseStartType === ReleaseStartType.Manual &&
            <div>
              <h4 className="ui dividing header">
                {
                  getI18n(this.props.langId, 'Manual')
                }
              </h4>

              <Form.Group widths='equal'>
                <Form.Field>

                  {
                    (this.props.exerciseRelease.hasAutomaticAssessmentStarted &&
                      !this.props.exerciseRelease.hasAutomaticAssessmentFinished) &&
                    <div>
                      {
                        getI18n(this.props.langId,
                                'Automatic assessment is still running... please reload the page to get new values'
                        )
                      }
                    </div>
                  }
                  {
                    (!this.props.exerciseRelease.hasAutomaticAssessmentStarted ||
                      this.props.exerciseRelease.hasAutomaticAssessmentFinished) &&
                    <div>

                      <Checkbox className="mar-right-half"
                                label={getI18n(this.props.langId, 'Is released')}
                                checked={this.props.exerciseRelease.isReleased}
                                onChange={(e, data: CheckboxOnChangeEventData) => {
                                  this.props.setIsReleased(data.checked)
                                }}
                      />
                      <HelpPopup
                        wide
                        defaultText={
                          <div>
                            {
                              getI18n(this.props.langId,
                                      'If you want to start the release then tick the checkbox and save. To end a release uncheck the checkbox and save. If you re-start a release the assessment results are not deleted, so there might be old results until a new assessment is started/finished'
                              )
                            }
                          </div>
                        }
                      />
                    </div>
                  }


                  {
                    //unrelease
                    //or
                    //unrelease (not the first unrelease)
                    (!this.props.exerciseRelease.isReleased && this.props.initialIsReleased === true
                      && this.props.exerciseRelease.releasedAt !== null
                      && !this.props.exerciseRelease.hasAutomaticAssessmentStarted
                      && !this.props.exerciseRelease.hasAutomaticAssessmentFinished
                      && this.props.exerciseRelease.shouldAutomaticAssessSubmissions
                      ||
                      !this.props.exerciseRelease.isReleased && this.props.initialIsReleased === true
                      && this.props.exerciseRelease.releasedAt !== null
                      && this.props.exerciseRelease.hasAutomaticAssessmentStarted
                      && this.props.exerciseRelease.hasAutomaticAssessmentFinished
                      && this.props.exerciseRelease.shouldAutomaticAssessSubmissions
                    ) &&
                    <div>
                      {
                        getI18n(this.props.langId, 'If you end the release now then the automatic tests will run')
                      }
                    </div>
                  }

                  {
                    (!this.props.exerciseRelease.isReleased && this.props.initialIsReleased === true
                      && this.props.exerciseRelease.releasedAt !== null
                      && !this.props.exerciseRelease.hasAutomaticAssessmentStarted
                      && !this.props.exerciseRelease.hasAutomaticAssessmentFinished
                      && !this.props.exerciseRelease.shouldAutomaticAssessSubmissions
                      ||
                      !this.props.exerciseRelease.isReleased && this.props.initialIsReleased === true
                      && this.props.exerciseRelease.releasedAt !== null
                      && this.props.exerciseRelease.hasAutomaticAssessmentStarted
                      && this.props.exerciseRelease.hasAutomaticAssessmentFinished
                      && !this.props.exerciseRelease.shouldAutomaticAssessSubmissions
                    ) &&
                    <div>
                      {
                        getI18n(this.props.langId,
                                "If you end the release now then the automatic tests will NOT run (because you didn't tick the option. If you want to run the tests tick the option first and then end the release."
                        )
                      }
                    </div>
                  }

                </Form.Field>
              </Form.Group>
            </div>
          }
          {
            //not supported (not tested + not used much)
            this.props.exerciseRelease.releaseStartType === ReleaseStartType.Automatic &&
            <div>
              <h4 className="ui dividing header">
                {
                  getI18n(this.props.langId, 'Automatic')
                }
                <HelpPopup className="mar-left-half" defaultText={getI18n(this.props.langId,
                                                                          "The automatic start/end can take up to 1 min (1 tick) until it's executed. If the automatic start is before now then the start is executed within the next tick."
                )}
                           wide
                />
                <HelpPopup icon="exclamation triangle" className="mar-left-half" defaultText={getI18n(this.props.langId,
                                                                                                      "Note that only the first automatic un-release will run the automatic assessment (if the option is checked), for all following un-releases you will need to use the manual (un-)release mode in order to run the automatic assessment"
                )}
                           wide
                />
              </h4>

              <Form.Group widths='equal'>
                <Form.Field>
                  <label>
                    {
                      getI18n(this.props.langId, 'Automatic start at')
                    }
                  </label>
                  <div className="flexed">
                    {
                      /*
                       <MaterialInput icon="calendar" className="mar-right-half"/>
                    <MaterialInput icon="time"/>
                       */
                    }
                    <MaterialDatetimeInput
                      locale="de"
                      value={this.props.exerciseRelease.automaticStartAt}
                      onChange={(e: Moment | string) => {
                        if (typeof e === 'string') {
                          //if the value is invalid...
                          this.props.setAutomaticStartAt(null)

                        } else {
                          this.props.setAutomaticStartAt(e)
                        }
                      }}
                    />

                  </div>
                </Form.Field>

                <Form.Field>
                  <label>
                    {
                      getI18n(this.props.langId, 'Automatic end at')
                    }
                  </label>
                  <div
                    className={this.props.exerciseRelease.releaseDurationType === ReleaseDurationType.Limited
                               ? null
                               : 'div-disabled'}>
                    <div className="flexed">
                      {
                        /*
                        <MaterialInput icon="calendar" className="mar-right-half"/>
                        <MaterialInput icon="time"/>
                        TODO make datetime material like...
                        */
                      }

                      <MaterialDatetimeInput
                        locale="de"
                        value={this.props.exerciseRelease.automaticEndAt}
                        onChange={(e: Moment | string) => {
                          if (typeof e === 'string') {
                            //if the value is invalid...
                            this.props.setAutomaticEndAt(null)

                          } else {
                            this.props.setAutomaticEndAt(e)
                          }
                        }}
                      />

                    </div>
                  </div>
                </Form.Field>
              </Form.Group>

              <Form.Field>
                <label>
                  {
                    getI18n(this.props.langId, 'Duration of the automatic release')
                  }
                </label>
                <div className="flexed">

                  <MyPopup
                    trigger={
                      <Radio name="releaseDurationMode" label={getI18n(this.props.langId, 'Infinitely')}
                             className="mar-right"
                             checked={this.props.exerciseRelease.releaseDurationType === ReleaseDurationType.Infinitely}
                             value={ReleaseDurationType.Infinitely}
                             onChange={(e, data: CheckboxOnChangeEventData) => {
                               this.props.setReleaseDurationType(data.value)
                             }}
                      />
                    }
                    content={getI18n(this.props.langId,
                                     'To end the release anyway select the manual mode and end the release manually.'
                    )}
                    wide
                  />
                  {
                    //then enable automatic end
                  }
                  <Radio name="releaseDurationMode" label={getI18n(this.props.langId, 'Limited')}
                         checked={this.props.exerciseRelease.releaseDurationType === ReleaseDurationType.Limited}
                         value={ReleaseDurationType.Limited}
                         onChange={(e, data: CheckboxOnChangeEventData) => {
                           this.props.setReleaseDurationType(data.value)
                         }}
                  />
                </div>
              </Form.Field>
            </div>
          }

          <h4 className="ui dividing header">
            {
              getI18n(this.props.langId, 'Options')
            }
          </h4>

          {
            this.props.exercises.isOnlyVisibleToMe &&
            <p>
              {
                getI18n(this.props.langId,
                        "Note that you will be the only user who can enter the release code because the exercise has 'is only visible to owner' set"
                )
              }
            </p>
          }

          <Form.Group widths='equal'>
            <Form.Field>
              <div className="form-field-label-with-icon">
                <label className="mar-right-half">
                  {
                    getI18n(this.props.langId, 'Programming language')
                  }
                </label>
                <HelpPopup
                  defaultText={getI18n(this.props.langId, "This cannot be changed after the release is created")}/>
              </div>

              <MaterialSelectWrapper
                value={this.props.exerciseRelease.pLangId === null
                       ? 0
                       : this.props.exerciseRelease.pLangId}
              >
                <select
                  disabled={this.props.exerciseRelease.id > 0}
                  value={this.props.exerciseRelease.pLangId === null
                         ? 0
                         : this.props.exerciseRelease.pLangId}
                  onChange={(e: SyntheticEvent<HTMLSelectElement>) => {

                    if (this.props.exerciseRelease.id > 0) return //old release, thus we cannot change the plang

                    const pLangId = parseId(e.currentTarget.value)
                    this.props.setPLangId(pLangId)

                  }}
                >
                  {
                    possiblePLangs.map((pLang, index) => {
                      return (
                        <option key={pLang.id} value={pLang.id}>{pLang.displayName}</option>
                      )
                    })
                  }
                </select>
              </MaterialSelectWrapper>


            </Form.Field>

            <Form.Field>
              <div className="form-field-label-with-icon">
                <label className="mar-right-half">
                  {
                    getI18n(this.props.langId, 'Working time in min')
                  }
                </label>
                <HelpPopup defaultText={getI18n(this.props.langId,
                                                "The user has the amount of working time to solve the exercise (started when the user opens the exercise for the first time). When re-releasing a release the working time won't reset!"
                )}/>
              </div>

              <div className="flexed">
                <Checkbox label={getI18n(this.props.langId, 'Limit')}
                          className="v-centered mar-right"
                          checked={this.props.exerciseRelease.hasLimitedWorkingTime}
                          onChange={(e, data: CheckboxOnChangeEventData) => {
                            this.props.setHasLimitedWorkingTime(data.checked)
                          }}
                />
                <MaterialInput
                  type="number"
                  style={{flex: '1'}}
                  value={this.props.exerciseRelease.availableWorkingTimeInMinutes}

                  validations={validationRules.availableWorkingTimeInMinutes}
                  validationMessageKey={validationMessageKeys.availableWorkingTimeInMinutes}

                  onFocus={selectAllTextEventHandler}
                  onChange={(e) => {
                    const time = parseInt(e.currentTarget.value, 10)

                    if (isNaN(time)) {
                      //TODO some error
                      this.props.setAvailableWorkingTimeInMinutes(null)
                    } else {

                      if (time < 1) {
                        //TODO some error
                      } else {
                        this.props.setAvailableWorkingTimeInMinutes(time)
                      }
                    }
                  }}
                  disabled={!this.props.exerciseRelease.hasLimitedWorkingTime}
                />
              </div>

            </Form.Field>

            <Form.Field>
              <div className="form-field-label-with-icon">
                <label className="mar-right-half">
                  {
                    getI18n(this.props.langId, 'Max manual points')
                  }
                </label>
                <HelpPopup defaultText={getI18n(this.props.langId,
                                                "The max manual points as note for the students. This is only a hint for the max manual points, you can assign more points than this value. Let this field empty to not set max manual points"
                )}/>
              </div>

              <MaterialInput
                type="number"
                value={this.props.exerciseRelease.maxManualPoints}

                validations={validationRules.maxManualPoints}
                validationMessageKey={validationMessageKeys.maxManualPoints}

                onFocus={selectAllTextEventHandler}
                onChange={(e) => {
                  const time = parseInt(e.currentTarget.value, 10)

                  if (isNaN(time)) {
                    this.props.setMaxManualPoints(null)
                    return
                  }

                  this.props.setMaxManualPoints(time)
                }}
              />


            </Form.Field>

          </Form.Group>

          <Form.Field>

            <Checkbox className="mar-right-half"
                      label={getI18n(this.props.langId, 'Is visible for all')}
                      checked={this.props.exerciseRelease.isVisibleToAllAfterRelease}
                      disabled={this.props.exercises.isOnlyVisibleToMe}
                      onChange={(e, data: CheckboxOnChangeEventData) => {
                        this.props.setIsVisibleToAllAfterRelease(data.checked)
                      }}
            />
            <HelpPopup
              wide
              defaultText={getI18n(this.props.langId,
                                   'If the release is visible for all then every user can see the release without entering the code. If the exercise \'is only visible to owner\' then the release cannot be visible for all and only the owner can enter the release code'
              )}/>
          </Form.Field>


          <Form.Group widths='equal'>

            <Form.Field>

              <Checkbox className="mar-right-half"
                        label={getI18n(this.props.langId, 'Run automatic assessment')}
                        checked={this.props.exerciseRelease.shouldAutomaticAssessSubmissions}
                        onChange={(e, data: CheckboxOnChangeEventData) => {
                          this.props.setShouldAutomaticAssessSubmissions(data.checked)
                        }}
              />
              <HelpPopup
                wide
                defaultText={getI18n(this.props.langId,
                                     'After the release ended the automatic tests will be run. Note that it can take up to 1 tick (1 minute) until the assessment will start.'
                )}/>
            </Form.Field>
            <Form.Field>

              <Checkbox className="mar-right-half"
                        label={getI18n(this.props.langId, 'Run normal tests too')}
                        checked={this.props.exerciseRelease.runAlsoNormalTests}
                        onChange={(e, data: CheckboxOnChangeEventData) => {
                          this.props.setRunAlsoNormalTests(data.checked)
                        }}
              />

              <HelpPopup
                wide='very'
                defaultText={getI18n(this.props.langId,
                                     'Normally only the submit tests are run. With this optin enabled the normal tests are also run. This will overwrite the test results from the users.'
                )}/>
            </Form.Field>
          </Form.Group>


          <Form.Field>
            <div>
              <label style={{
                color: 'rgba(0,0,0, 0.87)',
                fontWeight: 'bold',
                fontSize: '0.92857143em'
              }}
                     className="mar-right-half"
              >
                {
                  getI18n(this.props.langId, 'Note')
                }
              </label>
            </div>
            <MaterialTextArea
              rows={3}
              value={this.props.exerciseRelease.note}
              onChange={(e, value) => {
                this.props.setNote(value)
              }}
            />
          </Form.Field>

          <h4 className="ui dividing header">
            {
              getI18n(this.props.langId, 'Exam options')
            }
          </h4>


          <Form.Group widths='equal'>

            <Form.Field>

              <Checkbox className="mar-right-half"
                        label={getI18n(this.props.langId, 'Hide release in overviews')}
                        checked={this.props.exerciseRelease.hideInOverviews}
                        onChange={(e, data: CheckboxOnChangeEventData) => {
                          this.props.setHideInOverviews(data.checked)
                        }}
              />
              <HelpPopup
                wide
                defaultText={getI18n(this.props.langId,
                                     'Hides the release from the users in all exercise overviews and statistic views'
                )}/>
            </Form.Field>

            <Form.Field>

              <Checkbox className="mar-right-half"
                        label={getI18n(this.props.langId, 'Clear clipboard on focus (EXPERIMENTAL, known issues)')}
                        checked={this.props.exerciseRelease.shouldClearClipboard}
                        onChange={(e, data: CheckboxOnChangeEventData) => {
                          this.props.setShouldClearClipboard(data.checked)
                        }}
              />
              <HelpPopup
                wide
                defaultText={getI18n(this.props.langId,
                                     "Shows an overlay to resume when the window/tab lost focus. This will clear the clipboard when accepted else the user can't proceed."
                )}/>
            </Form.Field>

          </Form.Group>


          <Form.Group widths='equal'>

            <Form.Field>

              <Checkbox className="mar-right-half"
                        label={getI18n(this.props.langId, 'Hide site header bar')}
                        checked={this.props.exerciseRelease.hideSiteHeaderBar}
                        onChange={(e, data: CheckboxOnChangeEventData) => {
                          this.props.setHideSiteHeaderBar(data.checked)
                        }}
              />
              <HelpPopup
                wide
                defaultText={getI18n(this.props.langId,
                                     'Hides the header bar (checked) or displays it (unchecked). The site header bar is the topmost bar where the site menu, user menu is located.'
                )}/>
            </Form.Field>

            <Form.Field>
              <Checkbox className="mar-right-half"
                        label={getI18n(this.props.langId, 'Hide print options')}
                        checked={this.props.exerciseRelease.hidePrintOptions}
                        onChange={(e, data: CheckboxOnChangeEventData) => {
                          this.props.setHidePrintOptions(data.checked)
                        }}
              />
              <HelpPopup
                wide
                defaultText={getI18n(this.props.langId,
                                     'This will hide all print options (e.g. print task description, code editor)'
                )}/>
            </Form.Field>


          </Form.Group>

          <Form.Group widths='equal'>

            <Form.Field>
              <Checkbox className="mar-right-half"
                        label={getI18n(this.props.langId, 'Hide leave exercise actions')}
                        checked={this.props.exerciseRelease.hideExerciseLeaveActions}
                        onChange={(e, data: CheckboxOnChangeEventData) => {
                          this.props.setHideExerciseLeaveActions(data.checked)
                        }}
              />
              <HelpPopup
                wide
                defaultText={getI18n(this.props.langId,
                                     'This will hide all actions and buttons the user could use to leave the exercise (and e.g. select another exercise). The buttons in the site header bar are not affected (use the appropriate option for that).'
                )}/>
            </Form.Field>

          </Form.Group>

          <Form.Group widths='equal'>

            <Form.Field>
              <Checkbox className="mar-right-half"
                        label={getI18n(this.props.langId, 'Show additional log-button')}
                        checked={this.props.exerciseRelease.showAdditionalLogButton}
                        onChange={(e, data: CheckboxOnChangeEventData) => {
                          this.props.setShowAdditionalLogButton(data.checked)
                        }}
              />
              <HelpPopup
                wide
                defaultText={getI18n(this.props.langId,
                                     'This will show an additional log button because some options might hide the user header bar thus one cannot show the log and e.g. download the log'
                )}/>
            </Form.Field>

          </Form.Group>


          {/*<h4 className="ui dividing header">*/}
          {/*{*/}
          {/*getI18n(this.props.langId, 'Hints')*/}
          {/*}*/}
          {/*</h4>*/}

          {/*<Form.Field>*/}
          {/*<List divided relaxed>*/}
          {/*<List.Item>*/}
          {/*{*/}
          {/*getI18n(this.props.langId,*/}
          {/*'[message]'*/}
          {/*)*/}
          {/*}*/}
          {/*</List.Item>*/}
          {/*</List>*/}
          {/*</Form.Field>*/}

        </div>
      </div>
    )
  }
}

export default connect(mapStateToProps, mapDispatchToProps)(changeReleseView)
