/**
 * Created by janis dähne.
 */

import * as React from "react";
import {connect} from "react-redux";
import {bindActionCreators, Dispatch} from "redux";
import {returntypeof} from 'react-redux-typescript';
import {RootState} from "../../../../../../state/reducers";
import {Form, Checkbox, Icon} from "semantic-ui-react";

import {errorDialog} from "../../../../../../helpers/dialogHelper";
import {getI18n} from "../../../../../../../i18n/i18nRoot";
import {ExerciseReleaseFromBackend} from '../../../../../../types/exerciseRelease'
import {getDoExerciseSiteToLink} from '../../../../../../constants'
import {copyToClipboard} from '../../../../../../helpers/clipboard'
import Logger from '../../../../../../helpers/logger'

//const css = require('./styles.styl');

export interface MyProps {
  //readonly test: string
}

const mapStateToProps = (rootState: RootState /*, props: MyProps*/) => {
  return {
    //test0: rootState...
    //test: props.test
    release: rootState.zoomedExerciseReleaseState.release,
    pLangs: rootState.pLangsState.pLangs,
    langId: rootState.i18nState.langId
  }
}

const mapDispatchToProps = (dispatch: Dispatch) => bindActionCreators({
  //imported reducer funcs here

}, dispatch)


const stateProps = returntypeof(mapStateToProps);
const dispatchProps = returntypeof(mapDispatchToProps);
type Props = typeof stateProps & typeof dispatchProps;

class ReleaseCodeBigView extends React.Component<Props, any> {


  getLink(ex: ExerciseReleaseFromBackend): string {

    if (document.location.hash === '') {
      Logger.warn('the routing mechanism has changed but this file was not updated (zoomedExerciseReleaseView.getLink), thus the link will be wrong')
    }

    const host = `${document.location.protocol}//${document.location.host}/#`
    return `${host}${getDoExerciseSiteToLink(ex.generatedCode)}`
  }


  render(): JSX.Element {


    // noinspection TsLint
    let pLangName = getI18n(this.props.langId,'All')

    if (this.props.release.pLangId) {
      const pLang = this.props.pLangs.find(p => p.id === this.props.release.pLangId)

      if (pLang) {
        pLangName = pLang.displayName
      } else {
        errorDialog(getI18n(this.props.langId,'Error'), getI18n(this.props.langId,'Could not find programming language for given id'),this.props.langId)
      }
    }

    const doEditorLink = this.getLink(this.props.release)

    return (
      <div className="ui form">

        <Form.Group widths='equal'>
          <Form.Field>
            <label>
              {
                getI18n(this.props.langId,'Programming language')
              }
            </label>
            <span>
            {
              pLangName
            }
          </span>
          </Form.Field>

          <Form.Field>
            <label>
              {
                getI18n(this.props.langId,'Is released')
              }
            </label>
            <Checkbox checked={this.props.release.isReleased} disabled/>
          </Form.Field>

          <Form.Field>
            <label>
              {
                getI18n(this.props.langId,'Has limited working time')
              }
            </label>
            <Checkbox checked={this.props.release.hasLimitedWorkingTime} disabled/>
          </Form.Field>

          {
            this.props.release.hasLimitedWorkingTime &&
            <Form.Field>
              <label>
                {
                  getI18n(this.props.langId,'Working time in min')
                }
              </label>
              <span>
                {this.props.release.availableWorkingTimeInMinutes.toString()}
              </span>
            </Form.Field>
          }
        </Form.Group>

        <Form.Group>

          <Form.Field>
            <label>
              <span>
                {
                  getI18n(this.props.langId,'Link')
                }
              </span>
              <Icon name="copy" className="clickable mar-left-half"
                    onClick={() => {
                      copyToClipboard(doEditorLink)
                    }}
              />
            </label>
            <span>
            {
              doEditorLink
            }
          </span>
          </Form.Field>

        </Form.Group>

        <Form.Field>
          <label style={{marginBottom: '2em'}}>
            {
              getI18n(this.props.langId,'Release code')
            }
          </label>
          <span style={{fontSize: '70px'}}>
            {
              this.props.release &&
              this.props.release.generatedCode
            }
          </span>
        </Form.Field>


      </div>
    )
  }
}

export default connect(mapStateToProps, mapDispatchToProps)(ReleaseCodeBigView)
