import * as React from "react";
import {connect} from "react-redux";
import {bindActionCreators, Dispatch} from "redux";
import {returntypeof} from 'react-redux-typescript';
import {RootState} from '../../../../../state/reducers'
import Spinner from '../../../../helpers/spinner'
import {getI18n} from '../../../../../../i18n/i18nRoot'
import {Button, Divider} from 'semantic-ui-react'
import CancelButton from '../../../../buttons/cancelButton'
import {setIsExportCsvDialogDisplayed} from '../../../../../state/actions/ownOrGroupExerciseActions/submissionsSite/submissionsSiteActions'
import ExportCsvDialogView from './exportCsvDialogView'
import {FrontendSettingsManager} from '../../../../../helpers/frontendSettingsManager'
import {
  AdditionalCsvColumn,
  CsvExportAssessmentOption,
  CsvExportAssessmentOptionsMap,
  CsvExportReplaceRuleObj
} from '../../../../../types/csvExportAssessment'
import {
  set_csvDelimiter,
  set_submissionExportCsvDialog_csvExportAssessmentOptions,
  set_submissionExportCsvDialog_isBusy
} from '../../../../../state/reducers/ownOrGroupExerciseReducers/submissionsSite/exportCsvDialog/exportCsvDialogReducer'
import {downloadAssessmentsAsCsv} from '../../../../../communicationLayer/submissionsLayer'
import Logger from '../../../../../helpers/logger'
import {errorDialog} from '../../../../../helpers/dialogHelper'
import {ServerResponse} from '../../../../../types/response'
import {ResponseCodes} from '../../../../../helpers/responseCodes'

declare var require: any
const fileSaver = require('file-saver');

export interface MyProps {
  //readonly test: string
}

const mapStateToProps = (rootState: RootState /*, props: MyProps*/) => {
  return {
    //test0: rootState...
    //test: props.test
    exercise: rootState.submissionsSiteState.exercise,
    release: rootState.submissionsSiteState.release,
    dialogState: rootState.submissionExportCsvDialogState,
    langId: rootState.i18nState.langId,
    csvExportAssessmentOptions: rootState.submissionExportCsvDialogState.csvExportAssessmentOptions,
    csvDelimiter: rootState.submissionExportCsvDialogState.csvDelimiter,
    csvQuoteAllFields: rootState.submissionExportCsvDialogState.csvQuoteAllFields,
    submissions: rootState.submissionsSiteState.submissions,

    sortByKey: rootState.submissionExportCsvDialogState.orderByProp,
    sortDirection: rootState.submissionExportCsvDialogState.isOrderByDesc,

  }
}

const mapDispatchToProps = (dispatch: Dispatch) => bindActionCreators({
                                                                        //imported reducer funcs here
                                                                        setIsExportCsvDialogDisplayed,
                                                                        set_submissionExportCsvDialog_csvExportAssessmentOptions,
                                                                        set_csvDelimiter,
                                                                        set_submissionExportCsvDialog_isBusy,
                                                                      }, dispatch)


const stateProps = returntypeof(mapStateToProps);
const dispatchProps = returntypeof(mapDispatchToProps);
type Props = typeof stateProps & typeof dispatchProps;


class ExportCsvDialog extends React.Component<Props, any> {


  componentWillMount(): void {

    const frontendExportSettings = FrontendSettingsManager.getFrontendSettings().csvExportAssessmentsSettings

    if (frontendExportSettings) {
      const copy: CsvExportAssessmentOptionsMap = JSON.parse(JSON.stringify(this.props.csvExportAssessmentOptions))

      // noinspection TsLint
      let needToUpdate = false
      for (const key in frontendExportSettings) {

        if (!frontendExportSettings.hasOwnProperty(key)) continue

        //if (this.props.csvExportAssessmentOptions.hasOwnProperty(key)) //we cannot longer check for this because additional cols are also stored here...

        //we found old settings...
        needToUpdate = true

        const option = frontendExportSettings[key]


        if (option.kind === 'additional') {

          copy[key] = {
            ...option
          }
          continue
        } else if (option.kind === 'normal') {

        } else {
          //unknown or invalid... ignore
          continue
        }

        // tslint:disable-next-line
        let replaceObj: CsvExportReplaceRuleObj = option.replaceRulesObj

        //make sure we have default (e.g when we have an old frontend settings version)
        if (!replaceObj) {
          replaceObj = {
            replaceRules: [],
            elseReplace: null
          }
        }

        copy[key] = {
          ...option,
          replaceRulesObj: {
            ...option.replaceRulesObj,
          }
        } as CsvExportAssessmentOption


      }

      if (needToUpdate) {
        //this will also correct wrong indices
        this.props.set_submissionExportCsvDialog_csvExportAssessmentOptions(copy)
      }
    }


    const csvDelimiter = FrontendSettingsManager.getFrontendSettings().csvExportDelimiter
    this.props.set_csvDelimiter(csvDelimiter)

  }

  render(): JSX.Element {

    const selectedExercise = this.props.exercise
    const selectedRelease = this.props.release


    return (
      <div>
        {
          this.props.dialogState.isBusy &&
          <Spinner/>
        }

        <div className="view-padding">
          <div>
            <div className="view-caption">
              <h2>
                {
                  getI18n(this.props.langId, 'Export') + ' '
                }
                {
                  getI18n(this.props.langId, 'submissions of')
                }
                {
                  selectedExercise &&
                  ` ${selectedExercise.displayName}`
                }
                {
                  selectedRelease &&
                  ` (${selectedRelease.generatedCode}) `
                }
                {
                  getI18n(this.props.langId, "as csv")
                }
              </h2>
            </div>
          </div>

          <div className="view-content">
            <ExportCsvDialogView/>

            <Divider/>

            <div className="flexed">

              <div className={this.props.submissions.length === 0
                              ? 'div-disabled'
                              : ''}>
                <Button className="mar-right" onClick={async () => {

                  //some frontend checks e.g. that regex is valid

                  for (const key in this.props.csvExportAssessmentOptions) {

                    if (this.props.csvExportAssessmentOptions.hasOwnProperty(key) === false) continue

                    const obj = this.props.csvExportAssessmentOptions[key] as CsvExportAssessmentOption | AdditionalCsvColumn

                    if (obj.kind === 'additional') continue

                    for (const replaceRule of obj.replaceRulesObj.replaceRules) {

                      // tslint:disable-next-line
                      let isValid = true
                      try {

                        if (replaceRule.regex === '') {
                          //we don't allow this
                          throw new Error()
                        }

                        new RegExp(replaceRule.regex)
                      } catch (err) {
                        isValid = false
                      }

                      if (!isValid) {
                        errorDialog(getI18n(this.props.langId, "Error in column") + " '" + key + "'",
                                    getI18n(this.props.langId,
                                            "The given string is not a valid regex:"
                                    ) + " '" + replaceRule.regex + "'", this.props.langId
                        )
                        return
                      }
                    }
                  }


                  //split the object so that the backend can deserialize this better
                  // tslint:disable-next-line
                  const additionalColumns: AdditionalCsvColumn[] = []
                  const copy: CsvExportAssessmentOptionsMap = {}

                  // tslint:disable-next-line
                  for (const key in this.props.csvExportAssessmentOptions) {
                    const obj = this.props.csvExportAssessmentOptions[key]


                    if (obj.kind === 'additional') {

                      additionalColumns.push({
                                               ...obj,
                                               kind: undefined
                                             })

                    } else {
                      //normal
                      copy[key] = {
                        ...obj,
                        kind: undefined,
                      }
                    }

                  }

                  //we do not directly download the file because then we have not that much control...
                  //this way we can do stuff before (check if regex is valid) and after the download (hide dialog, show error)
                  //the downside is that for large files we need to store it in a temp variable vs saving to disk
                  try {

                    this.props.set_submissionExportCsvDialog_isBusy(true)
                    const response = await downloadAssessmentsAsCsv(this.props.release.generatedCode,
                                                                    this.props.sortByKey, this.props.sortDirection,
                                                                    this.props.csvDelimiter,
                                                                    this.props.csvQuoteAllFields,
                                                                    copy,
                                                                    additionalColumns
                    )

                    if (response.data.hasOwnProperty('code') && response.data.hasOwnProperty('message')) {
                      const data = response.data as ServerResponse

                      if (data.code !== ResponseCodes.ok) {
                        throw new Error(data.message)
                      }
                    }

                    const csvText = response.data as string

                    // tslint:disable-next-line
                    const blob = new Blob([csvText], {
                      type: 'text/comma-separated-values'
                    })

                    // tslint:disable-next-line
                    let fileName = `submissionResults_${this.props.release.generatedCode}.csv`
                    const contentDisposition = response.headers['content-disposition']

                    if (contentDisposition && contentDisposition.indexOf('attachment') !== -1) {
                      const filenameRegex = /filename[^;=\n]*=((['"]).*?\2|[^;\n]*)/;
                      const matches = filenameRegex.exec(contentDisposition);
                      if (matches !== null && matches[1]) {
                        fileName = matches[1].replace(/['"]/g, '');
                      }
                    }

                    this.props.set_submissionExportCsvDialog_isBusy(false)
                    fileSaver.saveAs(blob, fileName)

                    Logger.debug('[exportCsvDialog]', 'saved csv')

                  } catch (err) {

                    this.props.set_submissionExportCsvDialog_isBusy(false)

                    errorDialog(getI18n(this.props.langId, "Error"), getI18n(this.props.langId, "Error downloading the csv data. Make sure the delimiter is not empty."), this.props.langId)

                    Logger.error('error getting/processing submissions csv file, error: ' + JSON.stringify(err))

                    //the request call downloadAssessmentsAsCsv already displays error
                    // errorDialog(getI18n(this.props.langId, "Error"),
                    //             getI18n(this.props.langId, "Error while downloading the csv file"), this.props.langId
                    // )

                    return
                  }


                  FrontendSettingsManager.setCsvExportAssessmentsSettings(this.props.csvExportAssessmentOptions,
                                                                          this.props.csvDelimiter,
                                                                          this.props.csvQuoteAllFields
                  )
                  this.props.setIsExportCsvDialogDisplayed(false)

                }}
                        content={getI18n(this.props.langId, 'Export assessments as csv')}
                        icon='share'
                        labelPosition='left'
                        color="green"
                >
                </Button>
              </div>

              <CancelButton onClick={() => {

                FrontendSettingsManager.setCsvExportAssessmentsSettings(this.props.csvExportAssessmentOptions,
                                                                        this.props.csvDelimiter,
                                                                        this.props.csvQuoteAllFields
                )
                this.props.setIsExportCsvDialogDisplayed(false)
              }}/>
            </div>

          </div>
        </div>
      </div>
    )
  }
}

export default connect(mapStateToProps, mapDispatchToProps)(ExportCsvDialog)
