import * as React from "react";
import {connect} from "react-redux";
import {bindActionCreators, Dispatch} from "redux";
import {returntypeof} from 'react-redux-typescript';
import {RootState} from '../../../../../state/reducers'
import {Button, Checkbox, Form, Icon, Table} from 'semantic-ui-react'
import {isCurrentSortingColumnPaginated, rotateSorting} from '../../../../../helpers/tablesHelper'
import {AssessmentFullBase} from '../../../../../types/submissions'
import {getI18n} from '../../../../../../i18n/i18nRoot'
import {HelpPopup} from '../../../../helpers/helpPopup'
import {
  set_csvDelimiter, set_csvQuoteAllFields,
  set_submissionExportCsvDialog_csvExportAssessmentOptions,
  set_submissionExportCsvDialog_isOrderByDesc,
  set_submissionExportCsvDialog_orderByProp
} from '../../../../../state/reducers/ownOrGroupExerciseReducers/submissionsSite/exportCsvDialog/exportCsvDialogReducer'
import {CheckboxOnChangeEventData, SelectOnChangeEventData} from '../../../../../types/reactEvents'
import {
  AdditionalCsvColumn,
  CsvExportAssessmentOption,
  CsvExportAssessmentOptionsMap,
  CsvExportReplaceRule
} from '../../../../../types/csvExportAssessment'
import {MyPopup} from '../../../../helpers/myPopup'
import MaterialInput from '../../../../material/materialInput'
import {errorDialog} from '../../../../../helpers/dialogHelper'
import MaterialSelectWrapper from '../../../../material/materialSelectWrapper'
import {getGuid} from '../../../../../helpers/guid'

export interface MyProps {
  //readonly test: string
}

const mapStateToProps = (rootState: RootState /*, props: MyProps*/) => {
  return {
    //test0: rootState...
    //test: props.test
    sortByKey: rootState.submissionExportCsvDialogState.orderByProp,
    sortDirection: rootState.submissionExportCsvDialogState.isOrderByDesc,

    release: rootState.submissionsSiteState.release,
    submissions: rootState.submissionsSiteState.submissions,

    csvExportAssessmentOptionsMap: rootState.submissionExportCsvDialogState.csvExportAssessmentOptions,

    csvDelimiter: rootState.submissionExportCsvDialogState.csvDelimiter,

    csvQuoteAllFields: rootState.submissionExportCsvDialogState.csvQuoteAllFields,

    allPLangs: rootState.pLangsState.pLangs,

    langId: rootState.i18nState.langId
  }
}

const mapDispatchToProps = (dispatch: Dispatch) => bindActionCreators({
                                                                        //imported reducer funcs here
                                                                        set_submissionExportCsvDialog_csvExportAssessmentOptions,
                                                                        set_submissionExportCsvDialog_orderByProp,
                                                                        set_submissionExportCsvDialog_isOrderByDesc,
                                                                        set_csvDelimiter,
                                                                        set_csvQuoteAllFields,

                                                                      }, dispatch)


const stateProps = returntypeof(mapStateToProps);
const dispatchProps = returntypeof(mapDispatchToProps);
type Props = typeof stateProps & typeof dispatchProps;


const maxPreviewItemsCount = 5
const unknownPLangString = 'null' //same as backend
const assessment_csv_export_column_format = 'assessment_csv_export_column_format'

const defaultReplaceRuleRegex = '^$'
// tslint:disable-next-line
const possibleRegexReplaceRuleFlags = ['none', 'i', 'm', 'im']


// tslint:disable-next-line
let currentKey = -1

function getNextAdditionalColumnKey(): string {
  return `$_${(currentKey--)}-${getGuid()}`

}

class ExportCsvDialogView extends React.Component<Props, any> {


  /**
   * moves the given column by key
   * @param colKey (or string for additional col)
   * @param _newIndex can handle all indices (will clamp to valid ones) one can pass null to disable and hide the col for export, NOT NEEDED OR IMPLEMENTED YET)
   */
  moveColumn(colKey: keyof AssessmentFullBase, _newIndex: number | null): void {

    if (_newIndex === null) {

      const option = this.props.csvExportAssessmentOptionsMap[colKey]

      // tslint:disable-next-line
      let copy: CsvExportAssessmentOptionsMap

      if (option.kind === 'additional') {

        const key = colKey as string
        copy = {
          ...this.props.csvExportAssessmentOptionsMap,
          [key]: {
            ...option,
          }
        }
        //TODO if _newIndex is null we would remove this additional col?

      } else {

        copy = {
          ...this.props.csvExportAssessmentOptionsMap,
          [colKey]: {
            displayIndex: null,
            willExport: false,
            replaceRulesObj: {
              ...option.replaceRulesObj
            }
          } as CsvExportAssessmentOption
        }

      }

      this.props.set_submissionExportCsvDialog_csvExportAssessmentOptions(copy)
      return
    }

    //move col

    const keys = Object.keys(this.props.csvExportAssessmentOptionsMap)

    const keysToExport = keys
      .filter(key => {
        const tuple = this.props.csvExportAssessmentOptionsMap[key] as CsvExportAssessmentOption | AdditionalCsvColumn
        return tuple.displayIndex !== null
      })
      .sort((a, b) => {
        const tuple1 = this.props.csvExportAssessmentOptionsMap[a] as CsvExportAssessmentOption | AdditionalCsvColumn
        const tuple2 = this.props.csvExportAssessmentOptionsMap[b] as CsvExportAssessmentOption | AdditionalCsvColumn
        return tuple1.displayIndex - tuple2.displayIndex
      })

    // noinspection TsLint
    let newIndex = Math.min(keysToExport.length - 1, _newIndex)
    newIndex = Math.max(0, newIndex)


    const otherKeyAtPos = keysToExport.find(p => {
      const tuple = this.props.csvExportAssessmentOptionsMap[p] as CsvExportAssessmentOption | AdditionalCsvColumn
      return tuple.displayIndex === newIndex
    })

    const currentColTuple = this.props.csvExportAssessmentOptionsMap[colKey] as CsvExportAssessmentOption | AdditionalCsvColumn
    const oldTuple = this.props.csvExportAssessmentOptionsMap[otherKeyAtPos] as CsvExportAssessmentOption | AdditionalCsvColumn


    // tslint:disable-next-line
    let currentColTuple_copy: CsvExportAssessmentOption | AdditionalCsvColumn
    // tslint:disable-next-line
    let oldTuple_copy: CsvExportAssessmentOption | AdditionalCsvColumn

    if (oldTuple.kind === 'additional') {

      oldTuple_copy = {
        ...oldTuple,
        displayIndex: currentColTuple.displayIndex,
      } as AdditionalCsvColumn

    } else {

      oldTuple_copy = {
        ...oldTuple,
        displayIndex: currentColTuple.displayIndex,
        replaceRulesObj: {
          ...oldTuple.replaceRulesObj
        }
      } as CsvExportAssessmentOption
    }

    if (oldTuple.kind === 'additional') {

      currentColTuple_copy = {
        ...currentColTuple,
        displayIndex: oldTuple.displayIndex,
      } as AdditionalCsvColumn

    } else {

      currentColTuple_copy = {
        ...currentColTuple,
        displayIndex: oldTuple.displayIndex,
        replaceRulesObj: {
          ...oldTuple.replaceRulesObj
        }
      } as CsvExportAssessmentOption

    }

    const copy: CsvExportAssessmentOptionsMap = {
      ...this.props.csvExportAssessmentOptionsMap,

      [otherKeyAtPos]: oldTuple_copy,

      [colKey]: currentColTuple_copy
    }


    this.props.set_submissionExportCsvDialog_csvExportAssessmentOptions(copy)

  }

  /**
   * used to modify options (e.g. edit replace rules)
   * @param colKey
   * @param option
   */
  replaceCsvExportOption(colKey: string, option: CsvExportAssessmentOption | AdditionalCsvColumn): void {

    const copy = {
      ...this.props.csvExportAssessmentOptionsMap,
      [colKey]: option
    }

    this.props.set_submissionExportCsvDialog_csvExportAssessmentOptions(copy)
  }

  addAdditionalColumn(colName: string, value: string): void {

    //we need to renew the indices

    const keys = Object.keys(this.props.csvExportAssessmentOptionsMap)
    const sortedOptions = keys
      .filter(key => {
        const tuple = this.props.csvExportAssessmentOptionsMap[key] as CsvExportAssessmentOption | AdditionalCsvColumn
        return tuple.displayIndex !== null
      })
      .map((value, index) => {
        const tuple1 = this.props.csvExportAssessmentOptionsMap[value] as CsvExportAssessmentOption | AdditionalCsvColumn
        return {
          key: value,
          tuple: tuple1
        }
      })

    const maxIndex = Math.max(...sortedOptions.map(p => p.tuple.displayIndex))

    const newCol: AdditionalCsvColumn = {
      name: colName,
      displayIndex: maxIndex + 1, //will be set
      value,
      kind: 'additional',
      artificialKey: getNextAdditionalColumnKey()
    }


    const copy = {
      ...this.props.csvExportAssessmentOptionsMap,
      [newCol.artificialKey]: newCol
    }

    //this will also sort
    this.props.set_submissionExportCsvDialog_csvExportAssessmentOptions(copy)

  }

  /**
   *
   * @param artificialKey
   */
  removeAdditionalColumn(artificialKey: string): void {

    const keys = Object.keys(this.props.csvExportAssessmentOptionsMap).filter(p => p !== artificialKey)

    const copy: CsvExportAssessmentOptionsMap = {}

    for(const key of keys) {
        copy[key] = this.props.csvExportAssessmentOptionsMap[key]
    }

    //this will renew the indices
    this.props.set_submissionExportCsvDialog_csvExportAssessmentOptions(copy)

  }

  render(): JSX.Element {

    const list: ReadonlyArray<AssessmentFullBase> = this.props.submissions
      .filter((value, index) => index < maxPreviewItemsCount)
      .sort((a, b) => {

        if (this.props.sortDirection === null) return 0

        const val1 = a[this.props.sortByKey]
        const val2 = b[this.props.sortByKey]

        // noinspection TsLint
        let valCompare = 0

        // noinspection TsLint
        if (typeof val1 === 'number' && typeof val2 === 'number') {
          valCompare = val1 - val2
        } else {
          //assume string
          valCompare = val1.toString().localeCompare(val2.toString())
        }


        return this.props.sortDirection === false
               ? valCompare
               : -valCompare
      })

    const keys = Object.keys(this.props.csvExportAssessmentOptionsMap)

    const keysToExportSorted = keys
      .filter(key => {
        const tuple = this.props.csvExportAssessmentOptionsMap[key] as CsvExportAssessmentOption | AdditionalCsvColumn
        return tuple.displayIndex !== null
      })
      .sort((a, b) => {
        const tuple1 = this.props.csvExportAssessmentOptionsMap[a] as CsvExportAssessmentOption | AdditionalCsvColumn
        const tuple2 = this.props.csvExportAssessmentOptionsMap[b] as CsvExportAssessmentOption | AdditionalCsvColumn
        return tuple1.displayIndex - tuple2.displayIndex
      })

    // noinspection TsLint
    const elements: JSX.Element[] = keysToExportSorted.map<JSX.Element>((key, index) => {

      const _key = key as keyof AssessmentFullBase  //or string (when additional col)
      const tuple = this.props.csvExportAssessmentOptionsMap[_key] as CsvExportAssessmentOption | AdditionalCsvColumn

      if (tuple.kind === 'additional') {
        return (<Table.HeaderCell key={`${tuple.name}-${index}`}>
            <div className="flexed-h" style={{
              display: 'inline-block',
              minWidth: '92px'
            }}>
              <div className="flexed">
                <div className="clickable" onClick={() => {
                  this.moveColumn(_key, tuple.displayIndex - 1)
                }}>
                  <Icon name="chevron left"/>
                </div>

                <div className="clickable" onClick={() => {
                  this.moveColumn(_key, tuple.displayIndex + 1)
                }}>
                  <Icon name="chevron right"/>
                </div>
              </div>
              <span>
              {
                tuple.name
              }
            </span>
            </div>
          </Table.HeaderCell>
        )
      }

      // noinspection TsLint
      let isSortable = true

      if (_key === 'feedbackForStudent'
        || _key === 'noteForOtherTutors'
        || _key === 'lastAssessmentErrorMessage'
        || _key === 'lastEditingIpAddress'
      ) {
        isSortable = false
      }

      //replace rules or else rule
      const hasSomeReplaceStuff = tuple.replaceRulesObj.elseReplace !== null
        || tuple.replaceRulesObj.replaceRules.length > 0

      return (<Table.HeaderCell key={_key}
                                sorted={
                                  isCurrentSortingColumnPaginated<AssessmentFullBase>(this.props.sortByKey, _key,
                                                                                      this.props.sortDirection
                                  )
                                }
      >
        {
          //72px needed for checkbox + 3 icons to not break the line...
        }
        <div className="flexed-h" style={{
          display: 'inline-block',
          minWidth: '92px'
        }}>

          <div className="flexed">
            <MyPopup

              trigger={
                <Checkbox className="mar-right" checked={tuple.willExport}
                          onChange={(event, data: CheckboxOnChangeEventData) => {

                            const option = this.props.csvExportAssessmentOptionsMap[_key] as CsvExportAssessmentOption

                            const copy: CsvExportAssessmentOption = {
                              ...option,
                              willExport: data.checked,
                              replaceRulesObj: {
                                ...option.replaceRulesObj
                              }
                            }

                            this.replaceCsvExportOption(_key, copy)
                          }}/>
              }
              content={getI18n(this.props.langId, "If unchecked the column will not be exported")}
            />

            <div className="clickable" onClick={() => {
              this.moveColumn(_key, tuple.displayIndex - 1)
            }}>
              <Icon name="chevron left"/>
            </div>

            <div className="clickable" onClick={() => {
              this.moveColumn(_key, tuple.displayIndex + 1)
            }}>
              <Icon name="chevron right"/>
            </div>

            {
              //replace rules
            }
            <MyPopup
              header={getI18n(this.props.langId, "Replace rules")}
              trigger={
                <div className={hasSomeReplaceStuff
                                ? 'clickable-toggled'
                                : 'clickable'} onClick={() => {

                }}>
                  <Icon name="recycle"/>
                </div>
              }
              wide="very"
              content={
                <div style={{
                  maxHeight: '506px', //tested with 3 rules
                  overflow: 'auto'
                }}>

                  <p style={{marginTop: '0.5em'}}>
                    {getI18n(this.props.langId,
                             "Only first matched regex rule is applied, the whole cell value is replaced. If none matched (or no rules) the else rule is applied (if it has content). Also note that the replace rules are not applied in the preview."
                    )}
                  </p>
                  <p>
                    {getI18n(this.props.langId,
                             "To match empty content use ^$. It also a good idea to use the start position, end position operators and the ignore case flag. You can test your regex at"
                    ) + ' '}
                    <a href="https://regex101.com/" target="_blank">https://regex101.com/</a>
                  </p>

                  <Table selectable padded striped singleLine className="material-table">
                    <Table.Header>
                      <Table.Row>
                        <Table.HeaderCell/>
                        <Table.HeaderCell>
                          {getI18n(this.props.langId, "Regex")}
                        </Table.HeaderCell>
                        <Table.HeaderCell style={{minWidth: '75px'}}>
                          {getI18n(this.props.langId, "Flags")}
                        </Table.HeaderCell>
                        <Table.HeaderCell>
                          {getI18n(this.props.langId, "On match replace with")}
                        </Table.HeaderCell>
                      </Table.Row>
                    </Table.Header>
                    <Table.Body>
                      {
                        tuple.replaceRulesObj.replaceRules.map((replaceRule, index) => {
                          return (
                            <Table.Row key={index}>
                              <Table.Cell>
                                <div className="clickable" onClick={() => {

                                  const option = this.props.csvExportAssessmentOptionsMap[_key] as CsvExportAssessmentOption

                                  const copy: CsvExportAssessmentOption = {
                                    ...option,
                                    replaceRulesObj: {
                                      ...option.replaceRulesObj,
                                      replaceRules: option.replaceRulesObj.replaceRules.filter(
                                        (value, index1) => index1 !== index)
                                    }
                                  }
                                  this.replaceCsvExportOption(_key, copy)

                                }}>
                                  <Icon name="remove"/>
                                </div>
                              </Table.Cell>
                              <Table.Cell>
                                <MaterialInput placeholder={getI18n(this.props.langId, "e.g. ^test$ or just test")}
                                               value={replaceRule.regex}
                                               onChange={(event1, data) => {

                                                 const option = this.props.csvExportAssessmentOptionsMap[_key] as CsvExportAssessmentOption

                                                 const val = data.value
                                                 const copy: CsvExportAssessmentOption = {
                                                   ...option,
                                                   replaceRulesObj: {
                                                     ...option.replaceRulesObj,
                                                     replaceRules: option.replaceRulesObj.replaceRules.map(
                                                       (value, index1) =>
                                                         index1 !== index
                                                         ? value
                                                         : {
                                                             ...value,
                                                             regex: val
                                                           })
                                                   }
                                                 }
                                                 this.replaceCsvExportOption(_key, copy)
                                               }}
                                />
                              </Table.Cell>
                              <Table.Cell>
                                <MaterialSelectWrapper
                                  value={replaceRule.regexFlags}
                                >
                                  <select
                                    style={{width: '100%'}}
                                    value={replaceRule.regexFlags}
                                    onChange={(e: SelectOnChangeEventData) => {

                                      const option = this.props.csvExportAssessmentOptionsMap[_key] as CsvExportAssessmentOption

                                      const copy: CsvExportAssessmentOption = {
                                        ...option,
                                        replaceRulesObj: {
                                          ...option.replaceRulesObj,
                                          replaceRules: option.replaceRulesObj.replaceRules.map(
                                            (value, index1) =>
                                              index1 !== index
                                              ? value
                                              : {
                                                  ...value,
                                                  regexFlags: e.currentTarget.value
                                                })
                                        }
                                      }
                                      this.replaceCsvExportOption(_key, copy)
                                    }}
                                  >
                                    {
                                      possibleRegexReplaceRuleFlags.map((flag, index) => {
                                        //use _+flag because we have an empty flag...
                                        return (
                                          <option key={'_' + flag} value={flag}>
                                            {
                                              flag === ''
                                              ? getI18n(this.props.langId, "None")
                                              : flag
                                            }
                                          </option>
                                        )
                                      })
                                    }
                                  </select>
                                </MaterialSelectWrapper>
                              </Table.Cell>
                              <Table.Cell>
                                <MaterialInput
                                  value={replaceRule.replaceWith}
                                  onChange={(event1, data) => {
                                    const val = data.value

                                    const option = this.props.csvExportAssessmentOptionsMap[_key] as CsvExportAssessmentOption

                                    const copy: CsvExportAssessmentOption = {
                                      ...option,
                                      replaceRulesObj: {
                                        ...option.replaceRulesObj,
                                        replaceRules: option.replaceRulesObj.replaceRules.map(
                                          (value, index1) =>
                                            index1 !== index
                                            ? value
                                            : {
                                                ...value,
                                                replaceWith: val
                                              })
                                      }
                                    }
                                    this.replaceCsvExportOption(_key, copy)
                                  }}
                                />
                              </Table.Cell>
                            </Table.Row>
                          )
                        })
                      }
                      <Table.Row>
                        <Table.Cell colSpan={1000}>
                          <Button circular icon='add' onClick={() => {

                            const newEntry: CsvExportReplaceRule = {
                              replaceWith: '',
                              regex: defaultReplaceRuleRegex,
                              regexFlags: ''
                            }

                            const option = this.props.csvExportAssessmentOptionsMap[_key] as CsvExportAssessmentOption

                            const copy: CsvExportAssessmentOption = {
                              ...option,
                              replaceRulesObj: {
                                ...option.replaceRulesObj,
                                replaceRules: [...option.replaceRulesObj.replaceRules,
                                  newEntry],
                              }
                            }
                            this.replaceCsvExportOption(_key, copy)

                          }}/>
                        </Table.Cell>
                      </Table.Row>
                    </Table.Body>
                    <Table.Footer>
                      <Table.Row>
                        <Table.Cell>
                        </Table.Cell>
                        <Table.Cell colSpan={2}>
                          <MyPopup
                            trigger={
                              <span className={tuple.replaceRulesObj.elseReplace === null
                                               ? 'grey-colored'
                                               : ''}>
                                {getI18n(this.props.langId, "If none matched")}
                              </span>
                            }
                            content={getI18n(this.props.langId, "Only applied if the replace text is not empty")}
                          />
                        </Table.Cell>
                        <Table.Cell>
                          <MaterialInput
                            value={tuple.replaceRulesObj.elseReplace}
                            onChange={(event1, data) => {
                              const val = data.value === ''
                                          ? null
                                          : data.value

                              const option = this.props.csvExportAssessmentOptionsMap[_key] as CsvExportAssessmentOption

                              const copy: CsvExportAssessmentOption = {
                                ...option,
                                replaceRulesObj: {
                                  ...option.replaceRulesObj,
                                  elseReplace: val
                                }
                              }
                              this.replaceCsvExportOption(_key, copy)
                            }}
                          />
                        </Table.Cell>
                      </Table.Row>
                    </Table.Footer>
                  </Table>
                </div>
              }
              on="click"
            />
          </div>


          <span className={`${tuple.willExport === false
                              ? 'color-disabled'
                              : ''}${tuple.willExport && isSortable
                                     ? ' clickable'
                                     : ''}`}
                onClick={() => {

                  if (!isSortable) return

                  this.props.set_submissionExportCsvDialog_orderByProp(_key)
                  this.props.set_submissionExportCsvDialog_isOrderByDesc(
                    rotateSorting<AssessmentFullBase>(this.props.sortDirection,
                                                      this.props.sortByKey,
                                                      _key
                    )
                  )
                }}
          >
                {
                  _key === 'lastName' &&
                  getI18n(this.props.langId, 'Last name')
                }
            {
              _key === 'firstName' &&
              getI18n(this.props.langId, 'First name')
            }
            {
              _key === 'pLangId' &&
              <HelpPopup icon="puzzle" defaultText={getI18n(this.props.langId, 'Programming language')}/>
            }
            {
              _key === 'shouldNotCount' &&
              <HelpPopup icon="flag outline"
                         defaultText={getI18n(this.props.langId, 'Should not count')}/>
            }
            {
              _key === 'participationCount' &&
              <HelpPopup icon="repeat" wide
                         defaultText={getI18n(this.props.langId,
                                              "The times the user solved this exercise (e.g. in different releases). If a user solved the exercise (release) in different programming languages then two rows are shown but it's the same participation. The participation is counted as one because the restrictions (e.g. lef time, should not count) are stored in the participation."
                         )}/>
            }

            {
              _key === 'normalTestPoints' &&
              getI18n(this.props.langId, 'Normal points')
            }
            {
              _key === 'maxNormalTestPoints' &&
              getI18n(this.props.langId, 'Max normal points')
            }
            {
              _key === 'submitTestPoints' &&
              getI18n(this.props.langId, 'Submit points')
            }
            {
              _key === 'maxSubmitTestPoints' &&
              getI18n(this.props.langId, 'Max submit points')
            }
            {
              _key === 'manualPoints' &&
              getI18n(this.props.langId, 'Manual')
            }
            {
              _key === 'maxManualPoints' &&
              getI18n(this.props.langId, 'Max manual points')
            }
            {
              _key === 'noteForOtherTutors' &&
              getI18n(this.props.langId, 'Note for other tutors')
            }
            {
              _key === 'feedbackForStudent' &&
              getI18n(this.props.langId, 'Feedback for student')
            }
            {
              _key === 'token' &&
              getI18n(this.props.langId, 'Token')
            }
            {
              _key === 'lastAssessmentErrorMessage' &&
              getI18n(this.props.langId, 'Error during testing')
            }
            {
              _key === 'lastEditingIpAddress' &&
              getI18n(this.props.langId, 'Last editing ip address')
            }
              </span>
        </div>
      </Table.HeaderCell>)
    })


    const csvAdditionalColumnsKeys = keysToExportSorted.filter(p => {
      const tuple = this.props.csvExportAssessmentOptionsMap[p] as CsvExportAssessmentOption | AdditionalCsvColumn
      return tuple.kind === 'additional'
    })

    return (
      <div>


        <Form as="div">
          <Form.Group>

          <Form.Field>
            <label>
              {
                getI18n(this.props.langId, "Csv delimiter")
              }
            </label>
            <MaterialInput style={{width: '100px'}}
                           value={this.props.csvDelimiter}
                           onChange={(e) => {
                             this.props.set_csvDelimiter(e.currentTarget.value.trim())
                           }}
            />
          </Form.Field>

            <Form.Field>
              <label>
                {
                  getI18n(this.props.langId, "Quote all fields")
                }
              </label>
              <Checkbox className="mar-right" checked={this.props.csvQuoteAllFields}
                        onChange={(event, data: CheckboxOnChangeEventData) => {
                          this.props.set_csvQuoteAllFields(data.checked)
                        }}/>
            </Form.Field>

          </Form.Group>

          <Form.Field style={{width: '350px'}}>
            <label>
              {getI18n(this.props.langId, "Additional columns")}
            </label>

            <Table selectable sortable padded striped singleLine className="material-table">
              <Table.Header>
                <Table.Row>
                  <Table.HeaderCell>
                  </Table.HeaderCell>
                  <Table.HeaderCell>
                    {getI18n(this.props.langId, "Name")}
                  </Table.HeaderCell>
                  <Table.HeaderCell>
                    {getI18n(this.props.langId, "Value")}
                  </Table.HeaderCell>
                </Table.Row>
              </Table.Header>
              <Table.Body>
                {
                  csvAdditionalColumnsKeys.map((artificialKey, index) => {

                    const tuple = this.props.csvExportAssessmentOptionsMap[artificialKey] as AdditionalCsvColumn
                    return (
                      <Table.Row key={index}>
                        <Table.Cell>
                          <div className="clickable" onClick={() => {
                            this.removeAdditionalColumn(artificialKey)
                          }}>
                            <Icon name="remove"/>
                          </div>
                        </Table.Cell>
                        <Table.Cell>
                          <MaterialInput value={tuple.name}
                                         onChange={(event1, data) => {

                                           if (keys.some(p => p === data.value)) {
                                             //we use the names as key...
                                             errorDialog(getI18n(this.props.langId, "Error"), getI18n(this.props.langId,
                                                                                                      "There is already a column with this name:"
                                             ) + ' ' + data.value, this.props.langId)
                                             return
                                           }

                                           const copy = {
                                             ...tuple,
                                             name: data.value
                                           } as AdditionalCsvColumn

                                           this.replaceCsvExportOption(artificialKey, copy)

                                         }}
                          />
                        </Table.Cell>
                        <Table.Cell>
                          <MaterialInput value={tuple.value}
                                         onChange={(event1, data) => {

                                           const copy = {
                                             ...tuple,
                                             value: data.value
                                           } as AdditionalCsvColumn

                                           this.replaceCsvExportOption(artificialKey, copy)
                                         }}
                          />
                        </Table.Cell>
                      </Table.Row>
                    )
                  })
                }
              </Table.Body>
              <Table.Footer>
                <Table.Row>
                  <Table.Cell colSpan={1000}>
                    <Button circular icon='add' onClick={() => {
                      this.addAdditionalColumn('', '')
                    }}/>
                  </Table.Cell>
                </Table.Row>
              </Table.Footer>
            </Table>

          </Form.Field>

        </Form>


        <Table selectable sortable padded striped singleLine className="material-table">
          <Table.Header>
            <Table.Row>
              {
                elements
              }
            </Table.Row>
          </Table.Header>

          <Table.Body>
            {
              list.map((submission: AssessmentFullBase, index) => {

                const keys = Object.keys(this.props.csvExportAssessmentOptionsMap)

                const keysToExport = keys
                  .filter(key => {
                    const tuple = this.props.csvExportAssessmentOptionsMap[key] as CsvExportAssessmentOption | AdditionalCsvColumn
                    return tuple.displayIndex !== null
                  })
                  .sort((a, b) => {
                    const tuple1 = this.props.csvExportAssessmentOptionsMap[a] as CsvExportAssessmentOption | AdditionalCsvColumn
                    const tuple2 = this.props.csvExportAssessmentOptionsMap[b] as CsvExportAssessmentOption | AdditionalCsvColumn
                    return tuple1.displayIndex - tuple2.displayIndex
                  })

                //create rows
                // noinspection TsLint
                const elements: JSX.Element[] = keysToExport.map<JSX.Element>(key => {

                  const _key = key as keyof AssessmentFullBase //or string (when additional col)
                  const tuple = this.props.csvExportAssessmentOptionsMap[key] as CsvExportAssessmentOption | AdditionalCsvColumn


                  if (tuple.kind === 'additional') {

                    return (
                      <Table.Cell key={`${submission.userId}-${submission.pLangId}-${tuple.name}-${index}`}>
                        {
                          tuple.value
                        }
                      </Table.Cell>
                    )

                  }

                  // noinspection TsLint
                  let val = submission[key]

                  if (typeof val === 'boolean') {
                    val = val
                          ? 'true'
                          : 'false'
                  } else if (_key === 'pLangId') {
                    const plang = this.props.allPLangs.find(p => p.id === submission.pLangId)
                    val = plang
                          ? plang.displayName
                          : unknownPLangString
                  }

                  return <Table.Cell key={`${submission.userId}-${submission.pLangId}-${key}`}>
                    <div style={{
                      maxWidth: '200px',
                      display: 'block'
                    }} className={`text-ellipsis${tuple.willExport === false
                                                  ? ' div-disabled'
                                                  : ''}`}>
                      {
                        val
                      }
                    </div>
                  </Table.Cell>
                })


                return (
                  <Table.Row key={`${submission.userId}-${submission.pLangId}`}>
                    {
                      elements
                    }
                  </Table.Row>
                )
              })

            }

            {
              this.props.submissions.length > maxPreviewItemsCount &&
              <Table.Row>
                <Table.Cell colSpan={1000}>
                  <span>...</span>
                </Table.Cell>
              </Table.Row>
            }
          </Table.Body>
        </Table>
      </div>
    )
  }
}

export default connect(mapStateToProps, mapDispatchToProps)(ExportCsvDialogView)
