/**
 * Created by janis dähne.
 */

import * as React from "react";
import {connect} from "react-redux";
import {bindActionCreators, Dispatch} from "redux";
import {returntypeof} from 'react-redux-typescript';
import {RootState} from "../../../../state/reducers";
import {Icon} from "semantic-ui-react";
import CommonHeaderBarContentWrapper from '../../../commonHeaderBarContentWrapper'
import {SimpleVDivider} from "../../../helpers/simpleVDivider";
import {
  loadSubmissionsSite,
  rerunAllSubmissionsAsync,
  rerunSingleSubmissionAsync,
  setActivePaginationPage,
  setAutomaticRefreshViewIntervalCounter,
  setIsAutomaticRefreshViewEnabled,
  setIsExportCsvDialogDisplayed,
  setPaginationPageSize,
  setRunningAssessment
} from "../../../../state/actions/ownOrGroupExerciseActions/submissionsSite/submissionsSiteActions";
import {automaticRefreshSubmissionsListViewIntervalInS, getAllSubmissionDownloadLink} from "../../../../constants";
import {
  getSubmissions,
  uploadCsvAssessmentFile
} from "../../../../state/actions/ownOrGroupExerciseActions/submissionsSite/submissionsCrudActions";
import {askDialog, errorDialog, infoDialog} from "../../../../helpers/dialogHelper";


import {HelpPopup} from "../../../helpers/helpPopup";
import {getI18n} from "../../../../../i18n/i18nRoot";
import Logger from '../../../../helpers/logger'
import {ErrorHelper} from '../../../../helpers/errorHelper'
import {InputOnChangeEvent} from '../../../../types/reactEvents'
import {readFileAsAsset} from '../../../../helpers/fileReader'

//const css = require('./styles.styl');

export interface MyProps {
  //readonly test: string
}

const mapStateToProps = (rootState: RootState /*, props: MyProps*/) => {
  return {
    //test0: rootState...
    //test: props.test
    selectedReleaseId: rootState.submissionsSiteState.selectedReleaseId,
    release: rootState.submissionsSiteState.release,
    comesFromOwnExercisesSite: rootState.submissionsSiteState.comesFromOwnExercisesSite,
    submissions: rootState.submissionsSiteState.submissions,
    selectedExerciseId: rootState.submissionsSiteState.selectedExerciseId,
    isLoading: rootState.submissionsSiteState.isLoading
      || rootState.pLangsState.isLoading,

    exercisePreview: rootState.submissionsSiteState.exercise,

    isAutomaticRefreshViewEnabled: rootState.submissionsSiteState.isAutomaticRefreshViewEnabled,
    automaticRefreshViewIntervalCounter: rootState.submissionsSiteState.automaticRefreshViewIntervalCounter,

    isSomeDialogDisplayed: rootState.submissionsSiteState.isExportCsvDialogDisplayed,

    sortByKey: rootState.submissionsSiteState.pagination.orderByProp,
    sortDirection: rootState.submissionsSiteState.pagination.isOrderByDesc,
    pagination: rootState.submissionsSiteState.pagination,
    langId: rootState.i18nState.langId
  }
}

const mapDispatchToProps = (dispatch: Dispatch) => bindActionCreators({
                                                                        //imported reducer funcs here
                                                                        loadSubmissionsSite,
                                                                        rerunSingleSubmissionAsync,
                                                                        setRunningAssessment,
                                                                        getSubmissions,
                                                                        uploadCsvAssessmentFile,

                                                                        setIsAutomaticRefreshViewEnabled,
                                                                        setAutomaticRefreshViewIntervalCounter,
                                                                        setPaginationPageSize,
                                                                        setActivePaginationPage,
                                                                        rerunAllSubmissionsAsync,

                                                                        setIsExportCsvDialogDisplayed,
                                                                      }, dispatch)


const stateProps = returntypeof(mapStateToProps);
const dispatchProps = returntypeof(mapDispatchToProps);
type Props = typeof stateProps & typeof dispatchProps;


const importCsvAssessmentFileExerciseId = 'importCsvAssessmentFileExerciseId'
// tslint:disable-next-line
const maxCsvFileUploadSizeInByte = 1024 * 1024 // 1Mb?

class HeaderBar extends React.Component<Props, any> {


  automaticRefreshViewIntervalHandle: number | null = null

  componentWillUnmount(): void {
    this.clearAutoRefreshViewInterval()
  }

  clearAutoRefreshViewInterval(): void {
    if (this.automaticRefreshViewIntervalHandle) {
      //clear handle
      clearInterval(this.automaticRefreshViewIntervalHandle)
      this.automaticRefreshViewIntervalHandle = null
    }
  }

  createAutoRefreshViewInterval(): void {
    this.automaticRefreshViewIntervalHandle = window.setInterval(() => {

      this.props.setAutomaticRefreshViewIntervalCounter(this.props.automaticRefreshViewIntervalCounter - 1)

      if (this.props.automaticRefreshViewIntervalCounter <= 0) {
        this.props.setAutomaticRefreshViewIntervalCounter(automaticRefreshSubmissionsListViewIntervalInS)
        this.refreshView()
      }

    }, 1000)

  }

  refreshView(): void {
    this.props.loadSubmissionsSite(this.props.selectedExerciseId, this.props.comesFromOwnExercisesSite,
                                   this.props.selectedReleaseId, true
    )
  }

  async importCsvAssessmentFile(e: InputOnChangeEvent): Promise<void> {

    const input = document.getElementById(importCsvAssessmentFileExerciseId) as HTMLInputElement

    const file: File = input.files[0]

    const content = await readFileAsAsset(file)

    //if we changed the file and the file name does not change then we need to clear...
    input.value = ""

    if (file.size > maxCsvFileUploadSizeInByte) {
      Logger.error('file is too big')
    }

    try {
      await this.props.uploadCsvAssessmentFile(this.props.release.generatedCode, content)
    } catch (err) {
      Logger.error('error while uploading csv file')

      //not ok response will show right error
      // errorDialog(getI18n(globalState.getState().i18nState.langId, 'Error'),
      //             `${getI18n(globalState.getState().i18nState.langId, 'Error')}: ${err.message}`,
      //             globalState.getState().i18nState.langId
      // )

      return
    }

    //then refresh site...
    this.refreshView()

  }

  render(): JSX.Element {

    const selectedRelease = this.props.release

    if (!selectedRelease) {
      //TODO only error if really not found
      //on first render this might not be set because the list was not yet set
      return null
    }

    const canRerunAssessment =
      selectedRelease.hasAutomaticAssessmentStarted
      && selectedRelease.hasAutomaticAssessmentFinished

    const isExerciseLocked = this.props.exercisePreview !== null && this.props.exercisePreview.isPermanentlyLocked

    // noinspection RedundantConditionalExpressionJS
    const leftArea = (
      <div>
        <div className={this.props.isAutomaticRefreshViewEnabled
                        ? 'div-disabled'
                        : 'clickable'}
             onClick={() => {
               this.refreshView()
             }}
        >
          <Icon name="refresh"/>
          <span>
            {
              getI18n(this.props.langId, 'Refresh view')
            }
          </span>
        </div>

        <SimpleVDivider/>

        <div className="clickable" onClick={() => {
          this.props.setIsExportCsvDialogDisplayed(true)
        }}>
          <Icon name="share"/>
          <span>
            {
              getI18n(this.props.langId, 'Export assessments as csv')
            }
          </span>
        </div>

        <SimpleVDivider/>

        <div
        >
          <a target="_blank"
             download
             href={selectedRelease
                   ? getAllSubmissionDownloadLink(selectedRelease.generatedCode)
                   : null}>
            <Icon name="download" flipped="horizontally"/>
            <span className="mar-right-half">
              {
                getI18n(this.props.langId, 'Download all submissions')
              }
            </span>
          </a>
          <HelpPopup wide defaultText={getI18n(this.props.langId,
                                               "The .zip file contains a file called assessments.csv which can be filled and uploaded again (manual points, feedback, notes for other tutors are taken). Make sure you use a RFC 4180 compliant library for writing to the csv (that handled special cases e.g. new lines and escapes), comment lines start with #. Some users might have invalid characters in their names so the folder name contains then the user id instead."
          )}/>
          <HelpPopup className="mar-left-half" icon="exclamation triangle" defaultText={getI18n(this.props.langId,
                                                                                                'There is an option issue with this feature/function:'
          ) + '#19 Server'}/>
        </div>

        <SimpleVDivider/>

        {
          isExerciseLocked === false &&
          <div className={canRerunAssessment
                          ? 'clickable'
                          : ' div-disabled'}
               onClick={() => {
                 const input = document.getElementById(importCsvAssessmentFileExerciseId)
                 if (!input) {
                   Logger.error(`Could not get html element with specific id '${importCsvAssessmentFileExerciseId}'`)
                   errorDialog(getI18n(this.props.langId, 'Error'),
                               getI18n(this.props.langId, 'Could not get html element with specific id'),
                               this.props.langId
                   )
                   return
                 }

                 input.click()
               }}
          >
            <Icon name="upload"/>
            <span className="mar-right-half">
            {
              getI18n(this.props.langId, 'Upload assessments from csv')
            }
          </span>
            <HelpPopup wide defaultText={getI18n(this.props.langId,
                                                 "You can import the .csv file from the 'download all submissions' feature here. At least the columns UserId, PLangId, ManualPoints, NoteForOtherTutors, FeedbackForStudent are needed. The columns ManualPoints, NoteForOtherTutors, FeedbackForStudent are used to overwrite the values in the backend. The first line must be a comment (starting with #) containing the release code. If the upload was successful then the view is automatically refreshed"
            )}/>
            <input id={importCsvAssessmentFileExerciseId} type="file" className="collapsed" accept="*.csv"
                   onChange={(e: InputOnChangeEvent) => this.importCsvAssessmentFile(e)}
            />
          </div>
        }

        {
          isExerciseLocked === false &&
          <SimpleVDivider/>
        }

        {
          isExerciseLocked === false &&
          <div className={canRerunAssessment
                          ? 'clickable'
                          : ' div-disabled'}
               onClick={async () => {

                 const shouldReAssess = await askDialog(getI18n(this.props.langId, 'Re-run automatic assessments'),
                                                        getI18n(this.props.langId,
                                                                'Are you sure you want to re-run all automatic assessments?'
                                                        ), this.props.langId
                 )

                 if (!shouldReAssess) return


                 try {
                   this.props.rerunAllSubmissionsAsync(selectedRelease.generatedCode)
                   infoDialog(getI18n(this.props.langId, "Re-run automatic assessments"), getI18n(this.props.langId,
                                                                                                  "The re-run all automatic assessments has started. To get updates refresh the list"
                   ), this.props.langId)

                 } catch (err) {
                   ErrorHelper.makeServerErrorDialog(err, this.props.langId, 'Error',
                                                     'Could not re-run all automatic assessments'
                   )
                 }

                 this.props.setRunningAssessment(null)

               }}
          >
            <Icon name="refresh"/>
            <span className="mar-right-half">
            {
              getI18n(this.props.langId, 'Re-run automatic assessments')
            }
          </span>
            <HelpPopup wide defaultText={getI18n(this.props.langId,
                                                 'This re-runs all submit tests (and normal tests if specified in the release) and preserves the manual points and comments (for student, other tutors). This is also the case for single re-run.'
            )}/>
          </div>
        }
        {
          isExerciseLocked === false &&
          <SimpleVDivider/>
        }

        <div className="clickable"
             onClick={() => {

               //clear handle if any
               this.clearAutoRefreshViewInterval()
               this.props.setAutomaticRefreshViewIntervalCounter(1)

               if (this.props.isAutomaticRefreshViewEnabled === false) { //then enable
                 this.createAutoRefreshViewInterval()
               }

               this.props.setIsAutomaticRefreshViewEnabled(!this.props.isAutomaticRefreshViewEnabled)
             }}
        >
          <Icon name="refresh" className={this.props.isAutomaticRefreshViewEnabled
                                          ? 'clickable-toggled'
                                          : ''}/>
          {
            this.props.isAutomaticRefreshViewEnabled &&
            <span className="clickable-toggled">
              {
                getI18n(this.props.langId,
                        'Automatic refresh view in'
                ) +
                ` ${this.props.automaticRefreshViewIntervalCounter}` +
                `${getI18n(this.props.langId, 's')}`
              }
            </span>
          }

          {
            !this.props.isAutomaticRefreshViewEnabled &&
            <span>
            {
              getI18n(this.props.langId, 'Automatic refresh view')
            }
          </span>
          }


        </div>

      </div>
    )

    const rightArea = (
      <div>
        {
          isExerciseLocked &&
          <div>
            <HelpPopup icon="lock" defaultText={getI18n(this.props.langId, 'Exercise is lock')} />
          </div>
        }
      </div>
    )

    return (
      <CommonHeaderBarContentWrapper leftArea={leftArea} rightArea={rightArea}
                                     disabled={this.props.isLoading || this.props.isSomeDialogDisplayed}/>
    )
  }
}

export default connect(mapStateToProps, mapDispatchToProps)(HeaderBar)
