/**
 * Created by janis dähne.
 */

import * as React from "react";
import {connect} from "react-redux";
import {bindActionCreators, Dispatch} from "redux";
import {returntypeof} from 'react-redux-typescript';
import {RootState} from "../../../../state/reducers";

import ListView from './listView'
import SinglePanelSiteWrapper from '../../../singlePanelSiteWrapper'
import {Link} from "react-router-dom";
import * as constants from '../../../../constants'
import {getExerciseReleaseLinkPath} from '../../../../constants'
import MaterialSearchInput from '../../../material/materialSearchInput'
import {RouteComponentProps, withRouter} from "react-router";
import {parseBoolean, parseId} from "../../../../helpers/convertersAndTransformers";
import {
  setComesFromOwnExercisesSite,
  setSelectedExerciseId
} from "../../../../state/actions/ownOrGroupExerciseActions/releasesSiteActions/exerciseReleasesSiteActions";
import {
  loadSubmissionsSite,
  resetSubmissionsSite, setSearchText,
  setSelectedReleaseIdSubmissionsSite, setShowPassedTestsColumns
} from "../../../../state/actions/ownOrGroupExerciseActions/submissionsSite/submissionsSiteActions";
import Spinner from "../../../helpers/spinner";
import {BackArrow} from "../../../helpers/backArrow";
import {DoubleBackArrow} from "../../../helpers/doubleBackArrow";
import {ExerciseReleaseFromBackend} from "../../../../types/exerciseRelease";

import {getI18n} from "../../../../../i18n/i18nRoot";
import Logger from '../../../../helpers/logger'
import {CheckboxOnChangeEventData, InputOnChangeEvent} from '../../../../types/reactEvents'
import ExportCsvDialog from './exportCsvDialog/exportCsvDialog'
import {HelpPopup} from '../../../helpers/helpPopup'
import {CompareTestOptions} from '../../../../types/testResults'
import {Checkbox} from 'semantic-ui-react'

//const css = require('./styles.styl');

export interface MyProps {
  //readonly test: string
}

const mapStateToProps = (rootState: RootState /*, props: MyProps*/) => {
  return {
    //test0: rootState...
    //test: props.test

    comesFromOwnExercisesSite: rootState.submissionsSiteState.comesFromOwnExercisesSite,
    release: rootState.submissionsSiteState.release,
    selectedReleaseId: rootState.submissionsSiteState.selectedReleaseId,
    selectedExerciseId: rootState.submissionsSiteState.selectedExerciseId,
    exercise: rootState.submissionsSiteState.exercise,
    pagination: rootState.submissionsSiteState.pagination,

    isLoading: rootState.submissionsSiteState.isLoading
      || rootState.pLangsState.isLoading,

    isExportCsvDialogDisplayed: rootState.submissionsSiteState.isExportCsvDialogDisplayed,
    showPassedTestsColumns: rootState.submissionsSiteState.showPassedTestsColumns,

    langId: rootState.i18nState.langId
  }
}

const mapDispatchToProps = (dispatch: Dispatch) => bindActionCreators({
  //imported reducer funcs here
  setComesFromOwnExercisesSite,
  loadSubmissionsSite,
  resetSubmissionsSite,
  setSelectedReleaseIdSubmissionsSite,
  setSelectedExerciseId,
  setSearchText,
  setShowPassedTestsColumns,

}, dispatch)

interface MatchedProps {
  readonly exerciseId: string
  readonly comesFromOwnExercisesSite: string
  readonly releaseId: string
}

const stateProps = returntypeof(mapStateToProps);
const dispatchProps = returntypeof(mapDispatchToProps);
type Props = typeof stateProps & typeof dispatchProps;


//TODO enforce group role ... currently we don't know the group role ... maybe ask server exercise.userGroupId --> member --> role
class SubmissionsView extends React.Component<Props & RouteComponentProps<MatchedProps>, any> {

  componentWillMount(): void {
    const exerciseId = parseId(this.props.match.params.exerciseId)
    const comesFromOwnExercisesSite = parseBoolean(this.props.match.params.comesFromOwnExercisesSite)
    const releaseId = parseId(this.props.match.params.releaseId)


    this.props.resetSubmissionsSite()

    //don't do this in the load action because we need to set som props before (do we really need to??)
    //resetSubmissionsSite(false, false)

    this.props.setComesFromOwnExercisesSite(comesFromOwnExercisesSite)


    if (exerciseId === null && releaseId === null && comesFromOwnExercisesSite === null) {
      Logger.error('could not get url param exerciseId or releaseId or comesFromOwnExercisesSite')
      return
    }

    this.props.setSelectedExerciseId(exerciseId)
    this.props.setSelectedReleaseIdSubmissionsSite(releaseId)


    //this is possible if a user opens this page directly in the browser
    //then we need to load some more data e.g. from the releases site...
    //as a side effect (because we use the releases site actions
    //the data from the releases site is also set
    this.props.loadSubmissionsSite(exerciseId,
      comesFromOwnExercisesSite,
      releaseId,
      false
    )


  }

  getAssessmentState(selectedRelease: ExerciseReleaseFromBackend): string {

    if (selectedRelease.isReleased) return getI18n(this.props.langId, 'The release is still released (active).')

    if (!selectedRelease.releasedAt) {
      return getI18n(this.props.langId, 'The release was not released yet.')
    }


    if (selectedRelease.shouldAutomaticAssessSubmissions &&
      selectedRelease.hasAutomaticAssessmentStarted === false && selectedRelease.hasAutomaticAssessmentFinished === false) {
      return getI18n(this.props.langId, 'The automatic assessment is activated but has not ran yet.')
    }

    if (selectedRelease.hasAutomaticAssessmentStarted && selectedRelease.hasAutomaticAssessmentFinished === false) {
      return getI18n(this.props.langId,
        'The automatic assessment ist still running... Refresh the view to see progress.'
      )
    }

    if (selectedRelease.hasAutomaticAssessmentStarted
      && selectedRelease.hasAutomaticAssessmentFinished
      && selectedRelease.hadAutomaticAssessmentErrors) {
      return getI18n(this.props.langId, 'The automatic assessment has finished with errors!')
    }

    if (selectedRelease.shouldAutomaticAssessSubmissions
      && selectedRelease.hasAutomaticAssessmentStarted && selectedRelease.hasAutomaticAssessmentFinished
      && !selectedRelease.hadAutomaticAssessmentErrors) {
      return getI18n(this.props.langId, 'The automatic assessment has finished')
    }

    if (!selectedRelease.shouldAutomaticAssessSubmissions
      && selectedRelease.hasAutomaticAssessmentStarted
      && selectedRelease.hasAutomaticAssessmentFinished) {
      return getI18n(this.props.langId, 'The manually started automatic assessment has finished.')
    }


    if (!selectedRelease.shouldAutomaticAssessSubmissions) {


      if (selectedRelease.hasAutomaticAssessmentStarted === false && selectedRelease.hasAutomaticAssessmentFinished === false) {
        return getI18n(this.props.langId,
          'The automatic assessment option is not selected, tests won\'t run but a dummy assessment will be added automatically'
        )
      }


    }

    return getI18n(this.props.langId, "Error")
  }

  render(): JSX.Element {

    const selectedExercise = this.props.exercise
    const selectedRelease = this.props.release

    if (!selectedRelease) {
      //TODO only error if really not found
      //on first render this might not be set because the list was not yet set

      //this can also happen if the user opens this url and releases are not yet loaded...
    }

    return (
      <SinglePanelSiteWrapper>
        {
          this.props.isLoading &&
          <Spinner/>
        }

        {
          !this.props.isLoading && this.props.isExportCsvDialogDisplayed &&
          <ExportCsvDialog/>
        }

        {
          this.props.isExportCsvDialogDisplayed === false &&
          <div className="view-padding">
            <div className="view-head-line">
              <div className="view-caption">
                <div className="mar-right back-arrow">
                  <Link
                    to={this.props.comesFromOwnExercisesSite
                        ? constants.ownExercisesLinkPath
                        : constants.groupExercisesLinkPath}>
                    <DoubleBackArrow/>
                  </Link>
                </div>
                <div className="mar-right back-arrow">
                  <Link
                    to={getExerciseReleaseLinkPath(this.props.selectedExerciseId,
                      this.props.comesFromOwnExercisesSite
                    )}>
                    <BackArrow/>
                  </Link>
                </div>
                <h2 className="clear-margin">
                  {
                    getI18n(this.props.langId, 'Submissions of')
                  }
                  {
                    selectedExercise &&
                    ` ${selectedExercise.displayName}`
                  }
                  {
                    selectedRelease &&
                    ` (${selectedRelease.generatedCode}) `
                  }
                </h2>

                <HelpPopup className="mar-left-half v-centered" defaultText={getI18n(this.props.langId,
                  "The assessment (normal and submit tests) is run on the submit test server which might have other settings (e.g. timeout). The tutor view runs the submit tests on the submit test server and the normal test on the normal test server. If normal and submit tests are run and the normal tests produce some server error (e.g. connection to test server timed out) then the submit tests are not executed."
                )} wide={'very'}/>
              </div>
              <div className="view-options">

                <Checkbox className="mar-right" style={{alignSelf: 'center'}}
                          label={`${getI18n(this.props.langId, "Show passed tests columns")}`}
                          checked={this.props.showPassedTestsColumns}
                          onChange={(e, data) => {
                            this.props.setShowPassedTestsColumns(data.checked)
                          }}
                />

                <MaterialSearchInput
                  value={this.props.pagination.searchText}
                  onChange={(e: InputOnChangeEvent) => {
                    this.props.setSearchText(e.currentTarget.value)
                  }}
                />
              </div>
            </div>

            <div className="view-content">

              <div className="flexed pad-left-right">
                <div className="mar-right-half">
                  {getI18n(this.props.langId, 'State')}:
                </div>
                {
                  selectedRelease &&
                  <div className={selectedRelease.hadAutomaticAssessmentErrors
                                  ? 'error-colored mar-right-half'
                                  : 'mar-right-half'}>
                    {
                      this.getAssessmentState(selectedRelease)
                    }
                  </div>
                }
                {
                  selectedRelease && selectedRelease.isReleased &&
                  <div>
                  <span className="mar-right-half">{getI18n(this.props.langId,
                    'Nevertheless the normal test results of the users latest solution are displayed.'
                  )}</span>
                  </div>
                }
              </div>

              <ListView/>
            </div>
          </div>
        }


      </SinglePanelSiteWrapper>
    )
  }
}

export default withRouter(connect(mapStateToProps, mapDispatchToProps)(SubmissionsView))
