/**
 * Created by janis dähne.
 */

import * as React from "react";
import {connect} from "react-redux";
import {bindActionCreators, Dispatch} from "redux";
import {returntypeof} from 'react-redux-typescript';
import {RootState} from "../../../../state/reducers";
import ChangeTagView from './changeTagView'
import {Divider} from "semantic-ui-react";
import SaveButton from "../../../buttons/saveButton";
import CancelButton from "../../../buttons/cancelButton";
import {cancelEditTag} from "../../../../state/actions/manageTagsSite/manageTagsActions";
import {createTagAsync, updateTagAsync} from "../../../../state/actions/manageTagsSite/dialog/tagCrudActions";
import Spinner from "../../../helpers/spinner";


import {getI18n} from "../../../../../i18n/i18nRoot";

//const css = require('./styles.styl');

export interface MyProps {
  //readonly test: string
}

const mapStateToProps = (rootState: RootState /*, props: MyProps*/) => {
  return {
    //test0: rootState...
    //test: props.test
    dialogState: rootState.changeTagDialogState,
    isCreating: rootState.changeTagDialogState.isCreating,
    isBusy: rootState.changeTagDialogState.isBusy,
    langId: rootState.i18nState.langId
  }
}

const mapDispatchToProps = (dispatch: Dispatch) => bindActionCreators({
  //imported reducer funcs here
  createTagAsync,
  updateTagAsync,
  cancelEditTag
}, dispatch)


const stateProps = returntypeof(mapStateToProps);
const dispatchProps = returntypeof(mapDispatchToProps);
type Props = typeof stateProps & typeof dispatchProps;

class ChangeTagDialog extends React.Component<Props, any> {
  render(): JSX.Element {
    return (
      <div>

        {
          this.props.isBusy &&
          <Spinner/>
        }

        <div className="view-padding">
          <div>
            <div className="view-caption">
              <h2>
                {
                  this.props.isCreating &&
                  getI18n(this.props.langId,'Create tag')
                }
                {
                  !this.props.isCreating &&
                  getI18n(this.props.langId,'Change tag')
                }
              </h2>
            </div>
          </div>

          <div className="view-content">
            <ChangeTagView/>

            <Divider className="hidden"/>

            <div className="flexed">
              <SaveButton className="mar-right" disabled={!this.props.dialogState.isValid}
                          onClick={() => {
                            if (this.props.dialogState.isCreating) {
                              this.props.createTagAsync(this.props.dialogState.tag)
                            } else {
                              this.props.updateTagAsync(this.props.dialogState.tag)
                            }
                          }}
              />
              <CancelButton onClick={() => {
                this.props.cancelEditTag()
              }}/>
            </div>

          </div>
        </div>
      </div>
    )
  }
}

export default connect(mapStateToProps, mapDispatchToProps)(ChangeTagDialog)

