/**
 * Created by janis dähne.
 */

import * as React from "react";
import {connect} from "react-redux";
import {bindActionCreators, Dispatch} from "redux";
import {returntypeof} from 'react-redux-typescript';
import {RootState} from "../../../../state/reducers";
import {Form, Icon} from "semantic-ui-react";
import MaterialInput from '../../../material/materialInput'
import {ChromePicker, ColorResult} from 'react-color'
import {
  setDescription, setDisplayName, setHtmlBackgroundColor, setHtmlColor, setHtmlIcon, setId,
} from "../../../../state/actions/manageTagsSite/dialog/tagSetActions";
import {validationRules} from "../../../../state/reducers/manageTagsSite/dialog/tagReducer";
import {validationMessageKeys} from "../../../../state/reducers/manageTagsSite/dialog/tagReducerValidation";
import {
  setIsBackgroundColorPickerOpened, setIsColorPickerOpened
} from "../../../../state/actions/manageTagsSite/dialog/changeTagdialogActions";
import Tag from '../../../tagComponents/tag'
import {MyPopup} from "../../../helpers/myPopup";
import {HelpPopup} from "../../../helpers/helpPopup";
import {getI18n} from "../../../../../i18n/i18nRoot";

//const css = require('./styles.styl');

export interface MyProps {
  //readonly test: string
}

const mapStateToProps = (rootState: RootState /*, props: MyProps*/) => {
  return {
    //test0: rootState...
    //test: props.test
    tag: rootState.changeTagDialogState.tag,
    isColorPickerOpened: rootState.changeTagDialogState.isColorPickerOpened,
    isBackgroundColorPickerOpene: rootState.changeTagDialogState.isBackgroundColorPickerOpened,
    langId: rootState.i18nState.langId
  }
}

const mapDispatchToProps = (dispatch: Dispatch) => bindActionCreators({
  //imported reducer funcs here
  setId,
  setDisplayName,
  setDescription,
  setHtmlColor,
  setHtmlBackgroundColor,
  setHtmlIcon,
  setIsColorPickerOpened,
  setIsBackgroundColorPickerOpened
}, dispatch)


const stateProps = returntypeof(mapStateToProps);
const dispatchProps = returntypeof(mapDispatchToProps);
type Props = typeof stateProps & typeof dispatchProps;


/**
 * TODO make color picker icons clickable and display them
 */
class ChangeTagView extends React.Component<Props, any> {

  render(): JSX.Element {

    return (<div>
        <div className="ui form">
          <Form.Field>
            <div className="flexed">

              <div>
                <Tag tag={this.props.tag}/>
              </div>
            </div>
          </Form.Field>

          <Form.Group widths='equal'>
            <Form.Field required>
              <label>
                {getI18n(this.props.langId,'Name')}
              </label>
              <MaterialInput
                value={this.props.tag.displayName}
                validations={validationRules.displayName}
                validationMessageKey={validationMessageKeys.displayName}
                onChange={(e) => this.props.setDisplayName(e.currentTarget.value)}
              />
            </Form.Field>
            <Form.Field>
              <div className="form-field-label-with-icon">
                <label className="mar-right-half">
                  {getI18n(this.props.langId,'Icon')}
                </label>
                <MyPopup
                  trigger={<a href="https://react.semantic-ui.com/elements/icon" target="_blank" tabIndex={-1}>
                    <Icon name="question circle" className="hoverable"/>
                  </a>}
                  content={getI18n(this.props.langId,'Click to open a new tab with all possible icons. Then past the icon text here.')}
                />
              </div>

              <MaterialInput placeholder="z.B. block layout"
                             value={this.props.tag.htmlIcon}
                             validations={validationRules.htmlIcon}
                             validationMessageKey={validationMessageKeys.htmlIcon}
                             onChange={(e) => this.props.setHtmlIcon(e.currentTarget.value)}
              />
            </Form.Field>
          </Form.Group>

          <Form.Field>
            <div className="form-field-label-with-icon">
              <label className="mar-right-half">
                {getI18n(this.props.langId,'Description')}
              </label>
              <HelpPopup defaultText={getI18n(this.props.langId,'The description is only displayed on hover.')}
              />
            </div>
            <MaterialInput
              value={this.props.tag.description}
              validations={validationRules.description}
              validationMessageKey={validationMessageKeys.description}
              onChange={(e) => this.props.setDescription(e.currentTarget.value)}
            />
          </Form.Field>

          <Form.Group widths='equal'>
            <Form.Field>
              <div className="form-field-label-with-icon">
                <label className="mar-right-half">
                  {getI18n(this.props.langId,'Font color')}
                </label>
                <HelpPopup defaultText={getI18n(this.props.langId,'All css colors are allowed, so also red, green, ... You can click on the brush to open a color picker.')}/>
              </div>
              <MyPopup
                trigger={<div>
                  <MaterialInput icon="paint brush"
                                 value={this.props.tag.htmlColor}
                                 validations={validationRules.htmlColor}
                                 validationMessageKey={validationMessageKeys.htmlColor}
                                 onChange={(e) => this.props.setHtmlColor(e.currentTarget.value)}
                                 iconClickHandler={() => this.props.setIsColorPickerOpened(true)}
                  />
                </div>}
                content={<div className="react-color-picker">
                  <ChromePicker
                    color={this.props.tag.htmlColor
                           ? this.props.tag.htmlColor
                           : ''}
                    onChangeComplete={(color: ColorResult) => this.props.setHtmlColor(color.hex)}
                  />
                </div>}
                position="top left"
                open={this.props.isColorPickerOpened}
                onClose={() => this.props.setIsColorPickerOpened(false)}
                on='click'
              />
            </Form.Field>
            <Form.Field>
              <div className="form-field-label-with-icon">
                <label className="mar-right-half">
                  {getI18n(this.props.langId,'Background color')}
                </label>
                <HelpPopup defaultText={getI18n(this.props.langId,'All css colors are allowed, so also red, green, ... You can click on the brush to open a color picker.')}/>
              </div>

              <MyPopup
                trigger={<div>
                  <MaterialInput icon="paint brush"
                                 value={this.props.tag.htmlBackgroundColor}
                                 validations={validationRules.htmlBackgroundColor}
                                 validationMessageKey={validationMessageKeys.htmlBackgroundColor}
                                 onChange={(e) => this.props.setHtmlBackgroundColor(e.currentTarget.value)}
                                 iconClickHandler={() => this.props.setIsBackgroundColorPickerOpened(true)}

                  />
                </div>}
                content={<div className="react-color-picker">
                  <ChromePicker
                    color={this.props.tag.htmlBackgroundColor
                           ? this.props.tag.htmlBackgroundColor
                           : ''}
                    onChangeComplete={(color: ColorResult) => this.props.setHtmlBackgroundColor(color.hex)}
                  />
                </div>}
                position="top left"
                open={this.props.isBackgroundColorPickerOpene}
                onClose={() => this.props.setIsBackgroundColorPickerOpened(false)}
                on='click'
              />

            </Form.Field>
          </Form.Group>
        </div>
      </div>)
  }
}

export default connect(mapStateToProps, mapDispatchToProps)(ChangeTagView)