/**
 * Created by janis dähne.
 */

import * as React from "react";
import {connect} from "react-redux";
import {bindActionCreators, Dispatch} from "redux";
import {returntypeof} from 'react-redux-typescript';
import {RootState} from "../../../state/reducers";
import SinglePanelSiteWrapper from '../../singlePanelSiteWrapper'
import MaterialSearchInput from '../../material/materialSearchInput'
import Spinner from "../../helpers/spinner";
import ChangeTagDialog from "./dialogs/changeTagDialog";
import {loadManageTagsSite, openEditTagDialog} from "../../../state/actions/manageTagsSite/manageTagsActions";
import {deleteTagAsync, getTagsAsync} from "../../../state/actions/manageTagsSite/manageTagsSiteCrudActions";
import {TagFullBase} from "../../../types/tag";
import Tag from '../../tagComponents/tag'
import {askDialog, errorDialog} from "../../../helpers/dialogHelper";


import {orderBy} from 'lodash-es'
import {getI18n} from "../../../../i18n/i18nRoot";
import Logger from '../../../helpers/logger'
import {ErrorHelper} from '../../../helpers/errorHelper'

declare var require: any
const css = require('./styles.styl');

export interface MyProps {
  //readonly test: string
}

const mapStateToProps = (rootState: RootState /*, props: MyProps*/) => {
  return {
    //test0: rootState...
    //test: props.test
    tags: rootState.manageTagsState.tags,
    isLoading: rootState.manageTagsState.isLoading,
    isChangeDialogDisplayed: rootState.manageTagsState.isChangeDialogDisplayed,
    langId: rootState.i18nState.langId
  }
}

const mapDispatchToProps = (dispatch: Dispatch) => bindActionCreators({
  //imported reducer funcs here
  openEditTagDialog,
  deleteTagAsync,
  loadManageTagsSite
}, dispatch)


const stateProps = returntypeof(mapStateToProps);
const dispatchProps = returntypeof(mapDispatchToProps);
type Props = typeof stateProps & typeof dispatchProps;

class ManageTagsSite extends React.Component<Props, any> {

  async componentWillMount(): Promise<void> {
    try {
      await this.props.loadManageTagsSite()
    } catch(err) {
      ErrorHelper.makeServerErrorDialog(err, this.props.langId, 'Error', 'Could not load site data')
    }
  }

  render(): JSX.Element {


    const list: ReadonlyArray<TagFullBase> = orderBy(this.props.tags, (tag) => tag.displayName, 'asc')

    return (
      <SinglePanelSiteWrapper>

        {
          this.props.isLoading &&
          <Spinner/>
        }

        {
          !this.props.isChangeDialogDisplayed &&
          <div className="view-padding">
            <div className="view-head-line">
              <div className="view-caption">
                <h1>
                  {
                    getI18n(this.props.langId,'Tags')
                  }
                </h1>
              </div>
              {/*<div className="view-options">*/}
                {/*<MaterialSearchInput/>*/}
              {/*</div>*/}
            </div>

            <div className="view-content flexed separate-tabs">
              {
                list.map((tag, index2) => {
                  return (
                    <div key={tag.id}>
                      <Tag tag={tag}
                           leftIconName="write"
                           onLeftIconClicked={() => {
                             this.props.openEditTagDialog(tag)
                           }}
                           rightIconName="remove"
                           onRightIconClicked={async () => {

                             const shouldRemove = await askDialog(getI18n(this.props.langId,'Delete tag'),
                               `${getI18n(this.props.langId,'Are you sure you want to delete the tag?')} (${tag.displayName})`, this.props.langId)

                             if (shouldRemove) {
                               this.props.deleteTagAsync(tag)
                             }

                           }}
                      />
                    </div>
                  )
                })
              }
            </div>
          </div>
        }

        {
          !this.props.isLoading && this.props.isChangeDialogDisplayed &&
          <ChangeTagDialog/>
        }

      </SinglePanelSiteWrapper>
    )
  }
}

export default connect(mapStateToProps, mapDispatchToProps)(ManageTagsSite)
