/**
 * Created by janis dähne.
 */

import * as React from "react";
import {connect} from "react-redux";
import {bindActionCreators, Dispatch} from "redux";
import {returntypeof} from 'react-redux-typescript';
import {RootState} from "../../../state/reducers";
import SinglePanelSiteWrapper from '../../singlePanelSiteWrapper'
import {Icon, Form, Popup} from "semantic-ui-react";
import MaterialInput from '../../material/materialInput'
import {SimpleVDivider} from "../../helpers/simpleVDivider";
import Spinner from "../../helpers/spinner";
import {ChangeState} from "../../../state/reducers/settingsSite/settingsSiteReducers";
import {
  setUserDataChangeStateAction,
  setUserSettingsChangeStateAction
} from "../../../state/actions/settingsSite/userSettingsSiteActions";
import {updateUserDataAsync} from "../../../state/actions/settingsSite/userSettingsSiteCrudActions";
import {
  resetUserBaseData, setEmail, setNewPassword,
  setOldPassword
} from "../../../state/actions/settingsSite/setUserDataActions";
import OwnInformationSettingsChangeView from './ownInformationSettingsChangeView'
import {leftIconPopupOffset} from "../../../constants";
import * as cryptoHelper from '../../../helpers/cryptoHelper'


import {MyPopup} from "../../helpers/myPopup";
import {getI18n} from "../../../../i18n/i18nRoot";
import {errorDialog} from '../../../helpers/dialogHelper'

//const css = require('./styles.styl');

export interface MyProps {
  //readonly test: string
}

const mapStateToProps = (rootState: RootState /*, props: MyProps*/) => {
  return {
    userData: rootState.userDataSettingsSate.userData,
    systemRoleDisplayName: rootState.userDataSettingsSate.userData.systemRole
                           ? rootState.userDataSettingsSate.userData.systemRole.displayName
                           : null,
    isLoading: rootState.userDataSettingsSate.isLoadingUserData || rootState.langsState.isLoading,
    changeState: rootState.userDataSettingsSate.userDataChangeState,
    userDataCopy: rootState.userDataSettingsSate.userDataCopy,
    langId: rootState.i18nState.langId
  }
}

const mapDispatchToProps = (dispatch: Dispatch) => bindActionCreators({
                                                                        //imported reducer funcs here
                                                                        setUserDataChangeStateAction,
                                                                        resetUserBaseData,
                                                                        updateUserDataAsync
                                                                      }, dispatch)


const stateProps = returntypeof(mapStateToProps);
const dispatchProps = returntypeof(mapDispatchToProps);
type Props = typeof stateProps & typeof dispatchProps;

class OwnInformationSettingsView extends React.Component<Props, any> {
  render(): JSX.Element {
    return (
      <SinglePanelSiteWrapper>

        {
          this.props.isLoading &&
          <Spinner/>
        }

        <div className="view-padding">

          <div className="view-head-line">

            <div className="view-caption">
              <h2>
                {
                  getI18n(this.props.langId, 'Information')
                }
              </h2>
            </div>

            <div className="view-options">
              {
                this.props.changeState === ChangeState.notActive &&
                <div className="flexed view-head-line-centered">
                  <div className="clickable"
                       onClick={() => this.props.setUserDataChangeStateAction(ChangeState.active)}
                  >
                    <Icon name="write"/>
                    <span>
                      {
                        getI18n(this.props.langId, 'Change')
                      }
                    </span>
                  </div>
                </div>
              }

              {
                this.props.changeState === ChangeState.active &&
                <div className="flexed view-head-line-centered">
                  <div className={this.props.userDataCopy.isValid
                                  ? 'clickable'
                                  : 'div-disabled '}
                       onClick={() => {
                         if (this.props.userDataCopy.newPassword !== this.props.userDataCopy.newPasswordCheck) {
                           errorDialog('', getI18n(this.props.langId, "Password and Password again were different"),this.props.langId)
                           return
                         }

                         this.props.updateUserDataAsync({
                                                          email: this.props.userDataCopy.email,
                                                          oldPassword: cryptoHelper.hash(
                                                            this.props.userDataCopy.oldPassword),
                                                         oldPasswordBase64: cryptoHelper.hashBase64(
                                                           this.props.userDataCopy.oldPassword),
                                                          newPassword: cryptoHelper.hash(
                                                            this.props.userDataCopy.newPassword),
                                                          newPasswordBase64: cryptoHelper.hashBase64(
                                                            this.props.userDataCopy.newPassword)
                                                        })
                       }}
                  >
                    <Icon name="save"/>
                    <span>
                      {
                        getI18n(this.props.langId, 'Save')
                      }
                    </span>
                  </div>

                  <SimpleVDivider/>

                  <div className="clickable"
                       onClick={() => {
                         this.props.setUserDataChangeStateAction(ChangeState.notActive)
                         this.props.resetUserBaseData(this.props.userData.email)
                       }}
                  >
                    <Icon name="remove"/>
                    <span>
                      {
                        getI18n(this.props.langId, 'Discard')
                      }
                    </span>
                  </div>
                </div>
              }

            </div>
          </div>

          <div className="view-content">
            <div className="ui form">
              <Form.Group widths='equal'>
                <Form.Field>
                  {
                    //TODO if the user is an admin... he/she can change this??
                  }
                  <MyPopup
                    trigger={<Icon name="lock" className="hoverable"/>}
                    content={
                      <span>
                        {
                          getI18n(this.props.langId, 'This data can only be changed by an admin.')
                        }
                      </span>
                    }
                  />
                  <label>
                    {
                      getI18n(this.props.langId, 'First name')
                    }
                  </label>
                  <MaterialInput value={this.props.userData.firstName} disabled/>
                </Form.Field>

                <Form.Field>
                  <MyPopup
                    trigger={<Icon name="lock" className="hoverable"/>}
                    content={
                      <span>
                        {
                          getI18n(this.props.langId, 'This data can only be changed by an admin.')
                        }
                      </span>
                    }
                  />
                  <label>
                    {
                      getI18n(this.props.langId, 'Last name')
                    }
                  </label>
                  <MaterialInput value={this.props.userData.lastName} disabled/>
                </Form.Field>

                <Form.Field>
                  <MyPopup
                    trigger={<Icon name="lock" className="hoverable"/>}
                    content={
                      <span>
                        {
                          getI18n(this.props.langId, 'This data can only be changed by an admin.')
                        }
                      </span>
                    }
                  />
                  <label>
                    {
                      getI18n(this.props.langId, 'System role')
                    }
                  </label>

                  <MaterialInput
                    value={this.props.systemRoleDisplayName
                                        ? this.props.systemRoleDisplayName
                                        : getI18n(this.props.langId, "None")}
                                 disabled/>
                </Form.Field>

              </Form.Group>

              {
                this.props.changeState === ChangeState.notActive &&
                <Form.Group widths='equal'>
                  <Form.Field>
                    <label>
                      {
                        getI18n(this.props.langId, 'Email')
                      }
                    </label>
                    <MaterialInput value={this.props.userData.email} disabled/>
                  </Form.Field>
                  <Form.Field>
                    <label>
                      {
                        getI18n(this.props.langId, 'Password')
                      }
                    </label>
                    <MaterialInput value="****" type="password" disabled/>
                  </Form.Field>
                </Form.Group>
              }
              {
                this.props.changeState === ChangeState.active &&
                <OwnInformationSettingsChangeView/>
              }

            </div>
          </div>
        </div>

      </SinglePanelSiteWrapper>

    )
  }
}

export default connect(mapStateToProps, mapDispatchToProps)(OwnInformationSettingsView)
