/**
 * Created by janis dähne.
 */

import * as React from "react";
import {connect} from "react-redux";
import {bindActionCreators, Dispatch} from "redux";
import {returntypeof} from 'react-redux-typescript';
import {RootState} from "../../../state/reducers";
import {SelectableGroupUserFrontend} from "../../../types/group";
import {Checkbox, Icon, Popup, Table} from "semantic-ui-react";
import {CheckboxOnChangeEventData} from "../../../types/reactEvents";
import {
  removeMemberAndRefreshAsync,
  setAllMemberUiIsSelected,
  setMemberGroupRoleAndRefreshAsync,
  setMemberUiIsSelected,
  setUiIsChangeGroupRolePopupDisplayedAction
} from "../../../state/actions/singleGroupSite/singleGroupSiteActions";
import {copyToClipboard} from "../../../helpers/clipboard";
import {leftIconPopupOffset} from "../../../constants";
import SelectOptionsList, {SelectableOption} from '../../helpers/selectOptionList'
import {askDialog} from "../../../helpers/dialogHelper";


import {GroupRoleWithPermissionFullBase} from "../../../types/groupRoles";
import {MyPopup} from "../../helpers/myPopup";
import {HelpPopup} from "../../helpers/helpPopup";
import {getI18n} from "../../../../i18n/i18nRoot";

//const css = require('./styles.styl');

export interface MyProps {
  //readonly test: string
  readonly member: SelectableGroupUserFrontend
  readonly currentUserGroupRole: GroupRoleWithPermissionFullBase
}

const mapStateToProps = (rootState: RootState , props: MyProps) => {
  return {
    //test0: rootState...
    //test: props.test
    member: props.member,
    groupRoles: rootState.singleGroupState.groupRoles,
    groupId: rootState.singleGroupState.selectedGroupId,
    currentUserGroupRole: props.currentUserGroupRole,
    userData: rootState.userDataSettingsSate.userData,
    defaultUserGroupId: rootState.globalSystemSettingsState.systemSettings.defaultUserGroupId,
    langId: rootState.i18nState.langId
  }
}

const mapDispatchToProps = (dispatch: Dispatch) => bindActionCreators({
  //imported reducer funcs here

  setMemberGroupRoleAndRefreshAsync,
  setUiIsChangeGroupRolePopupDisplayedAction,
  removeMemberAndRefreshAsync,
  setAllMemberUiIsSelected,
  setMemberUiIsSelected,

}, dispatch)


const stateProps = returntypeof(mapStateToProps);
const dispatchProps = returntypeof(mapDispatchToProps);
type Props = typeof stateProps & typeof dispatchProps;

class listViewItem extends React.Component<Props, any> {

  render(): JSX.Element {

    const member = this.props.member

    return (
      <Table.Row>
        <Table.Cell>
          {
            !member.isCurrentUser &&
            <Checkbox checked={member.uiIsSelected}
                      onChange={(e, data: CheckboxOnChangeEventData) => {
                        this.props.setMemberUiIsSelected(member.id, data.checked)
                      }}
            />
          }
        </Table.Cell>

        <Table.Cell>
          <span>{member.lastName}</span>
        </Table.Cell>

        <Table.Cell>
          <span>{member.firstName}</span>
        </Table.Cell>

        <Table.Cell>
          <span>{member.token}</span>
        </Table.Cell>

        <Table.Cell>
          <div>
            {
              //TODO icon or checkbox??
              //checkbox is more precises than different icons??
              //<Icon name={member.isActivated ? 'user' : 'user outline'} />
            }
            {
              <Checkbox checked={member.isActivated} disabled/>
            }
          </div>
        </Table.Cell>

        <Table.Cell>
          <Icon name="copy" className="clickable"
                onClick={() => {
                  copyToClipboard(member.email)
                }}
          />
          <span>{member.email}</span>
        </Table.Cell>

        <Table.Cell>
          {
            member.groupRole && !member.isCurrentUser && this.props.currentUserGroupRole && this.props.currentUserGroupRole.canChangeOtherMembersRole
            && member.id !== this.props.userData.id &&
            <MyPopup
              trigger={
                <div className="clickable">
                  <Icon name="caret down" className="mar-right-half"/>
                  <span>{member.groupRole.displayName}</span>
                </div>
              }
              on="click"
              position="bottom left"
              content={
                <div>
                  <SelectOptionsList
                    options={this.props.groupRoles}
                    optionClicked={(option: SelectableOption) => {
                      const groupRole = this.props.groupRoles.find(p => p.id === option.id)

                      if (!groupRole) {
                        //TODO some error
                      } else {
                        this.props.setMemberGroupRoleAndRefreshAsync(member.id, this.props.groupId,
                          groupRole.id)
                      }
                    }}
                  />
                </div>
              }
              open={member.uiIsChangeGroupRolePopupDisplayed}
              onOpen={() => this.props.setUiIsChangeGroupRolePopupDisplayedAction(true, member.id)}
              onClose={() => this.props.setUiIsChangeGroupRolePopupDisplayedAction(false, member.id)}

            />
          }
          {
            member.groupRole && this.props.currentUserGroupRole && member.id === this.props.userData.id &&
            <div>
              <span>{member.groupRole.displayName}</span>
            </div>
          }
          {
            !member.groupRole &&
            <span>{getI18n(this.props.langId, "Error")}</span>
          }
        </Table.Cell>

        <Table.Cell>
          {
            member.isCurrentUser &&
            <HelpPopup className="warning-colored" defaultText={getI18n(this.props.langId,'This is you!')}/>
          }
          {
            this.props.groupId === this.props.defaultUserGroupId && !member.isCurrentUser &&
            <HelpPopup defaultText={getI18n(this.props.langId,'You cannot delete members from the default user group')}/>
          }
          {
            !member.isCurrentUser && this.props.groupId !== this.props.defaultUserGroupId && this.props.currentUserGroupRole && this.props.currentUserGroupRole.canRemoveMemberFromGroup &&
            <div className="clickable"
                 onClick={async () => {

                   const shouldRemove = await askDialog(getI18n(this.props.langId,'Release user from group'),
                     `${getI18n(this.props.langId,'Are you sure you want to release the user from this group?')} (${member.lastName}, ${member.firstName})`, this.props.langId)

                   if (shouldRemove) {
                     this.props.removeMemberAndRefreshAsync(member.id, this.props.groupId)
                   }

                 }}
            >
              <Icon name="remove" className="clickable"/>
            </div>

          }

        </Table.Cell>

      </Table.Row>
    )
  }
}

export default connect(mapStateToProps, mapDispatchToProps)(listViewItem)
