/**
 * Created by janis dähne.
 */

import * as React from "react";
import {connect} from "react-redux";
import {bindActionCreators, Dispatch} from "redux";
import {returntypeof} from 'react-redux-typescript';
import {RootState} from "../../../state/reducers";
import {Checkbox, Icon, Table} from "semantic-ui-react";
import {
  removeMemberAndRefreshAsync, setActivePaginationPage,
  setAllMemberUiIsSelected,
  setMemberGroupRoleAndRefreshAsync,
  setMemberUiIsSelected, setPaginationPageSize,
  setSortByKey,
  setUiIsChangeGroupRolePopupDisplayedAction
} from "../../../state/actions/singleGroupSite/singleGroupSiteActions";
import {CheckboxOnChangeEventData} from "../../../types/reactEvents";
import {SelectableGroupUserFrontend} from "../../../types/group";
import {isCurrentSortingColumnPaginated, rotateSorting} from "../../../helpers/tablesHelper";
import ListViewItem from './listViewItem'


import {MyPopup} from "../../helpers/myPopup";
import {getI18n} from "../../../../i18n/i18nRoot";
import PaginationTableFooter from '../../helpers/paginationTableFooter'

//const css = require('./styles.styl');

export interface MyProps {
  //readonly test: string
  readonly member: SelectableGroupUserFrontend
}

const mapStateToProps = (rootState: RootState /*, props: MyProps*/) => {
  return {
    //test0: rootState...
    //test: props.test
    members: rootState.singleGroupState.members,
    groupRoles: rootState.singleGroupState.groupRoles,
    groupId: rootState.singleGroupState.selectedGroupId,
    group: rootState.singleGroupState.group,
    sortByKey: rootState.singleGroupState.pagination.orderByProp,
    sortDirection: rootState.singleGroupState.pagination.isOrderByDesc,
    pagination: rootState.singleGroupState.pagination,
    langId: rootState.i18nState.langId
  }
}

const mapDispatchToProps = (dispatch: Dispatch) => bindActionCreators({
                                                                        //imported reducer funcs here
                                                                        setMemberGroupRoleAndRefreshAsync,
                                                                        setUiIsChangeGroupRolePopupDisplayedAction,
                                                                        removeMemberAndRefreshAsync,
                                                                        setAllMemberUiIsSelected,
                                                                        setMemberUiIsSelected,
                                                                        setSortByKey,
                                                                        setActivePaginationPage,
                                                                        setPaginationPageSize,
                                                                      }, dispatch)


const stateProps = returntypeof(mapStateToProps);
const dispatchProps = returntypeof(mapDispatchToProps);
type Props = typeof stateProps & typeof dispatchProps;

class MemberListView extends React.Component<Props, any> {
  render(): JSX.Element {

    const list: ReadonlyArray<SelectableGroupUserFrontend> = this.props.members

    return (
      <div>
        <Table sortable selectable padded striped singleLine className="material-table">

          <Table.Header>
            <Table.Row>
              <Table.HeaderCell>
                <Checkbox
                  checked={
                    this.props.members.every(p => p.uiIsSelected || p.isCurrentUser) && this.props.members.length > 0
                  }
                  onChange={(e, data: CheckboxOnChangeEventData) => {
                    this.props.setAllMemberUiIsSelected(data.checked)
                  }}
                />
              </Table.HeaderCell>
              <Table.HeaderCell
                sorted={
                  isCurrentSortingColumnPaginated<SelectableGroupUserFrontend>(this.props.sortByKey, 'lastName',
                                                                               this.props.sortDirection
                  )
                }
              >
                <span className="clickable"
                      onClick={() => {
                        this.props.setSortByKey('lastName',
                                                rotateSorting<SelectableGroupUserFrontend>(this.props.sortDirection,
                                                                                           this.props.sortByKey,
                                                                                           'lastName'
                                                )
                        )
                      }}
                >
                  {
                    getI18n(this.props.langId, 'Last name')
                  }
                </span>
              </Table.HeaderCell>
              <Table.HeaderCell
                sorted={
                  isCurrentSortingColumnPaginated<SelectableGroupUserFrontend>(this.props.sortByKey, 'firstName',
                                                                               this.props.sortDirection
                  )
                }
              >
                <span className="clickable"
                      onClick={() => {
                        this.props.setSortByKey('firstName',
                                                rotateSorting<SelectableGroupUserFrontend>(this.props.sortDirection,
                                                                                           this.props.sortByKey,
                                                                                           'firstName'
                                                )
                        )
                      }}
                >
                  {
                    getI18n(this.props.langId, 'First name')
                  }
                </span>
              </Table.HeaderCell>
              <Table.HeaderCell
                sorted={
                  isCurrentSortingColumnPaginated<SelectableGroupUserFrontend>(this.props.sortByKey, 'token',
                                                                               this.props.sortDirection
                  )
                }
              >
                <span className="clickable"
                      onClick={() => {
                        this.props.setSortByKey('token',
                                                rotateSorting<SelectableGroupUserFrontend>(this.props.sortDirection,
                                                                                           this.props.sortByKey,
                                                                                           'token'
                                                )
                        )
                      }}
                >
                  {
                    getI18n(this.props.langId, 'Token')
                  }
                </span>
              </Table.HeaderCell>
              <Table.HeaderCell
                sorted={
                  isCurrentSortingColumnPaginated<SelectableGroupUserFrontend>(this.props.sortByKey, 'isActivated',
                                                                               this.props.sortDirection
                  )
                }
              >
                <MyPopup
                  trigger={<Icon name="user outline" className="hoverable"/>}
                  content={getI18n(this.props.langId,
                                   'Indicates if the user is already active or if this is a new user (not activated yet).'
                  )}
                  wide
                />
                <span className="clickable"
                      onClick={() => {
                        this.props.setSortByKey('isActivated',
                                                rotateSorting<SelectableGroupUserFrontend>(this.props.sortDirection,
                                                                                           this.props.sortByKey,
                                                                                           'isActivated'
                                                )
                        )
                      }}
                >
                  {
                    getI18n(this.props.langId, 'Is activated')
                  }
                </span>
              </Table.HeaderCell>
              <Table.HeaderCell
                sorted={
                  isCurrentSortingColumnPaginated<SelectableGroupUserFrontend>(this.props.sortByKey, 'email',
                                                                               this.props.sortDirection
                  )
                }
              >
                <span className="clickable"
                      onClick={() => {
                        this.props.setSortByKey('email',
                                                rotateSorting<SelectableGroupUserFrontend>(this.props.sortDirection,
                                                                                           this.props.sortByKey,
                                                                                           'email'
                                                )
                        )
                      }}
                >
                  {
                    getI18n(this.props.langId, 'Email')
                  }
                </span>
              </Table.HeaderCell>
              <Table.HeaderCell
                sorted={
                  isCurrentSortingColumnPaginated<SelectableGroupUserFrontend>(this.props.sortByKey, 'groupRole',
                                                                               this.props.sortDirection
                  )
                }
              >
                <span className="clickable"
                      onClick={() => {
                        this.props.setSortByKey('groupRole',
                                                rotateSorting<SelectableGroupUserFrontend>(this.props.sortDirection,
                                                                                           this.props.sortByKey,
                                                                                           'groupRole'
                                                )
                        )
                      }}
                >
                  {
                    getI18n(this.props.langId, 'Group role')
                  }
                </span>
              </Table.HeaderCell>
              <Table.HeaderCell>
                <span>
                  {
                    getI18n(this.props.langId, 'Release member')
                  }
                </span>
              </Table.HeaderCell>
            </Table.Row>
          </Table.Header>

          <Table.Body>
            {
              list.map((member, index) => {
                return (
                  <ListViewItem key={member.id} member={member} currentUserGroupRole={this.props.group.groupRole}/>
                )
              })
            }

          </Table.Body>
          <PaginationTableFooter
            pagination={this.props.pagination}
            setActivePaginationPage={this.props.setActivePaginationPage}
            setPaginationPageSize={this.props.setPaginationPageSize}
          />
        </Table>

      </div>
    )
  }
}

export default connect(mapStateToProps, mapDispatchToProps)(MemberListView)
