import * as React from "react";
import {connect} from "react-redux";
import {bindActionCreators, Dispatch} from "redux";
import {returntypeof} from 'react-redux-typescript';
import {RootState} from '../../../state/reducers'
import SinglePanelSiteWrapper from '../../singlePanelSiteWrapper'
import {getI18n} from '../../../../i18n/i18nRoot'
import {Form, Icon, Table} from 'semantic-ui-react'
import Spinner from '../../helpers/spinner'
import {
  setEditingPLangId,
  setPLangDisplayName,
  setPLangEditorHighlightModeName,
  setPLangFileExtensionsWithDot,
  setPLangInternalName,
} from '../../../state/actions/plangSettingsView/pLangsActions'
import MaterialInput from '../../material/materialInput'
import {
  createPLangAsync,
  deletePLangAsync,
  updatePLangAsync
} from '../../../state/actions/plangSettingsView/pLangsCrudActions'
import {askDialog} from '../../../helpers/dialogHelper'
import {SelectOnChangeEventData} from '../../../types/reactEvents'

import MaterialSelectWrapper from '../../material/materialSelectWrapper'
import {possibleEditorHighlightPLanguages} from '../../../helpers/editorHelper'
import {HelpPopup} from '../../helpers/helpPopup'
import {MyPopup} from '../../helpers/myPopup'

export interface MyProps {
  //readonly test: string
}

const mapStateToProps = (rootState: RootState /*, props: MyProps*/) => {
  return {

    isLoading: rootState.pLangsState.isLoading,
    pLangsWithExtraData: rootState.pLangsState.pLangsWithExtraData,
    pLangCopy: rootState.pLangsState.pLangCopy,
    editingPLangId: rootState.pLangsState.editingPLangId,

    systemSettings: rootState.systemSettingsSiteState.systemSettings,

    langId: rootState.i18nState.langId,
  }
}

const mapDispatchToProps = (dispatch: Dispatch) => bindActionCreators({
                                                                        //imported reducer funcs here
                                                                        setEditingPLangId,
                                                                        setPLangDisplayName,
                                                                        setPLangInternalName,
                                                                        setPLangFileExtensionsWithDot,
                                                                        setPLangEditorHighlightModeName,

                                                                        updatePLangAsync,
                                                                        createPLangAsync,
                                                                        deletePLangAsync,
                                                                      }, dispatch)


const stateProps = returntypeof(mapStateToProps);
const dispatchProps = returntypeof(mapDispatchToProps);
type Props = typeof stateProps & typeof dispatchProps;

/**
 * this sites works on the plangs state
 */
class PlangSettingsView extends React.Component<Props, any> {


  render(): JSX.Element {

    // tslint:disable-next-line
    let pLangsList = this.props.pLangsWithExtraData

    if (this.props.editingPLangId !== null && this.props.editingPLangId <= 0) {
      pLangsList = pLangsList.concat([this.props.pLangCopy])
    }

    return (
      <SinglePanelSiteWrapper>

        {
          this.props.isLoading &&
          <Spinner/>
        }
        <div className="view-padding">
          <div className="view-head-line">
            <div className="view-caption">
              <h1 className="clear-margin">
                {
                  getI18n(this.props.langId, "Programming languages")
                }
              </h1>

              <HelpPopup className="mar-left-half v-centered"
                         defaultText={'This setting is only for the frontend and backend to know the programming languages. You need to add new programming languages also to the test server in order to run tests!'}
                         wide/>

              <MyPopup
                trigger={
                  <a className="mar-left-half" href={this.props.systemSettings.testServerConfigUiUrl
                                                     ? this.props.systemSettings.testServerConfigUiUrl
                                                     : '#'} target="_blank">
                    <Icon name="external alternate"/>
                  </a>
                }
                content={getI18n(this.props.langId, "Click to open the test server configuration page")}
              />
            </div>
            <div className="view-options">

            </div>
          </div>

          <div className="view-content">

            <div className="ui form">

              <Form.Field>

                <Table selectable padded striped singleLine className="material-table">
                  <Table.Header>
                    <Table.Row>
                      <Table.HeaderCell>
                        <span>
                          {
                            getI18n(this.props.langId, 'Edit')
                          }
                        </span>
                      </Table.HeaderCell>
                      <Table.HeaderCell>
                        <span>
                          {
                            getI18n(this.props.langId, 'Name')
                          }
                        </span>
                        <HelpPopup wide className="mar-left-half"
                                   defaultText={getI18n(this.props.langId, "Just for display purposes")}/>
                      </Table.HeaderCell>
                      <Table.HeaderCell>
                        <span>
                          {
                            getI18n(this.props.langId, 'Internal name')
                          }
                        </span>
                        <HelpPopup wide className="mar-left-half" defaultText={getI18n(this.props.langId,
                                                                                       "The internal name is used for requests to the test server to identify the programming language. It must match the some language known to the test server."
                        )}/>
                      </Table.HeaderCell>
                      <Table.HeaderCell>
                        <span>
                          {
                            getI18n(this.props.langId, 'File extensions with dot')
                          }
                        </span>
                        <HelpPopup wide className="mar-left-half" defaultText={getI18n(this.props.langId,
                                                                                       "You need to add the file extensions for this programming language in order to get syntax highlighting in the editor. Values must be separated by , (comma)."
                        )}/>
                      </Table.HeaderCell>
                      <Table.HeaderCell>
                        <span>
                          {
                            getI18n(this.props.langId, 'Editor highlight')
                          }
                        </span>
                        <HelpPopup wide className="mar-left-half" defaultText={getI18n(this.props.langId,
                                                                                       "The language mode for the editor to use when the file extension matches."
                        )}/>
                      </Table.HeaderCell>
                      <Table.HeaderCell>
                        <span>
                          {
                            //TODO add some more help tooltips (when we can delete a plang?)
                            getI18n(this.props.langId, 'Delete')
                          }
                        </span>
                      </Table.HeaderCell>
                    </Table.Row>
                  </Table.Header>

                  <Table.Body>
                    {
                      pLangsList.map((pLang, index) => {

                        const isEditingThisPlang = pLang.id === this.props.editingPLangId

                        //when we edit an old plang...
                        if (isEditingThisPlang) {
                          pLang = this.props.pLangCopy
                        }
                        //a new plang is also stored in this.props.pLangCopy but is added at the end of the list

                        return (
                          <Table.Row key={pLang.id}>
                            <Table.Cell>
                              <div className="flexed">

                                {
                                  this.props.editingPLangId === null &&
                                  <div className="clickable" onClick={() => {
                                    this.props.setEditingPLangId(pLang.id)
                                  }}>
                                    <Icon name="write"/>
                                  </div>
                                }

                                {
                                  isEditingThisPlang &&
                                  <div className="clickable" onClick={async () => {

                                    //createPLangAsync or updatePLangsAsync should clear the edit mode

                                    if (pLang.id <= 0) {
                                      //create
                                      await this.props.createPLangAsync(this.props.pLangCopy)
                                    } else {
                                      //update
                                      await this.props.updatePLangAsync(pLang)
                                    }
                                  }}>
                                    <Icon name="save"/>
                                  </div>
                                }
                                {
                                  isEditingThisPlang &&
                                  <div className="clickable mar-left" onClick={() => {
                                    this.props.setEditingPLangId(null)
                                  }}>
                                    <Icon name="remove"/>
                                  </div>
                                }


                              </div>

                            </Table.Cell>
                            <Table.Cell>
                              {
                                isEditingThisPlang === false &&
                                <span>{pLang.displayName}</span>
                              }

                              {
                                isEditingThisPlang &&
                                <MaterialInput value={pLang.displayName} onChange={(e) => {
                                  this.props.setPLangDisplayName(e.currentTarget.value)
                                }}/>

                              }
                            </Table.Cell>

                            <Table.Cell>
                              {
                                isEditingThisPlang === false &&
                                <span>{pLang.internalName}</span>
                              }

                              {
                                isEditingThisPlang &&
                                <MaterialInput value={pLang.internalName} onChange={(e) => {
                                  this.props.setPLangInternalName(e.currentTarget.value)
                                }}/>

                              }
                            </Table.Cell>

                            <Table.Cell>
                              {
                                isEditingThisPlang === false &&
                                <span>{pLang.fileExtensionsWithDot}</span>
                              }

                              {
                                isEditingThisPlang &&
                                <MaterialInput value={pLang.fileExtensionsWithDot} onChange={(e) => {
                                  this.props.setPLangFileExtensionsWithDot(e.currentTarget.value)
                                }}/>
                              }
                            </Table.Cell>

                            <Table.Cell>
                              {
                                isEditingThisPlang === false &&
                                <span>{pLang.editorHighlightModeName === '' || pLang.editorHighlightModeName === null
                                       ? getI18n(this.props.langId, "None")
                                       : pLang.editorHighlightModeName}
                                </span>
                              }

                              {
                                isEditingThisPlang &&
                                <MaterialSelectWrapper
                                  value={pLang.editorHighlightModeName}
                                >
                                  <select
                                    value={pLang.editorHighlightModeName}
                                    onChange={(e: SelectOnChangeEventData) => {
                                      this.props.setPLangEditorHighlightModeName(e.currentTarget.value)
                                    }}
                                  >
                                    <option key="">{getI18n(this.props.langId, "None")}</option>
                                    {
                                      possibleEditorHighlightPLanguages.map(editorPLangMode => {
                                        return (
                                          <option key={editorPLangMode} value={editorPLangMode}>
                                            {
                                              editorPLangMode
                                            }
                                          </option>
                                        )
                                      })
                                    }
                                  </select>
                                </MaterialSelectWrapper>
                              }
                            </Table.Cell>


                            <Table.Cell>


                              {
                                pLang.canBeDeleted === false && pLang.cannotBeDeletedReason !== null &&
                                <HelpPopup defaultText={pLang.cannotBeDeletedReason}/>
                              }

                              {
                                pLang.canBeDeleted && pLang.cannotBeDeletedReason === null &&
                                <div className="clickable" onClick={async () => {

                                  if (!pLang.canBeDeleted) return

                                  const shouldDelete = await askDialog(getI18n(this.props.langId, "Delete"),
                                                                       `${getI18n(this.props.langId,
                                                                                  "Are you sure you want to delete this programming language?"
                                                                       )} (${pLang.displayName})`,
                                                                       this.props.langId
                                  )

                                  if (!shouldDelete) return

                                  await this.props.deletePLangAsync(pLang)

                                }}>
                                  <Icon name="trash"/>
                                </div>
                              }

                            </Table.Cell>

                          </Table.Row>
                        )
                      })
                    }

                    {
                      this.props.editingPLangId === null &&
                      <Table.Row>
                        <Table.Cell colSpan={10}>

                          <div>
                            <div className="clickable" onClick={() => {
                              this.props.setEditingPLangId(-1)
                            }}>
                              <Icon name="plus"/>
                            </div>
                          </div>

                        </Table.Cell>

                      </Table.Row>
                    }
                  </Table.Body>

                </Table>


              </Form.Field>


            </div>

          </div>
        </div>

      </SinglePanelSiteWrapper>
    )
  }
}

export default connect(mapStateToProps, mapDispatchToProps)(PlangSettingsView)
