/**
 * Created by janis dähne.
 */

import * as React from "react";
import {connect} from "react-redux";
import {bindActionCreators, Dispatch} from "redux";
import {returntypeof} from 'react-redux-typescript';
import {RootState} from "../../../../state/reducers";
import {Icon} from "semantic-ui-react";
import {RunTutorTestCommandForBackend} from "../../../../types/testsAndTestProtocol";
import {HelpPopup} from "../../../helpers/helpPopup";
import Logger from "../../../../helpers/logger";
import {isTestResultValid} from "../../../../helpers/testHelper";
import {runSubmitTestAsTutor} from "../../../../state/actions/tutorViewSite/tutorViewSiteCrudActions";


import {resetTutorSubmitTestResults} from "../../../../state/actions/tutorViewSite/tutorViewSiteActions";
import {SimpleVDivider} from "../../../helpers/simpleVDivider";
import {percentageNeededToPassedAutomaticAssessment} from "../../../../constants";
import {getI18n} from "../../../../../i18n/i18nRoot";
import {orderBy} from 'lodash-es'
import {wait} from '../../../../../debug'

//const css = require('./styles.styl');

export interface MyProps {
  //readonly test: string
}

const mapStateToProps = (rootState: RootState /*, props: MyProps*/) => {
  return {
    //test0: rootState...
    //test: props.test
    tests: rootState.tutorViewSiteState.submitTestsForTutor,
    releaseCode: rootState.tutorViewSiteState.exercise.exerciseRelease.releaseCode,
    solutionPLangId: rootState.tutorViewSiteState.tutorEditorSolution.pLangId,
    systemSettings: rootState.globalSystemSettingsState.systemSettings,
    solution: rootState.tutorViewSiteState.tutorEditorSolution,

    isSomeTestRunning: rootState.tutorViewSiteState.exercise.tests.some(p => p.uiIsRunning)
    || rootState.tutorViewSiteState.exercise.customTests.some(p => p.uiIsRunning)
    || rootState.tutorViewSiteState.normalTestsForTutor.some(p => p.uiIsRunning)
    || rootState.tutorViewSiteState.submitTestsForTutor.some(p => p.uiIsRunning),

    maxNumberOfTestsWithOneRequest: rootState.globalSystemSettingsState.systemSettings
    && rootState.globalSystemSettingsState.systemSettings.maxNumberOfTestsWithOneRequest >= 0
                                    ? rootState.globalSystemSettingsState.systemSettings.maxNumberOfTestsWithOneRequest : 1,
    langId: rootState.i18nState.langId
  }
}

const mapDispatchToProps = (dispatch: Dispatch) => bindActionCreators({
  //imported reducer funcs here
  runSubmitTestAsTutor,
  resetTutorSubmitTestResults,
}, dispatch)


const stateProps = returntypeof(mapStateToProps);
const dispatchProps = returntypeof(mapDispatchToProps);
type Props = typeof stateProps & typeof dispatchProps;

class SubmitTestsPanelHeaderBar extends React.Component<Props, any> {

  render(): JSX.Element {

    const maxSummedPoints = this.props.tests
      .map(p => p.maxPoints)
      .reduce(((previousValue, currentValue) => previousValue + currentValue), 0)

    const reachedSummedPoints = this.props.tests
      .map(p => p.result && p.result.passed ? p.maxPoints : 0)
      .reduce(((previousValue, currentValue) => previousValue + currentValue), 0)

    // noinspection TsLint
    const reachedPercentage = maxSummedPoints > 0
      ? reachedSummedPoints * 100 / maxSummedPoints
      : 100

    // noinspection TsLint
    const hasPassedSummed = reachedPercentage >= percentageNeededToPassedAutomaticAssessment

    return (
      <div className="flexed">

        <div className={`flexed mar-right-half ${hasPassedSummed ? 'ok-colored' : 'error-colored'}`}>
          <div>
            {
              reachedSummedPoints
            }
          </div>
          /
          <div className="mar-right-half">
            {
              maxSummedPoints
            }
          </div>
          (
          <div>
            {
              reachedPercentage.toFixed(2)
            }
            %
          </div>
          )
        </div>

        <HelpPopup defaultText={`${getI18n(this.props.langId,'you need at least (>= ')}${percentageNeededToPassedAutomaticAssessment}${getI18n(this.props.langId,') to pass the exercise. The weight (points) of the tests is displayed as the background of each test.')}`}
                   wide
        />

        <SimpleVDivider/>

        <div className={`mar-right-half ${(!this.props.isSomeTestRunning && this.props.tests.length > 0) ? 'clickable' : 'div-disabled' }`}
             onClick={async () => {

               //this works but the lines after this still refer to the old this.props.test...
               //the props are only refreshed in a re-render
               this.props.resetTutorSubmitTestResults()
               //awaiting here will force a re-render and update the props so
               //the props after the await will be updated
               await wait(0)

               const maxNumberOfTestsWithOneRequest = this.props.maxNumberOfTestsWithOneRequest

               Logger.debug("submit test panel header bar (tutor view)",
                 `running all test with ${maxNumberOfTestsWithOneRequest} test in parallel`)

               const orderedTests = orderBy(this.props.tests, (test) => test.displayIndex)

               const validTests = orderedTests.filter(p => isTestResultValid(p))

               //a template
               // noinspection TsLint
               let tempCommand: RunTutorTestCommandForBackend = {
                 releaseCode: this.props.releaseCode,
                 isSubmitTest: true,
                 testIds: [],
                 isNormalTest: true,
                 plangId: this.props.solutionPLangId,
                 mainFileId: this.props.solution.mainFileId,
                 solutionFiles: this.props.solution.solutionFiles
               }

               // noinspection TsLint
               for (let i = 0; i < orderedTests.length; i++) {
                 const test = orderedTests[i]

                 if (isTestResultValid(test) === false || validTests.length === orderedTests.length) {
                   //add
                   tempCommand = {
                     ...tempCommand,
                     testIds: [...tempCommand.testIds, test.id]
                   }
                 } else {
                   //test was already valid
                 }

                 //we reached the limit
                 if (tempCommand.testIds.length === maxNumberOfTestsWithOneRequest) {
                   const hasAnyTestFailed = await this.props.runSubmitTestAsTutor(tempCommand)

                   //clear test ids
                   tempCommand = {
                     ...tempCommand,
                     testIds: []
                   }

                   //stop if at least 1 test failed
                   if (hasAnyTestFailed) {
                     //return instead of break else we would run the last check below which would be always true
                     //do not stop here because the tutor wan't to see all results
                     //we need this because we currently don't save the tutor editor and result (which would clear the test results)
                     // return
                   }
                 }
               }

               //we might have some last tests but not reached the max num of tests with one request...
               if (tempCommand.testIds.length > 0) {
                 //we missed the last < maxNumberOfTestsWithOneRequest tests
                 //these are still in the last tempCommand
                 await this.props.runSubmitTestAsTutor(tempCommand)
               }

             }}
        >
          <Icon name="forward"/>
          <span>
            {
              getI18n(this.props.langId,'Run all tests')
            }
          </span>
        </div>

        <HelpPopup
          defaultText={`${getI18n(this.props.langId,'Runs all not passed tests. If all tests have passed then all tests are re-run. When at least one test fails then the next x tests are not executed.')} ${getI18n(this.props.langId,'Max parallel tests')}: ${this.props.maxNumberOfTestsWithOneRequest}`}
          wide
        />

        <SimpleVDivider/>

        <div className="clickable mar-right-half"
             onClick={() => {
               this.props.resetTutorSubmitTestResults()
             }}
        >
          <Icon name="remove" />
          <span>
          {
            getI18n(this.props.langId, 'Clear test results')
          }
          </span>
        </div>
        <HelpPopup
          defaultText={getI18n(this.props.langId,'This only discards the results in the frontend, the results on the server are still there')}
          wide
        />

      </div>
    )
  }
}

export default connect(mapStateToProps, mapDispatchToProps)(SubmitTestsPanelHeaderBar)
