/**
 * Created by janis dähne.
 */

import * as React from "react";
import {connect} from "react-redux";
import {bindActionCreators, Dispatch} from "redux";
import {returntypeof} from 'react-redux-typescript';
import {RootState} from "../../../../state/reducers";
import {Icon} from "semantic-ui-react";
import {SimpleVDivider} from "../../../helpers/simpleVDivider";
import {openEditEditorSettingsDialog} from "../../../../state/actions/editorSettingsDialog/editorSettingsDialogActions";
import {
  resetTutorEditorToUserSolution,
  setTutorViewIsCodeEditorSettingsDialogDisplayed
} from "../../../../state/actions/tutorViewSite/tutorViewSiteActions";
import {askDialog} from "../../../../helpers/dialogHelper";


import {getI18n} from "../../../../../i18n/i18nRoot";

//const css = require('./styles.styl');

export interface MyProps {
  //readonly test: string
}

const mapStateToProps = (rootState: RootState /*, props: MyProps*/) => {
  return {
    //test0: rootState...
    //test: props.test
    codeEditorSettings: rootState.userDataSettingsSate.userData.settings ?
      rootState.userDataSettingsSate.userData.settings.codeEditorSettings
      : null,

    userSolution: rootState.tutorViewSiteState.userSolution,
    userAfterSolution: rootState.tutorViewSiteState.userAfterSolution,
    langId: rootState.i18nState.langId
  }
}

const mapDispatchToProps = (dispatch: Dispatch) => bindActionCreators({
  //imported reducer funcs here
  setTutorViewIsCodeEditorSettingsDialogDisplayed,
  openEditEditorSettingsDialog,
  resetTutorEditorToUserSolution
}, dispatch)


const stateProps = returntypeof(mapStateToProps);
const dispatchProps = returntypeof(mapDispatchToProps);
type Props = typeof stateProps & typeof dispatchProps;

class editorPanelHeaderBar extends React.Component<Props, any> {
  render(): JSX.Element {
    return (
      <div className="flexed">

        <div className="clickable"
             onClick={() => {
               this.props.openEditEditorSettingsDialog(this.props.codeEditorSettings,
                 this.props.setTutorViewIsCodeEditorSettingsDialogDisplayed)
             }}
        >
          <Icon name="settings"/>
          <span>
            {
              getI18n(this.props.langId,'Editor settings')
            }
          </span>
        </div>

        <SimpleVDivider/>

        <div className="clickable"
             onClick={async () => {

               const shouldReset = await askDialog(getI18n(this.props.langId,'Reset code to submission'),
                 getI18n(this.props.langId,'Are your sure you want to reset the code to the submissions code?'), this.props.langId
               )

               if (shouldReset) {
                 this.props.resetTutorEditorToUserSolution(this.props.userSolution)
               }
             }}
        >
          <Icon name="undo"/>
          <span>
            {
              getI18n(this.props.langId,'Reset code to submission')
            }
          </span>
        </div>

        {
          this.props.userAfterSolution !== null &&
          <SimpleVDivider/>
        }
        {
          this.props.userAfterSolution !== null &&
          <div className="clickable"
               onClick={async () => {

                 const shouldReset = await askDialog(getI18n(this.props.langId, 'Reset code to subsequent editor'),
                                                     getI18n(this.props.langId,
                                                             'Are your sure you want to reset the code to the subsequent editor code?'
                                                     ), this.props.langId
                 )

                 if (shouldReset) {
                   this.props.resetTutorEditorToUserSolution(this.props.userAfterSolution)
                 }
               }}
          >
            <Icon name="undo"/>
            <span>
            {
              getI18n(this.props.langId, 'Reset code to subsequent editor')
            }
          </span>
          </div>
        }

      </div>
    )
  }
}

export default connect(mapStateToProps, mapDispatchToProps)(editorPanelHeaderBar)
