/**
 * Created by janis dähne.
 */

import * as React from "react";
import {connect} from "react-redux";
import {bindActionCreators, Dispatch} from "redux";
import {returntypeof} from 'react-redux-typescript';
import {RootState} from "../../../../state/reducers";
import MultiTabCodeEditorWrapper from "../../../codeEditors/multiTabCodeEditorWrapper";
import {SolutionFileDoExerciseFullBase} from "../../../../types/exerciseSolution";
import Logger from "../../../../helpers/logger";
import {
  closeTabChangeTutorEditor,
  openTabChangeTutorEditor,
  setHasUnsavedChangesTutorEditor,
  setIsFileExplorerDisplayedTutorEditor, setIsRenamingFileTutorEditor, setNewFileNameTutorEditor,
  setRenamingFileIdTutorEditor, setRenamingFileNameTutorEditor,
  setSelectedTabIdTutorEditor
} from "../../../../state/actions/tutorViewSite/editorActions/tutorEditorActions";
import {setSolutionFilesTutorEditor} from "../../../../state/actions/tutorViewSite/tutorViewSiteActions";
import {errorDialog} from "../../../../helpers/dialogHelper";


import {EditorTab} from "../../../codeEditors/multiTabCodeEditorProps";
import {getTabModeFromExtension, setSolutionFiles_AndRefreshEditor} from "../../../../helpers/editorHelper";
import {getI18n} from "../../../../../i18n/i18nRoot";
import {multiFileEditorDebounceInMs, tutorViewEditorPanelViewEditorId} from '../../../../constants'

//const css = require('./styles.styl');

export interface MyProps {
  //readonly test: string
}

const mapStateToProps = (rootState: RootState /*, props: MyProps*/) => {
  return {
    //test0: rootState...
    //test: props.test

    solution: rootState.tutorViewSiteState.tutorEditorSolution,

    pLangs: rootState.pLangsState.pLangs,

    selectedTabIdDoEditor: rootState.tutorViewSiteState.tutorEditorState.selectedTabIdDoEditor,
    isFileExplorerDisplayedDoEditor: rootState.tutorViewSiteState.tutorEditorState.isFileExplorerDisplayedDoEditor,
    newFileNameDoEditor: rootState.tutorViewSiteState.tutorEditorState.newFileNameDoEditor,
    renamingFileIdDoEditor: rootState.tutorViewSiteState.tutorEditorState.renamingFileIdDoEditor,
    isRenamingFileDoEditor: rootState.tutorViewSiteState.tutorEditorState.isRenamingFileDoEditor,
    renamingFileNameDoEditor: rootState.tutorViewSiteState.tutorEditorState.renamingFileNameDoEditor,
    hasUnsavedChanges: rootState.tutorViewSiteState.tutorEditorState.hasUnsavedChanges,
    hasFileChangedMap: rootState.tutorViewSiteState.tutorEditorState.hasFileChangedMap,

    //other tests should not be runnable by the ui! from isSomeTestRunning
    canUserEditExercise: !(rootState.tutorViewSiteState.normalTestsForTutor.some(p => p.uiIsRunning)
      || rootState.tutorViewSiteState.submitTestsForTutor.some(p => p.uiIsRunning))
    ,

    langId: rootState.i18nState.langId
  }
}

const mapDispatchToProps = (dispatch: Dispatch) => bindActionCreators({
                                                                        //imported reducer funcs here

                                                                        setSolutionFilesTutorEditor,
                                                                        setSelectedTabIdTutorEditor,
                                                                        setIsFileExplorerDisplayedTutorEditor,
                                                                        setNewFileNameTutorEditor,
                                                                        setRenamingFileIdTutorEditor,
                                                                        setIsRenamingFileTutorEditor,
                                                                        setRenamingFileNameTutorEditor,
                                                                        setHasUnsavedChangesTutorEditor,
                                                                        openTabChangeTutorEditor,
                                                                        closeTabChangeTutorEditor,

                                                                        setSolutionFiles_AndRefreshEditor,
                                                                      }, dispatch)


const stateProps = returntypeof(mapStateToProps);
const dispatchProps = returntypeof(mapDispatchToProps);
type Props = typeof stateProps & typeof dispatchProps;

class editorPanelView extends React.Component<Props, any> {


  render(): JSX.Element {


    const pLang = this.props.pLangs.find(p => p.id === this.props.solution.pLangId)

    if (!pLang) {
      //TODO only error if really not found
      //on first render we have no plang but thats no and error!!

      return null
    }

    const editorTabs = this.props.solution.solutionFiles.map<EditorTab>(p => {
      return {
        id: p.id,
        fileNameWithExtensions: p.fileNameWithExtension,
        content: p.content,
        isMainFile: p.id === this.props.solution.mainFileId,
        isReadonly: !this.props.canUserEditExercise || !p.isEditableByUser,
        displayIndex: p.displayIndex,
        language: getTabModeFromExtension(p.fileNameWithExtension, this.props.pLangs),
        isContentVisibleForUser: p.isContentVisibleForUser
      }
    })
      .sort((a, b) => a.fileNameWithExtensions.localeCompare(b.fileNameWithExtensions))

    return (
      <div className="fh hidden-overflow-i">
        <MultiTabCodeEditorWrapper
          id={tutorViewEditorPanelViewEditorId}
          height="100%"
          debounceTimeInMs={multiFileEditorDebounceInMs}
          selectedTabId={this.props.selectedTabIdDoEditor}
          canPrint={true}
          onSelectedTabIdChanged={(tabId: number) => {
            this.props.setSelectedTabIdTutorEditor(tabId)
          }}

          isFileExplorerDisplayed={this.props.isFileExplorerDisplayedDoEditor}
          onChangeIsFileExplorerDisplayed={(isDisplayed: boolean) => {
            this.props.setIsFileExplorerDisplayedTutorEditor(isDisplayed)
          }}

          newFileName={this.props.newFileNameDoEditor}
          onNewFileNameChanged={(name: string) => {
            this.props.setNewFileNameTutorEditor(name)
          }}

          renamingFileId={this.props.renamingFileIdDoEditor}
          onRenamingFileIdChanged={(fileId: number) => {
            this.props.setRenamingFileIdTutorEditor(fileId)
          }}

          isRenamingFile={this.props.isRenamingFileDoEditor}
          onIsRenamingFileChanged={(isRenamingFile: boolean) => {
            this.props.setIsRenamingFileTutorEditor(isRenamingFile)
          }}

          renamingFileName={this.props.renamingFileNameDoEditor}
          onRenamingFileNameChanged={(newName: string) => {
            this.props.setRenamingFileNameTutorEditor(newName)
          }}

          hasUnsavedChanges={this.props.hasUnsavedChanges}
          onHasUnsavedChangesChanged={(hasChanged: boolean) => {
            this.props.setHasUnsavedChangesTutorEditor(hasChanged)
          }}

          userCanCreateNewFiles={true}

          onCreateFileClicked={(editorTab: EditorTab) => {

            const newFile: SolutionFileDoExerciseFullBase = {
              id: editorTab.id,
              content: editorTab.content,
              fileNameWithExtension: editorTab.fileNameWithExtensions,
              uiIsDisplayed: editorTab.displayIndex >= 0,
              displayIndex: editorTab.displayIndex,
              isEditableByUser: true,
              templateFileId: null,
              isContentVisibleForUser: true,
            }

            // this.props.setSolutionFilesTutorEditor(
            //   [...this.props.solution.solutionFiles, newFile]
            // )

            this.props.setSolutionFiles_AndRefreshEditor(
              [...this.props.solution.solutionFiles, newFile],
              this.props.setSolutionFilesTutorEditor,
            )

          }}

          onOpenTabClicked={(editorTab: EditorTab) => {
            const file = this.props.solution.solutionFiles.find(p => p.id === editorTab.id)

            if (!file) {
              Logger.error("file not found (onOpenTabClicked)")

              errorDialog(getI18n(this.props.langId, 'Error'),
                          getI18n(this.props.langId, 'Could not find editor file'), this.props.langId
              )

              return
            }

            const mappedFiles = this.props.solution.solutionFiles.map<SolutionFileDoExerciseFullBase>(p => {
              return p.id === file.id
                     ? {
                  ...file,
                  uiIsDisplayed: true,
                }
                     : p
            })

            this.props.openTabChangeTutorEditor(file, mappedFiles, this.props.selectedTabIdDoEditor)
          }}

          onCloseTabClicked={(editorTab: EditorTab) => {

            const file = this.props.solution.solutionFiles.find(p => p.id === editorTab.id)

            if (!file) {
              Logger.error("file not found (onCloseTabClicked)")

              errorDialog(getI18n(this.props.langId, 'Error'),
                          getI18n(this.props.langId, 'Could not find editor file'), this.props.langId
              )

              return
            }

            const mappedFiles = this.props.solution.solutionFiles.map<SolutionFileDoExerciseFullBase>(p => {
              return p.id === file.id
                     ? {
                  ...file,
                  uiIsDisplayed: false,
                }
                     : p
            })

            this.props.closeTabChangeTutorEditor(file, mappedFiles,
                                                 this.props.selectedTabIdDoEditor
            )
          }}

          onFileRename={(editorTab: EditorTab, newName: string) => {

            const currentFile = this.props.solution.solutionFiles.find(p => p.id === editorTab.id)

            if (!currentFile) {

              Logger.error("file not found (onFileRename)")

              errorDialog(getI18n(this.props.langId, 'Error'),
                          getI18n(this.props.langId, 'Could not find editor file'), this.props.langId
              )

              return
            }

            this.props.setSolutionFilesTutorEditor(
              this.props.solution.solutionFiles.map<SolutionFileDoExerciseFullBase>(file => {
                return file.id === currentFile.id
                       ? {
                    id: file.id,
                    displayIndex: file.displayIndex,
                    fileNameWithExtension: newName,
                    isEditableByUser: file.isEditableByUser,
                    content: file.content,
                    templateFileId: file.templateFileId,
                    uiIsDisplayed: file.uiIsDisplayed,
                    isContentVisibleForUser: file.isContentVisibleForUser,
                  }
                       : file
              })
            )

          }}

          onRemoveFile={(editorTab: EditorTab) => {

            this.props.setSolutionFiles_AndRefreshEditor(
              this.props.solution.solutionFiles.filter(p => p.id !== editorTab.id),
              this.props.setSolutionFilesTutorEditor,
            )

            // this.props.setSolutionFilesTutorEditor(
            //   this.props.solution.solutionFiles.filter(p => p.id !== editorTab.id)
            // )
          }}

          editorTabs={editorTabs}
          onBlur={(value, editorTab) => {
            const currentFile = this.props.solution.solutionFiles.find(p => p.id === editorTab.id)

            if (!currentFile) {

              Logger.error("file not found (onChange)")

              errorDialog(getI18n(this.props.langId, 'Error'),
                          getI18n(this.props.langId, 'Could not find editor file'), this.props.langId
              )

              return
            }

            this.props.setSolutionFilesTutorEditor(
              this.props.solution.solutionFiles.map<SolutionFileDoExerciseFullBase>(file => {
                return file.id === currentFile.id
                       ? {
                    id: file.id,
                    displayIndex: file.displayIndex,
                    fileNameWithExtension: file.fileNameWithExtension,
                    isEditableByUser: file.isEditableByUser,
                    content: value,
                    templateFileId: file.templateFileId,
                    uiIsDisplayed: file.uiIsDisplayed,
                    isContentVisibleForUser: file.isContentVisibleForUser,
                  }
                       : file
              })
            )
          }}
          // onChange={(editorTab: EditorTab, newValue: string) => {
          //
          //   const currentFile = this.props.solution.solutionFiles.find(p => p.id === editorTab.id)
          //
          //   if (!currentFile) {
          //
          //     Logger.error("file not found (onChange)")
          //
          //     errorDialog(getI18n(this.props.langId,'Error'),
          //       getI18n(this.props.langId,'Could not find editor file')
          //     )
          //
          //     return
          //   }
          //
          //   this.props.setSolutionFilesTutorEditor(
          //     this.props.solution.solutionFiles.map<SolutionFileDoExerciseFullBase>(file => {
          //       return file.id === currentFile.id
          //         ? {
          //           id: file.id,
          //           displayIndex: file.displayIndex,
          //           fileNameWithExtension: file.fileNameWithExtension,
          //           isEditableByUser: file.isEditableByUser,
          //           content: newValue,
          //           templateFileId: file.templateFileId,
          //           uiIsDisplayed: file.uiIsDisplayed,
          //           uiEditSession: editorTab.editSession
          //         }
          //         : file
          //     })
          //   )
          //
          // }}

          // onSomeEditorSessionChanged={(editorTabs: ReadonlyArray<EditorTab>) => {
          //   this.props.setSolutionFilesTutorEditor(
          //     this.props.solution.solutionFiles.map<SolutionFileDoExerciseFullBase>(file => {
          //       const changedTab = editorTabs.find(p => p.id === file.id)
          //       return changedTab
          //         ? {
          //           ...file,
          //           uiEditSession: changedTab.editSession
          //         }
          //         : file
          //     })
          //   )
          // }}

          hasFileChangedMap={this.props.hasFileChangedMap}
          onUpdateHasFileChangedMap={(changedFile: EditorTab, allTabs: ReadonlyArray<EditorTab>) => {
            //performance is tooooo bad...
            //this.props.setHasFileChangedMapDoEditor(updateHasFileChangedMap(changedFile, allTabs, this.props.hasFileChangedMap))
          }}

        />
      </div>
    )
  }
}

export default connect(mapStateToProps, mapDispatchToProps)(editorPanelView)
