/**
 * Created by janis dähne.
 */

import * as React from "react";
import {connect} from "react-redux";
import {bindActionCreators, Dispatch} from "redux";
import {returntypeof} from 'react-redux-typescript';
import {RootState} from "../../../../state/reducers";
import MultiTabCodeEditorWrapper from "../../../codeEditors/multiTabCodeEditorWrapper";
import {SolutionFileDoExerciseFullBase} from "../../../../types/exerciseSolution";
import Logger from "../../../../helpers/logger";
import {errorDialog} from "../../../../helpers/dialogHelper";
import {EditorTab} from "../../../codeEditors/multiTabCodeEditorProps";
import {getI18n} from "../../../../../i18n/i18nRoot";
import {tutorViewUserAfterSolutionEditorPanelViewEditorId} from '../../../../constants'
import {
  closeTabChangeAfterEditorTutorView,
  openTabChangeAfterEditorTutorView,
  setIsFileExplorerDisplayedAfterEditorTutorView,
  setSelectedTabIdAfterEditorTutorView
} from '../../../../state/actions/tutorViewSite/editorActions/userAfterSolutionEditorTutorActions'
import {getTabModeFromExtension} from '../../../../helpers/editorHelper'

//const css = require('./styles.styl');

export interface MyProps {
  //readonly test: string
}

const mapStateToProps = (rootState: RootState /*, props: MyProps*/) => {
  return {
    //test0: rootState...
    //test: props.test

    pLangs: rootState.pLangsState.pLangs,
    solution: rootState.tutorViewSiteState.userAfterSolution,

    selectedTabIdDoEditor: rootState.tutorViewSiteState.userAfterSolutionEditorState.selectedTabIdDoEditor,
    isFileExplorerDisplayedDoEditor: rootState.tutorViewSiteState.userAfterSolutionEditorState.isFileExplorerDisplayedDoEditor,
    newFileNameDoEditor: rootState.tutorViewSiteState.userAfterSolutionEditorState.newFileNameDoEditor,
    renamingFileIdDoEditor: rootState.tutorViewSiteState.userAfterSolutionEditorState.renamingFileIdDoEditor,
    isRenamingFileDoEditor: rootState.tutorViewSiteState.userAfterSolutionEditorState.isRenamingFileDoEditor,
    renamingFileNameDoEditor: rootState.tutorViewSiteState.userAfterSolutionEditorState.renamingFileNameDoEditor,
    hasUnsavedChanges: rootState.tutorViewSiteState.userAfterSolutionEditorState.hasUnsavedChanges,
    hasFileChangedMap: rootState.tutorViewSiteState.userAfterSolutionEditorState.hasFileChangedMap,
    langId: rootState.i18nState.langId
  }
}

const mapDispatchToProps = (dispatch: Dispatch) => bindActionCreators({
                                                                        //imported reducer funcs here
                                                                        setIsFileExplorerDisplayedAfterEditorTutorView,
                                                                        setSelectedTabIdAfterEditorTutorView,

                                                                        openTabChangeAfterEditorTutorView,
                                                                        closeTabChangeAfterEditorTutorView,
                                                                      }, dispatch)


const stateProps = returntypeof(mapStateToProps);
const dispatchProps = returntypeof(mapDispatchToProps);
type Props = typeof stateProps & typeof dispatchProps;

class editorPanelView extends React.Component<Props, any> {


  componentWillMount(): void {

  }

  render(): JSX.Element {

    if (this.props.solution === null) return null

    const pLang = this.props.pLangs.find(p => p.id === this.props.solution.pLangId)

    if (!pLang) {
      //TODO only error if really not found
      //on first render we have no plang but thats no and error!!
      return null
    }

    const editorTabs = this.props.solution.solutionFiles.map<EditorTab>(p => {
      return {
        id: p.id,
        fileNameWithExtensions: p.fileNameWithExtension,
        content: p.content,
        isMainFile: p.id === this.props.solution.mainFileId,
        isReadonly: true,
        displayIndex: p.displayIndex,
        language: getTabModeFromExtension(p.fileNameWithExtension, this.props.pLangs),
        isContentVisibleForUser: p.isContentVisibleForUser
      }
    })
      .sort((a, b) => a.fileNameWithExtensions.localeCompare(b.fileNameWithExtensions))

    return (<div className="fh hidden-overflow-i">
      <MultiTabCodeEditorWrapper
        id={tutorViewUserAfterSolutionEditorPanelViewEditorId}
        height="100%"
        debounceTimeInMs={undefined}
        selectedTabId={this.props.selectedTabIdDoEditor}
        canPrint={true}
        onSelectedTabIdChanged={(tabId: number) => {
          this.props.setSelectedTabIdAfterEditorTutorView(tabId)
        }}

        isFileExplorerDisplayed={this.props.isFileExplorerDisplayedDoEditor}
        onChangeIsFileExplorerDisplayed={(isDisplayed: boolean) => {
          this.props.setIsFileExplorerDisplayedAfterEditorTutorView(isDisplayed)
        }}

        newFileName={this.props.newFileNameDoEditor}
        onNewFileNameChanged={null}

        renamingFileId={this.props.renamingFileIdDoEditor}
        onRenamingFileIdChanged={null}

        isRenamingFile={this.props.isRenamingFileDoEditor}
        onIsRenamingFileChanged={null}

        renamingFileName={this.props.renamingFileNameDoEditor}
        onRenamingFileNameChanged={null}

        hasUnsavedChanges={false}
        onHasUnsavedChangesChanged={null}

        userCanCreateNewFiles={false}

        onCreateFileClicked={null}

        onOpenTabClicked={(editorTab: EditorTab) => {
          const file = this.props.solution.solutionFiles.find(p => p.id === editorTab.id)

          if (!file) {

            errorDialog(getI18n(this.props.langId, 'Error'), getI18n(this.props.langId, 'Could not find editor file'),this.props.langId)
            Logger.error("file not found (onOpenTabClicked)")

            return
          }

          const mappedFiles = this.props.solution.solutionFiles.map<SolutionFileDoExerciseFullBase>(p => {
            return p.id === file.id
                   ? {
                ...file,
                uiIsDisplayed: true,
              }
                   : p
          })

          this.props.openTabChangeAfterEditorTutorView(file, mappedFiles, this.props.selectedTabIdDoEditor)
        }}

        onCloseTabClicked={(editorTab: EditorTab) => {

          const file = this.props.solution.solutionFiles.find(p => p.id === editorTab.id)

          if (!file) {

            errorDialog(getI18n(this.props.langId, 'Error'), getI18n(this.props.langId, 'Could not find editor file'),this.props.langId)
            Logger.error("file not found (onCloseTabClicked)")

            return
          }

          const mappedFiles = this.props.solution.solutionFiles.map<SolutionFileDoExerciseFullBase>(p => {
            return p.id === file.id
                   ? {
                ...file,
                uiIsDisplayed: false,
              }
                   : p
          })

          this.props.closeTabChangeAfterEditorTutorView(file, mappedFiles, this.props.selectedTabIdDoEditor)
        }}

        onFileRename={null}

        onRemoveFile={null}

        editorTabs={editorTabs}
        onBlur={null}

        hasFileChangedMap={this.props.hasFileChangedMap}
        onUpdateHasFileChangedMap={(changedFile: EditorTab, allTabs: ReadonlyArray<EditorTab>) => {
        }}

      />
    </div>)
  }
}

export default connect(mapStateToProps, mapDispatchToProps)(editorPanelView)
