/**
 * Created by janis dähne.
 */

import * as React from "react";
import {connect} from "react-redux";
import {bindActionCreators, Dispatch} from "redux";
import {returntypeof} from 'react-redux-typescript';
import {RootState} from "../../../../state/reducers";
import {Icon} from "semantic-ui-react";
import {HelpPopup} from "../../../helpers/helpPopup";
import {SimpleVDivider} from "../../../helpers/simpleVDivider";
import {
  resetDoExerciseSolutionToTemplate,
  setIsDoCodeEditorSettingsDialogDisplayed
} from "../../../../state/actions/doExerciseSite/doExerciseSiteActions";
import {openEditEditorSettingsDialog} from "../../../../state/actions/editorSettingsDialog/editorSettingsDialogActions";
import {setTutorViewIsCodeEditorSettingsDialogDisplayed} from "../../../../state/actions/tutorViewSite/tutorViewSiteActions";


import {getI18n} from "../../../../../i18n/i18nRoot";
import {MyPopup} from '../../../helpers/myPopup'
import {displayDateAndTime} from '../../../../helpers/convertersAndTransformers'

//const css = require('./styles.styl');

export interface MyProps {
  //readonly test: string
}

const mapStateToProps = (rootState: RootState /*, props: MyProps*/) => {
  return {
    //test0: rootState...
    //test: props.test
    codeEditorSettings: rootState.userDataSettingsSate.userData.settings ?
      rootState.userDataSettingsSate.userData.settings.codeEditorSettings
      : null,
    userSolution: rootState.tutorViewSiteState.userSolution,
    langId: rootState.i18nState.langId
  }
}

const mapDispatchToProps = (dispatch: Dispatch) => bindActionCreators({
  //imported reducer funcs here
  setTutorViewIsCodeEditorSettingsDialogDisplayed,
  openEditEditorSettingsDialog,
}, dispatch)


const stateProps = returntypeof(mapStateToProps);
const dispatchProps = returntypeof(mapDispatchToProps);
type Props = typeof stateProps & typeof dispatchProps;

class editorPanelHeaderBar extends React.Component<Props, any> {
  render(): JSX.Element {
    return (
      <div className="flexed">

        <div className="clickable"
             onClick={() => {
               this.props.openEditEditorSettingsDialog(this.props.codeEditorSettings,
                 this.props.setTutorViewIsCodeEditorSettingsDialogDisplayed)
             }}
        >
          <Icon name="settings"/>
          <span>
            {
              getI18n(this.props.langId,'Editor settings')
            }
          </span>
        </div>

        <SimpleVDivider/>

        <MyPopup
          trigger={
            <div className="hoverable">

              <Icon className="mar-right-half" name="database"/>
              <span className="mar-right-half">
              {
                getI18n(this.props.langId,'Server version:')
              }
              </span>
              <span>
              {
                displayDateAndTime(this.props.userSolution.lastUpdatedAt, -1)
              }
              </span>
            </div>
          }
          content={getI18n(this.props.langId,'The time when the latest solution was saved/created on the server.')}
        />


      </div>
    )
  }
}

export default connect(mapStateToProps, mapDispatchToProps)(editorPanelHeaderBar)