/**
 * Created by janis dähne.
 */

import * as React from "react";
import {connect} from "react-redux";
import {bindActionCreators, Dispatch} from "redux";
import {returntypeof} from 'react-redux-typescript';
import {RootState} from "../../state/reducers";
import {Label, Icon, Popup, SemanticICONS} from "semantic-ui-react";
import {TagFullBase} from "../../types/tag";
import {HTMLProps} from "react";
import {MyPopup} from "../helpers/myPopup";

//const css = require('./styles.styl');

export interface MyProps {
  //readonly test: string
  readonly tag: TagFullBase

  readonly leftIconName?: string
  readonly onLeftIconClicked?: () => void

  readonly rightIconName?: string
  readonly onRightIconClicked?: () => void

  readonly className?: string

  readonly onClick?: (tag: TagFullBase) => void

}

const mapStateToProps = (rootState: RootState, props: MyProps) => {
  return {
    //test0: rootState...
    //test: props.test
    tag: props.tag,
    leftIconName: props.leftIconName,
    onLeftIconClicked: props.onLeftIconClicked,
    rightIconName: props.rightIconName,
    onRightIconClicked: props.onRightIconClicked,
    className: props.className,
    onClick: props.onClick
  }
}

const mapDispatchToProps = (dispatch: Dispatch) => bindActionCreators({
  //imported reducer funcs here

}, dispatch)


const stateProps = returntypeof(mapStateToProps);
const dispatchProps = returntypeof(mapDispatchToProps);
type Props = typeof stateProps & typeof dispatchProps;

/**
 *
 *
 * usage: (use a wrapping div element to apply classes, events...)
 * <div key={tag.id} onClick={() => {
 *   //
 *  }}
 * >
 *  <Tag tag={tag}/>
 * </div>
 * *
 */
class Tag extends React.Component<Props, any> {

  //this should not create a new function on every render
  onClick = () => {
    if (this.props.onClick) {
      this.props.onClick(this.props.tag)
    }
  }

  render(): JSX.Element {

    const label = (<Label className={`exercise-tag-label tags-wrapper ${this.props.className}`}
                          style={{
                            color: this.props.tag.htmlColor,
                            backgroundColor: this.props.tag.htmlBackgroundColor,
                            maxWidth: '20em',
                          }}
                          onClick={this.onClick}
    >

      {this.props.leftIconName && <Icon name={this.props.leftIconName as SemanticICONS} className="clickable"
                                        onClick={() => this.props.onLeftIconClicked()}
      />}

      {this.props.tag.htmlIcon && <Icon name={this.props.tag.htmlIcon as SemanticICONS}/>}

      <span className={(this.props.rightIconName
                        ? 'mar-right'
                        : null)}>
              {this.props.tag.displayName}
              </span>

      {this.props.rightIconName && <Icon name={this.props.rightIconName as SemanticICONS} className="mar-null clickable"
                                         onClick={() => this.props.onRightIconClicked()}
      />}

    </Label>)

    if (this.props.tag.description === null || this.props.tag.description.trim() === '') {
      return (label)
    }

    return (<MyPopup
      trigger={label}
      content={this.props.tag.description}
      position="top center"
    />)
  }
}

export default connect(mapStateToProps, mapDispatchToProps)(Tag)