/**
 * Created by janis dähne.
 */

import * as React from "react";
import {connect} from "react-redux";
import {bindActionCreators, Dispatch} from "redux";
import {returntypeof} from 'react-redux-typescript';
import {RootState} from "../../state/reducers";
import {TagFullBase} from "../../types/tag";
import SinglePanelSiteWrapper from '../singlePanelSiteWrapper'
import Tag from './tag'
import Spinner from "../helpers/spinner";
import {
  setTagFilterSelectedNegativeFilterTagsIds, setTagFilterSelectedTagIds, setTagFilterTagsConnector
} from "../../state/actions/tagsFilterActions";
import {HelpPopup} from "../helpers/helpPopup";
import {getI18n} from "../../../i18n/i18nRoot";
import {debounce} from 'lodash-es'
import {searchInputDebounceInMs} from '../../constants'
import {Icon} from 'semantic-ui-react'

//const css = require('./styles.styl');

export interface MyProps {
  //readonly test: string
  readonly allTags: ReadonlyArray<TagFullBase>
  readonly isLoading: boolean

  /**
   * e.g. recalculate pagination
   * is already debounced
   */
  readonly onTagsUpdated: () => void

}

const mapStateToProps = (rootState: RootState, props: MyProps) => {
  return {
    //test0: rootState...
    //test: props.test
    ...props,
    selectedTags: rootState.tagsFilterState.selectedFilterTagsIds,
    selectedNegativeFilterTagsIds: rootState.tagsFilterState.selectedNegativeFilterTagsIds,
    connector: rootState.tagsFilterState.logicalConnector,
    langId: rootState.i18nState.langId
  }
}

const mapDispatchToProps = (dispatch: Dispatch) => bindActionCreators({
                                                                        //imported reducer funcs here
                                                                        setTagFilterSelectedTagIds,
                                                                        setTagFilterSelectedNegativeFilterTagsIds,
                                                                        setTagFilterTagsConnector,
                                                                      }, dispatch)


const stateProps = returntypeof(mapStateToProps);
const dispatchProps = returntypeof(mapDispatchToProps);
type Props = typeof stateProps & typeof dispatchProps;


//one can use sortAndFilterExercisePreviews to apply sorting & tags filter
//also for all tags the manage tags site is useful
class tagsFilterPanel extends React.Component<Props, any> {


  onChange = debounce(() => {
    this.props.onTagsUpdated()
  }, searchInputDebounceInMs)

  render(): JSX.Element {

    const tagPair = this.props.allTags.map(p => {
      return {
        tag: p,
        isSelected: this.props.selectedTags.some(id => id === p.id),
        isSelectedNegative: this.props.selectedNegativeFilterTagsIds.some(id => id === p.id)
      }
    })

    return (
      <SinglePanelSiteWrapper outerClassName="tag-filter-sticky">


        <div className="view-head-line">

          <div className="view-caption">
            <h2 className="mar-right-half">
              {
                getI18n(this.props.langId, 'Tags filter')
              }
            </h2>
            <div className={`mar-right-half tab-filter-option ${this.props.connector === 'and' ? 'toggled-colored' : 'clickable'}`} onClick={() => {
              if (this.props.connector === 'and') return
              this.props.setTagFilterTagsConnector('and')
              this.onChange()
            }}>
              <span>
                AND
              </span>
            </div>
            <div className={`tab-filter-option ${this.props.connector === 'or' ? 'toggled-colored' : 'clickable'}`}  onClick={() => {
              if (this.props.connector === 'or') return
              this.props.setTagFilterTagsConnector('or')
              this.onChange()
            }}>
              <span>
                OR
              </span>
            </div>
            <HelpPopup className="mar-left-half" style={{marginTop: '0.6em'}} defaultText={getI18n(this.props.langId,
                                                                         'The selected tags are connected via the selected logical operation. E.g. if you select AND and two tags then both tags needs to be assigned to the exercise. If you click a tag twice then the exercise must not have the tag assigned.'
            )} wide/>
          </div>
          <div className="view-options">
            <div className="clickable" onClick={() => {
              this.props.setTagFilterSelectedTagIds([])
              this.props.setTagFilterSelectedNegativeFilterTagsIds([])
              //this.props.setTagFilterTagsConnector('and')
              this.onChange()
            }}>
              <Icon name="x"/>
            </div>
          </div>
        </div>


        {
          this.props.isLoading &&
          <Spinner/>
        }

        <div className="flexed separate-tabs" style={{
          maxHeight: '200px',
          overflow: 'auto'
        }}>
          {
            this.props.allTags.map(tag => {
              return (
                <div key={tag.id}>
                  <Tag tag={tag}
                       className={
                         tagPair.find(p => p.tag.id === tag.id).isSelected
                         ? 'selected-tag-filter clickable'
                         : tagPair.find(p => p.tag.id === tag.id).isSelectedNegative
                           ? 'clickable selected-negative-tag-filter'
                           : 'clickable selectable-tab-filter'
                       }
                       onClick={(tag: TagFullBase) => {

                         if (this.props.selectedTags.some(id => id === tag.id)) {
                           //after select -->  un select positive, negative select

                           this.props.setTagFilterSelectedTagIds(this.props.selectedTags.filter(id => id !== tag.id))
                           this.props.setTagFilterSelectedNegativeFilterTagsIds(
                             [...this.props.selectedNegativeFilterTagsIds, tag.id])

                         } else {

                           //not selected as positive

                           if (this.props.selectedNegativeFilterTagsIds.some(id => id === tag.id)) {

                             //after negative selected --> un select negative
                             this.props.setTagFilterSelectedNegativeFilterTagsIds(
                               this.props.selectedNegativeFilterTagsIds.filter(id => id !== tag.id))

                           } else {
                             //not yet selected at all --> select
                             this.props.setTagFilterSelectedTagIds([...this.props.selectedTags, tag.id])
                           }
                         }

                         this.onChange()
                       }}
                  />
                </div>
              )
            })
          }
        </div>


      </SinglePanelSiteWrapper>
    )
  }
}

export default connect(mapStateToProps, mapDispatchToProps)(tagsFilterPanel)
