import {browser_isSafari, tempPrintDivId} from '../constants'
import Logger from './logger'


//--- TODO
//add .js-printing to body and in behaviors.styl use

/*
body.js-printing {

  //all elements we need e.g.

  .site-content {
    display none !important
  }

}
 */
//this is easier to change when add new site elements that needs to be hidden

export class PrintHelper {
  private constructor() {}

  /**
   * this uses the hidden @see tempPrintDivId (id) div and places the inner html inside
   * then it hides the site content div because else firefox will display he site twice in the print preview
   * after printing the site is displayed again and the @see tempPrintDivId (id) div inner html is cleared
   * and it gets hidden again
   * @param printInnerHtml the html content to print
   * @param onContentInserted can be used to e.g. do syntax highlighting when printing code or set font size for
   *  @see tempPrintDivId manually
   * @param onFinishedPrinting tempPrintDivId is cleared automatically for you
   */
  static printContent(printInnerHtml: string, onContentInserted?: () => void, onFinishedPrinting?: () => void): void {


    const siteContents = document.getElementsByClassName('site-content')

    if (siteContents.length === 0) {
      //needed on ff else we print the content twice??
      Logger.warn('could not find site content')
      return
    }

    const siteContent = siteContents[0] as HTMLElement

    const siteHeaderBars = document.getElementsByClassName('site-header-bar')

    if (siteHeaderBars.length === 0) {
      Logger.warn('could not find site header bar')
      return
    }

    const siteHeaderBar = siteHeaderBars[0]

    const tempPrintDiv = document.getElementById(tempPrintDivId)

    if (!tempPrintDiv) {
      Logger.warn('could not find temp print div')
      return
    }

    tempPrintDiv.innerHTML = printInnerHtml

    if (onContentInserted) onContentInserted()


    //or use window focus event like in https://github.com/ajaxorg/ace/issues/1480#issuecomment-20201314 ??
    if (browser_isSafari()) {

      //https://www.tjvantoll.com/2012/06/15/detecting-print-requests-with-javascript/
      if (window.matchMedia) {
        const mediaQueryList = window.matchMedia('print')
        const handler = (mql:any) => {
          if (mql.matches) {
            // beforePrint();
          } else {
            Logger.debug('[printContent]', 'finished printing 1 ... clearing div')
            tempPrintDiv.innerHTML = ''
            tempPrintDiv.classList.remove('js-printing')
            siteContent.classList.remove('js-printing')
            siteHeaderBar.classList.remove('js-printing')
            mediaQueryList.removeListener(handler)

            if (onFinishedPrinting) onFinishedPrinting()
          }
        }
        mediaQueryList.addListener(handler)
      } else {
        Logger.warn('browser (safari) does not support printing (window.matchMedia)')
      }

    }
    else {
      window.onafterprint = (ev: Event) => {
        Logger.debug('[printContent]', 'finished printing 2 ... clearing div')
        tempPrintDiv.innerHTML = ''
        tempPrintDiv.classList.remove('js-printing')
        siteContent.classList.remove('js-printing')
        siteHeaderBar.classList.remove('js-printing')
        window.onafterprint = null
        if (onFinishedPrinting) onFinishedPrinting()
      }
    }

    //we need to hide the site (even if the temp div stretches the whole site) because
    //of ff we would get 2 pages??
    siteContent.classList.add('js-printing')
    siteHeaderBar.classList.add('js-printing')
    tempPrintDiv.classList.add('js-printing')

    //some browsers don't block the js here so we need listeners
    window.print()

  }
}
