/**
 * Created by janis dähne.
 */
import {
  ResetAction,
  SET_isEnterReleaseCodeDialogDisplayedAction,
  SET_isTagsFilterDisplayedAction,
  SET_openExercisesViaCodeAction, SET_openExercisesViaVisibilityAction,
  SET_sortByKeyAction,
  SetActivePaginationPageAction,
  SetPaginationPageSizeAction,
  SetSearchTextAction
} from "../../../reducers/allAccessibleExercises/openExercisesSite/openExercisesSiteReducer";
import {delay} from "../../../../../debug";
import {
  getOpenExercisePreviewsViaCode,
  getOpenExercisePreviewsViaVisibility
} from "../../../../communicationLayer/exercisesOverviewLayer";
import {ActionType} from "../../../reducers/allAccessibleExercises/openExercisesSite/openExercisesSiteActionTypes";
import {ExercisePreviewFromBackend} from "../../../../types/exercisePreview";
import {ExercisePreviewsFrontendExclusive} from "../../../../components/sites/allAccessibleExercisesSite/openExercises/allOpenExercisesSite";
import {AwaitActions, MultiActions} from "../../types";
import {loadManageTagsSite} from "../../manageTagsSite/manageTagsActions";
import {resetTagsFilter} from "../../tagsFilterActions";
import {debounce} from 'lodash-es'
import {searchInputDebounceInMs} from '../../../../constants'
import {PaginationPostData, PaginationPostDataWithTagsFilter} from '../../../../types/pagination'
import {PaginationHelper} from '../../../../helpers/paginationHelper'
import {GET_openExercisesViaCodeAction} from '../../../reducers/allAccessibleExercises/openExercisesSite/getOpenExercisesViaCodeCrudReducer'
import {GET_openExercisesViaVisibilityAction} from '../../../reducers/allAccessibleExercises/openExercisesSite/getOpenExercisesViaVisibilityCrudReducer'
import {getPLangsAsync} from '../../plangSettingsView/pLangsCrudActions'


export function setOpenExercisesViaCode(openExercisesViaCode: ReadonlyArray<ExercisePreviewsFrontendExclusive>): SET_openExercisesViaCodeAction {
  return {
    type: ActionType.SET_openExercisesViaCodeAction,
    openExercisesViaCode
  }
}

export function setOpenExercisesViaVisibility(openExercisesViaVisibility: ReadonlyArray<ExercisePreviewsFrontendExclusive>): SET_openExercisesViaVisibilityAction {
  return {
    type: ActionType.SET_openExercisesViaVisibility,
    openExercisesViaVisibility
  }
}


export function loadOpenExercisePreviewsViaCodeAsync(): AwaitActions {
  return async (dispatch) => {

    dispatch(resetOpenExercisesSite())

    dispatch(resetTagsFilter())
    await dispatch(loadManageTagsSite())

    await dispatch(getPLangsAsync())

    await dispatch(getOpenExercisesViaCode())
  }
}

export function loadOpenExercisePreviewsViaVisibilityAsync(): AwaitActions {
  return async (dispatch) => {

    dispatch(resetOpenExercisesSite())

    dispatch(resetTagsFilter())
    await dispatch(loadManageTagsSite())

    await dispatch(getPLangsAsync())

    await dispatch(getOpenExercisesViaVisibility())
  }
}

export function _getOpenExercisesViaCode(paginationPostData: PaginationPostDataWithTagsFilter): GET_openExercisesViaCodeAction {
  return {
    type: ActionType.GET_openExercisesViaCode,
    payload: delay(getOpenExercisePreviewsViaCode(paginationPostData))
  }
}

export function getOpenExercisesViaCode(): AwaitActions {
  return async (dispatch, getState) => {

    const state = getState().openExercisesState
    const tagsState = getState().tagsFilterState

    const paginationPostData = PaginationHelper.getPaginationPostDataWithTagsFilter(state.paginationViaCode,
                                                                                    tagsState.selectedFilterTagsIds,
                                                                                    tagsState.selectedNegativeFilterTagsIds,
                                                                                    tagsState.logicalConnector
    )
    await dispatch(_getOpenExercisesViaCode(paginationPostData))
  }
}

export function _getOpenExercisesViaVisibility(paginationPostData: PaginationPostDataWithTagsFilter): GET_openExercisesViaVisibilityAction {
  return {
    type: ActionType.GET_openExercisesViaVisibility,
    payload: delay(getOpenExercisePreviewsViaVisibility(paginationPostData))
  }
}

export function getOpenExercisesViaVisibility(): AwaitActions {
  return async (dispatch, getState) => {

    const state = getState().openExercisesState
    const tagsState = getState().tagsFilterState

    const paginationPostData = PaginationHelper.getPaginationPostDataWithTagsFilter(
      state.paginationViaVisibility,
      tagsState.selectedFilterTagsIds,
      tagsState.selectedNegativeFilterTagsIds,
      tagsState.logicalConnector
    )
    await dispatch(_getOpenExercisesViaVisibility(paginationPostData))
  }
}

export function setIsEnterReleaseCodeDialogDisplayed(isDisplayed: boolean): SET_isEnterReleaseCodeDialogDisplayedAction {
  return {
    type: ActionType.SET_isEnterReleaseCodeDialogDisplayed,
    isDisplayed
  }
}

export function setIsTagsFilterDisplayed(isTagsFilterDisplayed: boolean): SET_isTagsFilterDisplayedAction {
  return {
    type: ActionType.SET_isTagsFilterDisplayed,
    isTagsFilterDisplayed,
  }
}

export function resetOpenExercisesSite(): ResetAction {
  return {
    type: ActionType.RESET
  }
}


//--- pagination

export function _setSortByKey(sortByKey: keyof ExercisePreviewFromBackend,
                              sortDirection: boolean | null,
                              viaCode: boolean
): SET_sortByKeyAction {
  return {
    type: ActionType.SET_sortByKey,
    sortByKey,
    sortDirection,
    viaCode
  }
}

export function setSortByKey(sortByKey: keyof ExercisePreviewFromBackend,
                             sortDirection: boolean | null,
                             viaCode: boolean
): AwaitActions {
  return async (dispatch, getState) => {

    dispatch(_setSortByKey(sortByKey, sortDirection, viaCode))
    await dispatch(viaCode
             ? getOpenExercisesViaCode()
             : getOpenExercisesViaVisibility())
  }
}

export function _setActivePaginationPage(activePage: number, viaCode: boolean): SetActivePaginationPageAction {
  return {
    type: ActionType.setPaginationActivePage,
    activePage,
    viaCode
  }
}

export function setActivePaginationPage(activePage: number, viaCode: boolean): AwaitActions {
  return async (dispatch, getState) => {

    dispatch(_setActivePaginationPage(activePage, viaCode))
    await dispatch(viaCode
             ? getOpenExercisesViaCode()
             : getOpenExercisesViaVisibility())
  }
}

export function _setPaginationPageSize(pageSize: number, viaCode: boolean): SetPaginationPageSizeAction {
  return {
    type: ActionType.setPaginationPageSize,
    pageSize,
    viaCode
  }
}

export function setPaginationPageSize(pageSize: number, viaCode: boolean): AwaitActions {
  return async (dispatch, getState) => {

    dispatch(_setPaginationPageSize(pageSize, viaCode))

    await dispatch(viaCode
             ? getOpenExercisesViaCode()
             : getOpenExercisesViaVisibility())
  }
}

export function _setSearchText(searchText: string, viaCode: boolean): SetSearchTextAction {
  return {
    type: ActionType.setSearchText,
    searchText,
    viaCode
  }
}

// noinspection TsLint
const debouncedRefresh = debounce((dispatch, getState, viaCode: boolean) => {

  dispatch(viaCode
           ? getOpenExercisesViaCode()
           : getOpenExercisesViaVisibility())

}, searchInputDebounceInMs)

export function setSearchText(searchText: string, viaCode: boolean): MultiActions {
  return (dispatch, getState) => {
    dispatch(_setSearchText(searchText, viaCode))

    debouncedRefresh(dispatch, getState, viaCode)
  }
}
