/**
 * Created by janis dähne.
 */
import {
  SET_hasFileChangedMapAction,
  SET_hasUnsavedChangesAction,
  SET_isFileExplorerDisplayedDoEditorAction,
  SET_isRenamingFileDoEditorAction,
  SET_newFileNameDoEditorAction,
  SET_renamingFileIdDoEditorAction,
  SET_renamingFileNameDoEditorAction,
  SET_selectedTabIdDoEditorAction
} from "../../../reducers/doExerciseSite/editorReducers/doEditorReducer";
import {ActionType} from "../../../reducers/doExerciseSite/doExerciseSiteActionTypes";
import {
  changeTabVisibility,
  HasFileChangedMap,
  setSolutionFiles_AndRefreshEditor
} from "../../../../helpers/editorHelper";
import {AwaitActions, MultiActions} from "../../types";
import {SolutionFileDoExerciseFullBase} from "../../../../types/exerciseSolution";
import {setSolutionFiles} from "../doExerciseSiteActions";


export function setHasUnsavedChanges(hasUnsavedChanges: boolean): SET_hasUnsavedChangesAction {
  return {
    type: ActionType.SET_hasUnsavedChanges,
    hasUnsavedChanges
  }
}

/**
 * sets the file changed map to determine what files changed
 *
 *
 * pass {} to indicate that no file has changed
 * @param {HasFileChangedMap} hasFileChangedMap
 * @returns {SET_hasFileChangedMapAction}
 */
export function setHasFileChangedMapDoEditor(hasFileChangedMap: HasFileChangedMap): SET_hasFileChangedMapAction {
  return {
    type: ActionType.SET_hasFileChangedMap,
    hasFileChangedMap
  }
}

/**
 *
 * @param newFile
 * @param solutionFiles
 * @param selectedEditorTabId the currently (old) selected tab id
 */
export function onCreateFileDoEditor(newFile: SolutionFileDoExerciseFullBase,
  solutionFiles: ReadonlyArray<SolutionFileDoExerciseFullBase>,
  selectedEditorTabId: number): AwaitActions {
  return async (dispatch) => {

    const newSolutionFiles: ReadonlyArray<SolutionFileDoExerciseFullBase> = [...solutionFiles, newFile]

    await dispatch(setSolutionFiles_AndRefreshEditor(
      newSolutionFiles,
      setSolutionFiles,
      )
    )

    //open the new file
    dispatch(openTabChangeDoEditor(newFile, newSolutionFiles, selectedEditorTabId))
  }
}

/**
 * changes the visibility of a tab
 * @param {SolutionFileDoExerciseFullBase} file the file (won't be used to change something)
 * @param {ReadonlyArray<SolutionFileDoExerciseFullBase>} templateFiles
 * @param {number} selectedFileId needed because the user can close another file than the existing one
 * @returns {MultiActions}
 */
export function openTabChangeDoEditor(file: SolutionFileDoExerciseFullBase,
                                      templateFiles: ReadonlyArray<SolutionFileDoExerciseFullBase>,
                                      selectedFileId: number): MultiActions {
  return changeTabVisibility(file, true, templateFiles, selectedFileId, setSolutionFiles, setSelectedTabIdDoEditor)
}

export function closeTabChangeDoEditor(file: SolutionFileDoExerciseFullBase,
                                       templateFiles: ReadonlyArray<SolutionFileDoExerciseFullBase>,
                                       selectedFileId: number): MultiActions {
  return changeTabVisibility(file, false, templateFiles, selectedFileId, setSolutionFiles, setSelectedTabIdDoEditor)
}


export function setSelectedTabIdDoEditor(selectedTabIdDoEditor: number): SET_selectedTabIdDoEditorAction {
  return {
    type: ActionType.SET_selectedTabIdDoEditor,
    selectedTabIdDoEditor
  }
}

export function setIsFileExplorerDisplayedDoEditor(isFileExplorerDisplayedDoEditor: boolean): SET_isFileExplorerDisplayedDoEditorAction {
  return {
    type: ActionType.SET_isFileExplorerDisplayedDoEditor,
    isFileExplorerDisplayedDoEditor
  }
}

export function setNewFileNameDoEditor(newFileNameDoEditor: string): SET_newFileNameDoEditorAction {
  return {
    type: ActionType.SET_newFileNameDoEditor,
    newFileNameDoEditor
  }
}

export function setRenamingFileIdDoEditor(renamingFileIdDoEditor: number): SET_renamingFileIdDoEditorAction {
  return {
    type: ActionType.SET_renamingFileIdDoEditor,
    renamingFileIdDoEditor
  }
}

export function setIsRenamingFileDoEditor(isRenamingFileDoEditor: boolean): SET_isRenamingFileDoEditorAction {
  return {
    type: ActionType.SET_isRenamingFileDoEditor,
    isRenamingFileDoEditor
  }
}

export function setRenamingFileNameDoEditor(renamingFileNameDoEditor: string): SET_renamingFileNameDoEditorAction {
  return {
    type: ActionType.SET_renamingFileNameDoEditor,
    renamingFileNameDoEditor
  }
}
