/**
 * Created by janis dähne.
 */

import {UserGroupBase} from "../../../types/group";
import {
  ResetAction,
  SET_isChangeDialogDisplayedAction,
  SET_sortByKeyAction,
  SetActivePaginationPageAction,
  SetPaginationPageSizeAction,
  SetSearchTextAction
} from "../../reducers/groupsSite/groupsSiteReducer";
import {ActionType} from '../../reducers/groupsSite/groupsSiteActionTypes'
import {resetChangeGroupDialog, setIsCreating} from "./dialog/changeGroupDialogActions";
import {setTargetGroup} from "./dialog/groupSetActions";
import {getGroupsAsync, quitGroupAsync} from "./groupsCrudActions";
import {AwaitActions, MultiActions} from "../types";
import {debounce} from 'lodash-es'
import {searchInputDebounceInMs} from '../../../constants'
import {getGlobalSystemSettingsAsync} from '../globalSystemSettings/globalSystemSettingsCrudActions'


export function openCreateGroupDialog(): MultiActions {
  return dispatch => {

    dispatch(resetChangeGroupDialog())
    dispatch(setIsCreating(true))
    dispatch(setIsChangeDialogDisplayed(true))
  }
}

export function openEditGroupDialog(group: UserGroupBase): MultiActions {
  return dispatch => {

    dispatch(resetChangeGroupDialog())
    dispatch(setIsCreating(false))
    dispatch(setTargetGroup(group))
    dispatch(setIsChangeDialogDisplayed(true))

  }
}

export function cancelEditGroup(): MultiActions {
  return dispatch => {

    dispatch(setIsChangeDialogDisplayed(false))
    dispatch(resetChangeGroupDialog())

  }
}

export function quitGroup(group: UserGroupBase): AwaitActions {
  return async (dispatch) => {

    await dispatch(quitGroupAsync(group))
    dispatch(getGroupsAsync(null))

  }
}

export function setIsChangeDialogDisplayed(isDisplayed: boolean): SET_isChangeDialogDisplayedAction {
  return {
    type: ActionType.SET_isChangeDialogDisplayed,
    isDisplayed
  }
}


export function loadGroupsSite(): AwaitActions {
  return async(dispatch) => {

    dispatch(resetGroupsSite())
    await dispatch(getGroupsAsync(null))

    await dispatch(getGlobalSystemSettingsAsync()) //to get the default group
  }
}

export function resetGroupsSite(): ResetAction {
  return {
    type: ActionType.RESET
  }
}

//--- pagination


export function _setSortByKey(sortByKey: keyof UserGroupBase,
                              sortDirection: boolean | null
): SET_sortByKeyAction {
  return {
    type: ActionType.SET_sortByKey,
    sortByKey,
    sortDirection
  }
}

export function setSortByKey(sortByKey: keyof UserGroupBase,
                             sortDirection: boolean | null
): MultiActions {
  return (dispatch, getState) => {

    dispatch(_setSortByKey(sortByKey, sortDirection))
    dispatch(getGroupsAsync(null))

  }
}

export function _setActivePaginationPage(activePage: number): SetActivePaginationPageAction {
  return {
    type: ActionType.setPaginationActivePage,
    activePage
  }
}

export function setActivePaginationPage(activePage: number): MultiActions {
  return (dispatch, getState) => {

    dispatch(_setActivePaginationPage(activePage))
    dispatch(getGroupsAsync(null))
  }
}

export function _setPaginationPageSize(pageSize: number): SetPaginationPageSizeAction {
  return {
    type: ActionType.setPaginationPageSize,
    pageSize
  }
}

export function setPaginationPageSize(pageSize: number): MultiActions {
  return (dispatch, getState) => {

    dispatch(_setPaginationPageSize(pageSize))
    dispatch(getGroupsAsync(null))
  }
}

export function _setSearchText(searchText: string): SetSearchTextAction {
  return {
    type: ActionType.setSearchText,
    searchText
  }
}

// noinspection TsLint
const debouncedRefresh = debounce((dispatch) => {
  dispatch(getGroupsAsync(null))
}, searchInputDebounceInMs)

export function setSearchText(searchText: string): MultiActions {
  return (dispatch, getState) => {
    dispatch(_setSearchText(searchText))

    debouncedRefresh(dispatch)
  }
}
