/**
 * Created by janis dähne.
 */

import {debounce} from 'lodash-es'
import {SelectableUserFrontendExclusive, UserPreviewFromBackend} from "../../../types/users";
import {
  ResetAction,
  SET_isChangeUserDataDialogDisplayedAction,
  SET_activatedUser_uiIsSelectedAction,
  SET_all_activatedUsers_uiIsSelectedAction,
  SET_sortByKeyAction,
  SetActivePaginationPageAction,
  SetPaginationPageSizeAction, SetSearchTextAction
} from "../../reducers/manageActivatedUsersSite/manageActivatedUsersSiteReducer";
import {AwaitActions, MultiActions} from "../types";
import {getActivatedUsersAsync, getSystemRolesAsync} from "./manageActivatedUsersSiteCrudActions";
import {ActionType} from "../../reducers/manageActivatedUsersSite/manageActivatedUsersSiteActionTypes";
import {SortDirection} from "../../../types/sortDirection";
import {PaginationHelper} from '../../../helpers/paginationHelper'
import {PaginationPostData} from '../../../types/pagination'
import {searchInputDebounceInMs} from '../../../constants'


export function setActivatedUserUiIsSelected(activatedUserId: number,
                                             isSelected: boolean
): SET_activatedUser_uiIsSelectedAction {
  return {
    type: ActionType.SET_activatedUser_uiIsSelected,
    newUserId: activatedUserId,
    isSelected
  }
}
export function setAllActivatedUsersUiIsSelected(isSelected: boolean): SET_all_activatedUsers_uiIsSelectedAction {
  return {
    type: ActionType.SET_all_activatedUsers_uiIsSelected,
    isSelected
  }
}

export function loadManageActivatedUsersSite(): AwaitActions {
  return async (dispatch, getState) => {

    dispatch(resetManageActivatedUsersSite())

    //first load system roles
    //then load new users
    await dispatch(getSystemRolesAsync()) //this should ensure the system roles

    const systemRoles = getState().manageActivatedUsersState.systemRoles
    const currentUserId = getState().userDataSettingsSate.userData.id

    const postData = PaginationHelper.getPaginationPostData(getState().manageActivatedUsersState.pagination)

    dispatch(getActivatedUsersAsync(currentUserId, systemRoles, postData))

  }
}

export function setIsChangeUserDataDialogDisplayed(isDisplayed: boolean): SET_isChangeUserDataDialogDisplayedAction {
  return {
    type: ActionType.SET_isChangeUserDataDialogDisplayed,
    isChangeUserDataDialogDisplayed: isDisplayed
  }
}

export function resetManageActivatedUsersSite(): ResetAction {
  return {
    type: ActionType.RESET
  }
}

//--- pagination


export function _setSortByKey(sortByKey: keyof SelectableUserFrontendExclusive,
                              sortDirection: boolean | null
): SET_sortByKeyAction {
  return {
    type: ActionType.SET_sortByKey,
    sortByKey,
    sortDirection
  }
}

export function setSortByKey(sortByKey: keyof SelectableUserFrontendExclusive,
                             sortDirection: boolean | null
): MultiActions {
  return (dispatch, getState) => {

    dispatch(_setSortByKey(sortByKey, sortDirection))
    dispatch(refreshPagePaginatedDataManageActivatedUsersSite())

  }
}

export function _setActivePaginationPage(activePage: number): SetActivePaginationPageAction {
  return {
    type: ActionType.setPaginationActivePage,
    activePage
  }
}

export function setActivePaginationPage(activePage: number): MultiActions {
  return (dispatch, getState) => {

    dispatch(_setActivePaginationPage(activePage))
    dispatch(refreshPagePaginatedDataManageActivatedUsersSite())
  }
}

export function _setPaginationPageSize(pageSize: number): SetPaginationPageSizeAction {
  return {
    type: ActionType.setPaginationPageSize,
    pageSize
  }
}

export function setPaginationPageSize(pageSize: number): MultiActions {
  return (dispatch, getState) => {

    dispatch(_setPaginationPageSize(pageSize))
    dispatch(refreshPagePaginatedDataManageActivatedUsersSite())
  }
}

export function _setSearchText(searchText: string): SetSearchTextAction {
  return {
    type: ActionType.setSearchText,
    searchText
  }
}

// noinspection TsLint
const debouncedRefresh = debounce((dispatch) => {
  dispatch(refreshPagePaginatedDataManageActivatedUsersSite())
}, searchInputDebounceInMs)

export function setSearchText(searchText: string): MultiActions {
  return (dispatch, getState) => {
    dispatch(_setSearchText(searchText))

    debouncedRefresh(dispatch)
  }
}


export function refreshPagePaginatedDataManageActivatedUsersSite(): MultiActions {
  return (dispatch, getState) => {

    const systemRoles = getState().manageActivatedUsersState.systemRoles
    const currentUserId = getState().userDataSettingsSate.userData.id
    // noinspection TsLint
    const postData: PaginationPostData = PaginationHelper.getPaginationPostData(getState().manageActivatedUsersState.pagination)
    dispatch(getActivatedUsersAsync(currentUserId, systemRoles, postData))
  }
}
