/**
 * Created by janis dähne.
 */
import {GET_systemRolesAction} from "../../reducers/manageActivatedUsersSite/getSystemRolesReducer";
import {ActionType} from "../../reducers/manageActivatedUsersSite/manageActivatedUsersSiteActionTypes";
import {delay} from "../../../../debug";
import {getSystemRoles} from "../../../communicationLayer/systemRolesLayer";
import {deleteActive, getActivatedUsers} from "../../../communicationLayer/usersLayer";
import {GET_activatedUsersAction} from "../../reducers/manageActivatedUsersSite/getActivatedUsersReducer";
import {AwaitActions} from "../types";
import {UserPreviewFromBackend} from "../../../types/users";
import {DELETE_activatedUserAction} from "../../reducers/manageActivatedUsersSite/deleteActivatedUserReducer";
import {SystemRoleFullBase} from "../../../types/userData";
import {PaginationPostData} from '../../../types/pagination'
import {PaginationHelper} from '../../../helpers/paginationHelper'

export function deleteMultipleActivatedUserAsync(activatedUsers: ReadonlyArray<UserPreviewFromBackend>): AwaitActions {
  return async (dispatch, getState) => {

    for (const tuple of activatedUsers) {
      await dispatch(_deleteActivatedUserAsync(tuple))
    }

    const systemRoles = getState().manageActivatedUsersState.systemRoles
    const currentUserId = getState().userDataSettingsSate.userData.id


    const paginationPostData: PaginationPostData = PaginationHelper.getPaginationPostData(getState().manageActivatedUsersState.pagination);

    dispatch(getActivatedUsersAsync(currentUserId, systemRoles, paginationPostData))
  }
}

export function deleteActivatedUserAsync(activatedUser: UserPreviewFromBackend): AwaitActions {
  return async (dispatch, getState) => {
    await dispatch(_deleteActivatedUserAsync(activatedUser))

    const systemRoles = getState().manageActivatedUsersState.systemRoles
    const currentUserId = getState().userDataSettingsSate.userData.id

    const paginationPostData: PaginationPostData = PaginationHelper.getPaginationPostData(getState().manageActivatedUsersState.pagination);
    dispatch(getActivatedUsersAsync(currentUserId, systemRoles, paginationPostData))
  }
}

function _deleteActivatedUserAsync(activated: UserPreviewFromBackend): DELETE_activatedUserAction {
  return {
    type: ActionType.DELETE_activatedUser,
    payload: delay(deleteActive(activated))
  }
}


export function getSystemRolesAsync(): GET_systemRolesAction {
  return {
    type: ActionType.GET_systemRoles,
    payload: delay(getSystemRoles()),
  }
}


export function getActivatedUsersAsync(currentUserId: number, systemRoles: ReadonlyArray<SystemRoleFullBase>, paginationPostData: PaginationPostData): GET_activatedUsersAction {
  return {
    type: ActionType.GET_activatedUsers,
    payload: delay(getActivatedUsers(paginationPostData, null)),
    meta: {
      systemRoles,
      currentUserId
    }
  }
}
