/**
 * Created by janis dähne.
 */


import {NewUserFrontendExclusive, UserPreviewFromBackend} from "../../../types/users";
import {
  ResetAction,
  SET_all_newUsers_uiIsSelectedAction,
  SET_isLoadingAction,
  SET_newUser_systemRoleAction,
  SET_newUser_uiIsChangeSystemRolePopupDisplayedAction,
  SET_newUser_uiIsSelectedAction,
  SET_newUsersAction,
  SET_sortByKeyAction, SetActivePaginationPageAction, SetPaginationPageSizeAction, SetSearchTextAction
} from "../../reducers/manageNewUsersSite/manageNewUsersSiteReducer";
import {ActionType,} from '../../reducers/manageNewUsersSite/manageNewUsersSiteActionTypes'
import {SystemRoleFullBase} from "../../../types/userData";
import {AwaitActions, MultiActions} from "../types";
import {getNewUsersAsync, getSystemRolesAsync} from "./manageNewUserSiteCrudActions";
import {PaginationHelper} from '../../../helpers/paginationHelper'
import {PaginationPostData} from '../../../types/pagination'
import {debounce} from 'lodash-es'
import {searchInputDebounceInMs} from '../../../constants'

export function setNewUsers(newUsers: ReadonlyArray<NewUserFrontendExclusive>): SET_newUsersAction {
  return {
    type: ActionType.SET_newUsers,
    newUsers
  }
}

export function setIsLoading(isLoading: boolean): SET_isLoadingAction {
  return {
    type: ActionType.SET_isLoading,
    isLoading
  }
}

/**
 *
 * @param {number} newUserId better use id here else we could pass down an old user...
 * @param {boolean} isDisplayed
 * @returns {SET_newUser_uiIsChangeSystemRolePopupDisplayedAction}
 */
export function setNewUserUiIsChangeSystemRolePopupDisplayed(newUserId: number, isDisplayed: boolean): SET_newUser_uiIsChangeSystemRolePopupDisplayedAction {
  return {
    type: ActionType.SET_newUser_uiIsChangeSystemRolePopupDisplayed,
    newUserId,
    isDisplayed
  }
}

export function setNewUserUiIsSelected(newUserId: number, isSelected: boolean): SET_newUser_uiIsSelectedAction {
  return {
    type: ActionType.SET_newUser_uiIsSelected,
    newUserId,
    isSelected
  }
}
export function setAllNewUsersUiIsSelected(isSelected: boolean): SET_all_newUsers_uiIsSelectedAction {
  return {
    type: ActionType.SET_all_newUsers_uiIsSelected,
    isSelected
  }
}

export function setNewUserSystemRole(newUser: NewUserFrontendExclusive, systemRole: SystemRoleFullBase): SET_newUser_systemRoleAction {
  return {
    type: ActionType.SET_newUser_systemRole,
    newUser,
    systemRole
  }
}
export function loadManageNewUsersSite(): AwaitActions {
  return async (dispatch, getState) => {

    dispatch(resetManageNewUsersSite())

    //first load system roles
    //then load new users
    await dispatch(getSystemRolesAsync(true))

    const postData = PaginationHelper.getPaginationPostData(getState().manageNewUsersState.pagination)

    dispatch(getNewUsersAsync(postData))

  }
}

export function resetManageNewUsersSite(): ResetAction {
  return {
    type: ActionType.RESET
  }
}


//---pagination

export function _setSortByKey(sortByKey: keyof UserPreviewFromBackend, sortDirection: boolean | null): SET_sortByKeyAction {
  return {
    type: ActionType.SET_sortByKey,
    sortByKey,
    sortDirection
  }
}

export function setSortByKey(sortByKey: keyof UserPreviewFromBackend, sortDirection: boolean | null): MultiActions {
  return (dispatch, getState) => {

    dispatch(_setSortByKey(sortByKey, sortDirection))
    dispatch(refreshPagePaginatedDataManageNewUsersSite())
  }
}


export function _setActivePaginationPage(activePage: number): SetActivePaginationPageAction {
  return {
    type: ActionType.setPaginationActivePage,
    activePage
  }
}

export function setActivePaginationPage(activePage: number): MultiActions {
  return (dispatch, getState) => {

    dispatch(_setActivePaginationPage(activePage))
    dispatch(refreshPagePaginatedDataManageNewUsersSite())
  }
}

export function _setPaginationPageSize(pageSize: number): SetPaginationPageSizeAction {
  return {
    type: ActionType.setPaginationPageSize,
    pageSize
  }
}

export function setPaginationPageSize(pageSize: number): MultiActions {
  return (dispatch, getState) => {

    dispatch(_setPaginationPageSize(pageSize))
    dispatch(refreshPagePaginatedDataManageNewUsersSite())
  }
}

export function _setSearchText(searchText: string): SetSearchTextAction {
  return {
    type: ActionType.setSearchText,
    searchText
  }
}

// noinspection TsLint
const debouncedRefresh = debounce((dispatch) => {
  dispatch(refreshPagePaginatedDataManageNewUsersSite())
}, searchInputDebounceInMs)

export function setSearchText(searchText: string): MultiActions {
  return (dispatch, getState) => {
    dispatch(_setSearchText(searchText))

    debouncedRefresh(dispatch)
  }
}

export function refreshPagePaginatedDataManageNewUsersSite(): MultiActions {
  return (dispatch, getState) => {

    // noinspection TsLint
    const postData: PaginationPostData = PaginationHelper.getPaginationPostData(getState().manageNewUsersState.pagination)
    dispatch(getNewUsersAsync(postData))
  }
}
