/**
 * Created by janis dähne.
 */
import {
  ResetAction,
  SET_isTagsFilterDisplayedAction,
  SET_sortByKeyAction,
  SetActivePaginationPageAction, SetIsLoadingAction, SetPaginationPageSizeAction, SetSearchTextAction
} from "../../reducers/manageOwnExercisesSite/ownExercisesSiteReducer";
import {deleteExercise, getOwnExercisePreviews} from "../../../communicationLayer/exercisesOverviewLayer";
import {delay} from "../../../../debug";
import {GET_ownExercisesAction} from "../../reducers/manageOwnExercisesSite/getOwnExercisesReducer";
import {ActionType} from "../../reducers/manageOwnExercisesSite/ownExercisesSiteActionTypes";
import {AwaitActions, MultiActions} from "../types";
import {DELETE_ownExerciseAction} from "../../reducers/manageOwnExercisesSite/deleteOwnExerciseReducer";
import {loadManageTagsSite} from "../manageTagsSite/manageTagsActions";
import {resetTagsFilter} from "../tagsFilterActions";
import {debounce} from 'lodash-es'
import {searchInputDebounceInMs} from '../../../constants'
import {EditableExercisePreviewFromBackend} from '../../../types/exercisePreview'
import {PaginationPostDataWithTagsFilter} from '../../../types/pagination'
import {PaginationHelper} from '../../../helpers/paginationHelper'


export function _getOwnExercisesAsync(paginationPostData: PaginationPostDataWithTagsFilter): GET_ownExercisesAction {
  return {
    type: ActionType.GET_ownExercises,
    payload: delay(getOwnExercisePreviews(paginationPostData))
  }
}

export function getOwnExercisesAsync(): MultiActions {
  return (dispatch, getState) => {

    const tagsState = getState().tagsFilterState
    const paginationPostData = PaginationHelper.getPaginationPostDataWithTagsFilter(getState().manageOwnExercisesState.pagination,
                                                                                    tagsState.selectedFilterTagsIds,
                                                                                    tagsState.selectedNegativeFilterTagsIds,
                                                                                    tagsState.logicalConnector
    )
    dispatch(_getOwnExercisesAsync(paginationPostData))

  }
}

export function deleteOwnExerciseAsync(exerciseId:number): AwaitActions {
  return async (dispatch, getState) => {

    await dispatch(_deleteOwnExerciseAsync(exerciseId))

    dispatch(getOwnExercisesAsync())
  }
}


export function _deleteOwnExerciseAsync(exerciseId:number): DELETE_ownExerciseAction {
  return {
    type: ActionType.DELETE_ownExercise,
    payload: delay(deleteExercise(exerciseId))
  }
}


export function setIsTagsFilterDisplayed(isTagsFilterDisplayed: boolean): SET_isTagsFilterDisplayedAction {
  return {
    type: ActionType.SET_isTagsFilterDisplayed,
    isTagsFilterDisplayed,
  }
}

export function loadOwnExercisesSite(resetPaginationState: boolean): AwaitActions {
  return async (dispatch, getState) => {

    dispatch(resetOwnExercisesSite(resetPaginationState))

    // dispatch(resetTagsFilter()) //in most cases we go back from release and then we want to still have the filter

    await dispatch(loadManageTagsSite())

    await dispatch(getOwnExercisesAsync())
  }
}

export function resetOwnExercisesSite(resetPaginationState: boolean): ResetAction {
  return {
    type: ActionType.RESET,
    resetPaginationState
  }
}


//--- pagination

export function _setSortByKey(sortByKey: keyof EditableExercisePreviewFromBackend,
                              sortDirection: boolean | null
): SET_sortByKeyAction {
  return {
    type: ActionType.SET_sortByKey,
    sortByKey,
    sortDirection
  }
}

export function setSortByKey(sortByKey: keyof EditableExercisePreviewFromBackend,
                             sortDirection: boolean | null
): MultiActions {
  return (dispatch, getState) => {

    dispatch(_setSortByKey(sortByKey, sortDirection))
    dispatch(getOwnExercisesAsync())

  }
}

export function _setActivePaginationPage(activePage: number): SetActivePaginationPageAction {
  return {
    type: ActionType.setPaginationActivePage,
    activePage
  }
}

export function setActivePaginationPage(activePage: number): MultiActions {
  return (dispatch, getState) => {

    dispatch(_setActivePaginationPage(activePage))
    dispatch(getOwnExercisesAsync())
  }
}

export function _setPaginationPageSize(pageSize: number): SetPaginationPageSizeAction {
  return {
    type: ActionType.setPaginationPageSize,
    pageSize
  }
}

export function setPaginationPageSize(pageSize: number): MultiActions {
  return (dispatch, getState) => {

    dispatch(_setPaginationPageSize(pageSize))

    dispatch(getOwnExercisesAsync())
  }
}

export function _setSearchText(searchText: string): SetSearchTextAction {
  return {
    type: ActionType.setSearchText,
    searchText
  }
}

// noinspection TsLint
const debouncedRefresh = debounce((dispatch, getState) => {

  dispatch(getOwnExercisesAsync())

}, searchInputDebounceInMs)

export function setSearchText(searchText: string): MultiActions {
  return (dispatch, getState) => {
    dispatch(_setSearchText(searchText))

    debouncedRefresh(dispatch, getState)
  }
}

export function setIsLoading(isLoading: boolean): SetIsLoadingAction {
  return {
    type: ActionType.setIsLoading,
    isLoading
  }
}

