/**
 * Created by janis dähne.
 */


import {
  ResetAction,
  SET_comesFromOwnExercisesSiteAction,
  SET_isChangeDialogDisplayedAction,
  SET_isZoomedExerciseReleaseDisplayedAction,
  SET_selectedExerciseIdAction,
  SET_sortByKeyAction,
  SetActivePaginationPageAction,
  SetPaginationPageSizeAction,
  SetSearchTextAction
} from "../../../reducers/ownOrGroupExerciseReducers/exerciseReleasesSite/releaseSiteReducer";
import {ActionType} from "../../../reducers/ownOrGroupExerciseReducers/exerciseReleasesSite/releasesSiteActionTypes";
import {AwaitActions, MultiActions} from "../../types";
import {
  deleteExerciseReleaseAsync,
  getExerciseReleaseAsync,
  getSingleGroupExerciseAsync,
  getSingleOwnExerciseAsync
} from "./getReleasesSiteCrudAction";
import {ExerciseReleaseFromBackend} from "../../../../types/exerciseRelease";
import {errorDialog} from "../../../../helpers/dialogHelper";
import {getI18n} from "../../../../../i18n/i18nRoot";
import globalState from '../../../state'
import {debounce} from 'lodash-es'
import {searchInputDebounceInMs} from '../../../../constants'
import {getPLangsAsync} from '../../plangSettingsView/pLangsCrudActions'


export function setSelectedExerciseId(exerciseId: number): SET_selectedExerciseIdAction {
  return {
    type: ActionType.SET_selectedExerciseId,
    id: exerciseId
  }
}

export function setComesFromOwnExercisesSite(comesFromOwnExercisesSite: boolean): SET_comesFromOwnExercisesSiteAction {
  return {
    type: ActionType.SET_comesFromOwnExercisesSite,
    comesFromOwnExercisesSite
  }
}


export function deleteExerciseReleaseAsyncAndReload(exerciseReleaseId: number, exerciseId: number,
                                                    loadExercises: boolean, comesFromOwnExercisesSite: boolean
): AwaitActions {
  return async (dispatch) => {

    await dispatch(deleteExerciseReleaseAsync(exerciseReleaseId))


    // if (comesFromOwnExercisesSite) {
    //   await dispatch(getSingleOwnExerciseAsync(exerciseId))
    // } else {
    //   await dispatch(getSingleGroupExerciseAsync(exerciseId))
    // }

    await dispatch(getExerciseReleaseAsync())

  }
}


export function setIsZoomedExerciseReleaseDisplayed(isDisplayed: boolean): SET_isZoomedExerciseReleaseDisplayedAction {
  return {
    type: ActionType.SET_isZoomedExerciseReleaseDisplayed,
    isDisplayed
  }
}

export function setIsChangeDialogDisplayedAction(isDisplayed: boolean): SET_isChangeDialogDisplayedAction {
  return {
    type: ActionType.SET_isChangeDialogDisplayed,
    isDisplayed
  }
}

export function loadExerciseReleasesSite(exerciseId: number, comesFromOwnExercisesSite: boolean): AwaitActions {
  return async (dispatch, getState) => {


    if (comesFromOwnExercisesSite) {
      await dispatch(getSingleOwnExerciseAsync(exerciseId))
    } else {
      await dispatch(getSingleGroupExerciseAsync(exerciseId))
    }

    const exercise = getState().releaseSiteState.exercise
    if (!exercise) {
      errorDialog(getI18n(globalState.getState().i18nState.langId, 'Error'),
                  getI18n(globalState.getState().i18nState.langId, 'Could not find the exercise or no permission'),globalState.getState().i18nState.langId
      )
      return
    }

    // dispatch(getUserPermissionForGroup(exercise.managingGroupId))

    //we need the plangs for the dialog to select a plang
    await dispatch(getPLangsAsync())

    dispatch(getExerciseReleaseAsync())

  }
}

export function resetExerciseReleasesSite(resetPaginationState: boolean): ResetAction {
  return {
    type: ActionType.RESET,
    resetPaginationState
  }
}


//--- pagination


export function _setSortByKey(sortByKey: keyof ExerciseReleaseFromBackend,
                              sortDirection: boolean | null
): SET_sortByKeyAction {
  return {
    type: ActionType.SET_sortByKey,
    sortByKey,
    sortDirection
  }
}

export function setSortByKey(sortByKey: keyof ExerciseReleaseFromBackend,
                             sortDirection: boolean | null
): MultiActions {
  return (dispatch, getState) => {

    dispatch(_setSortByKey(sortByKey, sortDirection))
    dispatch(getExerciseReleaseAsync())

  }
}

export function _setActivePaginationPage(activePage: number): SetActivePaginationPageAction {
  return {
    type: ActionType.setPaginationActivePage,
    activePage
  }
}

export function setActivePaginationPage(activePage: number): MultiActions {
  return (dispatch, getState) => {

    dispatch(_setActivePaginationPage(activePage))
    dispatch(getExerciseReleaseAsync())
  }
}

export function _setPaginationPageSize(pageSize: number): SetPaginationPageSizeAction {
  return {
    type: ActionType.setPaginationPageSize,
    pageSize
  }
}

export function setPaginationPageSize(pageSize: number): MultiActions {
  return (dispatch, getState) => {

    dispatch(_setPaginationPageSize(pageSize))

    dispatch(getExerciseReleaseAsync())
  }
}

export function _setSearchText(searchText: string): SetSearchTextAction {
  return {
    type: ActionType.setSearchText,
    searchText
  }
}

// noinspection TsLint
const debouncedRefresh = debounce((dispatch, getState) => {

  dispatch(getExerciseReleaseAsync())

}, searchInputDebounceInMs)

export function setSearchText(searchText: string): MultiActions {
  return (dispatch, getState) => {
    dispatch(_setSearchText(searchText))

    debouncedRefresh(dispatch, getState)
  }
}
