/**
 * Created by janis dähne.
 */
import {GET_exerciseAction} from "../../reducers/tutorViewSite/getExerciseReducer";
import {ActionType} from "../../reducers/tutorViewSite/tutorViewSiteActionTypes";
import {delay} from "../../../../debug";
import {
  getAfterSubmissionTestResultsAsTutor,
  getAllAfterTestResultsAsTutor,
  getAllTestResultsAsTutor,
  getExerciseAsTutor,
  getSingleAfterCustomTestResultAsTutor,
  getSingleAfterNormalTestResultAsTutor,
  getSingleAfterSubmitTestResultAsTutor,
  getSingleAssessment,
  getSingleCustomTestResultAsTutor,
  getSingleNormalTestResultAsTutor,
  getSingleSubmitTestResultAsTutor,
  getSubmissionTestResultsAsTutor,
  getUserAfterSolutionAsTutor,
  getUserSolutionAsTutor,
  updateSingleAssessment,
} from "../../../communicationLayer/tutorViewAssessmentLayer";
import {GET_allUserTestResultsAction} from "../../reducers/tutorViewSite/crud/getAllUserTestResultsReducer";
import {GET_user_solutionVersionAction} from "../../reducers/tutorViewSite/crud/getUserSolutionReducer";
import {GET_userSubmitTestResultsAction} from "../../reducers/tutorViewSite/crud/getUserSubmitTestResults";
import {RUN_normalTestAsTutorAction} from "../../reducers/tutorViewSite/crud/runNormalTestAsTutorReducer";
import {
  JustRunProgramTestTutorViewCommandForBackend,
  RunCompileSingleFileCommandForBackend,
  RunTutorTestCommandForBackend
} from "../../../types/testsAndTestProtocol";
import {
  compileSingleFileTutorView,
  justRunProgramTestTutorView,
  runTestAsTutor
} from "../../../communicationLayer/testingLayer";
import {RUN_submitTestAsTutorAction} from "../../reducers/tutorViewSite/crud/runSubmitTestAsTutorReducer";
import {AwaitActions, AwaitActionsWithResult} from "../types";
import {setSelectedConsoleViewTest} from "./tutorViewSiteActions";
import {setNormalTestsForTutorAreRunningAndResetResult} from "./sub/normalTestActions";
import {RunnableExerciseTestFrontendExclusive} from "../../../types/exerciseEditor";
import {setSubmitTestsForTutorAreRunningAndResetResult} from "./sub/submitTestActions";
import {AssessedSubmitTestFrontendExclusive} from "../../../types/testResults";
import {GET_singleAssessmentAction} from "../../reducers/tutorViewSite/crud/getSingleAssessmentReducer";
import {AssessmentFullBase} from "../../../types/submissions";
import {UPDATE_singleAssessmentAction} from "../../reducers/tutorViewSite/crud/updateSingleAssessmentReducer";
import {notifySuccess} from "../../../helpers/notificationHelper";
import {errorDialog, warningDialog} from "../../../helpers/dialogHelper";
import {getI18n} from "../../../../i18n/i18nRoot";
import globalState from '../../state'
import {GET_user_afterSolutionAction} from '../../reducers/tutorViewSite/crud/afterSolution/getUserAfterSolutionReducer'
import {GET_afterTestResultsAction} from '../../reducers/tutorViewSite/crud/afterSolution/getAllUserAfterTestResultsReducer'
import {GET_afterSubmitTestResultsAction} from '../../reducers/tutorViewSite/crud/afterSolution/getAfterSubmitTestResultsReducer'
import {KnownInternalTestTypes, TestTypeFullBase} from '../../../types/testTypes'
import {JUST_RUN_programAction} from '../../reducers/tutorViewSite/crud/justRunProgramReducer'
import {RUN_compileSingleFileAction} from '../../reducers/tutorViewSite/crud/runCompileSingleFileReducer'
import {GET_singleUserNormalTestResultAction} from '../../reducers/tutorViewSite/crud/getSingleUserNormalTestResultReducer'
import {GET_singleUserCustomTestResultAction} from '../../reducers/tutorViewSite/crud/getSingleUserCustomTestResultReducer'
import {GET_singleUserSubmitTestResultAction} from '../../reducers/tutorViewSite/crud/getSingleUserSubmitTestResultReducer'
import Logger from '../../../helpers/logger'
import {_getDoExerciseSingleSubmitTestResult} from '../doExerciseSite/doExerciseCrudActions'
import {GET_singleAfterNormalTestResultAction} from '../../reducers/tutorViewSite/crud/afterSolution/getSingleUserAfterNormalTestResultReducer'
import {GET_singleAfterCustomTestResultAction} from '../../reducers/tutorViewSite/crud/afterSolution/getSingleUserAfterCustomTestResultReducer'
import {GET_singleAfterSubmitTestResultAction} from '../../reducers/tutorViewSite/crud/afterSolution/getSingleUserAfterSubmitTestResultReducer'


export function getExerciseTutorViewAsync(releaseId: number, userId: number, pLangId: number): GET_exerciseAction {
  return {
    type: ActionType.GET_exercise,
    payload: delay(getExerciseAsTutor(releaseId, userId, pLangId))
  }
}

export function getAllUserTestResultsAsTutorAsync(releaseId: number, userId: number, pLangId: number): GET_allUserTestResultsAction {
  return {
    type: ActionType.GET_allUserTestResults,
    payload: delay(getAllTestResultsAsTutor(releaseId, userId, pLangId))
  }
}


export function getSingleUserNormalTestResultAsTutorAsync(releaseId: number, userId: number, pLangId: number, testId: number): AwaitActions {
  return async(dispatch, getState) => {

    await dispatch(_getSingleUserNormalTestResultAsTutorAsync(releaseId, userId, pLangId, testId))

    const updatedTest = getState().tutorViewSiteState.exercise.tests.find(p => p.id === testId)

    if (!updatedTest) {
      Logger.warn(`could not find normal test with id ${testId} where we load the full console output`)
      return
    }

    dispatch(setSelectedConsoleViewTest(updatedTest))
  }
}
export function _getSingleUserNormalTestResultAsTutorAsync(releaseId: number, userId: number, pLangId: number, testId: number): GET_singleUserNormalTestResultAction {
  return {
    type: ActionType.GET_singleUserNormalTestResult,
    payload: delay(getSingleNormalTestResultAsTutor(releaseId, userId, pLangId, testId))
  }
}

export function getSingleUserCustomTestResultAsTutorAsync(releaseId: number, userId: number, pLangId: number, testId: number): AwaitActions {
  return async(dispatch, getState) => {

    await dispatch(_getSingleUserCustomTestResultAsTutorAsync(releaseId, userId, pLangId, testId))

    const updatedTest = getState().tutorViewSiteState.exercise.customTests.find(p => p.id === testId)

    if (!updatedTest) {
      Logger.warn(`could not find custom test with id ${testId} where we load the full console output`)
      return
    }

    dispatch(setSelectedConsoleViewTest(updatedTest))
  }
}
export function _getSingleUserCustomTestResultAsTutorAsync(releaseId: number, userId: number, pLangId: number, testId: number): GET_singleUserCustomTestResultAction {
  return {
    type: ActionType.GET_singleUserCustomTestResult,
    payload: delay(getSingleCustomTestResultAsTutor(releaseId, userId, pLangId, testId))
  }
}

export function getSingleUserSubmitTestResultAsTutorAsync(releaseId: number, userId: number, pLangId: number, testId: number): AwaitActions {
  return async(dispatch, getState) => {

    await dispatch(_getSingleUserSubmitTestResultAsTutorAsync(releaseId, userId, pLangId, testId))

    const updatedTest = getState().tutorViewSiteState.submitTestResults.find(p => p.id === testId)

    if (!updatedTest) {
      Logger.warn(`could not find submit test with id ${testId} where we load the full console output`)
      return
    }

    dispatch(setSelectedConsoleViewTest(updatedTest))
  }
}
export function _getSingleUserSubmitTestResultAsTutorAsync(releaseId: number, userId: number, pLangId: number, testId: number): GET_singleUserSubmitTestResultAction {
  return {
    type: ActionType.GET_singleUserSubmitTestResult,
    payload: delay(getSingleSubmitTestResultAsTutor(releaseId, userId, pLangId, testId))
  }
}



export function getAllUserAfterTestResultsAsTutorAsync(releaseId: number, userId: number, pLangId: number): GET_afterTestResultsAction {
  return {
    type: ActionType.GET_afterTestResults,
    payload: delay(getAllAfterTestResultsAsTutor(releaseId, userId, pLangId))
  }
}

export function getAllAfterSubmitTestResultsAsTutor(releaseId: number, userId: number, pLangId: number): GET_afterSubmitTestResultsAction {
  return {
    type: ActionType.GET_afterSubmitTestResults,
    payload: delay(getAfterSubmissionTestResultsAsTutor(releaseId, userId, pLangId))
  }
}


export function getSingleUserAfterNormalTestResultAsTutorAsync(releaseId: number, userId: number, pLangId: number, testId: number): AwaitActions {
  return async(dispatch, getState) => {

    await dispatch(_getSingleUserAfterNormalTestResultAsTutorAsync(releaseId, userId, pLangId, testId))

    const updatedTest = getState().tutorViewSiteState.afterNormalTests.find(p => p.id === testId)

    if (!updatedTest) {
      Logger.warn(`could not find after normal test with id ${testId} where we load the full console output`)
      return
    }

    dispatch(setSelectedConsoleViewTest(updatedTest))
  }
}
export function _getSingleUserAfterNormalTestResultAsTutorAsync(releaseId: number, userId: number, pLangId: number, testId: number): GET_singleAfterNormalTestResultAction {
  return {
    type: ActionType.GET_singleAfterNormalTestResult,
    payload: delay(getSingleAfterNormalTestResultAsTutor(releaseId, userId, pLangId, testId))
  }
}

export function getSingleUserAfterCustomTestResultAsTutorAsync(releaseId: number, userId: number, pLangId: number, testId: number): AwaitActions {
  return async(dispatch, getState) => {

    await dispatch(_getSingleUserAfterCustomTestResultAsTutorAsync(releaseId, userId, pLangId, testId))

    // const updatedTest = getState().tutorViewSiteState.afterNormalTests.find(p => p.id === testId)

    throw new Error(`currently not supported!`)

    // if (!updatedTest) {
    //   Logger.warn(`could not find after custom test with id ${testId} where we load the full console output`)
    //   return
    // }
    //
    // dispatch(setSelectedConsoleViewTest(updatedTest))
  }
}
export function _getSingleUserAfterCustomTestResultAsTutorAsync(releaseId: number, userId: number, pLangId: number, testId: number): GET_singleAfterCustomTestResultAction {
  return {
    type: ActionType.GET_singleAfterCustomTestResult,
    payload: delay(getSingleAfterCustomTestResultAsTutor(releaseId, userId, pLangId, testId))
  }
}

export function getSingleUserAfterSubmitTestResultAsTutorAsync(releaseId: number, userId: number, pLangId: number, testId: number): AwaitActions {
  return async(dispatch, getState) => {

    await dispatch(_getSingleUserAfterSubmitTestResultAsTutorAsync(releaseId, userId, pLangId, testId))

    const updatedTest = getState().tutorViewSiteState.afterSubmitTests.find(p => p.id === testId)

    if (!updatedTest) {
      Logger.warn(`could not find after submit test with id ${testId} where we load the full console output`)
      return
    }

    dispatch(setSelectedConsoleViewTest(updatedTest))
  }
}
export function _getSingleUserAfterSubmitTestResultAsTutorAsync(releaseId: number, userId: number, pLangId: number, testId: number): GET_singleAfterSubmitTestResultAction {
  return {
    type: ActionType.GET_singleAfterSubmitTestResult,
    payload: delay(getSingleAfterSubmitTestResultAsTutor(releaseId, userId, pLangId, testId))
  }
}





export function getUserSolutionAsTutorViewAsync(releaseId: number, userId: number, pLangId: number): GET_user_solutionVersionAction {
  return {
    type: ActionType.GET_user_solutionVersion,
    payload: delay(getUserSolutionAsTutor(releaseId, userId, pLangId))
  }
}

export function getUserAfterSolutionAsTutorViewAsync(releaseId: number, userId: number, pLangId: number): GET_user_afterSolutionAction {
  return {
    type: ActionType.GET_user_afterSolution,
    payload: delay(getUserAfterSolutionAsTutor(releaseId, userId, pLangId))
  }
}



export function getUserSubmissionTestResultsAsTutorAsync(releaseId: number, userId: number, pLangId: number): GET_userSubmitTestResultsAction {
  return {
    type: ActionType.GET_userSubmitTestResults,
    payload: delay(getSubmissionTestResultsAsTutor(releaseId, userId, pLangId))
  }
}

export function getSingleAssessmentTutorViewAsync(releaseId: number, userId: number, pLangId: number): GET_singleAssessmentAction {
  return {
    type: ActionType.GET_singleAssessment,
    payload: delay(getSingleAssessment(releaseId, userId, pLangId))
  }
}

export function changeSingleAssessmentTutorViewAsync(assessment: AssessmentFullBase): AwaitActions {
    return async(dispatch, getState) => {

      await dispatch(_changeSingleAssessmentTutorViewAsync(assessment))

      //TODO only if no error

      notifySuccess(getI18n(globalState.getState().i18nState.langId,'Manual assessment saved'))
    }
}

export function _changeSingleAssessmentTutorViewAsync(assessment: AssessmentFullBase): UPDATE_singleAssessmentAction {
  return {
    type: ActionType.UPDATE_singleAssessment,
    payload: delay(updateSingleAssessment(assessment))
  }
}


/**
 *
 * @param runTestAsTutorCommand
 * @returns [hasAnyTestFailed] true: some test has not compiled or not passed, false: all tests passed
 */
export function runNormalTestAsTutor(runTestAsTutorCommand: RunTutorTestCommandForBackend): AwaitActionsWithResult<boolean> {
  return async (dispatch, getState) => {
    //if we run the same test twice and the output won't change then this looks like nothing happened...
    //set console to blank so that the user sees that something happened
    dispatch(setSelectedConsoleViewTest(null))


    const runningTests = runTestAsTutorCommand.testIds
      .map<RunnableExerciseTestFrontendExclusive>(testId =>
        //don't use allTests because saveDoExerciseSolution could have reset the test (results) and allTest would be an old array
        getState().tutorViewSiteState.normalTestsForTutor
          .find(test =>
            test.id === testId
          )
      )

    //set test as running to show animation
    dispatch(
      setNormalTestsForTutorAreRunningAndResetResult(runningTests, getState().tutorViewSiteState.normalTestsForTutor))

    //run the test
    await dispatch(_runNormalTestAsTutor(runTestAsTutorCommand))


    const ranTests = runTestAsTutorCommand.testIds
      .map<RunnableExerciseTestFrontendExclusive>(testId =>
        getState().tutorViewSiteState.normalTestsForTutor
          .find(test =>
            test.id === testId
          )
      )

    if (ranTests.length === 0) {

      warningDialog(getI18n(globalState.getState().i18nState.langId,'Warning'),
        getI18n(globalState.getState().i18nState.langId,'Probably no test has run'), globalState.getState().i18nState.langId
        )
      return true
    }

    const firstFailedTest = ranTests.find(p => !p || !p.result || !p.result.hasCompiled || !p.result.passed)

    if (firstFailedTest) {
      //select the first failed test
      dispatch(setSelectedConsoleViewTest(firstFailedTest))
    } else {
      //select the last test
      dispatch(setSelectedConsoleViewTest(ranTests[ranTests.length - 1]))
    }

    return firstFailedTest !== undefined
  }
}

export function _runNormalTestAsTutor(runTestAsTutorCommand: RunTutorTestCommandForBackend): RUN_normalTestAsTutorAction {
  return {
    type: ActionType.RUN_normalTestAsTutor,
    payload: delay(runTestAsTutor(runTestAsTutorCommand))
  }
}

/**
 *
 * @param runTestAsTutorCommand
 * @returns [hasAnyTestFailed] true: some test has not compiled or not passed, false: all tests passed
 */
export function runSubmitTestAsTutor(runTestAsTutorCommand: RunTutorTestCommandForBackend): AwaitActionsWithResult<boolean> {
  return async (dispatch, getState) => {

    //if we run the same test twice and the output won't change then this looks like nothing happened...
    //set console to blank so that the user sees that something happened
    dispatch(setSelectedConsoleViewTest(null))


    const runningTests = runTestAsTutorCommand.testIds
      .map<AssessedSubmitTestFrontendExclusive>(testId =>
        //don't use allTests because saveDoExerciseSolution could have reset the test (results) and allTest would be an old array
        getState().tutorViewSiteState.submitTestsForTutor
          .find(test =>
            test.id === testId
          )
      )

    //set test as running to show animation
    dispatch(
      setSubmitTestsForTutorAreRunningAndResetResult(runningTests, getState().tutorViewSiteState.submitTestsForTutor))

    //run the test
    await dispatch(_runSubmitTestAsTutor(runTestAsTutorCommand))

    const ranTests = runTestAsTutorCommand.testIds
      .map<AssessedSubmitTestFrontendExclusive>(testId =>
        getState().tutorViewSiteState.submitTestsForTutor
          .find(test =>
            test.id === testId
          )
      )

    if (ranTests.length === 0) {

      warningDialog(getI18n(globalState.getState().i18nState.langId,'Warning'),
        getI18n(globalState.getState().i18nState.langId,'Probably no test has run'), globalState.getState().i18nState.langId
      )

      return true
    }

    const firstFailedTest = ranTests.find(p => !p || !p.result || !p.result.hasCompiled || !p.result.passed)

    if (firstFailedTest) {
      //select the first failed test
      dispatch(setSelectedConsoleViewTest(firstFailedTest))
    } else {
      //select the last test
      dispatch(setSelectedConsoleViewTest(ranTests[ranTests.length - 1]))
    }

    return firstFailedTest !== undefined
  }
}

export function _runSubmitTestAsTutor(runTestAsTutorCommand: RunTutorTestCommandForBackend): RUN_submitTestAsTutorAction {
  return {
    type: ActionType.RUN_submitTestAsTutor,
    payload: delay(runTestAsTutor(runTestAsTutorCommand))
  }
}


export function _runCompileSingleFileTutorView(runCompileSingleFileCommand: RunCompileSingleFileCommandForBackend): RUN_compileSingleFileAction {
  return {
    type: ActionType.RUN_compileSingleFile,
    payload: delay(compileSingleFileTutorView(runCompileSingleFileCommand))
  }
}

export function runCompileSingleFileTutorView(runCompileSingleFileCommand: RunCompileSingleFileCommandForBackend, testTypes: ReadonlyArray<TestTypeFullBase>): AwaitActions {
  return async (dispatch, getState) => {

    const compileTestType = testTypes.find(p => p.internalName === KnownInternalTestTypes.compileTest)

    if (!compileTestType) {

      errorDialog(getI18n(globalState.getState().i18nState.langId,'Error'),
                  getI18n(globalState.getState().i18nState.langId,'Unknown test type'),globalState.getState().i18nState.langId)

      return
    }

    await dispatch(_runCompileSingleFileTutorView(runCompileSingleFileCommand))

    const compileResult = getState().tutorViewSiteState.lastCompileSingleFileResult

    const tempTest: RunnableExerciseTestFrontendExclusive = {
      id: -1,
      displayName: 'Compile Single File',
      result: compileResult,
      content: '',
      uiIsRunning: false,
      files: [],
      displayIndex: -1,
      maxPoints: 0,
      isSubmitTest: false,
      uiIsAfterTest: false,
      testSettings: {
        id: -1,
        maxDiskSpaceInKb: -1,
        timeoutInMs: -1,
        memoryLimitInKb: -1,
        compileTimeoutInMs: -1,
        compilerOptions: '',
      },
      testTypeId: compileTestType.id,
      type: 'tmp'
    }

    dispatch(setSelectedConsoleViewTest(tempTest))

  }
}

export function _justRunProgramDoExercise(justRunProgramCommand: JustRunProgramTestTutorViewCommandForBackend): JUST_RUN_programAction {
  return {
    type: ActionType.JUST_RUN_program,
    payload: delay(justRunProgramTestTutorView(justRunProgramCommand))
  }
}


export function justRunProgramTutorViewAsync(justRunProgramCommand: JustRunProgramTestTutorViewCommandForBackend): AwaitActions {
  return async (dispatch, getState) => {


    await dispatch(_justRunProgramDoExercise(justRunProgramCommand))

    const testResult = getState().tutorViewSiteState.lastJustRunProgramResult

    const tempTest: RunnableExerciseTestFrontendExclusive = {
      id: -1,
      displayName: 'Just Run Program',
      result: testResult,
      content: '',
      uiIsRunning: false,
      files: [],
      displayIndex: -1,
      maxPoints: 0,
      isSubmitTest: false,
      uiIsAfterTest: false,
      testSettings: {
        id: -1,
        maxDiskSpaceInKb: -1,
        timeoutInMs: -1,
        memoryLimitInKb: -1,
        compileTimeoutInMs: -1,
        compilerOptions: '',
      },
      testTypeId: -1,
      type: 'tmp',
    }

    dispatch(setSelectedConsoleViewTest(tempTest))

  }
}
