import {Action, Reducer} from "redux";

/**
 * the idea is good but we only get generic arguments when calling actions/reducers (arg1, arg2, ...)
 * which is not that nice...
 */

//we already handled undefined in: (state: S, action: A) => S;
export type SimpleReducer<S, A extends Action> = (state: S, action: A) => S;


export type ActionCreator<S, TOut> = {
  readonly __kind: 'normal'
  readonly type: string
  readonly reducer: SimpleReducer<S, TOut & Action>
  (): { readonly type: string } & TOut
}

export  type ActionCreator1<S, Arg1, TOut> = {
  readonly __kind: 'normal'
  readonly type: string
  readonly reducer: SimpleReducer<S, TOut & Action>
  (arg1: Arg1): { readonly type: string } & TOut
}

export type ActionCreator2<S, Arg1, Arg2, TOut> = {
  readonly __kind: 'normal'
  readonly type: string
  readonly reducer: SimpleReducer<S, TOut & Action>
  (arg1: Arg1, arg2: Arg2): { readonly type: string } & TOut
}

export  type ActionCreator3<S, Arg1, Arg2, Arg3, TOut> = {
  readonly __kind: 'normal'
  readonly type: string
  readonly reducer: SimpleReducer<S, TOut & Action>
  (arg1: Arg1, arg2: Arg2, arg3: Arg3): { readonly type: string } & TOut
}

export type ActionCreator4<S, Arg1, Arg2, Arg3, Arg4, TOut> = {
  readonly __kind: 'normal'
  readonly type: string
  readonly reducer: SimpleReducer<S, TOut & Action>
  (arg1: Arg1, arg2: Arg2, arg3: Arg3, arg4: Arg4): { readonly type: string } & TOut
}

export type ActionCreator5<S, Arg1, Arg2, Arg3, Arg4, Arg5, TOut> = {
  readonly __kind: 'normal'
  readonly type: string
  readonly reducer: SimpleReducer<S, TOut & Action>
  (arg1: Arg1, arg2: Arg2, arg3: Arg3, arg4: Arg4, arg5: Arg5): { readonly type: string } & TOut
}

export  type AllActionCreators<S> =
  ActionCreator<S, any>
  | ActionCreator1<S, any, any>
  | ActionCreator2<S, any, any, any>
  | ActionCreator3<S, any, any, any, any>
  | ActionCreator4<S, any, any, any, any, any>
  | ActionCreator5<S, any, any, any, any, any, any>


//initial state to infer S and we don't need to specify the State type in the reducer func
//one could use initial state to check that we only used defined props at runtime... but overhead
export function createAction<S, TOut>(type: string, payload: () => TOut, reducer: (state: S, action: TOut) => S): ActionCreator<S, TOut>
export function createAction<S, Arg1, TOut>(type: string, payload: (arg1: Arg1) => TOut, reducer: (state: S, action: TOut) => S): ActionCreator1<S, Arg1, TOut>
export function createAction<S, Arg1, Arg2, TOut>(type: string, payload: (arg1: Arg1, arg2: Arg2) => TOut, reducer: (state: S, action: TOut) => S): ActionCreator2<S, Arg1, Arg2, TOut>
export function createAction<S, Arg1, Arg2, Arg3, TOut>(type: string, payload: (arg1: Arg1, arg2: Arg2, arg3: Arg3) => TOut, reducer: (state: S, action: TOut) => S): ActionCreator3<S, Arg1, Arg2, Arg3, TOut>
export function createAction<S, Arg1, Arg2, Arg3, Arg4, TOut>(type: string, payload: (arg1: Arg1, arg2: Arg2, arg3: Arg3, arg4: Arg4) => TOut, reducer: (state: S, action: TOut) => S): ActionCreator4<S, Arg1, Arg2, Arg3, Arg4, TOut>
export function createAction<S, Arg1, Arg2, Arg3, Arg4, Arg5, TOut>(type: string, payload: (arg1: Arg1, arg2: Arg2, arg3: Arg3, arg4: Arg4, arg5: Arg5) => TOut, reducer: (state: S, action: TOut) => S): ActionCreator5<S, Arg1, Arg2, Arg3, Arg4, Arg5, TOut> {

  // noinspection TsLint
  const aCreator = (...args: any[]) => {
    // noinspection TsLint
    return Object.assign({type}, (payload as any)(...args))
  }

  // noinspection TsLint
  return Object.assign(aCreator, {type, reducer, __kind: 'normal' as any})

}


//old we now support crud reducers

// noinspection TsLint
// export function createReducer<S>(initialState: S, ...actionCreators: AllActionCreators<S>[]): Reducer<S, Action> {
//
//   function reducer(state: S | undefined, action: Action): S {
//
//     if (state === undefined) return initialState
//
//     // noinspection TsLint
//     for (let i = 0; i < actionCreators.length; i++) {
//       if (actionCreators[i].type === action.type) {
//         return actionCreators[i].reducer(state, action)
//       }
//     }
//     return state
//   }
//
//   return reducer
// }


// noinspection TsLint
export function createReducer<S>(initialState: S, ...actionCreators: Array<AllActionCreators<S> | AllCrudActionCreators<S>>): Reducer<S, Action> {

  function reducer(state: S | undefined, action: Action): S {

    if (state === undefined) return initialState

    // noinspection TsLint
    for (let i = 0; i < actionCreators.length; i++) {

      const actionCreator = actionCreators[i]

      if (actionCreator.__kind === 'normal') {
        if (action.type === actionCreator.type) {
          return actionCreator.reducer(state, action)
        }
      }
      else if (actionCreator.__kind === 'crud') {

        //type is different...
        if (action.type === actionCreator.typePending) {
          return actionCreator.pendingHandler(state, action as any) //lets hope type is PendingCurdAction...
        }
        else if (action.type === actionCreator.typeFullFilled) {
          return actionCreator.fullFilledHandler(state, action as any)

        }
        //or use else if ??
        // else if (action.type === actionCreator.typeRejected) {
        return actionCreator.rejectedHandler(state, action as any)
        // }

      }
      else {
        throw new Error('unknown reducer type, ' + actionCreator)
      }

    }
    return state
  }

  return reducer
}


//---- crud reducers

export type CurdAction<T, Meta = void> = {
  // readonly type: string //this is set automatically
  readonly payload: Promise<T>
  readonly meta?: Meta
}

export type PendingCurdAction<T, Meta = void> = {
  readonly type: string //will be type_PENDING
  readonly payload: Promise<T>
  readonly meta?: Meta
}

export type FullFilledCurdAction<T, Meta = void> = {
  readonly type: string //will be type_FULLFILLED
  readonly payload: T
  readonly meta?: Meta
}

//TError is not used at all
export type RejectedCurdAction<Meta = void> = {
  readonly type: string //will be type_REJECTED
  readonly payload: Error
  readonly meta?: Meta
  readonly error: true

}

export type CrudActionCreator<S, TOut> = {
  readonly __kind: 'crud'

  readonly type: string
  readonly typePending: string
  readonly typeFullFilled: string
  readonly typeRejected: string

  (): { readonly type: string } & TOut

  readonly pendingHandler: SimpleReducer<S, PendingCurdAction<TOut>>
  readonly fullFilledHandler: SimpleReducer<S, FullFilledCurdAction<TOut>>
  readonly rejectedHandler: SimpleReducer<S, RejectedCurdAction>
}

export type CrudActionCreator1<S, Arg1, TOut> = {
  readonly __kind: 'crud'
  readonly type: string
  readonly typePending: string
  readonly typeFullFilled: string
  readonly typeRejected: string

  (arg1: Arg1): { readonly type: string } & TOut

  readonly pendingHandler: SimpleReducer<S, PendingCurdAction<TOut>>
  readonly fullFilledHandler: SimpleReducer<S, FullFilledCurdAction<TOut>>
  readonly rejectedHandler: SimpleReducer<S, RejectedCurdAction>
}

export type CrudActionCreator2<S, Arg1, Arg2, TOut> = {
  readonly __kind: 'crud'
  readonly type: string
  readonly typePending: string
  readonly typeFullFilled: string
  readonly typeRejected: string

  (arg1: Arg1, arg2: Arg2): { readonly type: string } & TOut

  readonly pendingHandler: SimpleReducer<S, PendingCurdAction<TOut>>
  readonly fullFilledHandler: SimpleReducer<S, FullFilledCurdAction<TOut>>
  readonly rejectedHandler: SimpleReducer<S, RejectedCurdAction>
}

export type CrudActionCreator3<S, Arg1, Arg2, Arg3, TOut> = {
  readonly __kind: 'crud'
  readonly type: string
  readonly typePending: string
  readonly typeFullFilled: string
  readonly typeRejected: string

  (arg1: Arg1, arg2: Arg2, arg3: Arg3): { readonly type: string } & TOut

  readonly pendingHandler: SimpleReducer<S, PendingCurdAction<TOut>>
  readonly fullFilledHandler: SimpleReducer<S, FullFilledCurdAction<TOut>>
  readonly rejectedHandler: SimpleReducer<S, RejectedCurdAction>
}

export type CrudActionCreator4<S, Arg1, Arg2, Arg3, Arg4, TOut> = {
  readonly __kind: 'crud'
  readonly type: string
  readonly typePending: string
  readonly typeFullFilled: string
  readonly typeRejected: string

  (arg1: Arg1, arg2: Arg2, arg3: Arg3, arg4: Arg4): { readonly type: string } & TOut

  readonly pendingHandler: SimpleReducer<S, PendingCurdAction<TOut>>
  readonly fullFilledHandler: SimpleReducer<S, FullFilledCurdAction<TOut>>
  readonly rejectedHandler: SimpleReducer<S, RejectedCurdAction>
}

export type CrudActionCreator5<S, Arg1, Arg2, Arg3, Arg4, Arg5, TOut> = {
  readonly __kind: 'crud'
  readonly type: string
  readonly typePending: string
  readonly typeFullFilled: string
  readonly typeRejected: string

  (arg1: Arg1, arg2: Arg2, arg3: Arg3, arg4: Arg4, arg5: Arg5): { readonly type: string } & TOut

  readonly pendingHandler: SimpleReducer<S, PendingCurdAction<TOut>>
  readonly fullFilledHandler: SimpleReducer<S, FullFilledCurdAction<TOut>>
  readonly rejectedHandler: SimpleReducer<S, RejectedCurdAction>
}

export  type AllCrudActionCreators<S, TMeta = void> =
  CrudActionCreator<S, any>
  | CrudActionCreator1<S, any, any>
  | CrudActionCreator2<S, any, any, any>
  | CrudActionCreator3<S, any, any, any, any>
  | CrudActionCreator4<S, any, any, any, any, any>
  | CrudActionCreator5<S, any, any, any, any, any, any>


export function createCrudAction<S, TMeta = void, TOut = {}>(
                                                             type: string,
                                                             createPayload: () => CurdAction<TOut, TMeta>,
                                                             pendingHandler: (state: S, action: PendingCurdAction<TOut, TMeta>) => S,
                                                             fullFilledHandler: (state: S, action: FullFilledCurdAction<TOut, TMeta>) => S,
                                                             rejectedHandler: (state: S, action: RejectedCurdAction<TMeta>) => S
): CrudActionCreator<S, TOut>

export function createCrudAction<S, Arg1, TMeta = void, TOut = {}>(
                                                                   type: string,
                                                                   createPayload: (arg1: Arg1) => CurdAction<TOut, TMeta>,
                                                                   pendingHandler: (state: S, action: PendingCurdAction<TOut, TMeta>) => S,
                                                                   fullFilledHandler: (state: S, action: FullFilledCurdAction<TOut, TMeta>) => S,
                                                                   rejectedHandler: (state: S, action: RejectedCurdAction<TMeta>) => S
): CrudActionCreator1<S, Arg1, TOut>

export function createCrudAction<S, Arg1, Arg2, TMeta = void, TOut = {}>(
                                                                         type: string,
                                                                         createPayload: (arg1: Arg1, arg2: Arg2) => CurdAction<TOut, TMeta>,
                                                                         pendingHandler: (state: S, action: PendingCurdAction<TOut, TMeta>) => S,
                                                                         fullFilledHandler: (state: S, action: FullFilledCurdAction<TOut, TMeta>) => S,
                                                                         rejectedHandler: (state: S, action: RejectedCurdAction<TMeta>) => S
): CrudActionCreator2<S, Arg1, Arg2, TOut>

export function createCrudAction<S, Arg1, Arg2, Arg3, TMeta = void, TOut = {}>(
                                                                               type: string,
                                                                               createPayload: (arg1: Arg1, arg2: Arg2, arg3: Arg3) => CurdAction<TOut, TMeta>,
                                                                               pendingHandler: (state: S, action: PendingCurdAction<TOut, TMeta>) => S,
                                                                               fullFilledHandler: (state: S, action: FullFilledCurdAction<TOut, TMeta>) => S,
                                                                               rejectedHandler: (state: S, action: RejectedCurdAction<TMeta>) => S
): CrudActionCreator3<S, Arg1, Arg2, Arg3, TOut>

export function createCrudAction<S, Arg1, Arg2, Arg3, Arg4, TMeta = void, TOut = {}>(
                                                                                     type: string,
                                                                                     createPayload: (arg1: Arg1, arg2: Arg2, arg3: Arg3, arg4: Arg4) => CurdAction<TOut, TMeta>,
                                                                                     pendingHandler: (state: S, action: PendingCurdAction<TOut, TMeta>) => S,
                                                                                     fullFilledHandler: (state: S, action: FullFilledCurdAction<TOut, TMeta>) => S,
                                                                                     rejectedHandler: (state: S, action: RejectedCurdAction<TMeta>) => S
): CrudActionCreator4<S, Arg1, Arg2, Arg3, Arg4, TOut>

export function createCrudAction<S, Arg1, Arg2, Arg3, Arg4, Arg5, TMeta = void, TOut = {}>(
                                                                                           type: string,
                                                                                           createPayload: (arg1: Arg1, arg2: Arg2, arg3: Arg3, arg4: Arg4, arg5: Arg5) => CurdAction<TOut, TMeta>,
                                                                                           pendingHandler: (state: S, action: PendingCurdAction<TOut, TMeta>) => S,
                                                                                           fullFilledHandler: (state: S, action: FullFilledCurdAction<TOut, TMeta>) => S,
                                                                                           rejectedHandler: (state: S, action: RejectedCurdAction<TMeta>) => S
): CrudActionCreator5<S, Arg1, Arg2, Arg3, Arg4, Arg5, TOut> {

  //this will call the actual action as a normal function: setX() and setX.type
  // noinspection TsLint
  const aCreator = (...args: any[]) => {
    // noinspection TsLint
    return Object.assign({type}, (createPayload as any)(...args))
  }

  // noinspection TsLint
  return Object.assign(aCreator, {
      type,
      typePending: `${type}_PENDING`,
      typeFullFilled: `${type}_FULFILLED`,
      typeRejected: `${type}_REJECTED`,
      pendingHandler: pendingHandler as any,
      fullFilledHandler: fullFilledHandler as any,
      rejectedHandler: rejectedHandler as any,
      __kind: 'crud' as any
    }
  )

}


///---- curd old

/**
 * used for everything else in CRUD than reading (reading is ReduxPromiseAction)
 */
export interface ReduxPromiseBaseAction<T, TMeta> {
  readonly payload: Promise<T>
  readonly meta?: TMeta //this is not really well documented ... see http://redux.js.org/docs/advanced/Middleware.html
}

/**
 * use this for the fetch actions (server communication)
 * this action is not in the reducer case because this doesn't get triggered
 * instead the redux promise middleware dispatch the [originalDispatchedName]_PENDING action
 *
 * so the types to use in the switch case in the reducer are:
 *
 * TPending: the pending type (just to ensure we not missed to implement this)
 * TFulfilled: the fulfilled type (just to ensure we not missed to implement this)
 * TRejected: the fulfilled type (just to ensure we not missed to implement this)
 *
 * however the actions (that are called in the components) only implement an action for [originalDispatchedName]
 * because the TPending, TFulfilled and TRejected are dispatched automatically
 *
 * usage:
 *
 * export interface GET_systemRolesAction
 *  extends ActionBase, ReduxPromiseAction<ReadonlyArray<SystemRoleFullBase>, GET_systemRoles_FULFILLEDAction, GET_systemRoles_REJECTEDAction> {
 *   readonly type: ActionType.GET_systemRoles
 *   readonly payload: Promise<ReadonlyArray<SystemRoleFullBase>>
 * }
 *
 * for GET_systemRoles_FULFILLEDAction and GET_systemRoles_REJECTEDAction see below
 */
export interface ReduxPromiseAction<T,
  TPending extends ReduxPromisePendingAction<TMeta>,
  TFulfilled extends ReduxPromiseFulfilledAction<T, TMeta>,
  TRejected extends ReduxPromiseRejectedAction<T, TMeta>,
  TMeta = void>
  extends ReduxPromiseBaseAction<T, TMeta> {
}

export interface ReduxPromisePendingAction<U> {
  readonly type: string //this is only used that typescript error goes away...
  readonly meta?: U
}

/**
 * use this for the fulfilled part of ReduxPromiseAction<T>
 * this is automatically dispatched by redux promise middleware
 *
 * usage:
 *
 * export interface GET_systemRoles_FULFILLEDAction
 *   extends ActionBase, ReduxPromiseFulfilledAction<ReadonlyArray<SystemRoleFullBase>> {
 * readonly type: ActionType.GET_systemRoles_FULFILLED
 * readonly payload: ReadonlyArray<SystemRoleFullBase>
 * }
 *
 */
export interface ReduxPromiseFulfilledAction<T, TMeta = void> {
  readonly payload: T
  readonly meta?: TMeta
}

/**
 *
 * use this for the rejected part of ReduxPromiseAction<T>
 * this is automatically dispatched by redux promise middleware
 *
 *
 * usage:
 *
 * export interface GET_systemRoles_REJECTEDAction
 *   extends ActionBase, ReduxPromiseRejectedAction<ReadonlyArray<SystemRoleFullBase>> {
 *  readonly type: ActionType.GET_systemRoles_REJECTED
 *  readonly payload: any
 * }
 */
export interface ReduxPromiseRejectedAction<T, TMeta = void> {
  readonly error: true
  readonly payload: any
  readonly meta?: TMeta
}
