/**
 * Created by janis dähne.
 */
import {notExhaustive} from "../../_notExhausiveHelper";
import {ExercisePreviewFromBackend} from "../../../../types/exercisePreview";
import {ActionBase, ActionType} from "./closedExercisesSiteActionTypes";
import {FrontendSettingsManager} from "../../../../helpers/frontendSettingsManager";
import {PaginationStateData} from '../../../../types/pagination'
import {getInitialPaginationData} from '../../../../constants'
import {
  AllActions as GetClosedExercisesViaCodeCrudActions,
  reducer as getClosedExercisesViaCodeCrudReducer
} from './getClosedExercisesViaCodeCrudReducer'
import {
  AllActions as GetClosedExercisesViaVisibilityCrudActions,
  reducer as getClosedExercisesViaVisibilityCrudReducer
} from './getClosedExercisesViaVisibilityCrudReducer'
import {ActionTypeGlobal, RESET_GlobalAction} from '../../globalActions'
// import {ActionType, RESET_GlobalAction} from '../../globalActions'


export type State = {

  readonly closedExercisePreviewsViaCode: ReadonlyArray<ExercisePreviewFromBackend>
  readonly paginationViaCode: PaginationStateData<ExercisePreviewFromBackend>
  readonly isLoadingViaCode: boolean

  readonly closedExercisePreviewsViaVisibility: ReadonlyArray<ExercisePreviewFromBackend>
  readonly paginationViaVisibility: PaginationStateData<ExercisePreviewFromBackend>
  readonly isLoadingViaVisibility: boolean

  readonly isLoading: boolean


  readonly isTagsFilterDisplayed: boolean
}

export const initial: State = {

  closedExercisePreviewsViaCode: [],
  paginationViaCode: {
    ...getInitialPaginationData(),
    orderByProp: 'lastUpdatedAt',
    isOrderByDesc: true
  },
  isLoadingViaCode: false,

  closedExercisePreviewsViaVisibility: [],
  paginationViaVisibility: {
    ...getInitialPaginationData(),
    orderByProp: 'displayName'
  },
  isLoadingViaVisibility: false,

  isLoading: false,

  isTagsFilterDisplayed: false,
}

//---pagination
export interface SetSearchTextAction extends ActionBase {
  readonly type: ActionType.setSearchText
  readonly searchText: string
  readonly viaCode: boolean
}

export interface SetActivePaginationPageAction extends ActionBase {
  readonly type: ActionType.setPaginationActivePage
  readonly activePage: number
  readonly viaCode: boolean
}

export interface SetPaginationPageSizeAction extends ActionBase {
  readonly type: ActionType.setPaginationPageSize
  readonly pageSize: number
  readonly viaCode: boolean
}

//---pagination end

export interface SET_sortByKeyAction extends ActionBase {
  readonly type: ActionType.SET_sortByKey
  readonly sortByKey: keyof ExercisePreviewFromBackend
  readonly sortDirection: boolean | null
  readonly viaCode: boolean
}

export interface SET_isTagsFilterDisplayedAction extends ActionBase {
  readonly type: ActionType.SET_isTagsFilterDisplayed
  readonly isTagsFilterDisplayed: boolean
}

export interface ResetAction extends ActionBase {
  readonly type: ActionType.RESET
}

export type AllActions =
  ResetAction
  | SET_sortByKeyAction
  | SET_isTagsFilterDisplayedAction
  | SetSearchTextAction
  | SetActivePaginationPageAction
  | SetPaginationPageSizeAction

  | GetClosedExercisesViaCodeCrudActions
  | GetClosedExercisesViaVisibilityCrudActions

  | RESET_GlobalAction

export function reducer(state: State = initial, action: AllActions): State {

  switch (action.type) {

    case ActionType.GET_closedExercisesViaCode_PENDING:
    case ActionType.GET_closedExercisesViaCode_FULFILLED:
    case ActionType.GET_closedExercisesViaCode_REJECTED:
      return getClosedExercisesViaCodeCrudReducer(state, action)

    case ActionType.GET_closedExercisesViaVisibility_PENDING:
    case ActionType.GET_closedExercisesViaVisibility_FULFILLED:
    case ActionType.GET_closedExercisesViaVisibility_REJECTED:
      return getClosedExercisesViaVisibilityCrudReducer(state, action)

    case ActionType.SET_isTagsFilterDisplayed: {

      FrontendSettingsManager.setAllClosedExercisesSiteTagsFilterIsDisplayed(action.isTagsFilterDisplayed)

      return {
        ...state,
        isTagsFilterDisplayed: action.isTagsFilterDisplayed
      }
    }

    case ActionType.SET_sortByKey: {
      if (action.viaCode) {
        return {
          ...state,
          paginationViaCode: {
            ...state.paginationViaCode,
            orderByProp: action.sortByKey,
            isOrderByDesc: action.sortDirection
          }
        }
      }

      return {
        ...state,
        paginationViaVisibility: {
          ...state.paginationViaVisibility,
          orderByProp: action.sortByKey,
          isOrderByDesc: action.sortDirection
        }
      }
    }

    case ActionType.setSearchText: {

      if (action.viaCode) {
        return {
          ...state,
          paginationViaCode: {
            ...state.paginationViaCode,
            searchText: action.searchText
          }
        }
      }
      return {
        ...state,
        paginationViaVisibility: {
          ...state.paginationViaVisibility,
          searchText: action.searchText
        }
      }
    }
    case ActionType.setPaginationActivePage: {

      if (action.viaCode) {
        return {
          ...state,
          paginationViaCode: {
            ...state.paginationViaCode,
            currentPage: action.activePage
          }
        }
      }
      return {
        ...state,
        paginationViaVisibility: {
          ...state.paginationViaVisibility,
          currentPage: action.activePage
        }
      }
    }

    case ActionType.setPaginationPageSize: {

      if (action.viaCode) {
        FrontendSettingsManager.set_closedExercisesViaCodePageSize(action.pageSize)
        return {
          ...state,
          paginationViaCode: {
            ...state.paginationViaCode,
            pageSize: action.pageSize,
            currentPage: 1,
          }
        }
      }

      FrontendSettingsManager.set_closedExercisesViaVisibilityPageSize(action.pageSize)
      return {
        ...state,
        paginationViaVisibility: {
          ...state.paginationViaVisibility,
          pageSize: action.pageSize
        }
      }
    }


    case ActionType.RESET:
    case ActionTypeGlobal.RESET_Global:
      return {
        ...initial,
        isTagsFilterDisplayed: FrontendSettingsManager.getFrontendSettings().allClosedExercisesSiteTagsFilterIsDisplayed,
        paginationViaCode: {
          ...initial.paginationViaCode,
          pageSize: FrontendSettingsManager.getFrontendSettings().closedExercisesViaCodePageSize
        },
        paginationViaVisibility: {
          ...initial.paginationViaVisibility,
          pageSize: FrontendSettingsManager.getFrontendSettings().closedExercisesViaVisibilityPageSize
        }
      }

    default:
      notExhaustive(action)
      return state
  }
}

