import {ActionBase, ActionType} from './closedExercisesSiteActionTypes'
import {initial, State} from './closedExercisesSiteReducer'
import {PaginatedData} from '../../../../types/pagination'
import {ExercisePreviewFromBackend} from '../../../../types/exercisePreview'
import {PaginationHelper} from '../../../../helpers/paginationHelper'
import {ReduxPromiseAction, ReduxPromiseFulfilledAction, ReduxPromiseRejectedAction} from '../../index'
import {notExhaustive} from '../../_notExhausiveHelper'


export interface GET_closedExercisesViaCodeAction
  extends ActionBase, ReduxPromiseAction<PaginatedData<ExercisePreviewFromBackend>, GET_closedExercisesViaCode_PENDINGAction, GET_closedExercisesViaCode_FULFILLEDAction, GET_closedExercisesViaCode_REJECTEDAction> {
  readonly type: ActionType.GET_closedExercisesViaCode
  readonly payload: Promise<PaginatedData<ExercisePreviewFromBackend>>
}

//automatically dispatched by redux promise middleware
export interface GET_closedExercisesViaCode_PENDINGAction
  extends ActionBase {
  readonly type: ActionType.GET_closedExercisesViaCode_PENDING
}

//automatically dispatched by redux promise middleware
export interface GET_closedExercisesViaCode_FULFILLEDAction
  extends ActionBase, ReduxPromiseFulfilledAction<PaginatedData<ExercisePreviewFromBackend>> {
  readonly type: ActionType.GET_closedExercisesViaCode_FULFILLED
  readonly payload: PaginatedData<ExercisePreviewFromBackend>
}

//automatically dispatched by redux promise middleware
export interface GET_closedExercisesViaCode_REJECTEDAction
  extends ActionBase, ReduxPromiseRejectedAction<void> {
  readonly type: ActionType.GET_closedExercisesViaCode_REJECTED
  readonly payload: any
}


export type AllActions =
  GET_closedExercisesViaCode_PENDINGAction
  | GET_closedExercisesViaCode_FULFILLEDAction
  | GET_closedExercisesViaCode_REJECTEDAction


export function reducer(state: State = initial, action: AllActions): State {

  switch (action.type) {

    case ActionType.GET_closedExercisesViaCode_PENDING:
      return {
        ...state,
        isLoadingViaCode: true
      }
    case ActionType.GET_closedExercisesViaCode_FULFILLED:
      return {
        ...state,
        isLoadingViaCode: false,
        closedExercisePreviewsViaCode: action.payload.items,
        paginationViaCode: PaginationHelper.getPaginationStateSpreadObj(action.payload),
      }
    case ActionType.GET_closedExercisesViaCode_REJECTED:
      return {
        ...state,
        isLoadingViaCode: false,
        closedExercisePreviewsViaCode: [],
      }

    default:
      notExhaustive(action)
      return state
  }
}

