
import {ExercisePreviewFromBackend} from '../../../../types/exercisePreview'
import {PaginatedData} from '../../../../types/pagination'
import {ActionBase, ActionType} from './openExercisesSiteActionTypes'
import {initial, State} from './openExercisesSiteReducer'
import {ReduxPromiseAction, ReduxPromiseFulfilledAction, ReduxPromiseRejectedAction} from '../../index'
import {notExhaustive} from '../../_notExhausiveHelper'
import {PaginationHelper} from '../../../../helpers/paginationHelper'
import {convertOpenExercisePreviewToFrontendOpenExercisePreview} from '../../../../helpers/convertersAndTransformers'


export interface GET_openExercisesViaVisibilityAction
  extends ActionBase, ReduxPromiseAction<PaginatedData<ExercisePreviewFromBackend>, GET_openExercisesViaVisibility_PENDINGAction, GET_openExercisesViaVisibility_FULFILLEDAction, GET_openExercisesViaVisibility_REJECTEDAction> {
  readonly type: ActionType.GET_openExercisesViaVisibility
  readonly payload: Promise<PaginatedData<ExercisePreviewFromBackend>>
}

//automatically dispatched by redux promise middleware
export interface GET_openExercisesViaVisibility_PENDINGAction
  extends ActionBase {
  readonly type: ActionType.GET_openExercisesViaVisibility_PENDING
}

//automatically dispatched by redux promise middleware
export interface GET_openExercisesViaVisibility_FULFILLEDAction
  extends ActionBase, ReduxPromiseFulfilledAction<PaginatedData<ExercisePreviewFromBackend>> {
  readonly type: ActionType.GET_openExercisesViaVisibility_FULFILLED
  readonly payload: PaginatedData<ExercisePreviewFromBackend>
}

//automatically dispatched by redux promise middleware
export interface GET_openExercisesViaVisibility_REJECTEDAction
  extends ActionBase, ReduxPromiseRejectedAction<void> {
  readonly type: ActionType.GET_openExercisesViaVisibility_REJECTED
  readonly payload: any
}


export type AllActions =
  GET_openExercisesViaVisibility_PENDINGAction
  | GET_openExercisesViaVisibility_FULFILLEDAction
  | GET_openExercisesViaVisibility_REJECTEDAction


export function reducer(state: State = initial, action: AllActions): State {

  switch (action.type) {

    case ActionType.GET_openExercisesViaVisibility_PENDING:
      return {
        ...state,
        isLoadingViaVisibility: true
      }
    case ActionType.GET_openExercisesViaVisibility_FULFILLED:
      return {
        ...state,
        openExercisePreviewsViaVisibility: convertOpenExercisePreviewToFrontendOpenExercisePreview(action.payload.items),
        paginationViaVisibility: PaginationHelper.getPaginationStateSpreadObj(action.payload),
        isLoadingViaVisibility: false
      }
    case ActionType.GET_openExercisesViaVisibility_REJECTED:
      return {
        ...state,
        isLoadingViaVisibility: true,
        openExercisePreviewsViaVisibility: []
      }

    default:
      notExhaustive(action)
      return state
  }
}

