/**
 * Created by janis dähne.
 */
import {notExhaustive} from "../../_notExhausiveHelper";
import {ExercisePreviewFromBackend} from "../../../../types/exercisePreview";
import {ActionBase, ActionType} from "./openExercisesSiteActionTypes";
import {ExercisePreviewsFrontendExclusive} from "../../../../components/sites/allAccessibleExercisesSite/openExercises/allOpenExercisesSite";
import {FrontendSettingsManager} from "../../../../helpers/frontendSettingsManager";
import {
  AllActions as GetOpenExercisesViaCodeCrudActions,
  reducer as getOpenExercisesViaCodeCrudReducer
} from './getOpenExercisesViaCodeCrudReducer'
import {
  AllActions as GetClosedExercisesViaVisibilityCrudActions,
  reducer as getClosedExercisesViaVisibilityCrudReducer
} from './getOpenExercisesViaVisibilityCrudReducer'
import {PaginationStateData} from '../../../../types/pagination'
import {getInitialPaginationData} from '../../../../constants'
import {ActionTypeGlobal, RESET_GlobalAction} from '../../globalActions'


export type State = {

  readonly openExercisePreviewsViaCode: ReadonlyArray<ExercisePreviewsFrontendExclusive>
  readonly paginationViaCode: PaginationStateData<ExercisePreviewFromBackend>
  readonly isLoadingViaCode: boolean


  readonly openExercisePreviewsViaVisibility: ReadonlyArray<ExercisePreviewsFrontendExclusive>
  readonly paginationViaVisibility: PaginationStateData<ExercisePreviewFromBackend>
  readonly isLoadingViaVisibility: boolean

  readonly isLoading: boolean
  readonly isEnterReleaseCodeDialogDisplayed: boolean

  readonly isTagsFilterDisplayed: boolean
}

export const initial: State = {

  openExercisePreviewsViaCode: [],
  paginationViaCode: {
    ...getInitialPaginationData(),
    orderByProp: 'lastUpdatedAt',
    isOrderByDesc: true
  },
  isLoadingViaCode: false,
  openExercisePreviewsViaVisibility: [],
  paginationViaVisibility: {
    ...getInitialPaginationData(),
    orderByProp: 'displayName'
  },
  isLoadingViaVisibility: false,

  isLoading: false,
  isEnterReleaseCodeDialogDisplayed: false,


  isTagsFilterDisplayed: false,
}


//---pagination
export interface SetSearchTextAction extends ActionBase {
  readonly type: ActionType.setSearchText
  readonly searchText: string
  readonly viaCode: boolean
}

export interface SetActivePaginationPageAction extends ActionBase {
  readonly type: ActionType.setPaginationActivePage
  readonly activePage: number
  readonly viaCode: boolean
}

export interface SetPaginationPageSizeAction extends ActionBase {
  readonly type: ActionType.setPaginationPageSize
  readonly pageSize: number
  readonly viaCode: boolean
}

//---pagination end

//used to update the left time
export interface SET_openExercisesViaCodeAction extends ActionBase {
  readonly type: ActionType.SET_openExercisesViaCodeAction
  readonly openExercisesViaCode: ReadonlyArray<ExercisePreviewsFrontendExclusive>
}

//used to update the left time
export interface SET_openExercisesViaVisibilityAction extends ActionBase {
  readonly type: ActionType.SET_openExercisesViaVisibility
  readonly openExercisesViaVisibility: ReadonlyArray<ExercisePreviewsFrontendExclusive>
}

export interface SET_isEnterReleaseCodeDialogDisplayedAction extends ActionBase {
  readonly type: ActionType.SET_isEnterReleaseCodeDialogDisplayed
  readonly isDisplayed: boolean
}

export interface SET_sortByKeyAction extends ActionBase {
  readonly type: ActionType.SET_sortByKey
  readonly sortByKey: keyof ExercisePreviewFromBackend
  readonly sortDirection: boolean | null
  readonly viaCode: boolean
}

export interface SET_isTagsFilterDisplayedAction extends ActionBase {
  readonly type: ActionType.SET_isTagsFilterDisplayed
  readonly isTagsFilterDisplayed: boolean
}

export interface ResetAction extends ActionBase {
  readonly type: ActionType.RESET
}

export type AllActions =
  ResetAction
  | SET_sortByKeyAction
  | SET_isEnterReleaseCodeDialogDisplayedAction
  | SET_isTagsFilterDisplayedAction
  | SetSearchTextAction
  | SetActivePaginationPageAction
  | SetPaginationPageSizeAction
  | SET_openExercisesViaCodeAction
  | SET_openExercisesViaVisibilityAction

  | GetOpenExercisesViaCodeCrudActions
  | GetClosedExercisesViaVisibilityCrudActions

  | RESET_GlobalAction


export function reducer(state: State = initial, action: AllActions): State {

  switch (action.type) {

    case ActionType.GET_openExercisesViaCode_PENDING:
    case ActionType.GET_openExercisesViaCode_FULFILLED:
    case ActionType.GET_openExercisesViaCode_REJECTED:
      return getOpenExercisesViaCodeCrudReducer(state, action)

    case ActionType.GET_openExercisesViaVisibility_PENDING:
    case ActionType.GET_openExercisesViaVisibility_FULFILLED:
    case ActionType.GET_openExercisesViaVisibility_REJECTED:
      return getClosedExercisesViaVisibilityCrudReducer(state, action)

    case ActionType.SET_isTagsFilterDisplayed:

      FrontendSettingsManager.setAllOpenExercisesSiteTagsFilterIsDisplayed(action.isTagsFilterDisplayed)

      return {
        ...state,
        isTagsFilterDisplayed: action.isTagsFilterDisplayed
      }

    case ActionType.SET_isEnterReleaseCodeDialogDisplayed:
      return {
        ...state,
        isEnterReleaseCodeDialogDisplayed: action.isDisplayed
      }

    case ActionType.SET_openExercisesViaCodeAction:
      return {
        ...state,
        openExercisePreviewsViaCode: action.openExercisesViaCode
      }
    case ActionType.SET_openExercisesViaVisibility:
      return {
        ...state,
        openExercisePreviewsViaVisibility: action.openExercisesViaVisibility
      }

    case ActionType.SET_sortByKey: {
      if (action.viaCode) {
        return {
          ...state,
          paginationViaCode: {
            ...state.paginationViaCode,
            orderByProp: action.sortByKey,
            isOrderByDesc: action.sortDirection
          }
        }
      }

      return {
        ...state,
        paginationViaVisibility: {
          ...state.paginationViaVisibility,
          orderByProp: action.sortByKey,
          isOrderByDesc: action.sortDirection
        }
      }
    }

    case ActionType.setSearchText: {

      if (action.viaCode) {
        return {
          ...state,
          paginationViaCode: {
            ...state.paginationViaCode,
            searchText: action.searchText
          }
        }
      }
      return {
        ...state,
        paginationViaVisibility: {
          ...state.paginationViaVisibility,
          searchText: action.searchText
        }
      }
    }
    case ActionType.setPaginationActivePage: {

      if (action.viaCode) {
        return {
          ...state,
          paginationViaCode: {
            ...state.paginationViaCode,
            currentPage: action.activePage
          }
        }
      }
      return {
        ...state,
        paginationViaVisibility: {
          ...state.paginationViaVisibility,
          currentPage: action.activePage
        }
      }
    }

    case ActionType.setPaginationPageSize: {

      if (action.viaCode) {
        FrontendSettingsManager.set_openExercisesViaCodePageSize(action.pageSize)
        return {
          ...state,
          paginationViaCode: {
            ...state.paginationViaCode,
            pageSize: action.pageSize,
            currentPage: 1,
          }
        }
      }

      FrontendSettingsManager.set_openExercisesViaVisibilityPageSize(action.pageSize)
      return {
        ...state,
        paginationViaVisibility: {
          ...state.paginationViaVisibility,
          pageSize: action.pageSize
        }
      }
    }


    case ActionType.RESET:
    case ActionTypeGlobal.RESET_Global:
      return {
        ...initial,
        isTagsFilterDisplayed: FrontendSettingsManager.getFrontendSettings().allOpenExercisesSiteTagsFilterIsDisplayed,
        paginationViaCode: {
          ...initial.paginationViaCode,
          pageSize: FrontendSettingsManager.getFrontendSettings().openExercisesViaCodePageSize
        },
        paginationViaVisibility: {
          ...initial.paginationViaVisibility,
          pageSize: FrontendSettingsManager.getFrontendSettings().openExercisesViaVisibilityPageSize
        }
      }

    default:
      notExhaustive(action)
      return state
  }
}

