import {ActionBase, ActionType} from '../dashboardSiteActionTypes'
import {initial, State} from '../dashboardSiteReducer'
import {ReduxPromiseAction, ReduxPromiseFulfilledAction, ReduxPromiseRejectedAction} from '../../index'
import {notExhaustive} from '../../_notExhausiveHelper'
import {ClientServerAppSettings} from '../../../../types/dashboard'


export interface GET_clientServerAppSettingsAction
  extends ActionBase, ReduxPromiseAction<ClientServerAppSettings, GET_clientServerAppSettings_PENDINGAction, GET_clientServerAppSettings_FULFILLEDAction, GET_clientServerAppSettings_REJECTEDAction> {
  readonly type: ActionType.GET_clientServerAppSettings
  readonly payload: Promise<ClientServerAppSettings>
}

//automatically dispatched by redux promise middleware
export interface GET_clientServerAppSettings_PENDINGAction
  extends ActionBase {
  readonly type: ActionType.GET_clientServerAppSettings_PENDING
}

//automatically dispatched by redux promise middleware
export interface GET_clientServerAppSettings_FULFILLEDAction
  extends ActionBase, ReduxPromiseFulfilledAction<ClientServerAppSettings> {
  readonly type: ActionType.GET_clientServerAppSettings_FULFILLED
  readonly payload: ClientServerAppSettings
}

//automatically dispatched by redux promise middleware
export interface GET_clientServerAppSettings_REJECTEDAction
  extends ActionBase, ReduxPromiseRejectedAction<void> {
  readonly type: ActionType.GET_clientServerAppSettings_REJECTED
  readonly payload: any
}


export type AllActions =
  GET_clientServerAppSettings_PENDINGAction
  | GET_clientServerAppSettings_FULFILLEDAction
  | GET_clientServerAppSettings_REJECTEDAction


export function reducer(state: State = initial, action: AllActions): State {

  switch (action.type) {

    case ActionType.GET_clientServerAppSettings_PENDING:
      return {
        ...state,
        isLoading: true
      }
    case ActionType.GET_clientServerAppSettings_FULFILLED:
      return {
        ...state,
        isLoading: false,
        clientServerAppSettings: action.payload
      }
    case ActionType.GET_clientServerAppSettings_REJECTED:
      return {
        ...state,
        isLoading: false,
        clientServerAppSettings: null
      }

    default:
      notExhaustive(action)
      return state
  }
}

