import {ActionBase, ActionType} from "../dashboardSiteActionTypes";
import {initial, State} from "../dashboardSiteReducer";
import {ReduxPromiseAction, ReduxPromiseFulfilledAction, ReduxPromiseRejectedAction} from "../../index";
import {SystemSettingsFromBackend} from '../../../../types/systemSettings'
import {notExhaustive} from '../../_notExhausiveHelper'



export interface GET_clientServerSettingsAction
  extends ActionBase, ReduxPromiseAction<SystemSettingsFromBackend, GET_clientServerSettings_PENDINGAction, GET_clientServerSettings_FULFILLEDAction, GET_clientServerSettings_REJECTEDAction> {
  readonly type: ActionType.GET_clientServerSettings
  readonly payload: Promise<SystemSettingsFromBackend>
}

//automatically dispatched by redux promise middleware
export interface GET_clientServerSettings_PENDINGAction
  extends ActionBase {
  readonly type: ActionType.GET_clientServerSettings_PENDING
}

//automatically dispatched by redux promise middleware
export interface GET_clientServerSettings_FULFILLEDAction
  extends ActionBase, ReduxPromiseFulfilledAction<SystemSettingsFromBackend> {
  readonly type: ActionType.GET_clientServerSettings_FULFILLED
  readonly payload: SystemSettingsFromBackend
}

//automatically dispatched by redux promise middleware
export interface GET_clientServerSettings_REJECTEDAction
  extends ActionBase, ReduxPromiseRejectedAction<void> {
  readonly type: ActionType.GET_clientServerSettings_REJECTED
  readonly payload: any
}


export type AllActions =
  GET_clientServerSettings_PENDINGAction
  | GET_clientServerSettings_FULFILLEDAction
  | GET_clientServerSettings_REJECTEDAction


export function reducer(state: State = initial, action: AllActions): State {

  switch (action.type) {

    case ActionType.GET_clientServerSettings_PENDING:
      return {
        ...state,
        isLoading: true
      }
    case ActionType.GET_clientServerSettings_FULFILLED:
      return {
        ...state,
        isLoading: false,
        clientServerSystemSettings: action.payload
      }
    case ActionType.GET_clientServerSettings_REJECTED:
      return {
        ...state,
        isLoading: false,
        clientServerSystemSettings: null
      }

    default:
      notExhaustive(action)
      return state
  }
}

