import {ReduxPromiseAction, ReduxPromiseFulfilledAction, ReduxPromiseRejectedAction} from "../../../index";
import {ActionType} from '../../doExerciseSiteActionTypes'
import {Action} from 'redux'
import {StoredTestResultFromBackend} from '../../../../../types/testResults'
import {initial, State} from '../../doExerciseSiteReducer'
import {RunnableAfterCustomTestFrontendExclusive} from '../../../../../types/customTests'
import {_storedTestResultToTestResult} from '../../../../../helpers/convertersAndTransformers'
import {notExhaustive} from '../../../_notExhausiveHelper'
import {RunnableExerciseTestFrontendExclusive} from '../../../../../types/exerciseEditor'
import {upState} from '../../../../../helpers/typeHelpers'

export interface ActionBase extends Action {
  readonly type: ActionType
}

export interface GET_singleAfterCustomTTestResultAction
  extends ActionBase, ReduxPromiseAction<StoredTestResultFromBackend, GET_singleAfterCustomTTestResult_PENDINGAction, GET_singleAfterCustomTTestResult_FULFILLEDAction, GET_singleAfterCustomTTestResult_REJECTEDAction> {
  readonly type: ActionType.GET_singleAfterCustomTestResult
  readonly payload: Promise<StoredTestResultFromBackend>
}

//automatically dispatched by redux promise middleware
export interface GET_singleAfterCustomTTestResult_PENDINGAction
  extends ActionBase {
  readonly type: ActionType.GET_singleAfterCustomTestResult_PENDING
}

//automatically dispatched by redux promise middleware
export interface GET_singleAfterCustomTTestResult_FULFILLEDAction
  extends ActionBase, ReduxPromiseFulfilledAction<StoredTestResultFromBackend> {
  readonly type: ActionType.GET_singleAfterCustomTestResult_FULFILLED
  readonly payload: StoredTestResultFromBackend
}

//automatically dispatched by redux promise middleware
export interface GET_singleAfterCustomTTestResult_REJECTEDAction
  extends ActionBase, ReduxPromiseRejectedAction<void> {
  readonly type: ActionType.GET_singleAfterCustomTestResult_REJECTED
  readonly payload: any
}


export type AllActions =
  GET_singleAfterCustomTTestResult_PENDINGAction
  | GET_singleAfterCustomTTestResult_FULFILLEDAction
  | GET_singleAfterCustomTTestResult_REJECTEDAction


export function reducer(state: State = initial, action: AllActions): State {

  switch (action.type) {

    case ActionType.GET_singleAfterCustomTestResult_PENDING:
      return {
        ...state,
        isLoading: true
      }
    case ActionType.GET_singleAfterCustomTestResult_FULFILLED:
      return {
        ...state,
        isLoading: false,
        afterCustomTestResults: state.afterCustomTestResults.map<RunnableAfterCustomTestFrontendExclusive>(p =>
          p.id !== action.payload.testId
          ? p
          : upState(p, {
            ...p,
            result: _storedTestResultToTestResult(action.payload),
            uiIsAfterTest: true
          }))
      }
    case ActionType.GET_singleAfterCustomTestResult_REJECTED:
      return {
        ...state,
        isLoading: false
      }

    default:
      notExhaustive(action)
      return state
  }
}

