import {ReduxPromiseAction, ReduxPromiseFulfilledAction, ReduxPromiseRejectedAction} from "../../../index";
import {ActionType} from '../../doExerciseSiteActionTypes'
import {Action} from 'redux'
import {AssessedSubmitTestFrontendExclusive, StoredTestResultFromBackend} from '../../../../../types/testResults'
import {initial, State} from '../../doExerciseSiteReducer'
import {RunnableAfterCustomTestFrontendExclusive} from '../../../../../types/customTests'
import {_storedTestResultToTestResult} from '../../../../../helpers/convertersAndTransformers'
import {notExhaustive} from '../../../_notExhausiveHelper'
import {RunnableExerciseTestFrontendExclusive} from '../../../../../types/exerciseEditor'
import {upState} from '../../../../../helpers/typeHelpers'

export interface ActionBase extends Action {
  readonly type: ActionType
}

export interface GET_singleAfterSubmitTestResultAction
  extends ActionBase, ReduxPromiseAction<StoredTestResultFromBackend, GET_singleAfterSubmitTestResult_PENDINGAction, GET_singleAfterSubmitTestResult_FULFILLEDAction, GET_singleAfterSubmitTestResult_REJECTEDAction> {
  readonly type: ActionType.GET_singleAfterSubmitTestResult
  readonly payload: Promise<StoredTestResultFromBackend>
}

//automatically dispatched by redux promise middleware
export interface GET_singleAfterSubmitTestResult_PENDINGAction
  extends ActionBase {
  readonly type: ActionType.GET_singleAfterSubmitTestResult_PENDING
}

//automatically dispatched by redux promise middleware
export interface GET_singleAfterSubmitTestResult_FULFILLEDAction
  extends ActionBase, ReduxPromiseFulfilledAction<StoredTestResultFromBackend> {
  readonly type: ActionType.GET_singleAfterSubmitTestResult_FULFILLED
  readonly payload: StoredTestResultFromBackend
}

//automatically dispatched by redux promise middleware
export interface GET_singleAfterSubmitTestResult_REJECTEDAction
  extends ActionBase, ReduxPromiseRejectedAction<void> {
  readonly type: ActionType.GET_singleAfterSubmitTestResult_REJECTED
  readonly payload: any
}


export type AllActions =
  GET_singleAfterSubmitTestResult_PENDINGAction
  | GET_singleAfterSubmitTestResult_FULFILLEDAction
  | GET_singleAfterSubmitTestResult_REJECTEDAction


export function reducer(state: State = initial, action: AllActions): State {

  switch (action.type) {

    case ActionType.GET_singleAfterSubmitTestResult_PENDING:
      return {
        ...state,
        isLoading: true
      }
    case ActionType.GET_singleAfterSubmitTestResult_FULFILLED:
      return {
        ...state,
        isLoading: false,
        afterSubmitTests: state.afterSubmitTests.map<AssessedSubmitTestFrontendExclusive>(p =>
          p.id !== action.payload.testId
          ? p
          : upState(p, {
            ...p,
            uiIsAfterTest: true,
            result: _storedTestResultToTestResult(action.payload)
          }))
      }
    case ActionType.GET_singleAfterSubmitTestResult_REJECTED:
      return {
        ...state,
        isLoading: false
      }

    default:
      notExhaustive(action)
      return state
  }
}

