/**
 * Created by janis dähne.
 */
import {ActionBase, ActionType} from "../../doExerciseSiteActionTypes";
import {initial, State} from "../../doExerciseSiteReducer";
import {ReduxPromiseAction, ReduxPromiseFulfilledAction, ReduxPromiseRejectedAction} from "../../../index";
import Logger from "../../../../../helpers/logger";
import {notExhaustive} from "../../../_notExhausiveHelper";
import {mergeSubmitTestResults} from "../../../../../helpers/convertersAndTransformers";
import {TestAnswerFromBackend} from "../../../../../types/testsAndTestProtocol";


export interface RUN_afterSubmitTestAction
  extends ActionBase, ReduxPromiseAction<ReadonlyArray<TestAnswerFromBackend>, RUN_afterSubmitTest_PENDINGAction, RUN_afterSubmitTest_FULFILLEDAction, RUN_afterSubmitTest_REJECTEDAction> {
  readonly type: ActionType.RUN_afterSubmitTest
  readonly payload: Promise<ReadonlyArray<TestAnswerFromBackend>>
}

//automatically dispatched by redux promise middleware
export interface RUN_afterSubmitTest_PENDINGAction
  extends ActionBase {
  readonly type: ActionType.RUN_afterSubmitTest_PENDING
}

//automatically dispatched by redux promise middleware
export interface RUN_afterSubmitTest_FULFILLEDAction
  extends ActionBase, ReduxPromiseFulfilledAction<ReadonlyArray<TestAnswerFromBackend>> {
  readonly type: ActionType.RUN_afterSubmitTest_FULFILLED
  readonly payload: ReadonlyArray<TestAnswerFromBackend>
}

//automatically dispatched by redux promise middleware
export interface RUN_afterSubmitTest_REJECTEDAction
  extends ActionBase, ReduxPromiseRejectedAction<void> {
  readonly type: ActionType.RUN_afterSubmitTest_REJECTED
  readonly payload: any
}


export type AllActions =
  RUN_afterSubmitTest_PENDINGAction
  | RUN_afterSubmitTest_FULFILLEDAction
  | RUN_afterSubmitTest_REJECTEDAction


export function reducer(state: State = initial, action: AllActions): State {

  switch (action.type) {

    case ActionType.RUN_afterSubmitTest_PENDING:
      return {
        ...state,
        isLoading: false
      }
    case ActionType.RUN_afterSubmitTest_FULFILLED:
      return {
        ...state,
        isLoading: false,
        afterSubmitTests: mergeSubmitTestResults(state.afterSubmitTests, action.payload)
      }
    case ActionType.RUN_afterSubmitTest_REJECTED:
      return {
        ...state,
        isLoading: false,
        afterSubmitTests: mergeSubmitTestResults(state.afterSubmitTests, [])
      }

    default:
      notExhaustive(action)
      return state
  }
}

