import {ReduxPromiseAction, ReduxPromiseFulfilledAction, ReduxPromiseRejectedAction} from "../../../index";
import {Action} from 'redux'
import {ActionType} from '../../doExerciseSiteActionTypes'
import {initial, State} from '../../doExerciseSiteReducer'
import {notExhaustive} from '../../../_notExhausiveHelper'
import {AfterSolutionTupleFromBackend} from '../../../../../types/exerciseAfterSolution'
import {AceEditorsStorageHelper} from '../../../../../components/codeEditors/aceEditors/aceEditorsStorageHelper'
import {doEditorAfterEditorPanelViewEditorId} from '../../../../../constants'
import {updateFileReferencesAceStorage, updateFilesReference} from '../../../../../helpers/editorHelper'
import {KnownLangs} from '../../../../../../i18n/i18nRoot'

interface Meta {
  readonly langId: KnownLangs
}

export interface ActionBase extends Action {
  readonly type: ActionType
}

export interface UPDATE_doExercise_afterSolutionAction
  extends ActionBase, ReduxPromiseAction<AfterSolutionTupleFromBackend, UPDATE_doExercise_afterSolution_PENDINGAction, UPDATE_doExercise_afterSolution_FULFILLEDAction, UPDATE_doExercise_afterSolution_REJECTEDAction, Meta> {
  readonly type: ActionType.UPDATE_doExercise_afterSolution
  readonly payload: Promise<AfterSolutionTupleFromBackend>
  readonly meta: Meta
}

//automatically dispatched by redux promise middleware
export interface UPDATE_doExercise_afterSolution_PENDINGAction
  extends ActionBase {
  readonly type: ActionType.UPDATE_doExercise_afterSolution_PENDING
  readonly meta: Meta
}

//automatically dispatched by redux promise middleware
export interface UPDATE_doExercise_afterSolution_FULFILLEDAction
  extends ActionBase, ReduxPromiseFulfilledAction<AfterSolutionTupleFromBackend, Meta> {
  readonly type: ActionType.UPDATE_doExercise_afterSolution_FULFILLED
  readonly payload: AfterSolutionTupleFromBackend
}

//automatically dispatched by redux promise middleware
export interface UPDATE_doExercise_afterSolution_REJECTEDAction
  extends ActionBase, ReduxPromiseRejectedAction<void, Meta> {
  readonly type: ActionType.UPDATE_doExercise_afterSolution_REJECTED
  readonly payload: any
}


export type AllActions =
  UPDATE_doExercise_afterSolution_PENDINGAction
  | UPDATE_doExercise_afterSolution_FULFILLEDAction
  | UPDATE_doExercise_afterSolution_REJECTEDAction


export function reducer(state: State = initial, action: AllActions): State {

  switch (action.type) {

    case ActionType.UPDATE_doExercise_afterSolution_PENDING:
      return {
        ...state,
        isLoading: true
      }
    case ActionType.UPDATE_doExercise_afterSolution_FULFILLED:


      const storedTabs = AceEditorsStorageHelper.getEditor(doEditorAfterEditorPanelViewEditorId)
      updateFileReferencesAceStorage(storedTabs, action.payload.solution.solutionFiles,
                                     action.payload.fileIdAssociationMap, action.meta.langId
      )

      //for the initial create we need to map the frontend id to the backend id...
      //all other updates will have the same frontned id = backend id because the ids don't change
      const tuple = action.payload.fileIdAssociationMap.find(
        p => p.frontendFileId === state.afterEditorState.selectedTabIdDoEditor)

      const newSelectedTabId = tuple
                               ? tuple.backendFileId
                               : -1

      return {
        ...state,
        isLoading: false,

        //this needs to be BEFORE setting solution somehow...
        afterEditorState: {
          ...state.afterEditorState,
          selectedTabIdDoEditor: newSelectedTabId,
        },
        afterSolution: {
          ...action.payload.solution,
          //the display order is preserved because we take the new ids (and some props) with updateFilesReference
          solutionFiles: updateFilesReference(state.afterSolution.solutionFiles,
                                              action.payload.solution.solutionFiles,
                                              action.payload.fileIdAssociationMap, action.meta.langId
          ),
        },
        solution: {
          ...state.solution,
          hasLoadAfterSolution: true,
          shouldLoadAfterSolution: true
        }
      }
    case ActionType.UPDATE_doExercise_afterSolution_REJECTED:
      return {
        ...state,
        isLoading: false
      }

    default:
      notExhaustive(action)
      return state
  }
}

