import {ActionBase, ActionType} from '../../doExerciseSiteActionTypes'
import {ReduxPromiseAction, ReduxPromiseFulfilledAction, ReduxPromiseRejectedAction} from '../../../index'
import {initial, State} from '../../doExerciseSiteReducer'
import {notExhaustive} from '../../../_notExhausiveHelper'
import {FilePreviewFromBackend} from '../../../../../types/exerciseEditor'


interface Meta {
  readonly customTestId: number
  readonly assetPreviewId: number
}

export interface DETACH_customTestAssetPreviewAction
  extends ActionBase, ReduxPromiseAction<void, DETACH_customTestAssetPreview_PENDINGAction, DETACH_customTestAssetPreview_FULFILLEDAction, DETACH_customTestAssetPreview_REJECTEDAction, Meta> {
  readonly type: ActionType.DETACH_customTestAssetPreview
  readonly payload: Promise<void>
  readonly meta: Meta
}

//automatically dispatched by redux promise middleware
export interface DETACH_customTestAssetPreview_PENDINGAction
  extends ActionBase {
  readonly type: ActionType.DETACH_customTestAssetPreview_PENDING
  readonly meta: Meta
}

//automatically dispatched by redux promise middleware
export interface DETACH_customTestAssetPreview_FULFILLEDAction
  extends ActionBase, ReduxPromiseFulfilledAction<void, Meta> {
  readonly type: ActionType.DETACH_customTestAssetPreview_FULFILLED
  readonly payload: void
  readonly meta: Meta
}

//automatically dispatched by redux promise middleware
export interface DETACH_customTestAssetPreview_REJECTEDAction
  extends ActionBase, ReduxPromiseRejectedAction<void, Meta> {
  readonly type: ActionType.DETACH_customTestAssetPreview_REJECTED
  readonly payload: any
  readonly meta: Meta
}


export type AllActions =
  DETACH_customTestAssetPreview_PENDINGAction
  | DETACH_customTestAssetPreview_FULFILLEDAction
  | DETACH_customTestAssetPreview_REJECTEDAction


export function reducer(state: State = initial, action: AllActions): State {

  switch (action.type) {

    case ActionType.DETACH_customTestAssetPreview_PENDING:
      return {
        ...state,
        isAddCustomTestAssetFilesModalBusy: true
      }
    case ActionType.DETACH_customTestAssetPreview_FULFILLED:
      return {
        ...state,
        isAddCustomTestAssetFilesModalBusy: false,
        exercise: {
          ...state.exercise,
          customTests: state.exercise.customTests.map(p => p.id !== action.meta.customTestId
                                                           ? p
                                                           : {
              ...p,
              files: p.files.filter(k => k.id !== action.meta.assetPreviewId)
            })
        }
      }
    case ActionType.DETACH_customTestAssetPreview_REJECTED:
      return {
        ...state,
        isAddCustomTestAssetFilesModalBusy: false
      }

    default:
      notExhaustive(action)
      return state
  }
}

