/**
 * Created by janis dähne.
 */

import {ActionBase, ActionType} from "../doExerciseSiteActionTypes";
import {DoExerciseFromBackend} from "../../../../types/doExercise";
import {initial, State} from "../doExerciseSiteReducer";
import Logger from "../../../../helpers/logger";
import {ReduxPromiseAction, ReduxPromiseFulfilledAction, ReduxPromiseRejectedAction} from "../../index";
import {notExhaustive} from "../../_notExhausiveHelper";
import {
  convertDoExerciseFromBackendToFrontend,
  convertTestFromBackendToFrontend
} from "../../../../helpers/convertersAndTransformers";
import {RunnableAfterCustomTestFrontendExclusive} from '../../../../types/customTests'
import {initial as initialDoExerciseSiteState} from '../doExerciseSiteReducer'


export interface GET_doExerciseAction
  extends ActionBase, ReduxPromiseAction<DoExerciseFromBackend, GET_doExercise_PENDINGAction, GET_doExercise_FULFILLEDAction, GET_doExercise_REJECTEDAction> {
  readonly type: ActionType.GET_doExercise
  readonly payload: Promise<DoExerciseFromBackend>
}

//automatically dispatched by redux promise middleware
export interface GET_doExercise_PENDINGAction
  extends ActionBase {
  readonly type: ActionType.GET_doExercise_PENDING
}

//automatically dispatched by redux promise middleware
export interface GET_doExercise_FULFILLEDAction
  extends ActionBase, ReduxPromiseFulfilledAction<DoExerciseFromBackend> {
  readonly type: ActionType.GET_doExercise_FULFILLED
  readonly payload: DoExerciseFromBackend
}

//automatically dispatched by redux promise middleware
export interface GET_doExercise_REJECTEDAction
  extends ActionBase, ReduxPromiseRejectedAction<void> {
  readonly type: ActionType.GET_doExercise_REJECTED
  readonly payload: any
}


export type AllActions =
  GET_doExercise_PENDINGAction
  | GET_doExercise_FULFILLEDAction
  | GET_doExercise_REJECTEDAction


export function reducer(state: State = initial, action: AllActions): State {

  switch (action.type) {

    case ActionType.GET_doExercise_PENDING:
      return {
        ...state,
        isLoading: true
      }
    case ActionType.GET_doExercise_FULFILLED:
      return {
        ...state,
        isLoading: false,
        exercise: convertDoExerciseFromBackendToFrontend(action.payload),
        afterNormalTests: convertTestFromBackendToFrontend(action.payload.tests, true),
        afterCustomTestResults: action.payload.customTests.map<RunnableAfterCustomTestFrontendExclusive>(p => {
          return {
            id: p.id,
            result: null,
            uiIsRunning: false,
            uiIsAfterTest: true
          }
        }),
        lockSolutionsFlag: false,
        isLeftTimeStringDisplayed: true
      }
    case ActionType.GET_doExercise_REJECTED:
      return initialDoExerciseSiteState //this is the first thing we load so if this fails nothing else is loaded...

    default:
      notExhaustive(action)
      return state
  }
}

