/**
 * Created by janis dähne.
 */


import {ActionBase, ActionType} from "../doExerciseSiteActionTypes";
import {initial, State} from "../doExerciseSiteReducer";
import {mergeTestResultsIntoDoExercise} from "../../../../helpers/convertersAndTransformers";
import {notExhaustive} from "../../_notExhausiveHelper";
import {ReduxPromiseAction, ReduxPromiseFulfilledAction, ReduxPromiseRejectedAction} from "../../index";
import {StoredTestResultFromBackend, StoredTestResultTupleFromBackend} from '../../../../types/testResults'

//this is actually only the normal tests, custom tests

export interface GET_singleCustomTestResultAction
  extends ActionBase, ReduxPromiseAction<StoredTestResultFromBackend, GET_singleCustomTestResult_PENDINGAction, GET_singleCustomTestResult_FULFILLEDAction, GET_singleCustomTestResult_REJECTEDAction> {
  readonly type: ActionType.GET_singleCustomTestResults
  readonly payload: Promise<StoredTestResultFromBackend>
}

//automatically dispatched by redux promise middleware
export interface GET_singleCustomTestResult_PENDINGAction
  extends ActionBase {
  readonly type: ActionType.GET_singleCustomTestResults_PENDING
}

//automatically dispatched by redux promise middleware
export interface GET_singleCustomTestResult_FULFILLEDAction
  extends ActionBase, ReduxPromiseFulfilledAction<StoredTestResultFromBackend> {
  readonly type: ActionType.GET_singleCustomTestResults_FULFILLED
  readonly payload: StoredTestResultFromBackend
}

//automatically dispatched by redux promise middleware
export interface GET_singleCustomTestResult_REJECTEDAction
  extends ActionBase, ReduxPromiseRejectedAction<void> {
  readonly type: ActionType.GET_singleCustomTestResults_REJECTED
  readonly payload: any
}


export type AllActions =
  GET_singleCustomTestResult_PENDINGAction
  | GET_singleCustomTestResult_FULFILLEDAction
  | GET_singleCustomTestResult_REJECTEDAction


export function reducer(state: State = initial, action: AllActions): State {

  switch (action.type) {

    case ActionType.GET_singleCustomTestResults_PENDING:
      return {
        ...state,
        isLoading: true
      }
    case ActionType.GET_singleCustomTestResults_FULFILLED:

      const testResults: StoredTestResultTupleFromBackend = {
        testResults: state.exercise.tests.map(p => p.result),
        customTestResults: state.exercise.customTests.map(p =>
          p.id === action.payload.testId
          ? action.payload
          : p.result
        )
      }

      return {
        ...state,
        isLoading: false,
        exercise: mergeTestResultsIntoDoExercise(state.exercise, testResults)
      }
    case ActionType.GET_singleCustomTestResults_REJECTED:
      return {
        ...state,
        isLoading: false,
      }

    default:
      notExhaustive(action)
      return state
  }
}

