/**
 * Created by janis dähne.
 */


import {ActionBase, ActionType} from "../doExerciseSiteActionTypes";
import {initial, State} from "../doExerciseSiteReducer";
import {mergeTestResultsIntoDoExercise} from "../../../../helpers/convertersAndTransformers";
import {notExhaustive} from "../../_notExhausiveHelper";
import {ReduxPromiseAction, ReduxPromiseFulfilledAction, ReduxPromiseRejectedAction} from "../../index";
import {StoredTestResultFromBackend, StoredTestResultTupleFromBackend} from '../../../../types/testResults'

//this is actually only the normal tests, custom tests

export interface GET_singleNormalTestResultAction
  extends ActionBase, ReduxPromiseAction<StoredTestResultFromBackend, GET_singleNormalTestResult_PENDINGAction, GET_singleNormalTestResult_FULFILLEDAction, GET_singleNormalTestResult_REJECTEDAction> {
  readonly type: ActionType.GET_singleNormalTestResults
  readonly payload: Promise<StoredTestResultFromBackend>
}

//automatically dispatched by redux promise middleware
export interface GET_singleNormalTestResult_PENDINGAction
  extends ActionBase {
  readonly type: ActionType.GET_singleNormalTestResults_PENDING
}

//automatically dispatched by redux promise middleware
export interface GET_singleNormalTestResult_FULFILLEDAction
  extends ActionBase, ReduxPromiseFulfilledAction<StoredTestResultFromBackend> {
  readonly type: ActionType.GET_singleNormalTestResults_FULFILLED
  readonly payload: StoredTestResultFromBackend
}

//automatically dispatched by redux promise middleware
export interface GET_singleNormalTestResult_REJECTEDAction
  extends ActionBase, ReduxPromiseRejectedAction<void> {
  readonly type: ActionType.GET_singleNormalTestResults_REJECTED
  readonly payload: any
}


export type AllActions =
  GET_singleNormalTestResult_PENDINGAction
  | GET_singleNormalTestResult_FULFILLEDAction
  | GET_singleNormalTestResult_REJECTEDAction


export function reducer(state: State = initial, action: AllActions): State {

  switch (action.type) {

    case ActionType.GET_singleNormalTestResults_PENDING:
      return {
        ...state,
        isLoading: true
      }
    case ActionType.GET_singleNormalTestResults_FULFILLED:

      const testResults: StoredTestResultTupleFromBackend = {
        testResults: state.exercise.tests.map(p =>
          p.id === action.payload.testId
          ? action.payload
          : p.result
        ),
        customTestResults: state.exercise.customTests.map(p => p.result)
      }

      return {
        ...state,
        isLoading: false,
        exercise: mergeTestResultsIntoDoExercise(state.exercise, testResults)
      }
    case ActionType.GET_singleNormalTestResults_REJECTED:
      return {
        ...state,
        isLoading: false,
      }

    default:
      notExhaustive(action)
      return state
  }
}

