/**
 * Created by janis dähne.
 */


import {ActionBase, ActionType} from "../doExerciseSiteActionTypes";
import {initial, State} from "../doExerciseSiteReducer";
import {mergeTestResultsIntoDoExercise} from "../../../../helpers/convertersAndTransformers";
import {notExhaustive} from "../../_notExhausiveHelper";
import {ReduxPromiseAction, ReduxPromiseFulfilledAction, ReduxPromiseRejectedAction} from "../../index";
import {StoredTestResultFromBackend, StoredTestResultTupleFromBackend} from '../../../../types/testResults'
import {upState} from '../../../../helpers/typeHelpers'

//this is actually only the normal tests, custom tests

export interface GET_singleSubmitTestResultAction
  extends ActionBase, ReduxPromiseAction<StoredTestResultFromBackend, GET_singleSubmitTestResult_PENDINGAction, GET_singleSubmitTestResult_FULFILLEDAction, GET_singleSubmitTestResult_REJECTEDAction> {
  readonly type: ActionType.GET_singleSubmitTestResults
  readonly payload: Promise<StoredTestResultFromBackend>
}

//automatically dispatched by redux promise middleware
export interface GET_singleSubmitTestResult_PENDINGAction
  extends ActionBase {
  readonly type: ActionType.GET_singleSubmitTestResults_PENDING
}

//automatically dispatched by redux promise middleware
export interface GET_singleSubmitTestResult_FULFILLEDAction
  extends ActionBase, ReduxPromiseFulfilledAction<StoredTestResultFromBackend> {
  readonly type: ActionType.GET_singleSubmitTestResults_FULFILLED
  readonly payload: StoredTestResultFromBackend
}

//automatically dispatched by redux promise middleware
export interface GET_singleSubmitTestResult_REJECTEDAction
  extends ActionBase, ReduxPromiseRejectedAction<void> {
  readonly type: ActionType.GET_singleSubmitTestResults_REJECTED
  readonly payload: any
}


export type AllActions =
  GET_singleSubmitTestResult_PENDINGAction
  | GET_singleSubmitTestResult_FULFILLEDAction
  | GET_singleSubmitTestResult_REJECTEDAction


export function reducer(state: State = initial, action: AllActions): State {

  switch (action.type) {

    case ActionType.GET_singleSubmitTestResults_PENDING:
      return {
        ...state,
        isLoading: true
      }
    case ActionType.GET_singleSubmitTestResults_FULFILLED:
      return {
        ...state,
        isLoading: false,
        submitTestResults: state.submitTestResults.map(p =>
          p.id !== action.payload.testId
          ? p
          : upState(p, {
            ...p,
            result: action.payload
          })
        )
      }
    case ActionType.GET_singleSubmitTestResults_REJECTED:
      return {
        ...state,
        isLoading: false,
      }

    default:
      notExhaustive(action)
      return state
  }
}

