/**
 * Created by janis dähne.
 */
import {notExhaustive} from "../../_notExhausiveHelper";
import {ReduxPromiseAction, ReduxPromiseFulfilledAction, ReduxPromiseRejectedAction} from "../../index";
import {ActionBase, ActionType} from "../doExerciseSiteActionTypes";
import {initial, State} from "../doExerciseSiteReducer";
import {SolutionFileDoExerciseFullBase, SolutionTupleFromBackend} from "../../../../types/exerciseSolution";

export interface GET_doExercise_solutionVersionAction
  extends ActionBase, ReduxPromiseAction<SolutionTupleFromBackend, GET_doExercise_solutionVersion_PENDINGAction, GET_doExercise_solutionVersion_FULFILLEDAction, GET_doExercise_solutionVersion_REJECTEDAction> {
  readonly type: ActionType.GET_doExercise_solutionVersion
  readonly payload: Promise<SolutionTupleFromBackend>
}

//automatically dispatched by redux promise middleware
export interface GET_doExercise_solutionVersion_PENDINGAction
  extends ActionBase {
  readonly type: ActionType.GET_doExercise_solutionVersion_PENDING
}

//automatically dispatched by redux promise middleware
export interface GET_doExercise_solutionVersion_FULFILLEDAction
  extends ActionBase, ReduxPromiseFulfilledAction<SolutionTupleFromBackend> {
  readonly type: ActionType.GET_doExercise_solutionVersion_FULFILLED
  readonly payload: SolutionTupleFromBackend
}

//automatically dispatched by redux promise middleware
export interface GET_doExercise_solutionVersion_REJECTEDAction
  extends ActionBase, ReduxPromiseRejectedAction<void> {
  readonly type: ActionType.GET_doExercise_solutionVersion_REJECTED
  readonly payload: any
}


export type AllActions =
  GET_doExercise_solutionVersion_PENDINGAction
  | GET_doExercise_solutionVersion_FULFILLEDAction
  | GET_doExercise_solutionVersion_REJECTEDAction


export function reducer(state: State = initial, action: AllActions): State {

  switch (action.type) {

    case ActionType.GET_doExercise_solutionVersion_PENDING:
      return {
        ...state,
        isLoading: true
      }
    case ActionType.GET_doExercise_solutionVersion_FULFILLED:


      // noinspection TsLint
      const solutionFromBackend = {
        ...action.payload.solution,
        //ensure same file display order (because display indices are not applied)
        //we cannot use ids because the backend saves files in any order
        //we do sorting in the editorPanel when we convert the files into editor tabs
        solutionFiles: (action.payload.solution.solutionFiles as SolutionFileDoExerciseFullBase[])
      }

      return {
        ...state,
        isLoading: false,
        solution: solutionFromBackend,

      }
    case ActionType.GET_doExercise_solutionVersion_REJECTED:
      return {
        ...state,
        isLoading: false,
        solution: null,
      }

    default:
      notExhaustive(action)
      return state
  }
}

