/**
 * Created by janis dähne.
 */
import {ActionBase, ActionType} from "../doExerciseSiteActionTypes";
import {initial, State} from "../doExerciseSiteReducer";
import Logger from "../../../../helpers/logger";
import {ReduxPromiseAction, ReduxPromiseFulfilledAction, ReduxPromiseRejectedAction} from "../../index";
import {notExhaustive} from "../../_notExhausiveHelper";
import {
  AssessedSubmitTestFromBackend, AssessedSubmitTestFrontendExclusive,
  AssessmentForUserWithTestResults
} from "../../../../types/testResults";
import {convertAssessmentFromBackendToFrontend} from "../../../../helpers/convertersAndTransformers";

export interface GET_submitTestResultsAction
  extends ActionBase, ReduxPromiseAction<AssessmentForUserWithTestResults, GET_submitTestResults_PENDINGAction, GET_submitTestResults_FULFILLEDAction, GET_submitTestResults_REJECTEDAction> {
  readonly type: ActionType.GET_submitTestResults
  readonly payload: Promise<AssessmentForUserWithTestResults>
}

//automatically dispatched by redux promise middleware
export interface GET_submitTestResults_PENDINGAction
  extends ActionBase {
  readonly type: ActionType.GET_submitTestResults_PENDING
}

//automatically dispatched by redux promise middleware
export interface GET_submitTestResults_FULFILLEDAction
  extends ActionBase, ReduxPromiseFulfilledAction<AssessmentForUserWithTestResults> {
  readonly type: ActionType.GET_submitTestResults_FULFILLED
  readonly payload: AssessmentForUserWithTestResults
}

//automatically dispatched by redux promise middleware
export interface GET_submitTestResults_REJECTEDAction
  extends ActionBase, ReduxPromiseRejectedAction<void> {
  readonly type: ActionType.GET_submitTestResults_REJECTED
  readonly payload: any
}


export type AllActions =
  GET_submitTestResults_PENDINGAction
  | GET_submitTestResults_FULFILLEDAction
  | GET_submitTestResults_REJECTEDAction


export function reducer(state: State = initial, action: AllActions): State {

  switch (action.type) {

    case ActionType.GET_submitTestResults_PENDING:
      return {
        ...state,
        isLoading: true
      }
    case ActionType.GET_submitTestResults_FULFILLED:
      return {
        ...state,
        isLoading: false,
        submitTestResults: convertAssessmentFromBackendToFrontend(action.payload.submitTestResults, false),
        assessmentResult: action.payload.assessmentResult,
        //when we have no after solution and submit test results this would be empty because we never set this
        afterSubmitTests: convertAssessmentFromBackendToFrontend(action.payload.submitTestResults, true).map<AssessedSubmitTestFrontendExclusive>(p => {
          return {
            ...p,
            result: null
          }
        })
      }
    case ActionType.GET_submitTestResults_REJECTED:
      return {
        ...state,
        isLoading: false,
        submitTestResults: [],
        assessmentResult: null
      }

    default:
      notExhaustive(action)
      return state
  }
}

