import {Action} from 'redux';
import {ActionType} from '../doExerciseSiteActionTypes'
import {initial, State} from '../doExerciseSiteReducer'
import {SolutionDoExerciseFullBase, SolutionFileDoExerciseFullBase} from '../../../../types/exerciseSolution'
import {ReduxPromiseFulfilledAction, ReduxPromiseRejectedAction, ReduxPromiseAction} from '../../index'
import {notExhaustive} from '../../_notExhausiveHelper'
import {
  AceEditorsStorageHelper,
  AceStoredTabLike
} from '../../../../components/codeEditors/aceEditors/aceEditorsStorageHelper'
import {customProjectEditorPanelViewEditorId, doEditorEditorPanelViewEditorId} from '../../../../constants'


export interface ActionBase extends Action {
  readonly type: ActionType
}

export interface RESET_userSolutionAction
  extends ActionBase, ReduxPromiseAction<SolutionDoExerciseFullBase, RESET_userSolution_PENDINGAction, RESET_userSolution_FULFILLEDAction, RESET_userSolution_REJECTEDAction> {
  readonly type: ActionType.RESET_userSolution
  readonly payload: Promise<SolutionDoExerciseFullBase>
}

//automatically dispatched by redux promise middleware
export interface RESET_userSolution_PENDINGAction
  extends ActionBase {
  readonly type: ActionType.RESET_userSolution_PENDING
}

//automatically dispatched by redux promise middleware
export interface RESET_userSolution_FULFILLEDAction
  extends ActionBase, ReduxPromiseFulfilledAction<SolutionDoExerciseFullBase> {
  readonly type: ActionType.RESET_userSolution_FULFILLED
  readonly payload: SolutionDoExerciseFullBase
}

//automatically dispatched by redux promise middleware
export interface RESET_userSolution_REJECTEDAction
  extends ActionBase, ReduxPromiseRejectedAction<void> {
  readonly type: ActionType.RESET_userSolution_REJECTED
  readonly payload: any
}


export type AllActions =
  RESET_userSolution_PENDINGAction
  | RESET_userSolution_FULFILLEDAction
  | RESET_userSolution_REJECTEDAction


export function reducer(state: State = initial, action: AllActions): State {

  switch (action.type) {

    case ActionType.RESET_userSolution_PENDING:
      return {
        ...state,
        isLoading: true
      }
    case ActionType.RESET_userSolution_FULFILLED:

      // noinspection TsLint
      const solutionFromBackend: SolutionDoExerciseFullBase = {
        ...action.payload,
        //ensure same file display order (because display indices are not applied)
        //we cannot use ids because the backend saves files in any order
        solutionFiles: (action.payload.solutionFiles as SolutionFileDoExerciseFullBase[]).sort((a, b) => a.fileNameWithExtension.localeCompare(b.fileNameWithExtension)),
      }

      return {
        ...state,
        isLoading: false,
        solution: solutionFromBackend,
        doEditorState: {
          ...state.doEditorState,
          hasUnsavedChanges: false //we war up-to-date
        }
      }
    case ActionType.RESET_userSolution_REJECTED:
      return {
        ...state,
        isLoading: false
      }

    default:
      notExhaustive(action)
      return state
  }
}

