import {ActionBase, ActionType} from "../doExerciseSiteActionTypes";
import {initial, State} from "../doExerciseSiteReducer";
import {TestAnswerFromBackend} from "../../../../types/testsAndTestProtocol";
import {ReduxPromiseAction, ReduxPromiseFulfilledAction, ReduxPromiseRejectedAction} from "../../index";
import {notExhaustive} from "../../_notExhausiveHelper";
import {mergeNormalTestResults} from "../../../../helpers/convertersAndTransformers";


export interface RUN_testAction
  extends ActionBase, ReduxPromiseAction<ReadonlyArray<TestAnswerFromBackend>, RUN_test_PENDINGAction, RUN_test_FULFILLEDAction, RUN_test_REJECTEDAction> {
  readonly type: ActionType.RUN_test
  readonly payload: Promise<ReadonlyArray<TestAnswerFromBackend>>
}

//automatically dispatched by redux promise middleware
export interface RUN_test_PENDINGAction
  extends ActionBase {
  readonly type: ActionType.RUN_test_PENDING
}

//automatically dispatched by redux promise middleware
export interface RUN_test_FULFILLEDAction
  extends ActionBase, ReduxPromiseFulfilledAction<ReadonlyArray<TestAnswerFromBackend>> {
  readonly type: ActionType.RUN_test_FULFILLED
  readonly payload: ReadonlyArray<TestAnswerFromBackend>
}

//automatically dispatched by redux promise middleware
export interface RUN_test_REJECTEDAction
  extends ActionBase, ReduxPromiseRejectedAction<void> {
  readonly type: ActionType.RUN_test_REJECTED
  readonly payload: any
}


export type AllActions =
  RUN_test_PENDINGAction
  | RUN_test_FULFILLEDAction
  | RUN_test_REJECTEDAction


export function reducer(state: State = initial, action: AllActions): State {

  switch (action.type) {

    case ActionType.RUN_test_PENDING:
      return {
        ...state,
        isLoading: false //we use the nice test number spinner for loading
      }
    case ActionType.RUN_test_FULFILLED:
      return {
        ...state,
        isLoading: false,
        exercise: {
          ...state.exercise,
          tests: mergeNormalTestResults(state.exercise.tests, action.payload)
        }
      }
    case ActionType.RUN_test_REJECTED:
      return {
        ...state,
        isLoading: false,
        exercise: {
          ...state.exercise,
          tests: mergeNormalTestResults(state.exercise.tests, [])
        }
      }

    default:
      notExhaustive(action)
      return state
  }
}

