/**
 * Created by janis dähne.
 */
import {ActionBase, ActionType} from "../doExerciseSiteActionTypes";
import {SolutionTupleFromBackend} from "../../../../types/exerciseSolution";
import {initial, State} from "../doExerciseSiteReducer";
import {ReduxPromiseAction, ReduxPromiseFulfilledAction, ReduxPromiseRejectedAction} from "../../index";
import {notExhaustive} from "../../_notExhausiveHelper";
import {updateFileReferencesAceStorage, updateFilesReference} from "../../../../helpers/editorHelper";
import {AceEditorsStorageHelper} from "../../../../components/codeEditors/aceEditors/aceEditorsStorageHelper";
import {doEditorEditorPanelViewEditorId} from "../../../../constants";
import {KnownLangs} from '../../../../../i18n/i18nRoot'


interface Meta {
  readonly langId: KnownLangs
}

export interface UPDATE_doExerciseSolutionAction
  extends ActionBase, ReduxPromiseAction<SolutionTupleFromBackend, UPDATE_doExerciseSolution_PENDINGAction, UPDATE_doExerciseSolution_FULFILLEDAction, UPDATE_doExerciseSolution_REJECTEDAction, Meta> {
  readonly type: ActionType.UPDATE_doExerciseSolution
  readonly payload: Promise<SolutionTupleFromBackend>
  readonly meta: Meta
}

//automatically dispatched by redux promise middleware
export interface UPDATE_doExerciseSolution_PENDINGAction
  extends ActionBase {
  readonly type: ActionType.UPDATE_doExerciseSolution_PENDING
  readonly meta: Meta
}

//automatically dispatched by redux promise middleware
export interface UPDATE_doExerciseSolution_FULFILLEDAction
  extends ActionBase, ReduxPromiseFulfilledAction<SolutionTupleFromBackend, Meta> {
  readonly type: ActionType.UPDATE_doExerciseSolution_FULFILLED
  readonly payload: SolutionTupleFromBackend
}

//automatically dispatched by redux promise middleware
export interface UPDATE_doExerciseSolution_REJECTEDAction
  extends ActionBase, ReduxPromiseRejectedAction<void, Meta> {
  readonly type: ActionType.UPDATE_doExerciseSolution_REJECTED
  readonly payload: any
}


export type AllActions =
  UPDATE_doExerciseSolution_PENDINGAction
  | UPDATE_doExerciseSolution_FULFILLEDAction
  | UPDATE_doExerciseSolution_REJECTEDAction


export function reducer(state: State = initial, action: AllActions): State {

  switch (action.type) {

    case ActionType.UPDATE_doExerciseSolution_PENDING:
      return {
        ...state,
        isLoading: true
      }
    case ActionType.UPDATE_doExerciseSolution_FULFILLED:

      /** this is the same as {@link UPDATE_doExerciseSolution_FULFILLEDAction} */

      const storedTabs = AceEditorsStorageHelper.getEditor(doEditorEditorPanelViewEditorId)
      updateFileReferencesAceStorage(storedTabs, action.payload.solution.solutionFiles, action.payload.fileIdAssociationMap, action.meta.langId)

      //for the initial create we need to map the frontend id to the backend id... (even though the initial creation is on server side
      //but maybe for further versions...)
      //all other updates will have the same frontned id = backend id because the ids don't change

      const tuple = action.payload.fileIdAssociationMap.find(
        p => p.frontendFileId === state.doEditorState.selectedTabIdDoEditor)

      let newSelectedTabId = tuple
                               ? tuple.backendFileId
                               : -1

      return {
        ...state,
        isLoading: false,

        //this needs to be BEFORE setting solution somehow...
        doEditorState: {
          ...state.doEditorState,
          selectedTabIdDoEditor: newSelectedTabId,
        },
        solution: {
          ...state.solution,
          lastUpdatedAt: action.payload.solution.lastUpdatedAt,
          pLangId: action.payload.solution.pLangId,
          mainFileId: action.payload.solution.mainFileId,
          //the display order is preserved because we take the new ids (and some props) with updateFilesReference
          solutionFiles: updateFilesReference(state.solution.solutionFiles,
                                              action.payload.solution.solutionFiles, action.payload.fileIdAssociationMap, action.meta.langId)
        },


        //solution: action.payload.solutionVersion //do NOT take the new version else the editor has no undo action??
      }
    case ActionType.UPDATE_doExerciseSolution_REJECTED:
      return {
        ...state,
        isLoading: false
      }

    default:
      notExhaustive(action)
      return state
  }
}

