/**
 * Created by janis dähne.
 */
import {ActionBase, ActionType} from "./changeCustomTestDialogActionTypes";
import {initial, State} from "./changeCustomTestDialogReducer";
import Logger from "../../../../../helpers/logger";
import {ReduxPromiseAction, ReduxPromiseFulfilledAction, ReduxPromiseRejectedAction} from "../../../index";
import {notExhaustive} from "../../../_notExhausiveHelper";
import {CustomTestFromBackend} from "../../../../../types/customTests";
import {convertCustomTestFromBackendToFrontend} from "../../../../../helpers/convertersAndTransformers";

export interface CREATE_customTestAction
  extends ActionBase, ReduxPromiseAction<CustomTestFromBackend, CREATE_customTest_PENDINGAction, CREATE_customTest_FULFILLEDAction, CREATE_customTest_REJECTEDAction> {
  readonly type: ActionType.CREATE_customTest
  readonly payload: Promise<CustomTestFromBackend>
}

//automatically dispatched by redux promise middleware
export interface CREATE_customTest_PENDINGAction
  extends ActionBase {
  readonly type: ActionType.CREATE_customTest_PENDING
}

//automatically dispatched by redux promise middleware
export interface CREATE_customTest_FULFILLEDAction
  extends ActionBase, ReduxPromiseFulfilledAction<CustomTestFromBackend> {
  readonly type: ActionType.CREATE_customTest_FULFILLED
  readonly payload: CustomTestFromBackend
}

//automatically dispatched by redux promise middleware
export interface CREATE_customTest_REJECTEDAction
  extends ActionBase, ReduxPromiseRejectedAction<void> {
  readonly type: ActionType.CREATE_customTest_REJECTED
  readonly payload: any
}


export type AllActions =
  CREATE_customTest_PENDINGAction
  | CREATE_customTest_FULFILLEDAction
  | CREATE_customTest_REJECTEDAction


export function reducer(state: State = initial(), action: AllActions): State {

  switch (action.type) {

    case ActionType.CREATE_customTest_PENDING:
      return {
        ...state,
        isBusy: true
      }
    case ActionType.CREATE_customTest_FULFILLED:
      return {
        ...state,
        isBusy: false,
        customTest: convertCustomTestFromBackendToFrontend(action.payload)
      }
    case ActionType.CREATE_customTest_REJECTED:
      return {
        ...state,
        isBusy: false
      }

    default:
      notExhaustive(action)
      return state
  }
}

