/**
 * Created by janis dähne.
 */

import {notExhaustive} from "../_notExhausiveHelper";
import {DoExerciseFromBackend, DoExerciseFrontendExclusive} from "../../../types/doExercise";
import * as moment from "moment";
import {ActionBase, ActionType} from "./doExerciseSiteActionTypes";
import {initial as initialCodeTemplates} from './subSets/setDoExerciseCodeTemplatesReducer'
import {
  initial as initialExerciseDescription,
  AllActions as SetExerciseDescriptionActions,
  reducer as setDoExerciseDescriptionReducer
} from './subSets/setDoExerciseDescriptionReducer'
import {initial as initialDefaultTestSettings} from './subSets/setDoExerciseDefaultTestSettingsReducer'
import {initial as initialExerciseRelease} from './subSets/setDoExerciseReleaseReducer'
import {convertDoExerciseFromBackendToFrontend} from "../../../helpers/convertersAndTransformers";
import {RunnableExerciseTestFrontendExclusive} from "../../../types/exerciseEditor";
import {RunnableCustomTestFrontendExclusive} from "../../../types/customTests";


export type State = DoExerciseFrontendExclusive


export const initial: State = {
  id: -1,
  displayName: '',
  shortDescription: '',
  canUserCreateFiles: false,
  canUserCreateCustomTests: false,
  lastEditedPLangId: -1,
  isPermanentlyLocked: false,
  firstAccessedAt: moment(),
  codeTemplates: initialCodeTemplates,
  exerciseDescription: initialExerciseDescription,
  customTests: [],
  defaultCustomTestSettings: initialDefaultTestSettings,
  tests: [],
  exerciseRelease: initialExerciseRelease,
  lockSolutionsFlag: false
}

export interface SET_doExerciseAction extends ActionBase {
  readonly type: ActionType.SET_doExercise
  readonly doExercise: DoExerciseFromBackend
}


export interface SET_testsAction extends ActionBase {
  readonly type: ActionType.SET_tests
  readonly tests: ReadonlyArray<RunnableExerciseTestFrontendExclusive>
}


export interface SET_customTestsAction extends ActionBase {
  readonly type: ActionType.SET_customTests
  readonly customTests: ReadonlyArray<RunnableCustomTestFrontendExclusive>
}


export type AllActions =
  SET_doExerciseAction
  | SET_testsAction
  | SET_customTestsAction



export function reducer(state: State = initial, action: AllActions): State {

  switch (action.type) {

    case ActionType.SET_doExercise:
      return convertDoExerciseFromBackendToFrontend(action.doExercise)

    case ActionType.SET_tests:
      return {
        ...state,
        tests: action.tests
      }

    case ActionType.SET_customTests:
      return {
        ...state,
        customTests: action.customTests
      }

    default:
      notExhaustive(action)
      return state
  }
}

