/**
 * Created by janis dähne.
 */
import {notExhaustive} from "../_notExhausiveHelper";
import {ActionBase, ActionType} from "./doExerciseSiteActionTypes";
import {DoExerciseFrontendExclusive} from "../../../types/doExercise";

import {
  AllActions as DoExerciseActions,
  initial as initialDoExercise,
  reducer as doExerciseReducer
} from './doExerciseReducer'
import {AllActions as GetDoExerciseActions, reducer as getDoExerciseReducer} from './crud/getDoExerciseReducer'
import {SolutionDoExerciseFullBase} from "../../../types/exerciseSolution";
import {
  AllActions as DoExerciseSolutionActions,
  initial as initialSolution,
  reducer as setDoExerciseSolutionReducer
} from './subSets/setDoExerciseSolutionReducer'
import {AllActions as GetSolutionActions, reducer as getSolutionReducer} from './crud/getSolutionReducer'
import {
  AllActions as GetAllTestResultActions,
  reducer as getAllTestResultsReducer
} from './crud/getAllTestResultsReducer'
import {
  RunnableAfterCustomTestFrontendExclusive,
  RunnableCustomTestFrontendExclusive
} from "../../../types/customTests";
import {
  FilePreviewFromBackend,
  RunnableExerciseTestFrontendExclusive
} from "../../../types/exerciseEditor";

import {
  AllActions as DoEditorActions,
  DoEditorState as DoEditorState,
  initial as initialDoEditorState,
  reducer as doEditorReducer
} from './editorReducers/doEditorReducer'


import {
  AllActions as UpdateDoExerciseSolutionActions,
  reducer as updateDoEditorSolutionReducer
} from './crud/updateDoEditorSolutionReducer'

import {AllActions as RunTestActions, reducer as runTestReducer} from './crud/runTestReducer'
import {AllActions as RunCustomTestActions, reducer as runCustomTestReducer} from './crud/runCustomTestReducer'
import {
  AllActions as RunCompileSingleFileActions,
  reducer as runCompileSingleFileReducer
} from './crud/runCompileSingleFileReducer'

import {AllActions as DeleteCustomTestActions, reducer as deleteCustomTestReducer} from './crud/deleteCustomTestReducer'
import {TestAnswerFromBackend} from "../../../types/testsAndTestProtocol";
import {ExercisePreviewsFrontendExclusive} from "../../../components/sites/allAccessibleExercisesSite/openExercises/allOpenExercisesSite";
import {AssessedSubmitTestFrontendExclusive, AssessmentResultForUser} from "../../../types/testResults";

import {AllActions as GetSubmitTestResultsActions, reducer as getSubmitTestResults} from './crud/getSubmitTestResults'
import {PanelViewSize} from "../../../components/panelSystem/panelGrid/panelGrid";
import {FrontendSettingsManager} from "../../../helpers/frontendSettingsManager";
import {
  AllActions as AfterEditorActions,
  initial as initialAfterEditorState,
  reducer as afterEditorReducer
} from './editorReducers/afterEditorReducer'
import {
  AllActions as AllSetAfterSolutionActions,
  initial as initialAfterSolution,
  reducer as setAfterSolutionReducer
} from './subSets/setAfterSolutionReducer'

import {
  AllActions as AllRunAfterNormalTestActions,
  reducer as runAfterNormalTest
} from './crud/afterSolution/runAfterNormalTest'
import {
  AllActions as AllRunAfterSubmitTestActions,
  reducer as runAfterSubmitTest
} from './crud/afterSolution/runAfterSubmitTest'
import Logger from "../../../helpers/logger";

import {
  AllActions as SetShouldNotCountParticipationActions,
  reducer as setShouldNotCountParticipationReducer
} from './crud/setShouldNotCountParticipationReducer'
import {RunnableCustomProjectTestFrontendExclusive} from "../../../types/customProjects";


import {AllActions as AllJustRunProgramActions, reducer as justRunProgramReducer} from './crud/justRunProgramReducer'
import {AfterSolutionDoExerciseFullBase} from '../../../types/exerciseAfterSolution'
import {
  AllActions as GetAfterSolutionActions,
  reducer as getAfterSolutionReducer
} from './crud/afterSolution/getAfterSolutionReducer'
import {
  AllActions as UpdateAfterSolutionActions,
  reducer as updateAfterSolution
} from './crud/afterSolution/updateAfterSolution'
import {
  AllActions as GetAllAfterTestResultsActions,
  reducer as getAllAfterTestResultsReducer
} from './crud/afterSolution/getAllAfterTestResultsReducer'
import {
  AllActions as GetAfterSubmitTestResultsActions,
  reducer as getAfterSubmitTestResultsReducer
} from './crud/afterSolution/getAfterSubmitTestResultsReducer'
import {
  AllActions as RunAfterCustomTestActions,
  reducer as runAfterCustomTest
} from './crud/afterSolution/runAfterCustomTest'
import {AllActions as ResetUserCodeActions, reducer as resetUserSolution} from './crud/resetUserSolution'
import {
  AllActions as AllSetLockSolutionsFlagActions,
  reducer as setLockSolutionsFlagReducer
} from './crud/setLockSolutionsFlagReducer'
import {ActionTypeGlobal, RESET_GlobalAction} from '../globalActions'
import {
  AllActions as AllGetCustomTestAssetPreviewsActions,
  reducer as getCustomTestAssetPreviewsReducer
} from './crud/assets/getAllUserAssetPreviewsReducer'
import {
  AllActions as AllAttachCustomTestAssetPreviewRActions,
  reducer as attachCustomTestAssetPreviewReducer
} from './crud/assets/attachCustomTestAssetPreviewReducer'
import {
  AllActions as AllDetachCustomTestAssetPreviewActions,
  reducer as detachCustomTestAssetPreviewReducer
} from './crud/assets/detachCustomTestAssetPreviewReducer'
import {
  AllActions as AllGetSingleNormalTestResultActions,
  reducer as getSingleNormalTestResultReducer
} from './crud/getSingleNormalTestResultReducer'
import {
  AllActions as AllGetSingleSubmitTestResultActions,
  reducer as getSingleSubmitTestResultReducer
} from './crud/getSingleSubmitlTestResultReducer'
import {
  AllActions as AllGetSingleCustomTestResultActions,
  reducer as getSingleCustomTestResultReducer
} from './crud/getSingleCustomTestResultReducer'
import {
  AllActions as AllGetSingleAfterNormalTestResultActions,
  reducer as getSingleAfterNormalTestResultReducer
} from './crud/afterSolution/getSingleAfterNormalTestResultReducer'
import {
  AllActions as AllGetSingleAfterCustomTestResultActions,
  reducer as getSingleAfterCustomTestResultReducer
} from './crud/afterSolution/getSingleAfterCustomTestResultReducer'
import {
  AllActions as AllGetSingleAfterSubmitTestResultActions,
  reducer as getSingleAfterSubmitTestResultReducer
} from './crud/afterSolution/getSingleAfterSubmitTestResultReducer'
import {PanelSystemStateAddition} from '../../../types/panelSystem'

export const justRunProgramArgsMaxLength = 1000

export type State = {

  readonly exercise: DoExerciseFrontendExclusive

  /**
   * the submit test results, only available if the release has ended
   */
  readonly submitTestResults: ReadonlyArray<AssessedSubmitTestFrontendExclusive>

  readonly assessmentResult: AssessmentResultForUser | null

  /**
   * THIS IST NOT RECEIVED FROM BACKEND... just to store some data...
   *
   * stores some more infos than exercise.release
   *
   * e.g. left time
   */
  readonly exercisePreviewFrontendTemp: ExercisePreviewsFrontendExclusive | null

  readonly solution: SolutionDoExerciseFullBase


  /**
   * the currently selected test to display in the console view
   *
   * no id because it could be RunnableExerciseTestFrontendExclusive | RunnableCustomTestFrontendExclusive | AssessedSubmitTestFrontendExclusive
   */
  readonly selectedConsoleViewTest: RunnableExerciseTestFrontendExclusive | RunnableCustomTestFrontendExclusive | AssessedSubmitTestFrontendExclusive | null

  /**
   * normally the test results are attached to the tests... but for this command we have no test...
   * we only check if the CURRENT (not saved in db) state of the current file compiles
   *
   * this is set via async crud action callback
   */
  readonly lastCompileSingleFileResult: TestAnswerFromBackend | null

  /**
   * the last just run program result (like lastCompileSingleFileResult)
   *
   * this is set via async crud action callback
   */
  readonly lastJustRunProgramResult: TestAnswerFromBackend | null

  readonly isLoading: boolean

  readonly isOnGotFocusInterceptionOverlayDisplayed: boolean

  /**
   * true: the left time string is displayed, false: not
   * can be changed by the user or
   * @see lockSolutionsFlag
   */
  readonly isLeftTimeStringDisplayed: boolean

  //for the exercise description assets
  readonly isAddCustomTestAssetFilesModalDisplayed: boolean
  readonly isAddCustomTestAssetFilesModalBusy: boolean
  readonly addCustomTestAssetFilesModalFilterText: string
  readonly addCustomTestAssetFilesModalUploadPercentage: number | null | 'active'
  /**
   * all possible custom test asset file previews
   * set via CRUD
   */
  readonly allCustomTestAssetFilePreviews: ReadonlyArray<FilePreviewFromBackend>

  /**
   * the args for the just run operation
   *
   * must be <=
   * @see justRunProgramArgsMaxLength
   */
  readonly justRunProgramArgs: string

  readonly isJustRunProgramArgsInputDisplayed: boolean

  readonly doEditorState: DoEditorState
  /**
   * editor after the release finished or the time is up
   */
  readonly afterEditorState: DoEditorState

  /**
   * same as afterEditorState but for the solution
   * null if don't exists yet
   */
  readonly afterSolution: AfterSolutionDoExerciseFullBase | null

  /**
   * same as afterEditorState but for the normal tests
   *
   * set initially via CRUD when we receive in {@link GET_doExerciseAction}
   */
  readonly afterNormalTests: ReadonlyArray<RunnableExerciseTestFrontendExclusive>

  /**
   * WE ONLY STORE ui props and after results here...
   * for e.g. lists, edit we use the original exercise custom tests because they need to stay in sync (this and the exercise custom tests)
   *
   * set initially via CRUD when we receive in {@link GET_doExerciseAction}
   */
  readonly afterCustomTestResults: ReadonlyArray<RunnableAfterCustomTestFrontendExclusive>
  /**
   * same as afterEditorState but for the normal submit tests
   *
   * set initially via CRUD when received in {@link GET_submitTestResultsAction}
   */
  readonly afterSubmitTests: ReadonlyArray<AssessedSubmitTestFrontendExclusive>


  //TODO do we need this?
  readonly selectedPLangId: number

  readonly isCustomTestDialogDisplayed: boolean
  readonly isViewTestDialogDisplayed: boolean
  readonly isCodeEditorSettingsDialogDisplayed: boolean

  /**
   * toggle this to force a re render of hte preview panel (when assets are finished creating)
   * & math jax should be rendered (else we would render mathjax 2 times which is bad and could
   * render some formulas twice
   *
   * TODO rename... e.g. finished creating dependencies or smth
   */
  readonly dummy: boolean

  /**
   * if true then lock the site (readonly) and hide the left time
   * this pretends the user 'submitted' the solution (even if it's just a checkbox
   * @ignore obsolete... currently not used and always set to false in CRUD even if backend returns true
   *  maybe we use it some day again...
   */
  readonly lockSolutionsFlag: boolean


  /**
   * we store the last known editor content
   * after x mins we check it again and if it has changed then we don't update
   * the auth token lifetime
   */
  readonly lastKnownEditorContent: string

} & PanelSystemStateAddition

export const initial: State = {

  exercise: initialDoExercise,
  exercisePreviewFrontendTemp: null,

  submitTestResults: [],
  assessmentResult: null,

  solution: initialSolution,
  afterSolution: null,
  selectedConsoleViewTest: null,
  lastCompileSingleFileResult: null,
  lastJustRunProgramResult: null,


  isLeftTimeStringDisplayed: true,
  isLoading: false,
  isOnGotFocusInterceptionOverlayDisplayed: false,

  isAddCustomTestAssetFilesModalDisplayed: false,
  isAddCustomTestAssetFilesModalBusy: false,
  addCustomTestAssetFilesModalFilterText: '',
  addCustomTestAssetFilesModalUploadPercentage: null,
  allCustomTestAssetFilePreviews: [],

  topRightSelectedTabIndex: 0,
  topLeftSelectedTabIndex: 0,
  bottomRightSelectedTabIndex: 0,
  bottomLeftSelectedTabIndex: 0,

  topRightPanelViewSize: PanelViewSize.normal,
  topLeftPanelViewSize: PanelViewSize.normal,
  botRightPanelViewSize: PanelViewSize.normal,
  botLeftPanelViewSize: PanelViewSize.normal,

  hDividerPosOffset: 0,
  vDividerPosOffset: 150,


  isJustRunProgramArgsInputDisplayed: false,
  justRunProgramArgs: '',

  doEditorState: {...initialDoEditorState},
  afterEditorState: {...initialAfterEditorState},

  afterNormalTests: [],
  afterCustomTestResults: [],
  afterSubmitTests: [],

  selectedPLangId: -1,

  isCustomTestDialogDisplayed: false,
  isViewTestDialogDisplayed: false,
  isCodeEditorSettingsDialogDisplayed: false,

  dummy: true,
  lockSolutionsFlag: false,
  lastKnownEditorContent: ''
}

export interface SET_dummyAction extends ActionBase {
  readonly type: ActionType.SET_dummy
  readonly dummy: boolean
}

export interface SET_lastKnownEditorContentAction extends ActionBase {
  readonly type: ActionType.SET_lastKnownEditorContent
  readonly lastKnownEditorContent: string
}

export interface SET_justRunProgramArgsAction extends ActionBase {
  readonly type: ActionType.SET_justRunProgramArgs
  readonly justRunProgramArgs: string
}

export interface SET_isJustRunProgramArgsInputDisplayedAction extends ActionBase {
  readonly type: ActionType.SET_isJustRunProgramArgsInputDisplayed
  readonly isJustRunProgramArgsInputDisplayed: boolean
}

export interface SET_isLoadingAction extends ActionBase {
  readonly type: ActionType.SET_isLoading
  readonly isLoading: boolean
}

export interface SET_isOnGotFocusInterceptionOverlayDisplayedAction extends ActionBase {
  readonly type: ActionType.SET_isOnGotFocusInterceptionOverlayDisplayed
  readonly isOnGotFocusInterceptionOverlayDisplayed: boolean
}

//-- after normal tests, submit tests


export interface SET_afterNormalTestsAction extends ActionBase {
  readonly type: ActionType.SET_afterNormalTests
  readonly afterNormalTests: ReadonlyArray<RunnableExerciseTestFrontendExclusive>
}

export interface SET_afterCustomTestsAction extends ActionBase {
  readonly type: ActionType.SET_afterCustomTests
  readonly afterCustomTests: ReadonlyArray<RunnableAfterCustomTestFrontendExclusive>
}

export interface SET_afterSubmitTestsAction extends ActionBase {
  readonly type: ActionType.SET_afterSubmitTests
  readonly afterSubmitTests: ReadonlyArray<AssessedSubmitTestFrontendExclusive>
}


export interface SET_isLeftTimeStringDisplayedAction extends ActionBase {
  readonly type: ActionType.SET_isLeftTimeStringDisplayed
  readonly isLeftTimeStringDisplayed: boolean
}

export interface SET_exercisePreviewFrontendAction extends ActionBase {
  readonly type: ActionType.SET_exercisePreviewFrontend
  readonly exercisePreviewFrontendTemp: ExercisePreviewsFrontendExclusive | null
}

export interface SET_solutionAction extends ActionBase {
  readonly type: ActionType.SET_solution
  readonly solution: SolutionDoExerciseFullBase
}


export interface SET_selectedConsoleViewTestAction extends ActionBase {
  readonly type: ActionType.SET_selectedConsoleViewTest
  readonly selectedConsoleViewTest: RunnableExerciseTestFrontendExclusive | RunnableCustomTestFrontendExclusive | AssessedSubmitTestFrontendExclusive | null
}

export interface SET_selectedPLangIdAction extends ActionBase {
  readonly type: ActionType.SET_selectedPLangId
  readonly selectedPLangId: number
}


// --- panel view sizes

export interface SET_topRightPanelViewSizeAction extends ActionBase {
  readonly type: ActionType.SET_topRightPanelViewSize
  readonly topRightPanelViewSize: PanelViewSize
}

export interface SET_topLeftPanelViewSizeAction extends ActionBase {
  readonly type: ActionType.SET_topLeftPanelViewSize
  readonly topLeftPanelViewSize: PanelViewSize
}

export interface SET_botRightPanelViewSizeAction extends ActionBase {
  readonly type: ActionType.SET_botRightPanelViewSize
  readonly botRightPanelViewSize: PanelViewSize
}

export interface SET_botLeftPanelViewSizeAction extends ActionBase {
  readonly type: ActionType.SET_botLeftPanelViewSize
  readonly botLeftPanelViewSize: PanelViewSize
}


//--- panel selected indices

export interface SET_topRightSelectedTabIndexAction extends ActionBase {
  readonly type: ActionType.SET_topRightSelectedTabIndex
  readonly topRightSelectedTabIndex: number
}

export interface SET_topLeftSelectedTabIndexAction extends ActionBase {
  readonly type: ActionType.SET_topLeftSelectedTabIndex
  readonly topLeftSelectedTabIndex: number
}

export interface SET_bottomRightSelectedTabIndexAction extends ActionBase {
  readonly type: ActionType.SET_bottomRightSelectedTabIndex
  readonly bottomRightSelectedTabIndex: number
}

export interface SET_bottomLeftSelectedTabIndexAction extends ActionBase {
  readonly type: ActionType.SET_bottomLeftSelectedTabIndex
  readonly bottomLeftSelectedTabIndex: number
}


//--- panel grip

export interface SET_hDividerPosOffsetAction extends ActionBase {
  readonly type: ActionType.SET_hDividerPosOffset
  readonly hDividerPosOffset: number
}

export interface SET_vDividerPosOffsetAction extends ActionBase {
  readonly type: ActionType.SET_vDividerPosOffset
  readonly vDividerPosOffset: number
}


//--- dialogs

export interface SET_isCustomTestDialogDisplayedAction extends ActionBase {
  readonly type: ActionType.SET_isCustomTestDialogDisplayed
  readonly isCustomTestDialogDisplayed: boolean
}

export interface SET_isViewTestDialogDisplayedAction extends ActionBase {
  readonly type: ActionType.SET_isViewTestDialogDisplayed
  readonly isViewTestDialogDisplayed: boolean
}

export interface SET_isCodeEditorSettingsDialogDisplayedAction extends ActionBase {
  readonly type: ActionType.SET_isCodeEditorSettingsDialogDisplayed
  readonly isDisplayed: boolean
}

//--- custom test asset files

export interface SET_isAddCustomTestAssetFilesModalDisplayedAction extends ActionBase {
  readonly type: ActionType.SET_isAddCustomTestAssetFilesModalDisplayed
  readonly isDisplayed: boolean
  readonly isBusy: boolean
}

export interface SET_addCustomTestAssetFilesModalFilterTextAction extends ActionBase {
  readonly type: ActionType.SET_addCustomTestAssetFilesModalFilterText
  readonly addCustomTestAssetFilesModalFilterText: string
}

export interface SET_addCustomTestAssetFilesModalUploadPercentageAction extends ActionBase {
  readonly type: ActionType.SET_addCustomTestAssetFilesModalUploadPercentage
  readonly addCustomTestAssetFilesModalUploadPercentage: number | null | 'active'
}

//reset test results after saving code
//this also resets the custom test results
export interface RESET_TestResultsAction extends ActionBase {
  readonly type: ActionType.RESET_TestResults
}

export interface RESET_AfterTestResultsAction extends ActionBase {
  readonly type: ActionType.RESET_AfterTestResults
}


export interface ResetAction extends ActionBase {
  readonly type: ActionType.RESET
}

export type AllActions =
  ResetAction
  | RESET_TestResultsAction
  | RESET_AfterTestResultsAction

  | GetDoExerciseActions
  | GetSolutionActions
  | GetAllTestResultActions
  | GetAllAfterTestResultsActions
  | GetAfterSubmitTestResultsActions

  | AllSetLockSolutionsFlagActions

  | SET_exercisePreviewFrontendAction
  | SET_isLeftTimeStringDisplayedAction

  | SET_topRightSelectedTabIndexAction
  | SET_topLeftSelectedTabIndexAction
  | SET_bottomRightSelectedTabIndexAction
  | SET_bottomLeftSelectedTabIndexAction

  | SET_topRightPanelViewSizeAction
  | SET_topLeftPanelViewSizeAction
  | SET_botRightPanelViewSizeAction
  | SET_botLeftPanelViewSizeAction

  | SET_hDividerPosOffsetAction
  | SET_vDividerPosOffsetAction

  | SET_selectedPLangIdAction
  | SET_isCustomTestDialogDisplayedAction
  | SET_isViewTestDialogDisplayedAction
  | SET_isCodeEditorSettingsDialogDisplayedAction

  | RunTestActions
  | RunCustomTestActions
  | RunCompileSingleFileActions
  | RunAfterCustomTestActions

  | SET_dummyAction
  | SET_isLoadingAction
  | SET_isOnGotFocusInterceptionOverlayDisplayedAction

  | DeleteCustomTestActions
  | SetShouldNotCountParticipationActions

  | DoEditorActions
  | DoExerciseSolutionActions
  | DoExerciseActions

  | GetSubmitTestResultsActions

  | UpdateDoExerciseSolutionActions

  | SET_solutionAction
  | SET_selectedConsoleViewTestAction

  | AfterEditorActions

  | AllSetAfterSolutionActions

  | SET_afterNormalTestsAction
  | SET_afterCustomTestsAction
  | SET_afterSubmitTestsAction

  | AllRunAfterNormalTestActions
  | AllRunAfterSubmitTestActions
  | AllJustRunProgramActions

  | GetAfterSolutionActions
  | UpdateAfterSolutionActions

  | SET_justRunProgramArgsAction
  | SET_isJustRunProgramArgsInputDisplayedAction
  | ResetUserCodeActions

  | SET_lastKnownEditorContentAction

  | SET_isAddCustomTestAssetFilesModalDisplayedAction
  | SET_addCustomTestAssetFilesModalFilterTextAction
  | SET_addCustomTestAssetFilesModalUploadPercentageAction

  | AllGetCustomTestAssetPreviewsActions
  | AllAttachCustomTestAssetPreviewRActions
  | AllDetachCustomTestAssetPreviewActions
  | AllGetSingleNormalTestResultActions
  | AllGetSingleSubmitTestResultActions
  | AllGetSingleCustomTestResultActions
  | AllGetSingleAfterNormalTestResultActions
  | AllGetSingleAfterCustomTestResultActions
  | AllGetSingleAfterSubmitTestResultActions

  | RESET_GlobalAction

export function reducer(state: State = initial, action: AllActions): State {

  switch (action.type) {

    case ActionType.UPDATE_lockSolutionsFlag_PENDING:
    case ActionType.UPDATE_lockSolutionsFlag_FULFILLED:
    case ActionType.UPDATE_lockSolutionsFlag_REJECTED:
      return setLockSolutionsFlagReducer(state, action)

    case ActionType.RESET_userSolution_PENDING:
    case ActionType.RESET_userSolution_FULFILLED:
    case ActionType.RESET_userSolution_REJECTED:
      return resetUserSolution(state, action)


    case ActionType.GET_doExercise_afterSolution_PENDING:
    case ActionType.GET_doExercise_afterSolution_FULFILLED:
    case ActionType.GET_doExercise_afterSolution_REJECTED:
      return getAfterSolutionReducer(state, action)

    case ActionType.UPDATE_doExercise_afterSolution_PENDING:
    case ActionType.UPDATE_doExercise_afterSolution_FULFILLED:
    case ActionType.UPDATE_doExercise_afterSolution_REJECTED:
      return updateAfterSolution(state, action)

    case ActionType.JUST_RUN_program_PENDING:
    case ActionType.JUST_RUN_program_FULFILLED:
    case ActionType.JUST_RUN_program_REJECTED:
      return justRunProgramReducer(state, action)

    case ActionType.SET_shouldNotCountParticipation_PENDING:
    case ActionType.SET_shouldNotCountParticipation_FULFILLED:
    case ActionType.SET_shouldNotCountParticipation_REJECTED:
      return setShouldNotCountParticipationReducer(state, action)

    case ActionType.RUN_afterNormalTest_PENDING:
    case ActionType.RUN_afterNormalTest_FULFILLED:
    case ActionType.RUN_afterNormalTest_REJECTED:
      return runAfterNormalTest(state, action)

    case ActionType.RUN_afterSubmitTest_PENDING:
    case ActionType.RUN_afterSubmitTest_FULFILLED:
    case ActionType.RUN_afterSubmitTest_REJECTED:
      return runAfterSubmitTest(state, action)

    case ActionType.DELETE_customTest_PENDING:
    case ActionType.DELETE_customTest_FULFILLED:
    case ActionType.DELETE_customTest_REJECTED:
      return deleteCustomTestReducer(state, action)

    case ActionType.GET_allTestResults_PENDING:
    case ActionType.GET_allTestResults_FULFILLED:
    case ActionType.GET_allTestResults_REJECTED:
      return getAllTestResultsReducer(state, action)

    case ActionType.GET_singleNormalTestResults_PENDING:
    case ActionType.GET_singleNormalTestResults_FULFILLED:
    case ActionType.GET_singleNormalTestResults_REJECTED:
      return getSingleNormalTestResultReducer(state, action)

    case ActionType.GET_singleCustomTestResults_PENDING:
    case ActionType.GET_singleCustomTestResults_FULFILLED:
    case ActionType.GET_singleCustomTestResults_REJECTED:
      return getSingleCustomTestResultReducer(state, action)

    case ActionType.GET_allAfterTestResults_PENDING:
    case ActionType.GET_allAfterTestResults_FULFILLED:
    case ActionType.GET_allAfterTestResults_REJECTED:
      return getAllAfterTestResultsReducer(state, action)

    case ActionType.GET_singleAfterNormalTestResult_PENDING:
    case ActionType.GET_singleAfterNormalTestResult_FULFILLED:
    case ActionType.GET_singleAfterNormalTestResult_REJECTED:
      return getSingleAfterNormalTestResultReducer(state, action)

    case ActionType.GET_singleAfterCustomTestResult_PENDING:
    case ActionType.GET_singleAfterCustomTestResult_FULFILLED:
    case ActionType.GET_singleAfterCustomTestResult_REJECTED:
      return getSingleAfterCustomTestResultReducer(state, action)

    case ActionType.GET_singleAfterSubmitTestResult_PENDING:
    case ActionType.GET_singleAfterSubmitTestResult_FULFILLED:
    case ActionType.GET_singleAfterSubmitTestResult_REJECTED:
      return getSingleAfterSubmitTestResultReducer(state, action)

    case ActionType.GET_doExercise_solutionVersion_PENDING:
    case ActionType.GET_doExercise_solutionVersion_FULFILLED:
    case ActionType.GET_doExercise_solutionVersion_REJECTED:
      return getSolutionReducer(state, action)

    case ActionType.GET_doExercise_PENDING:
    case ActionType.GET_doExercise_FULFILLED:
    case ActionType.GET_doExercise_REJECTED:
      return getDoExerciseReducer(state, action)


    case ActionType.UPDATE_doExerciseSolution_PENDING:
    case ActionType.UPDATE_doExerciseSolution_FULFILLED:
    case ActionType.UPDATE_doExerciseSolution_REJECTED:
      return updateDoEditorSolutionReducer(state, action)

    case ActionType.RUN_test_PENDING:
    case ActionType.RUN_test_FULFILLED:
    case ActionType.RUN_test_REJECTED:
      return runTestReducer(state, action)

    case ActionType.RUN_customTest_PENDING:
    case ActionType.RUN_customTest_FULFILLED:
    case ActionType.RUN_customTest_REJECTED:
      return runCustomTestReducer(state, action)

    case ActionType.RUN_compileSingleFile_PENDING:
    case ActionType.RUN_compileSingleFile_FULFILLED:
    case ActionType.RUN_compileSingleFile_REJECTED:
      return runCompileSingleFileReducer(state, action)

    case ActionType.RUN_afterCustomTest_PENDING:
    case ActionType.RUN_afterCustomTest_FULFILLED:
    case ActionType.RUN_afterCustomTest_REJECTED:
      return runAfterCustomTest(state, action)

    case ActionType.GET_submitTestResults_PENDING:
    case ActionType.GET_submitTestResults_FULFILLED:
    case ActionType.GET_submitTestResults_REJECTED:
      return getSubmitTestResults(state, action)

    case ActionType.GET_singleSubmitTestResults_PENDING:
    case ActionType.GET_singleSubmitTestResults_FULFILLED:
    case ActionType.GET_singleSubmitTestResults_REJECTED:
      return getSingleSubmitTestResultReducer(state, action)

    case ActionType.GET_afterSubmitTestResults_PENDING:
    case ActionType.GET_afterSubmitTestResults_FULFILLED:
    case ActionType.GET_afterSubmitTestResults_REJECTED:
      return getAfterSubmitTestResultsReducer(state, action)

    case ActionType.SET_doExercise:
    case ActionType.SET_tests:
    case ActionType.SET_customTests:
      return {
        ...state,
        exercise: doExerciseReducer(state.exercise, action)
      }

    case ActionType.SET_afterNormalTests:
      return {
        ...state,
        afterNormalTests: action.afterNormalTests
      }

    case ActionType.SET_afterCustomTests:
      return {
        ...state,
        afterCustomTestResults: action.afterCustomTests
      }

    case ActionType.SET_afterSubmitTests:
      return {
        ...state,
        afterSubmitTests: action.afterSubmitTests
      }

    case ActionType.SET_exercisePreviewFrontend: {
      return {
        ...state,
        exercisePreviewFrontendTemp: action.exercisePreviewFrontendTemp
      }
    }
    case ActionType.SET_isLeftTimeStringDisplayed: {
      return {
        ...state,
        isLeftTimeStringDisplayed: action.isLeftTimeStringDisplayed
      }
    }

    case ActionType.SET_isCustomTestDialogDisplayed: {
      return {
        ...state,
        isCustomTestDialogDisplayed: action.isCustomTestDialogDisplayed
      }
    }

    case ActionType.SET_isViewTestDialogDisplayed: {
      return {
        ...state,
        isViewTestDialogDisplayed: action.isViewTestDialogDisplayed
      }
    }

    case ActionType.SET_isCodeEditorSettingsDialogDisplayed: {
      return {
        ...state,
        isCodeEditorSettingsDialogDisplayed: action.isDisplayed
      }
    }

    case ActionType.RESET_TestResults: {
      return {
        ...state,
        exercise: {
          ...state.exercise,
          tests: state.exercise.tests.map<RunnableExerciseTestFrontendExclusive>(p => {
            return {
              ...p,
              result: null
            }
          }),
          customTests: state.exercise.customTests.map<RunnableCustomTestFrontendExclusive>(p => {
            return {
              ...p,
              result: null
            }
          })
        }
      }
    }

    case ActionType.RESET_AfterTestResults: {
      return {
        ...state,
        afterNormalTests: state.afterNormalTests.map<RunnableExerciseTestFrontendExclusive>(p => {
          return {
            ...p,
            result: null
          }
        }),
        afterCustomTestResults: state.afterCustomTestResults.map<RunnableAfterCustomTestFrontendExclusive>(p => {
          return {
            ...p,
            result: null
          }
        }),
        afterSubmitTests: state.afterSubmitTests.map<AssessedSubmitTestFrontendExclusive>(p => {
          return {
            ...p,
            result: null
          }
        }),
      }
    }

    //--- only used for directly settings the solution (not from backend)
    // this is used to set the code from frontend settings (recovery) TODO remove
    case ActionType.SET_solution:
      return {
        ...state,
        solution: action.solution,
      }

    //-- normal solution
    case ActionType.SET_solutionFiles:

      // FrontendSettingsManager.setDoEditorSolution({
      //     ...state.solution,
      //     solutionFiles: [...action.solutionFiles]
      //   }
      // )

      return {
        ...state,
        solution: setDoExerciseSolutionReducer(state.solution, action),
      }

    case ActionType.SET_mainFileId:

      // FrontendSettingsManager.setDoEditorSolution({
      //     ...FrontendSettingsManager.getFrontendSettings().doEditorSolution,
      //     mainFileId: action.mainFileId
      //   }
      // )

      return {
        ...state,
        solution: setDoExerciseSolutionReducer(state.solution, action),
        doEditorState: {
          ...state.doEditorState,
          hasUnsavedChanges: true
        }
      }

    //-- after solution
    case ActionType.SET_afterSolutionFirstDummy:
      return {
        ...state,
        afterSolution: setAfterSolutionReducer(state.afterSolution, action),
      }
    case ActionType.SET_afterSolutionFiles:
      return {
        ...state,
        afterSolution: setAfterSolutionReducer(state.afterSolution, action),
      }
    case ActionType.SET_afterMainFileId:
      return {
        ...state,
        afterSolution: setAfterSolutionReducer(state.afterSolution, action),
        afterEditorState: {
          ...state.afterEditorState,
          hasUnsavedChanges: true
        }
      }


    case ActionType.SET_selectedPLangId:
      return {
        ...state,
        selectedPLangId: action.selectedPLangId
      }

    case ActionType.SET_hasFileChangedMap:
    case ActionType.SET_hasUnsavedChanges:
    case ActionType.SET_selectedTabIdDoEditor:
    case ActionType.SET_isFileExplorerDisplayedDoEditor:
    case ActionType.SET_newFileNameDoEditor:
    case ActionType.SET_renamingFileIdDoEditor:
    case ActionType.SET_isRenamingFileDoEditor:
    case ActionType.SET_renamingFileNameDoEditor:
      return {
        ...state,
        doEditorState: doEditorReducer(state.doEditorState, action)
      }

    case ActionType.SET_afterEditor_hasFileChangedMap:
    case ActionType.SET_afterEditor_hasUnsavedChanges:
    case ActionType.SET_afterEditor_selectedTabIdDoEditor:
    case ActionType.SET_afterEditor_isFileExplorerDisplayedDoEditor:
    case ActionType.SET_afterEditor_newFileNameDoEditor:
    case ActionType.SET_afterEditor_renamingFileIdDoEditor:
    case ActionType.SET_afterEditor_isRenamingFileDoEditor:
    case ActionType.SET_afterEditor_renamingFileNameDoEditor:
      return {
        ...state,
        afterEditorState: afterEditorReducer(state.afterEditorState, action)
      }


    case ActionType.SET_selectedConsoleViewTest:
      return {
        ...state,
        selectedConsoleViewTest: action.selectedConsoleViewTest
      }

    case ActionType.SET_topRightSelectedTabIndex:
      return {
        ...state,
        topRightSelectedTabIndex: action.topRightSelectedTabIndex
      }

    case ActionType.SET_topLeftSelectedTabIndex:
      return {
        ...state,
        topLeftSelectedTabIndex: action.topLeftSelectedTabIndex
      }

    case ActionType.SET_bottomRightSelectedTabIndex:
      return {
        ...state,
        bottomRightSelectedTabIndex: action.bottomRightSelectedTabIndex
      }

    case ActionType.SET_bottomLeftSelectedTabIndex:
      return {
        ...state,
        bottomLeftSelectedTabIndex: action.bottomLeftSelectedTabIndex
      }

    case ActionType.SET_hDividerPosOffset:
      return {
        ...state,
        hDividerPosOffset: action.hDividerPosOffset
      }
    case ActionType.SET_vDividerPosOffset:
      return {
        ...state,
        vDividerPosOffset: action.vDividerPosOffset
      }

    case ActionType.SET_topRightPanelViewSize:
      return {
        ...state,
        topRightPanelViewSize: action.topRightPanelViewSize
      }

    case ActionType.SET_topLeftPanelViewSize:
      return {
        ...state,
        topLeftPanelViewSize: action.topLeftPanelViewSize
      }

    case ActionType.SET_botRightPanelViewSize:
      return {
        ...state,
        botRightPanelViewSize: action.botRightPanelViewSize
      }

    case ActionType.SET_botLeftPanelViewSize:
      return {
        ...state,
        botLeftPanelViewSize: action.botLeftPanelViewSize
      }

    case ActionType.SET_dummy:
      return {
        ...state,
        dummy: action.dummy
      }

    case ActionType.SET_isLoading:
      return {
        ...state,
        isLoading: action.isLoading
      }

    case ActionType.SET_isOnGotFocusInterceptionOverlayDisplayed:
      return {
        ...state,
        isOnGotFocusInterceptionOverlayDisplayed: action.isOnGotFocusInterceptionOverlayDisplayed
      }

    case ActionType.SET_justRunProgramArgs:

      if (action.justRunProgramArgs.length > justRunProgramArgsMaxLength) {
        Logger.log(`too large just run program args, length: ${action.justRunProgramArgs.length}`)
        return state
      }

      return {
        ...state,
        justRunProgramArgs: action.justRunProgramArgs
      }

    case ActionType.SET_isJustRunProgramArgsInputDisplayed:
      return {
        ...state,
        isJustRunProgramArgsInputDisplayed: action.isJustRunProgramArgsInputDisplayed
      }
    case ActionType.SET_lastKnownEditorContent:
      return {
        ...state,
        lastKnownEditorContent: action.lastKnownEditorContent
      }

    case ActionType.SET_isAddCustomTestAssetFilesModalDisplayed:
      return {
        ...state,
        isAddCustomTestAssetFilesModalDisplayed: action.isDisplayed,
        isAddCustomTestAssetFilesModalBusy: action.isBusy
      }
    case ActionType.SET_addCustomTestAssetFilesModalFilterText:
      return {
        ...state,
        addCustomTestAssetFilesModalFilterText: action.addCustomTestAssetFilesModalFilterText
      }
    case ActionType.SET_addCustomTestAssetFilesModalUploadPercentage:
      return {
        ...state,
        addCustomTestAssetFilesModalUploadPercentage: action.addCustomTestAssetFilesModalUploadPercentage
      }

    case ActionType.GET_allUserAssetPreviews_PENDING:
    case ActionType.GET_allUserAssetPreviews_FULFILLED:
    case ActionType.GET_allUserAssetPreviews_REJECTED:
      return getCustomTestAssetPreviewsReducer(state, action)

    case ActionType.ATTACH_customTestAssetPreview_PENDING:
    case ActionType.ATTACH_customTestAssetPreview_FULFILLED:
    case ActionType.ATTACH_customTestAssetPreview_REJECTED:
      return attachCustomTestAssetPreviewReducer(state, action)

    case ActionType.DETACH_customTestAssetPreview_PENDING:
    case ActionType.DETACH_customTestAssetPreview_FULFILLED:
    case ActionType.DETACH_customTestAssetPreview_REJECTED:
      return detachCustomTestAssetPreviewReducer(state, action)

    case ActionType.RESET:
    case ActionTypeGlobal.RESET_Global:
      return {
        ...initial,
      }

    default:
      notExhaustive(action)
      return state
  }
}

