import {FilePreviewFromBackend} from '../../../../../types/exerciseEditor'
import {ActionBase, ActionType} from '../../editCustomProjectSiteActionTypes'
import {ReduxPromiseAction, ReduxPromiseFulfilledAction, ReduxPromiseRejectedAction} from '../../../index'
import {State, initial} from '../../editCustomProjectSiteReducer'
import {notExhaustive} from '../../../_notExhausiveHelper'


interface Meta {
  readonly customTestId: number
}

export interface ATTACH_customProjectCustomTestAssetPreviewEAction
  extends ActionBase, ReduxPromiseAction<FilePreviewFromBackend, ATTACH_customProjectCustomTestAssetPreviewE_PENDINGAction, ATTACH_customProjectCustomTestAssetPreviewE_FULFILLEDAction, ATTACH_customProjectCustomTestAssetPreviewE_REJECTEDAction, Meta> {
  readonly type: ActionType.ATTACH_customProjectCustomTestAssetPreview
  readonly payload: Promise<FilePreviewFromBackend>
  readonly meta: Meta
}

//automatically dispatched by redux promise middleware
export interface ATTACH_customProjectCustomTestAssetPreviewE_PENDINGAction
  extends ActionBase {
  readonly type: ActionType.ATTACH_customProjectCustomTestAssetPreview_PENDING
  readonly meta: Meta
}

//automatically dispatched by redux promise middleware
export interface ATTACH_customProjectCustomTestAssetPreviewE_FULFILLEDAction
  extends ActionBase, ReduxPromiseFulfilledAction<FilePreviewFromBackend, Meta> {
  readonly type: ActionType.ATTACH_customProjectCustomTestAssetPreview_FULFILLED
  readonly payload: FilePreviewFromBackend
  readonly meta: Meta
}

//automatically dispatched by redux promise middleware
export interface ATTACH_customProjectCustomTestAssetPreviewE_REJECTEDAction
  extends ActionBase, ReduxPromiseRejectedAction<void, Meta> {
  readonly type: ActionType.ATTACH_customProjectCustomTestAssetPreview_REJECTED
  readonly payload: any
  readonly meta: Meta
}


export type AllActions =
  ATTACH_customProjectCustomTestAssetPreviewE_PENDINGAction
  | ATTACH_customProjectCustomTestAssetPreviewE_FULFILLEDAction
  | ATTACH_customProjectCustomTestAssetPreviewE_REJECTEDAction


export function reducer(state: State = initial(), action: AllActions): State {

  switch (action.type) {

    case ActionType.ATTACH_customProjectCustomTestAssetPreview_PENDING:
      return {
        ...state,
        isAddCustomProjectCustomTestAssetFilesModalBusy: true
      }
    case ActionType.ATTACH_customProjectCustomTestAssetPreview_FULFILLED:
      return {
        ...state,
        isAddCustomProjectCustomTestAssetFilesModalBusy: false,
        customProject: {
          ...state.customProject,
          tests: state.customProject.tests.map(p => p.id !== action.meta.customTestId
                                                    ? p
                                                    : {
              ...p,
              files: p.files.concat(action.payload)
            })
        }
      }
    case ActionType.ATTACH_customProjectCustomTestAssetPreview_REJECTED:
      return {
        ...state,
        isAddCustomProjectCustomTestAssetFilesModalBusy: false
      }

    default:
      notExhaustive(action)
      return state
  }
}

