/**
 * Created by janis dähne.
 */
import {ActionBase, ActionType} from "../editCustomProjectSiteActionTypes";
import {State, initial} from "../editCustomProjectSiteReducer";
import {ReduxPromiseAction, ReduxPromiseFulfilledAction, ReduxPromiseRejectedAction} from "../../index";
import {notExhaustive} from "../../_notExhausiveHelper";
import {CustomProjectFullBase} from "../../../../types/customProjects";
import {convertCustomProjectFromBackendToFrontend} from "../../../../helpers/convertersAndTransformers";


export interface GET_customProjectAction
  extends ActionBase, ReduxPromiseAction<CustomProjectFullBase, GET_customProject_PENDINGAction, GET_customProject_FULFILLEDAction, GET_customProject_REJECTEDAction> {
  readonly type: ActionType.GET_customProject
  readonly payload: Promise<CustomProjectFullBase>
}

//automatically dispatched by redux promise middleware
export interface GET_customProject_PENDINGAction
  extends ActionBase {
  readonly type: ActionType.GET_customProject_PENDING
}

//automatically dispatched by redux promise middleware
export interface GET_customProject_FULFILLEDAction
  extends ActionBase, ReduxPromiseFulfilledAction<CustomProjectFullBase> {
  readonly type: ActionType.GET_customProject_FULFILLED
  readonly payload: CustomProjectFullBase
}

//automatically dispatched by redux promise middleware
export interface GET_customProject_REJECTEDAction
  extends ActionBase, ReduxPromiseRejectedAction<void> {
  readonly type: ActionType.GET_customProject_REJECTED
  readonly payload: any
}


export type AllActions =
  GET_customProject_PENDINGAction
  | GET_customProject_FULFILLEDAction
  | GET_customProject_REJECTEDAction


export function reducer(state: State = initial(), action: AllActions): State {

  switch (action.type) {

    case ActionType.GET_customProject_PENDING:
      return {
        ...state,
        isLoading: true
      }
    case ActionType.GET_customProject_FULFILLED:
      return {
        ...state,
        isLoading: false,
        customProject: convertCustomProjectFromBackendToFrontend(action.payload),
        initialDescriptionFromBackend: {
          ...action.payload.description
        },
        initialSolutionFromBackend: {
          ...action.payload.solution
        }
      }
    case ActionType.GET_customProject_REJECTED:
      return {
        ...state,
        isLoading: false
      }

    default:
      notExhaustive(action)
      return state
  }
}

