/**
 * Created by janis dähne.
 */
import {ActionBase, ActionType} from "../editCustomProjectSiteActionTypes";
import {State, initial} from "../editCustomProjectSiteReducer";
import {ReduxPromiseAction, ReduxPromiseFulfilledAction, ReduxPromiseRejectedAction} from "../../index";
import {notExhaustive} from "../../_notExhausiveHelper";
import {StoredTestResultFromBackend} from "../../../../types/testResults";
import {mergeTestResultsIntoCustomProject} from "../../../../helpers/convertersAndTransformers";

export interface GET_customProjectSingleTestResultAction
  extends ActionBase, ReduxPromiseAction<StoredTestResultFromBackend, GET_customProjectSingleTestResult_PENDINGAction, GET_customProjectSingleTestResult_FULFILLEDAction, GET_customProjectSingleTestResult_REJECTEDAction> {
  readonly type: ActionType.GET_customProjectSingleTestResult
  readonly payload: Promise<StoredTestResultFromBackend>
}

//automatically dispatched by redux promise middleware
export interface GET_customProjectSingleTestResult_PENDINGAction
  extends ActionBase {
  readonly type: ActionType.GET_customProjectSingleTestResult_PENDING
}

//automatically dispatched by redux promise middleware
export interface GET_customProjectSingleTestResult_FULFILLEDAction
  extends ActionBase, ReduxPromiseFulfilledAction<StoredTestResultFromBackend> {
  readonly type: ActionType.GET_customProjectSingleTestResult_FULFILLED
  readonly payload: StoredTestResultFromBackend
}

//automatically dispatched by redux promise middleware
export interface GET_customProjectSingleTestResult_REJECTEDAction
  extends ActionBase, ReduxPromiseRejectedAction<void> {
  readonly type: ActionType.GET_customProjectSingleTestResult_REJECTED
  readonly payload: any
}


export type AllActions =
  GET_customProjectSingleTestResult_PENDINGAction
  | GET_customProjectSingleTestResult_FULFILLEDAction
  | GET_customProjectSingleTestResult_REJECTEDAction


export function reducer(state: State = initial(), action: AllActions): State {

  switch (action.type) {

    case ActionType.GET_customProjectSingleTestResult_PENDING:
      return {
        ...state,
        isLoading: true
      }
    case ActionType.GET_customProjectSingleTestResult_FULFILLED:

      const testResultsWithNewTestResult = state.customProject.tests.map(p =>
        p.id === action.payload.testId
        ? action.payload
        : p.result
      )

      return {
        ...state,
        isLoading: false,
        customProject: mergeTestResultsIntoCustomProject(state.customProject, testResultsWithNewTestResult)
      }
    case ActionType.GET_customProjectSingleTestResult_REJECTED:
      return {
        ...state,
        isLoading: false
      }

    default:
      notExhaustive(action)
      return state
  }
}

