/**
 * Created by janis dähne.
 */
import {Action} from "redux";
import {ActionType} from "../editCustomProjectSiteActionTypes";
import {TestAnswerFromBackend} from "../../../../types/testsAndTestProtocol";
import {State, initial} from "../editCustomProjectSiteReducer";
import {ReduxPromiseAction, ReduxPromiseFulfilledAction, ReduxPromiseRejectedAction} from "../../index";
import {notExhaustive} from "../../_notExhausiveHelper";


export interface ActionBase extends Action {
  readonly type: ActionType
}

export interface JUST_RUN_programAction
  extends ActionBase, ReduxPromiseAction<TestAnswerFromBackend, JUST_RUN_program_PENDINGAction, JUST_RUN_program_FULFILLEDAction, JUST_RUN_program_REJECTEDAction> {
  readonly type: ActionType.JUST_RUN_program
  readonly payload: Promise<TestAnswerFromBackend>
}

//automatically dispatched by redux promise middleware
export interface JUST_RUN_program_PENDINGAction
  extends ActionBase {
  readonly type: ActionType.JUST_RUN_program_PENDING
}

//automatically dispatched by redux promise middleware
export interface JUST_RUN_program_FULFILLEDAction
  extends ActionBase, ReduxPromiseFulfilledAction<TestAnswerFromBackend> {
  readonly type: ActionType.JUST_RUN_program_FULFILLED
  readonly payload: TestAnswerFromBackend
}

//automatically dispatched by redux promise middleware
export interface JUST_RUN_program_REJECTEDAction
  extends ActionBase, ReduxPromiseRejectedAction<void> {
  readonly type: ActionType.JUST_RUN_program_REJECTED
  readonly payload: any
}


export type AllActions =
  JUST_RUN_program_PENDINGAction
  | JUST_RUN_program_FULFILLEDAction
  | JUST_RUN_program_REJECTEDAction


export function reducer(state: State = initial(), action: AllActions): State {

  switch (action.type) {

    case ActionType.JUST_RUN_program_PENDING:
      return {
        ...state,
        isLoading: true
      }
    case ActionType.JUST_RUN_program_FULFILLED:
      return {
        ...state,
        isLoading: false,
        lastJustRunProgramResult: action.payload
      }
    case ActionType.JUST_RUN_program_REJECTED:
      return {
        ...state,
        isLoading: false,
        lastJustRunProgramResult: null
      }

    default:
      notExhaustive(action)
      return state
  }
}

