/**
 * Created by janis dähne.
 */

import {ActionBase, ActionType} from "../editCustomProjectSiteActionTypes";
import {initial, State} from "../editCustomProjectSiteReducer";
import {CustomProjectSolutionFullBase, CustomSolutionTupleFromBackend} from "../../../../types/customProjects";
import {ReduxPromiseAction, ReduxPromiseFulfilledAction, ReduxPromiseRejectedAction} from "../../index";
import {notExhaustive} from "../../_notExhausiveHelper";
import {KnownLangs} from '../../../../../i18n/i18nRoot'
import {
  AceEditorsStorageHelper,
  AceStoredTabLike
} from '../../../../components/codeEditors/aceEditors/aceEditorsStorageHelper'
import {customProjectEditorPanelViewEditorId, doEditorEditorPanelViewEditorId} from '../../../../constants'
import {updateFileReferencesAceStorage} from '../../../../helpers/editorHelper'


interface Meta {
  readonly langId: KnownLangs
}

export interface UPDATE_customProjectSolutionAction
  extends ActionBase, ReduxPromiseAction<CustomSolutionTupleFromBackend, UPDATE_customProjectSolution_PENDINGAction, UPDATE_customProjectSolution_FULFILLEDAction, UPDATE_customProjectSolution_REJECTEDAction, Meta> {
  readonly type: ActionType.UPDATE_customProjectSolution
  readonly payload: Promise<CustomSolutionTupleFromBackend>
  readonly meta: Meta
}

//automatically dispatched by redux promise middleware
export interface UPDATE_customProjectSolution_PENDINGAction
  extends ActionBase {
  readonly type: ActionType.UPDATE_customProjectSolution_PENDING
  readonly meta: Meta
}

//automatically dispatched by redux promise middleware
export interface UPDATE_customProjectSolution_FULFILLEDAction
  extends ActionBase, ReduxPromiseFulfilledAction<CustomSolutionTupleFromBackend, Meta> {
  readonly type: ActionType.UPDATE_customProjectSolution_FULFILLED
  readonly payload: CustomSolutionTupleFromBackend
}

//automatically dispatched by redux promise middleware
export interface UPDATE_customProjectSolution_REJECTEDAction
  extends ActionBase, ReduxPromiseRejectedAction<void, Meta> {
  readonly type: ActionType.UPDATE_customProjectSolution_REJECTED
  readonly payload: any
}


export type AllActions =
  UPDATE_customProjectSolution_PENDINGAction
  | UPDATE_customProjectSolution_FULFILLEDAction
  | UPDATE_customProjectSolution_REJECTEDAction


export function reducer(state: State = initial(), action: AllActions): State {

  switch (action.type) {

    case ActionType.UPDATE_customProjectSolution_PENDING:
      return {
        ...state,
        isLoading: true
      }
    case ActionType.UPDATE_customProjectSolution_FULFILLED:

      /** this is the same as {@link UPDATE_customProjectSolution_FULFILLEDAction} */

      //we need to update the ace storage because we might have added new files where we need to id from the backend
      const storedTabs = AceEditorsStorageHelper.getEditor(customProjectEditorPanelViewEditorId)
      updateFileReferencesAceStorage(storedTabs, action.payload.solution.solutionFiles, action.payload.fileIdAssociationMap, action.meta.langId)

      const tuple = action.payload.fileIdAssociationMap.find(
        p => p.frontendFileId === state.solutionEditorState.selectedTabIdDoEditor)

      let newSelectedFileId = tuple
                                ? tuple.backendFileId
                                : -1

      return {
        ...state,
        isLoading: false,
        solutionEditorState: {
          ...state.solutionEditorState,
          selectedTabIdDoEditor: newSelectedFileId,
        },
        initialSolutionFromBackend: {
          ...action.payload.solution //this makes only a shallow copy but we use readonly so should be ok
        },
        customProject: {
          ...state.customProject,
          solution: action.payload.solution
        },
      }
    case ActionType.UPDATE_customProjectSolution_REJECTED:
      return {
        ...state,
        isLoading: false
      }

    default:
      notExhaustive(action)
      return state
  }
}

