/**
 * Created by janis dähne.
 */

import {ActionBase, ActionType} from "./changeCustomProjectTestDialogActionTypes";
import {notExhaustive} from "../../../_notExhausiveHelper";
import {CustomProjectTestFullBase} from "../../../../../types/customProjects";
import {isValidDialog} from "../../../../../validation/dialogValidations";
import {
  validationRules,
  initial as initialTest,
  AllActions as SetCustomProjectTestActions,
  reducer as customProjectTestReducer
} from "./customProjectTestReducer";

import {
  AllActions as CreateCustomProjectTestActions,
  reducer as createCustomProjectTestReducer
} from './createCustomProjectTestReducer'
import {
  AllActions as UpdateCustomProjectTestActions,
  reducer as updateCustomProjectTestReducer
} from './updateCustomProjectTestReducer'
import {ActionTypeGlobal, RESET_GlobalAction} from '../../../globalActions'
import {ActionType as EditCustomProjectSiteActionTypes} from '../../editCustomProjectSiteActionTypes'

export type State = {
  readonly isBusy: boolean
  readonly isHelpDisplayed: boolean
  readonly isCreating: boolean
  readonly isValid: boolean
  readonly selectedTabIndex: number
  readonly test: CustomProjectTestFullBase
}

export const initial: () => State = () => {
  return {

    isBusy: false,
    isHelpDisplayed: false,
    isCreating: false,
    selectedTabIndex: 0,

    test: initialTest(),
    isValid: isValidDialog(initialTest(), validationRules),
  }
}

export interface SET_selectedTabIndexAction extends ActionBase {
  readonly type: ActionType.SET_selectedTabIndex
  readonly index: number
}

export interface SET_isHelpDisplayedAction extends ActionBase {
  readonly type: ActionType.SET_isHelpDisplayed
  readonly isDisplayed: boolean
}

export interface SET_isCreatingAction extends ActionBase {
  readonly type: ActionType.SET_isCreating
  readonly isCreating: boolean
}


export interface ResetAction extends ActionBase {
  readonly type: ActionType.RESET
}

export type AllActions =
  ResetAction
  | SET_isHelpDisplayedAction
  | SET_isCreatingAction
  | SET_selectedTabIndexAction
  | SetCustomProjectTestActions

  | CreateCustomProjectTestActions
  | UpdateCustomProjectTestActions

  | RESET_GlobalAction

export function reducer(state: State = initial(), action: AllActions): State {

  switch (action.type) {

    case ActionType.CREATE_customProjectTest_PENDING:
    case ActionType.CREATE_customProjectTest_FULFILLED:
    case ActionType.CREATE_customProjectTest_REJECTED:
      return createCustomProjectTestReducer(state, action)

    case ActionType.UPDATE_customProjectReducer_PENDING:
    case ActionType.UPDATE_customProjectReducer_FULFILLED:
    case ActionType.UPDATE_customProjectReducer_REJECTED:
      return updateCustomProjectTestReducer(state, action)

    case ActionType.SET_testId:
    case ActionType.SET_content:
    case ActionType.SET_displayName:
    case ActionType.SET_displayIndex:
    case ActionType.SET_weight:
    case ActionType.SET_testTypeId:
    case ActionType.SET_editorTest_memoryLimitInKb:
    case ActionType.SET_editorTest_maxDiskSpaceInKb:
    case ActionType.SET_editorTest_timeoutInMs:
    case ActionType.SET_editorTest_compileTimeoutInMs:
    case ActionType.SET_editorTest_compilerOptions:
    case ActionType.SET_editorTest_files:
    case EditCustomProjectSiteActionTypes.ATTACH_customProjectCustomTestAssetPreview_FULFILLED:
    case EditCustomProjectSiteActionTypes.DETACH_customProjectCustomTestAssetPreview_FULFILLED:
      const temp = customProjectTestReducer(state.test, action)
      return {
        ...state,
        isValid: isValidDialog(temp, validationRules),
        test: customProjectTestReducer(state.test, action)
      }

    case ActionType.SET_isCreating:
      return {
        ...state,
        isCreating: action.isCreating
      }

    case ActionType.SET_isHelpDisplayed:
      return {
        ...state,
        isHelpDisplayed: action.isDisplayed
      }

    case ActionType.SET_selectedTabIndex:
      return {
        ...state,
        selectedTabIndex: action.index
      }

    case ActionType.RESET:
    case ActionTypeGlobal.RESET_Global:
      return initial()

    default:
      notExhaustive(action)
      return state
  }
}

