/**
 * Created by janis dähne.
 */
import {notExhaustive} from "../_notExhausiveHelper";
import {ActionBase, ActionType} from "./editorSettingsDialogActionTypes";
import {CodeEditorSettingsFullBase} from "../../../types/userData";
import {
  AllActions as EditorSettingsActions, initial as initialEditorSettings, reducer as editorSettingsReducer,
  validationRules
} from './editorSettingsReducer'
import {isValidDialog} from "../../../validation/dialogValidations";
import {ActionTypeGlobal, RESET_GlobalAction} from '../globalActions'

export type  SetIsDialogDisplayedActionCallback = (isDisplayed: boolean) => { readonly type: string, readonly isDisplayed: boolean }

export type State = {

  readonly editorSettings: CodeEditorSettingsFullBase

  readonly setIsDialogDisplayedActionCallback: SetIsDialogDisplayedActionCallback | null

  readonly isBusy: boolean
  readonly isValid: boolean
}


export const initial: State = {

  editorSettings: initialEditorSettings,
  setIsDialogDisplayedActionCallback: null,
  isBusy: false,
  isValid: isValidDialog(initialEditorSettings, validationRules),
}


export interface SET_editorSettingsAction extends ActionBase {
  readonly type: ActionType.SET_editorSettings
  readonly editorSettings: CodeEditorSettingsFullBase
}


export interface SET_setIsDialogDisplayedActionCallbackAction extends ActionBase {
  readonly type: ActionType.SET_setIsDialogDisplayedActionCallback
  readonly setIsDialogDisplayedActionCallback: SetIsDialogDisplayedActionCallback
}

export interface ResetAction extends ActionBase {
  readonly type: ActionType.RESET
}

export type AllActions =
  ResetAction
  | SET_editorSettingsAction

  | EditorSettingsActions

  | SET_setIsDialogDisplayedActionCallbackAction

  | RESET_GlobalAction

export function reducer(state: State = initial, action: AllActions): State {

  switch (action.type) {

    case ActionType.SET_theme:
    case ActionType.SET_fontSize:
    case ActionType.SET_highlightCurrentLine:
    case ActionType.SET_showInvisibles:
    case ActionType.SET_showLineNumbers:
    case ActionType.SET_tabSize:
    case ActionType.SET_useWrapping:
    case ActionType.SET_showLineIndentions: {

      const temp = editorSettingsReducer(state.editorSettings, action)

      return {
        ...state,
        editorSettings: temp,
        isValid: isValidDialog(temp, validationRules)
      }
    }

    case ActionType.SET_editorSettings:
      return {
        ...state,
        editorSettings: action.editorSettings,
        isValid: isValidDialog(action.editorSettings, validationRules)
      }

    case ActionType.SET_setIsDialogDisplayedActionCallback:
      return {
        ...state,
        setIsDialogDisplayedActionCallback: action.setIsDialogDisplayedActionCallback,
      }


    case ActionType.RESET:
    case ActionTypeGlobal.RESET_Global:
      return {
        ...initial
      }

    default:
      notExhaustive(action)
      return state
  }
}

