import {ActionBase, ActionType} from '../../editorExerciseSiteActionTypes'
import {State, initial} from '../../exerciseEditorSiteReducer'
import {ReduxPromiseAction, ReduxPromiseFulfilledAction, ReduxPromiseRejectedAction} from '../../../index'
import {notExhaustive} from '../../../_notExhausiveHelper'
import {ExerciseTestForBackend, FilePreviewFromBackend} from '../../../../../types/exerciseEditor'


interface Meta {
  readonly testId: number
}

export interface ATTACH_testAssetAction
  extends ActionBase, ReduxPromiseAction<FilePreviewFromBackend, ATTACH_testAsset_PENDINGAction, ATTACH_testAsset_FULFILLEDAction, ATTACH_testAsset_REJECTEDAction, Meta> {
  readonly type: ActionType.ATTACH_testAsset
  readonly payload: Promise<FilePreviewFromBackend>
  readonly meta: Meta
}

//automatically dispatched by redux promise middleware
export interface ATTACH_testAsset_PENDINGAction
  extends ActionBase {
  readonly type: ActionType.ATTACH_testAsset_PENDING
  readonly meta: Meta
}

//automatically dispatched by redux promise middleware
export interface ATTACH_testAsset_FULFILLEDAction
  extends ActionBase, ReduxPromiseFulfilledAction<FilePreviewFromBackend, Meta> {
  readonly type: ActionType.ATTACH_testAsset_FULFILLED
  readonly payload: FilePreviewFromBackend
  readonly meta: Meta
}

//automatically dispatched by redux promise middleware
export interface ATTACH_testAsset_REJECTEDAction
  extends ActionBase, ReduxPromiseRejectedAction<void, Meta> {
  readonly type: ActionType.ATTACH_testAsset_REJECTED
  readonly payload: any
  readonly meta: Meta
}


export type AllActions =
  ATTACH_testAsset_PENDINGAction
  | ATTACH_testAsset_FULFILLEDAction
  | ATTACH_testAsset_REJECTEDAction


export function reducer(state: State = initial(), action: AllActions): State {

  switch (action.type) {

    case ActionType.ATTACH_testAsset_PENDING:
      return {
        ...state,
        isAddTestAssetFilesModalBusy: true
      }
    case ActionType.ATTACH_testAsset_FULFILLED:
      return {
        ...state,
        isAddTestAssetFilesModalBusy: false,
        editorExercise: {
          ...state.editorExercise,
          tests: state.editorExercise.tests.map(p =>
                                                  p.id !== action.meta.testId
                                                  ? p
                                                  : {
                                                      ...p,
                                                      files: p.files.concat(action.payload)
                                                    })
        }
      }
    case ActionType.ATTACH_testAsset_REJECTED:
      return {
        ...state,
        isAddTestAssetFilesModalBusy: false
      }

    default:
      notExhaustive(action)
      return state
  }
}

