import {ActionBase, ActionType} from '../../editorExerciseSiteActionTypes'
import {State, initial} from '../../exerciseEditorSiteReducer'
import {ReduxPromiseAction, ReduxPromiseFulfilledAction, ReduxPromiseRejectedAction} from '../../../index'
import {notExhaustive} from '../../../_notExhausiveHelper'
import {createAndRefreshAssetUrlsAndObjects} from '../../../../../helpers/assetHelper'


export interface Meta {
  readonly assetPreviewId: number
}

export interface DETACH_markdownAssetPreviewAction
  extends ActionBase, ReduxPromiseAction<void, DETACH_markdownAssetPreview_PENDINGAction, DETACH_markdownAssetPreview_FULFILLEDAction, DETACH_markdownAssetPreview_REJECTEDAction, Meta> {
  readonly type: ActionType.DETACH_markdownAsset
  readonly payload: Promise<void>
  readonly meta: Meta
}

//automatically dispatched by redux promise middleware
export interface DETACH_markdownAssetPreview_PENDINGAction
  extends ActionBase {
  readonly type: ActionType.DETACH_markdownAsset_PENDING
  readonly meta: Meta
}

//automatically dispatched by redux promise middleware
export interface DETACH_markdownAssetPreview_FULFILLEDAction
  extends ActionBase, ReduxPromiseFulfilledAction<void, Meta> {
  readonly type: ActionType.DETACH_markdownAsset_FULFILLED
  readonly payload: void
  readonly meta: Meta
}

//automatically dispatched by redux promise middleware
export interface DETACH_markdownAssetPreview_REJECTEDAction
  extends ActionBase, ReduxPromiseRejectedAction<void, Meta> {
  readonly type: ActionType.DETACH_markdownAsset_REJECTED
  readonly payload: any
  readonly meta: Meta
}


export type AllActions =
  DETACH_markdownAssetPreview_PENDINGAction
  | DETACH_markdownAssetPreview_FULFILLEDAction
  | DETACH_markdownAssetPreview_REJECTEDAction


export function reducer(state: State = initial(), action: AllActions): State {

  switch (action.type) {

    case ActionType.DETACH_markdownAsset_PENDING:
      return {
        ...state,
        isAddAssetFilesModalBusy: true
      }
    case ActionType.DETACH_markdownAsset_FULFILLED:
      //remove the asset we detached

      const newAssetList = state.editorExercise.exerciseDescription.assets.filter(p => p.id !== action.meta.assetPreviewId)

      createAndRefreshAssetUrlsAndObjects(newAssetList)

      return {
        ...state,
        isAddAssetFilesModalBusy: false,
        editorExercise: {
          ...state.editorExercise,
          exerciseDescription: {
            ...state.editorExercise.exerciseDescription,
            assets: newAssetList
          },
        }

      }
    case ActionType.DETACH_markdownAsset_REJECTED:
      return {
        ...state,
        isAddAssetFilesModalBusy: false
      }

    default:
      notExhaustive(action)
      return state
  }
}


