import {ActionBase, ActionType} from '../../editorExerciseSiteActionTypes'
import {State, initial} from '../../exerciseEditorSiteReducer'
import {ReduxPromiseAction, ReduxPromiseFulfilledAction, ReduxPromiseRejectedAction} from '../../../index'
import {notExhaustive} from '../../../_notExhausiveHelper'
import {FilePreviewFromBackend} from '../../../../../types/exerciseEditor'


export interface GET_markdownAssetPreviewsAction
  extends ActionBase, ReduxPromiseAction<ReadonlyArray<FilePreviewFromBackend>, GET_markdownAssetPreviews_PENDINGAction, GET_markdownAssetPreviews_FULFILLEDAction, GET_markdownAssetPreviews_REJECTEDAction> {
  readonly type: ActionType.GET_markdownAssetPreviews
  readonly payload: Promise<ReadonlyArray<FilePreviewFromBackend>>
}

//automatically dispatched by redux promise middleware
export interface GET_markdownAssetPreviews_PENDINGAction
  extends ActionBase {
  readonly type: ActionType.GET_markdownAssetPreviews_PENDING
}

//automatically dispatched by redux promise middleware
export interface GET_markdownAssetPreviews_FULFILLEDAction
  extends ActionBase, ReduxPromiseFulfilledAction<ReadonlyArray<FilePreviewFromBackend>> {
  readonly type: ActionType.GET_markdownAssetPreviews_FULFILLED
  readonly payload: ReadonlyArray<FilePreviewFromBackend>
}

//automatically dispatched by redux promise middleware
export interface GET_markdownAssetPreviews_REJECTEDAction
  extends ActionBase, ReduxPromiseRejectedAction<void> {
  readonly type: ActionType.GET_markdownAssetPreviews_REJECTED
  readonly payload: any
}


export type AllActions =
  GET_markdownAssetPreviews_PENDINGAction
  | GET_markdownAssetPreviews_FULFILLEDAction
  | GET_markdownAssetPreviews_REJECTEDAction


export function reducer(state: State = initial(), action: AllActions): State {

  switch (action.type) {

    case ActionType.GET_markdownAssetPreviews_PENDING:
      return {
        ...state,
        isLoading: true,
        isAddAssetFilesModalBusy: true,
      }
    case ActionType.GET_markdownAssetPreviews_FULFILLED:
      return {
        ...state,
        isLoading: false,
        isAddAssetFilesModalBusy: false,
        allMarkdownAssetFilePreviews: action.payload
      }
    case ActionType.GET_markdownAssetPreviews_REJECTED:
      return {
        ...state,
        isLoading: false,
        isAddAssetFilesModalBusy: false,
        allMarkdownAssetFilePreviews: []
      }

    default:
      notExhaustive(action)
      return state
  }
}

