/**
 * Created by janis dähne.
 */
import {notExhaustive} from "../_notExhausiveHelper";
import {ReduxPromiseAction, ReduxPromiseFulfilledAction, ReduxPromiseRejectedAction} from "../index";
import {ActionBase, ActionType} from "./editorExerciseSiteActionTypes";
import {initial, State} from "./exerciseEditorSiteReducer";
import {EditorExerciseFromBackend} from "../../../types/exerciseEditor";
import {convertEditorExerciseFromBackendToFrontend} from "../../../helpers/convertersAndTransformers";
import {AceEditorsStorageHelper} from '../../../components/codeEditors/aceEditors/aceEditorsStorageHelper'
import {editExerciseExerciseDescriptionEditorPanelViewEditorId} from '../../../constants'
import {initial as initialEditorExercise} from './editorExerciseReducer'


export interface GET_editorExerciseAction
  extends ActionBase, ReduxPromiseAction<EditorExerciseFromBackend, GET_editorExercise_PENDINGAction, GET_editorExercise_FULFILLEDAction, GET_editorExercise_REJECTEDAction> {
  readonly type: ActionType.GET_editorExercise
  readonly payload: Promise<EditorExerciseFromBackend>
}

//automatically dispatched by redux promise middleware
export interface GET_editorExercise_PENDINGAction
  extends ActionBase {
  readonly type: ActionType.GET_editorExercise_PENDING
}

//automatically dispatched by redux promise middleware
export interface GET_editorExercise_FULFILLEDAction
  extends ActionBase, ReduxPromiseFulfilledAction<EditorExerciseFromBackend> {
  readonly type: ActionType.GET_editorExercise_FULFILLED
  readonly payload: EditorExerciseFromBackend
}

//automatically dispatched by redux promise middleware
export interface GET_editorExercise_REJECTEDAction
  extends ActionBase, ReduxPromiseRejectedAction<void> {
  readonly type: ActionType.GET_editorExercise_REJECTED
  readonly payload: any
}


export type AllActions =
  GET_editorExercise_PENDINGAction
  | GET_editorExercise_FULFILLEDAction
  | GET_editorExercise_REJECTEDAction


export function reducer(state: State = initial(), action: AllActions): State {

  switch (action.type) {

    case ActionType.GET_editorExercise_PENDING:
      return {
        ...state,
        isLoading: true
      }
    case ActionType.GET_editorExercise_FULFILLED:

      //reset the ace editor session to clear the history... we have the default text stored
      AceEditorsStorageHelper.removeEditor(editExerciseExerciseDescriptionEditorPanelViewEditorId)

      //these ares the same defaults as in the descriptionEditorPanelView (we need the same id)
      //here the editor settings are not yet applied but we apply them in the componentDidUpdate method of the editor wrapper
      AceEditorsStorageHelper.addEditor(editExerciseExerciseDescriptionEditorPanelViewEditorId, {
        id: 0,
        content: action.payload.exerciseDescription.content,
        language: 'markdown'
      })

      return {
        ...state,
        isLoading: false,
        editorExercise: convertEditorExerciseFromBackendToFrontend(action.payload),
        oldGroupId: action.payload.exerciseProperties.userGroupId,
        oldIsOnlyVisibleToMe: action.payload.exerciseProperties.isOnlyVisibleToMe,
      }
    case ActionType.GET_editorExercise_REJECTED:
      return {
        ...state,
        isLoading: false,
        editorExercise: initialEditorExercise()
      }

    default:
      notExhaustive(action)
      return state
  }
}

