import {ActionBase, ActionType} from '../editorExerciseSiteActionTypes'
import {State, initial} from '../exerciseEditorSiteReducer'
import {ReduxPromiseAction, ReduxPromiseFulfilledAction, ReduxPromiseRejectedAction} from '../../index'
import {notExhaustive} from '../../_notExhausiveHelper'
import {ExerciseTestForBackend, FilePreviewFromBackend} from '../../../../types/exerciseEditor'


interface Meta {
  readonly testId: number
}

export interface UPLOADED_TestAssetFileAction
  extends ActionBase, ReduxPromiseAction<FilePreviewFromBackend, UPLOADED_TestAssetFile_PENDINGAction, UPLOADED_TestAssetFile_FULFILLEDAction, UPLOADED_TestAssetFile_REJECTEDAction, Meta> {
  readonly type: ActionType.UPLOADED_TestAssetFile
  readonly payload: Promise<FilePreviewFromBackend>
  readonly meta: Meta
}

//automatically dispatched by redux promise middleware
export interface UPLOADED_TestAssetFile_PENDINGAction
  extends ActionBase {
  readonly type: ActionType.UPLOADED_TestAssetFile_PENDING
  readonly meta: Meta
}

//automatically dispatched by redux promise middleware
export interface UPLOADED_TestAssetFile_FULFILLEDAction
  extends ActionBase, ReduxPromiseFulfilledAction<FilePreviewFromBackend, Meta> {
  readonly type: ActionType.UPLOADED_TestAssetFile_FULFILLED
  readonly payload: FilePreviewFromBackend
  readonly meta: Meta
}

//automatically dispatched by redux promise middleware
export interface UPLOADED_TestAssetFile_REJECTEDAction
  extends ActionBase, ReduxPromiseRejectedAction<void, Meta> {
  readonly type: ActionType.UPLOADED_TestAssetFile_REJECTED
  readonly payload: any
  readonly meta: Meta
}


export type AllActions =
  UPLOADED_TestAssetFile_PENDINGAction
  | UPLOADED_TestAssetFile_FULFILLEDAction
  | UPLOADED_TestAssetFile_REJECTEDAction


export function reducer(state: State = initial(), action: AllActions): State {

  switch (action.type) {

    case ActionType.UPLOADED_TestAssetFile_PENDING:
      return {
        ...state,
        isAddTestAssetFilesModalBusy: true
      }
    case ActionType.UPLOADED_TestAssetFile_FULFILLED:
      return {
        ...state,
        isAddTestAssetFilesModalBusy: false,
        editorExercise: {
          ...state.editorExercise,
          tests: state.editorExercise.tests.map(p =>
                                                  p.id !== action.meta.testId
                                                  ? p
                                                  : {
                                                      ...p,
                                                      files: p.files.concat(action.payload)
                                                    })
        }
      }
    case ActionType.UPLOADED_TestAssetFile_REJECTED:
      return {
        ...state,
        isAddTestAssetFilesModalBusy: false
      }

    default:
      notExhaustive(action)
      return state
  }
}

