import {notExhaustive} from "../_notExhausiveHelper";
import {ReduxPromiseAction, ReduxPromiseFulfilledAction, ReduxPromiseRejectedAction} from "../index";
import {ActionBase, ActionType} from './globalSystemSettingsActionTypes'
import {initial, State} from './globalSystemSettingsReducer'



export interface GET_serverMessageAction
  extends ActionBase, ReduxPromiseAction<string, GET_serverMessage_PENDINGAction, GET_serverMessage_FULFILLEDAction, GET_serverMessage_REJECTEDAction> {
  readonly type: ActionType.GET_serverMessage
  readonly payload: Promise<string>
}

//automatically dispatched by redux promise middleware
export interface GET_serverMessage_PENDINGAction
  extends ActionBase {
  readonly type: ActionType.GET_serverMessage_PENDING
}

//automatically dispatched by redux promise middleware
export interface GET_serverMessage_FULFILLEDAction
  extends ActionBase, ReduxPromiseFulfilledAction<string> {
  readonly type: ActionType.GET_serverMessage_FULFILLED
  readonly payload: string
}

//automatically dispatched by redux promise middleware
export interface GET_serverMessage_REJECTEDAction
  extends ActionBase, ReduxPromiseRejectedAction<void> {
  readonly type: ActionType.GET_serverMessage_REJECTED
  readonly payload: any
}


export type AllActions =
  GET_serverMessage_PENDINGAction
  | GET_serverMessage_FULFILLEDAction
  | GET_serverMessage_REJECTEDAction


export function reducer(state: State = initial, action: AllActions): State {

  //don't show loader here because not needed... all sites depending on global system settings would show a loader

  switch (action.type) {

    case ActionType.GET_serverMessage_PENDING:
      return {
        ...state,
      }
    case ActionType.GET_serverMessage_FULFILLED:
      return {
        ...state,
        serverMessage: action.payload,
        clickedOnServerMessage: state.lastKnownServerMessage === action.payload
                                ? state.clickedOnServerMessage //if we already know the message, then keep the state
                                : false //else reset
      }
    case ActionType.GET_serverMessage_REJECTED:
      return {
        ...state,
      }

    default:
      notExhaustive(action)
      return state
  }
}

