import {ActionBase, ActionType} from './globalSystemSettingsActionTypes'
import {ReduxPromiseAction, ReduxPromiseFulfilledAction, ReduxPromiseRejectedAction} from '../index'
import {SystemSettingsFromBackend} from '../../../types/systemSettings'
import {initial, State} from './globalSystemSettingsReducer'
import {notExhaustive} from '../_notExhausiveHelper'

export interface GET_globalSystemSettingsAction
  extends ActionBase, ReduxPromiseAction<SystemSettingsFromBackend, GET_globalSystemSettings_PENDINGAction, GET_globalSystemSettings_FULFILLEDAction, GET_globalSystemSettings_REJECTEDAction> {
  readonly type: ActionType.GET_systemSettings
  readonly payload: Promise<SystemSettingsFromBackend>
}

//automatically dispatched by redux promise middleware
export interface GET_globalSystemSettings_PENDINGAction
  extends ActionBase {
  readonly type: ActionType.GET_systemSettings_PENDING
}

//automatically dispatched by redux promise middleware
export interface GET_globalSystemSettings_FULFILLEDAction
  extends ActionBase, ReduxPromiseFulfilledAction<SystemSettingsFromBackend> {
  readonly type: ActionType.GET_systemSettings_FULFILLED
  readonly payload: SystemSettingsFromBackend
}

//automatically dispatched by redux promise middleware
export interface GET_globalSystemSettings_REJECTEDAction
  extends ActionBase, ReduxPromiseRejectedAction<void> {
  readonly type: ActionType.GET_systemSettings_REJECTED
  readonly payload: any
}



export type AllActions =
  GET_globalSystemSettings_PENDINGAction
  | GET_globalSystemSettings_FULFILLEDAction
  | GET_globalSystemSettings_REJECTEDAction


export function reducer(state: State = initial, action: AllActions): State {

  switch (action.type) {

    case ActionType.GET_systemSettings_PENDING:
      return {
        ...state,
        isLoading: true
      }
    case ActionType.GET_systemSettings_FULFILLED:
      return {
        ...state,
        isLoading: false,
        systemSettings: action.payload,
        serverMessage: action.payload.currentServerMessage,
        clickedOnServerMessage: state.lastKnownServerMessage === action.payload.currentServerMessage
                                ? state.clickedOnServerMessage //if we already know the message, then keep the state
                                : false //else reset
      }
    case ActionType.GET_systemSettings_REJECTED:
      return {
        ...state,
        isLoading: false,
      }

    default:
      notExhaustive(action)
      return state
  }
}

