/**
 * Created by janis dähne.
 */

import {GroupRoleWithPermissionFullBase} from "../../../types/groupRoles";
import {notExhaustive} from "../_notExhausiveHelper";
import {ActionType, ActionBase} from "./groupRolesSiteActionTypes";
import {AllActions as GetActions, reducer as getReducer} from './getGroupRolesReducer'
import {AllActions as DeleteActions, reducer as deleteReducer} from './deleteGroupRoleReducer'
import {ActionTypeGlobal, RESET_GlobalAction} from '../globalActions'

export type State = {
  readonly groupRoles: ReadonlyArray<GroupRoleWithPermissionFullBase>

  /**
   * true: display the dialog,
   * false: display main view
   */
  readonly isChangeDialogDisplayed: boolean

  /**
   * true: fetching data, display spinner,
   * false: not (finished error/or success)
   */
  readonly isLoading: boolean
}


export const initial: State = {
  groupRoles: [],
  isChangeDialogDisplayed: false,
  isLoading: false
}

export interface SET_groupRolesAction extends ActionBase {
  readonly type: ActionType.SET_groupRoles
  readonly groupRoles: ReadonlyArray<GroupRoleWithPermissionFullBase>
}


export interface SET_isChangeDialogDisplayedAction extends ActionBase {
  readonly type: ActionType.SET_isChangeDialogDisplayed
  readonly isDisplayed: boolean
}


export interface ResetAction extends ActionBase {
  readonly type: ActionType.RESET
}

export type AllActions =
  ResetAction
  | SET_groupRolesAction
  | SET_isChangeDialogDisplayedAction
  | GetActions
  | DeleteActions

  | RESET_GlobalAction


export function reducer(state: State = initial, action: AllActions): State {

  switch (action.type) {

    case ActionType.SET_groupRoles:
      return {
        ...state,
        groupRoles: action.groupRoles
      }
    case ActionType.SET_isChangeDialogDisplayed:
      return {
        ...state,
        isChangeDialogDisplayed: action.isDisplayed
      }

    case ActionType.GET_groupRoles_PENDING:
    case ActionType.GET_groupRoles_FULFILLED:
    case ActionType.GET_groupRoles_REJECTED:
      return getReducer(state, action)


    case ActionType.DELETE_groupRole_PENDING:
    case ActionType.DELETE_groupRole_FULFILLED:
    case ActionType.DELETE_groupRole_REJECTED:
      return deleteReducer(state, action)


    case ActionType.RESET:
    case ActionTypeGlobal.RESET_Global:
      return {
        ...initial
      }

    default:
      notExhaustive(action)
      return state
  }
}

