/**
 * Created by janis dähne.
 */
import {ReduxPromiseAction, ReduxPromiseFulfilledAction, ReduxPromiseRejectedAction} from "../index";
import {ActionBase, ActionType} from "./groupsSiteActionTypes";
import {UserGroupBase, UserGroupWithExerciseCountFromBackend} from "../../../types/group";
import Logger from "../../../helpers/logger";
import {notExhaustive} from "../_notExhausiveHelper";
import {State, initial} from "./groupsSiteReducer";
import {PaginatedData} from '../../../types/pagination'
import {PaginationHelper} from '../../../helpers/paginationHelper'


export interface GET_groupsAction
  extends ActionBase, ReduxPromiseAction<PaginatedData<UserGroupWithExerciseCountFromBackend>, GET_groups_PENDINGAction, GET_groups_FULFILLEDAction, GET_groups_REJECTEDAction> {
  readonly type: ActionType.GET_groups
  readonly payload: Promise<PaginatedData<UserGroupWithExerciseCountFromBackend>>
}

//automatically dispatched by redux promise middleware
export interface GET_groups_PENDINGAction
  extends ActionBase {
  readonly type: ActionType.GET_groups_PENDING
}

//automatically dispatched by redux promise middleware
export interface GET_groups_FULFILLEDAction
  extends ActionBase, ReduxPromiseFulfilledAction<PaginatedData<UserGroupWithExerciseCountFromBackend>> {
  readonly type: ActionType.GET_groups_FULFILLED
  readonly payload: PaginatedData<UserGroupWithExerciseCountFromBackend>
}

//automatically dispatched by redux promise middleware
export interface GET_groups_REJECTEDAction
  extends ActionBase, ReduxPromiseRejectedAction<PaginatedData<UserGroupWithExerciseCountFromBackend>> {
  readonly type: ActionType.GET_groups_REJECTED
  readonly payload: any
}


export type AllActions =
  GET_groups_PENDINGAction
  | GET_groups_FULFILLEDAction
  | GET_groups_REJECTEDAction


export function reducer(state: State = initial, action: AllActions): State {

  switch (action.type) {

    case ActionType.GET_groups_PENDING:
      return {
        ...state,
        isLoading: true
      }

    case ActionType.GET_groups_FULFILLED:
      return {
        ...state,
        groups: action.payload.items,
        isLoading: false,
        pagination: PaginationHelper.getPaginationStateSpreadObj(action.payload)
      }

    case ActionType.GET_groups_REJECTED:
      return {
        ...state,
        groups: [],
        isLoading: false
      }

    default:
      notExhaustive(action)
      return state
  }
}

