import {ActionBase, ActionType} from '../langsViewActionTypes'
import {initial, State} from '../langsViewReducer'
import {ReduxPromiseAction, ReduxPromiseFulfilledAction, ReduxPromiseRejectedAction} from '../../index'
import {notExhaustive} from '../../_notExhausiveHelper'
import {LangFullBase} from '../../../../types/languages'


export interface UPDATE_langAction
  extends ActionBase, ReduxPromiseAction<LangFullBase, UPDATE_lang_PENDINGAction, UPDATE_lang_FULFILLEDAction, UPDATE_lang_REJECTEDAction> {
  readonly type: ActionType.UPDATE_lang
  readonly payload: Promise<LangFullBase>
}

//automatically dispatched by redux promise middleware
export interface UPDATE_lang_PENDINGAction
  extends ActionBase {
  readonly type: ActionType.UPDATE_lang_PENDING
}

//automatically dispatched by redux promise middleware
export interface UPDATE_lang_FULFILLEDAction
  extends ActionBase, ReduxPromiseFulfilledAction<LangFullBase> {
  readonly type: ActionType.UPDATE_lang_FULFILLED
  readonly payload: LangFullBase
}

//automatically dispatched by redux promise middleware
export interface UPDATE_lang_REJECTEDAction
  extends ActionBase, ReduxPromiseRejectedAction<void> {
  readonly type: ActionType.UPDATE_lang_REJECTED
  readonly payload: any
}


export type AllActions =
  UPDATE_lang_PENDINGAction
  | UPDATE_lang_FULFILLEDAction
  | UPDATE_lang_REJECTEDAction


export function reducer(state: State = initial, action: AllActions): State {

  switch (action.type) {

    case ActionType.UPDATE_lang_PENDING:
      return {
        ...state,
        isLoading: true
      }
    case ActionType.UPDATE_lang_FULFILLED:
      return {
        ...state,
        isLoading: false,
        langs: state.langs.map(p => p.id !== action.payload.id
                                    ? p
                                    : action.payload
        ),
        langCopy: null,
        editingLangId: null,
      }
    case ActionType.UPDATE_lang_REJECTED:
      return {
        ...state,
        isLoading: false,
      }

    default:
      notExhaustive(action)
      return state
  }
}

