/**
 * Created by janis dähne.
 */
import {notExhaustive} from "../_notExhausiveHelper";
import {ReduxPromiseAction, ReduxPromiseFulfilledAction, ReduxPromiseRejectedAction} from "../index";
import {ActionBase, ActionType} from "./manageCustomProjectsSiteActionTypes";
import {State, initial} from "./manageCustomProjectsSiteReducer";
import {CustomProjectPreviewFullBase} from "../../../types/customProjects";
import {PaginatedData} from '../../../types/pagination'
import {PaginationHelper} from '../../../helpers/paginationHelper'


export interface GET_customProjectOverviewsAction
  extends ActionBase, ReduxPromiseAction<PaginatedData<CustomProjectPreviewFullBase>, GET_customProjectOverviews_PENDINGAction, GET_customProjectOverviews_FULFILLEDAction, GET_customProjectOverviews_REJECTEDAction> {
  readonly type: ActionType.GET_customProjectOverviews
  readonly payload: Promise<PaginatedData<CustomProjectPreviewFullBase>>
}

//automatically dispatched by redux promise middleware
export interface GET_customProjectOverviews_PENDINGAction
  extends ActionBase {
  readonly type: ActionType.GET_customProjectOverviews_PENDING
}

//automatically dispatched by redux promise middleware
export interface GET_customProjectOverviews_FULFILLEDAction
  extends ActionBase, ReduxPromiseFulfilledAction<PaginatedData<CustomProjectPreviewFullBase>> {
  readonly type: ActionType.GET_customProjectOverviews_FULFILLED
  readonly payload: PaginatedData<CustomProjectPreviewFullBase>
}

//automatically dispatched by redux promise middleware
export interface GET_customProjectOverviews_REJECTEDAction
  extends ActionBase, ReduxPromiseRejectedAction<void> {
  readonly type: ActionType.GET_customProjectOverviews_REJECTED
  readonly payload: any
}


export type AllActions =
  GET_customProjectOverviews_PENDINGAction
  | GET_customProjectOverviews_FULFILLEDAction
  | GET_customProjectOverviews_REJECTEDAction


export function reducer(state: State = initial, action: AllActions): State {

  switch (action.type) {

    case ActionType.GET_customProjectOverviews_PENDING:
      return {
        ...state,
        isLoading: true
      }
    case ActionType.GET_customProjectOverviews_FULFILLED:
      return {
        ...state,
        isLoading: false,
        customProjectPreviews: action.payload.items,
        pagination: PaginationHelper.getPaginationStateSpreadObj(action.payload)
      }
    case ActionType.GET_customProjectOverviews_REJECTED:
      return {
        ...state,
        isLoading: false
      }

    default:
      notExhaustive(action)
      return state
  }
}

