/**
 * Created by janis dähne.
 */
import {notExhaustive} from "../_notExhausiveHelper";
import {
  ReduxPromiseAction,
  ReduxPromiseFulfilledAction,
  ReduxPromisePendingAction,
  ReduxPromiseRejectedAction
} from "../index";
import {ActionBase, ActionType} from "./groupExercisesSiteActionTypes";
import {EditableExercisePreviewFromBackend} from "../../../types/exercisePreview";
import {initial, State} from "./groupExercisesSiteReducer";
import {PaginatedData} from '../../../types/pagination'
import {PaginationHelper} from '../../../helpers/paginationHelper'
import {FrontendSettingsManager, frontendSettings_syncDisplaySortingItems} from '../../../helpers/frontendSettingsManager'

interface Meta {
  readonly groupId: number
}

export interface GET_groupExercisesAction
  extends ActionBase, ReduxPromiseAction<PaginatedData<EditableExercisePreviewFromBackend>, GET_groupExercises_PENDINGAction, GET_groupExercises_FULFILLEDAction, GET_groupExercises_REJECTEDAction, Meta> {
  readonly type: ActionType.GET_groupExercises
  readonly payload: Promise<PaginatedData<EditableExercisePreviewFromBackend>>
  readonly meta: Meta
}

//automatically dispatched by redux promise middleware
export interface GET_groupExercises_PENDINGAction extends ActionBase, ReduxPromisePendingAction<Meta> {
  readonly type: ActionType.GET_groupExercises_PENDING
}

//automatically dispatched by redux promise middleware
export interface GET_groupExercises_FULFILLEDAction
  extends ActionBase, ReduxPromiseFulfilledAction<PaginatedData<EditableExercisePreviewFromBackend>, Meta> {
  readonly type: ActionType.GET_groupExercises_FULFILLED
  readonly payload: PaginatedData<EditableExercisePreviewFromBackend>
}

//automatically dispatched by redux promise middleware
export interface GET_groupExercises_REJECTEDAction
  extends ActionBase, ReduxPromiseRejectedAction<void, Meta> {
  readonly type: ActionType.GET_groupExercises_REJECTED
  readonly payload: any
}


export type AllActions =
  GET_groupExercises_PENDINGAction
  | GET_groupExercises_FULFILLEDAction
  | GET_groupExercises_REJECTEDAction


export function reducer(state: State = initial, action: AllActions): State {

  switch (action.type) {

    case ActionType.GET_groupExercises_PENDING:
      return {
        ...state,
        // isLoading: true
        groupExercises: state.groupExercises.map(p =>
                                                   p.userGroup.id === action.meta.groupId
                                                   ? {
                                                       ...p,
                                                       isLoading: true
                                                     }
                                                   : p
        )
      }
    case ActionType.GET_groupExercises_FULFILLED:
      return {
        ...state,
        // isLoading: false,
        groupExercises: state.groupExercises.map(p =>
                                                   p.userGroup.id === action.meta.groupId
                                                   ? {
                                                       ...p,
                                                       exercisePreviews: action.payload.items,
                                                       pagination: PaginationHelper.getPaginationStateSpreadObj(
                                                         action.payload),
                                                       isLoading: false
                                                     }
                                                   : p
        )
      }
    case ActionType.GET_groupExercises_REJECTED:
      return {
        ...state,
        // isLoading: false,
        groupExercises: state.groupExercises.map(p =>
                                                   p.userGroup.id === action.meta.groupId
                                                   ? {
                                                       ...p,
                                                       exercisePreviews: [] as any,
                                                       pagination: {
                                                         ...p.pagination,
                                                         totalPages: 0,
                                                       },
                                                       isLoading: false
                                                     }
                                                   : p
        )
      }

    default:
      notExhaustive(action)
      return state
  }
}

