/**
 * Created by janis dähne.
 */
import {Action} from "redux";
import {notExhaustive} from "../_notExhausiveHelper";
import {ActionBase, ActionType} from "./manageNewUsersSiteActionTypes";
import {ReduxPromiseAction, ReduxPromiseFulfilledAction, ReduxPromiseRejectedAction} from "../index";
import {State, initial} from './manageNewUsersSiteReducer'
import Logger from "../../../helpers/logger";


export type Meta = {
  readonly shouldHideLoading: boolean
}


export interface DELETE_newUserAction
  extends ActionBase, ReduxPromiseAction<void, DELETE_newUser_PENDINGAction, DELETE_newUser_FULFILLEDAction, DELETE_newUser_REJECTEDAction, Meta> {
  readonly type: ActionType.DELETE_newUser
  readonly payload: Promise<void>
  readonly meta: Meta
}

//automatically dispatched by redux promise middleware
export interface DELETE_newUser_PENDINGAction
  extends ActionBase {
  readonly type: ActionType.DELETE_newUser_PENDING
}

//automatically dispatched by redux promise middleware
export interface DELETE_newUser_FULFILLEDAction
  extends ActionBase, ReduxPromiseFulfilledAction<void, Meta> {
  readonly type: ActionType.DELETE_newUser_FULFILLED
  readonly payload: void
  readonly meta: Meta
}

//automatically dispatched by redux promise middleware
export interface DELETE_newUser_REJECTEDAction
  extends ActionBase, ReduxPromiseRejectedAction<void, Meta> {
  readonly type: ActionType.DELETE_newUser_REJECTED
  readonly payload: any
  readonly meta: Meta
}

export type AllActions =
  DELETE_newUser_PENDINGAction
  | DELETE_newUser_FULFILLEDAction
  | DELETE_newUser_REJECTEDAction


export function reducer(state: State = initial, action: AllActions): State {

  switch (action.type) {

    case ActionType.DELETE_newUser_PENDING:
      return {
        ...state,
        isLoading: true
      }
    case ActionType.DELETE_newUser_FULFILLED:
      return {
        ...state,
        isLoading: action.meta.shouldHideLoading ? false : state.isLoading
      }
    case ActionType.DELETE_newUser_REJECTED:
      Logger.error('rejected delete data, prev state', state)
      return {
        ...state,
        isLoading: action.meta.shouldHideLoading ? false : state.isLoading
      }

    default:
      notExhaustive(action)
      return state
  }
}

