/**
 * Created by janis dähne.
 */
import {Action} from "redux";
import {notExhaustive} from "../_notExhausiveHelper";
import {ActionBase, ActionType} from "./manageTagsSiteActionTypes";
import {ReduxPromiseAction, ReduxPromiseFulfilledAction, ReduxPromiseRejectedAction} from "../index";
import {TagFullBase} from "../../../types/tag";
import {initial, State} from "./manageTagsSiteReducer";
import Logger from "../../../helpers/logger";


export interface GET_tagsAction
  extends ActionBase, ReduxPromiseAction<ReadonlyArray<TagFullBase>, GET_tags_PENDINGAction, GET_tags_FULFILLEDAction, GET_tags_REJECTEDAction> {
  readonly type: ActionType.GET_tags
  readonly payload: Promise<ReadonlyArray<TagFullBase>>
}

//automatically dispatched by redux promise middleware
export interface GET_tags_PENDINGAction
  extends ActionBase {
  readonly type: ActionType.GET_tags_PENDING
}

//automatically dispatched by redux promise middleware
export interface GET_tags_FULFILLEDAction
  extends ActionBase, ReduxPromiseFulfilledAction<ReadonlyArray<TagFullBase>> {
  readonly type: ActionType.GET_tags_FULFILLED
  readonly payload: ReadonlyArray<TagFullBase>
}

//automatically dispatched by redux promise middleware
export interface GET_tags_REJECTEDAction
  extends ActionBase, ReduxPromiseRejectedAction<ReadonlyArray<TagFullBase>> {
  readonly type: ActionType.GET_tags_REJECTED
  readonly payload: any
}


export type AllActions =
  GET_tags_PENDINGAction
  | GET_tags_FULFILLEDAction
  | GET_tags_REJECTEDAction


export function reducer(state: State = initial, action: AllActions): State {

  switch (action.type) {

    case ActionType.GET_tags_PENDING:
      return {
        ...state,
        isLoading: true
      }
    case ActionType.GET_tags_FULFILLED:
      return {
        ...state,
        tags: action.payload,
        isLoading: false
      }
    case ActionType.GET_tags_REJECTED:
      return {
        ...state,
        tags: [],
        isLoading: false
      }

    default:
      notExhaustive(action)
      return state
  }
}

