import {createAction, createReducer} from '../../../../reducerHelper'
import {AssessmentFullBase} from '../../../../../types/submissions'
import {
  AdditionalCsvColumn,
  CsvExportAssessmentOption,
  CsvExportAssessmentOptionsMap
} from '../../../../../types/csvExportAssessment'


export type State = {
  readonly isBusy: boolean

  readonly orderByProp: keyof AssessmentFullBase | null
  readonly isOrderByDesc: boolean | null //false = ascending, true: descending, null: not sorted

  /**
   * this also contains the additional columns... check kind
   */
  readonly csvExportAssessmentOptions: CsvExportAssessmentOptionsMap

  /**
   * the delimiter
   */
  readonly csvDelimiter: string

  readonly csvQuoteAllFields: boolean

}

// noinspection TsLint
let _index = 0

export const initial: State = {
  isBusy: false,


  csvDelimiter: ',',

  orderByProp: 'lastName',
  isOrderByDesc: false,
  csvQuoteAllFields: true,

  csvExportAssessmentOptions: {

    token: {
      kind: 'normal',
      displayIndex: _index++,
      willExport: true,
      replaceRulesObj: {
        replaceRules: [],
        elseReplace: null
      },
    },
    lastName: {
      kind: 'normal',
      displayIndex: _index++,
      willExport: true,
      replaceRulesObj: {
        replaceRules: [],
        elseReplace: null
      },
    },
    firstName: {
      kind: 'normal',
      displayIndex: _index++,
      willExport: true,
      replaceRulesObj: {
        replaceRules: [],
        elseReplace: null
      },
    },
    pLangId: {
      kind: 'normal',
      displayIndex: _index++,
      willExport: true,
      replaceRulesObj: {
        replaceRules: [],
        elseReplace: null
      },
    },
    shouldNotCount: {
      kind: 'normal',
      displayIndex: _index++,
      willExport: true,
      replaceRulesObj: {
        replaceRules: [],
        elseReplace: null
      },
    },
    participationCount: {
      kind: 'normal',
      displayIndex: _index++,
      willExport: true,
      replaceRulesObj: {
        replaceRules: [],
        elseReplace: null
      },
    },
    normalTestPoints: {
      kind: 'normal',
      displayIndex: _index++,
      willExport: true,
      replaceRulesObj: {
        replaceRules: [],
        elseReplace: null
      },
    },
    maxNormalTestPoints: {
      kind: 'normal',
      displayIndex: _index++,
      willExport: true,
      replaceRulesObj: {
        replaceRules: [],
        elseReplace: null
      },
    },
    submitTestPoints: {
      kind: 'normal',
      displayIndex: _index++,
      willExport: true,
      replaceRulesObj: {
        replaceRules: [],
        elseReplace: null
      },
    },
    maxSubmitTestPoints: {
      kind: 'normal',
      displayIndex: _index++,
      willExport: true,
      replaceRulesObj: {
        replaceRules: [],
        elseReplace: null
      },
    },
    manualPoints: {
      kind: 'normal',
      displayIndex: _index++,
      willExport: true,
      replaceRulesObj: {
        replaceRules: [],
        elseReplace: null
      },
    },
    maxManualPoints: {
      kind: 'normal',
      displayIndex: _index++,
      willExport: true,
      replaceRulesObj: {
        replaceRules: [],
        elseReplace: null
      },
    },

    //these cannot be sorted on backend
    noteForOtherTutors: {
      kind: 'normal',
      displayIndex: _index++,
      willExport: true,
      replaceRulesObj: {
        replaceRules: [],
        elseReplace: null
      },
    },
    feedbackForStudent: {
      kind: 'normal',
      displayIndex: _index++,
      willExport: true,
      replaceRulesObj: {
        replaceRules: [],
        elseReplace: null
      },
    },
    lastEditingIpAddress: {
      kind: 'normal',
      displayIndex:  _index++,
      willExport: true,
      replaceRulesObj: {
        replaceRules: [],
        elseReplace: null
      },
    },
    lastAssessmentErrorMessage: {
      kind: 'normal',
      displayIndex:  _index++,
      willExport: true,
      replaceRulesObj: {
        replaceRules: [],
        elseReplace: null
      },
    },
    userId: {
      kind: 'normal',
      displayIndex: null,
      willExport: false,
      replaceRulesObj: {
        replaceRules: [],
        elseReplace: null
      },
    },
    exerciseReleaseId: {
      kind: 'normal',
      displayIndex: null,
      willExport: false,
      replaceRulesObj: {
        replaceRules: [],
        elseReplace: null
      },
    },
  }
}

const initialColKeys = Object.keys(initial.csvExportAssessmentOptions)

export const set_csvDelimiter = createAction(
  'exportCsvDialogReducer_set_csvDelimiter',
  (csvDelimiter: string) => {
    return {
      csvDelimiter
    }
  },
  (state: State, action) => {
    return {
      ...state,
      csvDelimiter: action.csvDelimiter
    }
  }
)

export const set_csvQuoteAllFields = createAction(
  'exportCsvDialogReducer_set_csvQuoteAllFields',
  (csvQuoteAllFields: boolean) => {
    return {
      csvQuoteAllFields
    }
  },
  (state: State, action) => {
    return {
      ...state,
      csvQuoteAllFields: action.csvQuoteAllFields
    }
  }
)

/**
 * will also renew the indices
 */
export const set_submissionExportCsvDialog_csvExportAssessmentOptions = createAction(
  'exportCsvDialogReducer_set_csvExportAssessmentOptions',
  (csvExportAssessmentOptionsMap: CsvExportAssessmentOptionsMap) => {
    return {
      csvExportAssessmentOptionsMap
    }
  },
  (state: State, action) => {

    //self correct indices we want not holes in the sequence... 0,1,2...

    return {
      ...state,
      csvExportAssessmentOptions: correctIndices(action.csvExportAssessmentOptionsMap)
    }
  }
)

export const set_submissionExportCsvDialog_orderByProp = createAction(
  'exportCsvDialogReducer_set_orderByProp',
  (orderByProp: keyof AssessmentFullBase | null) => {
    return {
      orderByProp
    }
  },
  (state: State, action) => {
    return {
      ...state,
      orderByProp: action.orderByProp
    }
  }
)


export const set_submissionExportCsvDialog_isOrderByDesc = createAction(
  'exportCsvDialogReducer_set_isOrderByDesc',
  (isOrderByDesc: boolean | null) => {
    return {
      isOrderByDesc
    }
  },
  (state: State, action) => {
    return {
      ...state,
      isOrderByDesc: action.isOrderByDesc
    }
  }
)

export const set_submissionExportCsvDialog_isBusy = createAction(
  'exportCsvDialogReducer_set_submissionExportCsvDialog_isBusy',
  (isBusy: boolean) => {
    return {
      isBusy
    }
  },
  (state: State, action) => {
    return {
      ...state,
      isBusy: action.isBusy
    }
  }
)


export const reset_submissionExportCsvDialog_Action = createAction('exportCsvDialogReducer_RESET', () => {
}, () => {
  return {
    ...initial
  }
})

/**
 * @see RESET_GlobalAction use the same string so we can react to the action
 */
export const globalActions_RESET_GlobalAction = createAction('globalActions_RESET_Global', () => {
}, () => {
  return {
    ...initial
  }
})


export const reducer = createReducer(
  initial,
  reset_submissionExportCsvDialog_Action,
  set_submissionExportCsvDialog_orderByProp,
  set_submissionExportCsvDialog_isOrderByDesc,
  set_submissionExportCsvDialog_csvExportAssessmentOptions,
  set_csvDelimiter,
  set_submissionExportCsvDialog_isBusy,

  globalActions_RESET_GlobalAction
)

function correctIndices(map: CsvExportAssessmentOptionsMap): CsvExportAssessmentOptionsMap {

  const copy: CsvExportAssessmentOptionsMap  = {}
  const keys = Object.keys(map)

  const keysToExportSorted = keys
    // .filter(key => {
    //   const tuple = map[key] as CsvExportAssessmentOption | AdditionalCsvColumn
    //   return tuple.displayIndex !== null
    // })
    .sort((a, b) => {
      const tuple1 = map[a] as CsvExportAssessmentOption | AdditionalCsvColumn
      const tuple2 = map[b] as CsvExportAssessmentOption | AdditionalCsvColumn

      if (tuple1.displayIndex === null) {
        return 1
      }
      if (tuple2.displayIndex === null) {
        return -1
      }

      return tuple1.displayIndex - tuple2.displayIndex
    })
    .map(p => {

      const tuple = map[p]

      return {
        key: p,
        tuple
      }
    })

  // tslint:disable-next-line
  for (let i = 0; i < keysToExportSorted.length; i++) {
    const _tuple = keysToExportSorted[i]

    copy[_tuple.key] = {
      ..._tuple.tuple,
      displayIndex: _tuple.tuple.displayIndex === null ? null : i //null display index is always last so this is ok
    }
  }

  console.log(copy)

  return copy
}
