/**
 * Created by janis dähne.
 */
import {notExhaustive} from "../../_notExhausiveHelper";
import {ReduxPromiseAction, ReduxPromiseFulfilledAction, ReduxPromiseRejectedAction} from "../../index";
import {ActionBase, ActionType} from "./submissionsSiteActionTypes";
import {initial, State} from "./submissionsSiteReducer";
import {AssessmentFullBase} from "../../../../types/submissions";
import {PaginatedData} from '../../../../types/pagination'
import {PaginationHelper} from '../../../../helpers/paginationHelper'


export interface GET_submissionsAction
  extends ActionBase, ReduxPromiseAction<PaginatedData<AssessmentFullBase>, GET_submissions_PENDINGAction, GET_submissions_FULFILLEDAction, GET_submissions_REJECTEDAction> {
  readonly type: ActionType.GET_submissions

  readonly payload: Promise<PaginatedData<AssessmentFullBase>>
}

//automatically dispatched by redux promise middleware
export interface GET_submissions_PENDINGAction
  extends ActionBase {
  readonly type: ActionType.GET_submissions_PENDING
}

//automatically dispatched by redux promise middleware
export interface GET_submissions_FULFILLEDAction
  extends ActionBase, ReduxPromiseFulfilledAction<PaginatedData<AssessmentFullBase>> {
  readonly type: ActionType.GET_submissions_FULFILLED
  readonly payload: PaginatedData<AssessmentFullBase>
}

//automatically dispatched by redux promise middleware
export interface GET_submissions_REJECTEDAction
  extends ActionBase, ReduxPromiseRejectedAction<void> {
  readonly type: ActionType.GET_submissions_REJECTED
  readonly payload: any
}


export type AllActions =
  GET_submissions_PENDINGAction
  | GET_submissions_FULFILLEDAction
  | GET_submissions_REJECTEDAction


export function reducer(state: State = initial, action: AllActions): State {

  switch (action.type) {

    case ActionType.GET_submissions_PENDING:
      return {
        ...state,
        isLoading: true
      }
    case ActionType.GET_submissions_FULFILLED:
      return {
        ...state,
        isLoading: false,
        submissions: action.payload.items,
        pagination: PaginationHelper.getPaginationStateSpreadObj(action.payload)
      }
    case ActionType.GET_submissions_REJECTED:
      return {
        ...state,
        isLoading: false,
        submissions: []
      }

    default:
      notExhaustive(action)
      return state
  }
}

