/**
 * Created by janis dähne.
 */

import {notExhaustive} from "../../_notExhausiveHelper";
import {AssessmentFullBase} from "../../../../types/submissions";
import {ActionBase, ActionType} from "./submissionsSiteActionTypes";
import {AllActions as GetSubmissionsActions, reducer as getSubmissionsReducer} from './getSubmissionsReducer'
import {
  AllActions as RerunAutomaticAssessmentActions,
  reducer as rerunAutomaticAssessmentReducer
} from './rerunAutomaticAssessmentReducer'
import {
  AllActions as RerunSingleAutomaticAssessmentActions,
  reducer as rerunSingleAutomaticAssessmentReducer
} from './rerunSingleAutomaticAssessmentReducer'
import {
  SET_comesFromOwnExercisesSiteAction,
  SET_selectedExerciseIdAction,
} from "../exerciseReleasesSite/releaseSiteReducer";
import {
  ActionType as ReleaseSiteActionType,
  ActionType as ReleasesSiteActionTypes
} from "../exerciseReleasesSite/releasesSiteActionTypes";
import {ExerciseReleaseFromBackend} from "../../../../types/exerciseRelease";
import {EditableExercisePreviewFromBackend} from "../../../../types/exercisePreview";
import {
  AllActions as GetOwnExercisesActions,
  reducer as getSingleOwnExerciseCrudReducer
} from './getSingleOwnExerciseCrudReducer'
import {
  AllActions as GetGroupExercisesActions,
  reducer as getSingleGroupExerciseCrudReducer
} from './getSingleGroupExerciseCrudReducer'
import {GroupPreviewFromBackend} from '../../../../types/group'
import {GroupRoleWithPermissionFullBase} from '../../../../types/groupRoles'
import {PaginationStateExtend} from '../../../../types/pagination'
import {getInitialPaginationData, paginationPageSizeOptionAll} from '../../../../constants'
import {
  AllActions as GetSingleReleaseActions,
  reducer as getSingleReleaseCrudReducer
} from './getSingleReleaseCrudReducer'
import {FrontendSettingsManager} from '../../../../helpers/frontendSettingsManager'
import {
  AllActions as UploadCsvAssessmentFileActions,
  reducer as uploadCsvAssessmentFileReducer
} from './uploadCsvAssessmentFileReducer'
import {ActionTypeGlobal, RESET_GlobalAction} from '../../globalActions'


export type State = {

  readonly submissions: ReadonlyArray<AssessmentFullBase>

  readonly isLoading: boolean

  /**
   * null on error or init
   */
  readonly release: ExerciseReleaseFromBackend | null

  /**
   * TODO do we really need this???
   * can't we use release directly??
   */
  readonly selectedReleaseId: number

  /**
   * true: we use the own exercises (maybe we need to load them), we don't need the group role, we have the permission
   * false: we use the group exercises (maybe we need to load them)
   */
  readonly comesFromOwnExercisesSite: boolean
  /**
   * is null if we comesFromOwnExercisesSite  but also on error
   */
  readonly group: GroupPreviewFromBackend | null

  /**
   * is null if we comesFromOwnExercisesSite but also on error
   */
  readonly groupRole: GroupRoleWithPermissionFullBase | null

  /**
   * TODO do we really need this???
   * can't we use release directly??
   */
  readonly selectedExerciseId: number
  /**
   * null on error or init
   */
  readonly exercise: EditableExercisePreviewFromBackend | null

  readonly isAutomaticRefreshViewEnabled: boolean
  readonly automaticRefreshViewIntervalCounter: number

  readonly runningAssessment: AssessmentFullBase | null

  readonly isExportCsvDialogDisplayed: boolean

  readonly showLastEditingIpAddress: boolean

  /**
   * in the submissions view the columns for passed tests can be hidden or displayed
   * true: show them
   * false: hide them
   */
  readonly showPassedTestsColumns: boolean

} & PaginationStateExtend<AssessmentFullBase>


export const initial: State = {
  submissions: [],
  comesFromOwnExercisesSite: false,
  isLoading: true,
  selectedReleaseId: -1,
  release: null,
  exercise: null,
  selectedExerciseId: -1,
  isAutomaticRefreshViewEnabled: false,
  automaticRefreshViewIntervalCounter: 1, //could be 0
  group: null,
  groupRole: null,

  runningAssessment: null,
  isExportCsvDialogDisplayed: false,
  showLastEditingIpAddress: false,

  showPassedTestsColumns: FrontendSettingsManager.getFrontendSettings().showPassedTestsColumns,

  pagination: { //can be overwritten by frontend settings
    ...getInitialPaginationData(),
    orderByProp: 'lastName',
    pageSize: paginationPageSizeOptionAll //currently the backend calcs all submissions and then does pagination so we can load all...
  }
}


//---pagination
export interface SetSearchTextAction extends ActionBase {
  readonly type: ActionType.setSearchText
  readonly searchText: string
}

export interface SetActivePaginationPageAction extends ActionBase {
  readonly type: ActionType.setPaginationActivePage
  readonly activePage: number
}

export interface SetPaginationPageSizeAction extends ActionBase {
  readonly type: ActionType.setPaginationPageSize
  readonly pageSize: number
}

//---pagination end

export interface SET_isExportCsvDialogDisplayedAction extends ActionBase {
  readonly type: ActionType.SET_isExportCsvDialogDisplayed
  readonly isExportCsvDialogDisplayed: boolean
}

export interface SET_isAutomaticRefreshViewEnabledAction extends ActionBase {
  readonly type: ActionType.SET_isAutomaticRefreshViewEnabled
  readonly isAutomaticRefreshViewEnabled: boolean
}

export interface SET_automaticRefreshViewIntervalCounterAction extends ActionBase {
  readonly type: ActionType.SET_automaticRefreshViewIntervalCounter
  readonly automaticRefreshViewIntervalCounter: number
}

export interface SET_sortByKeyAction extends ActionBase {
  readonly type: ActionType.SET_sortByKey
  readonly sortByKey: keyof AssessmentFullBase
  readonly sortDirection: boolean | null
}


export interface SET_selectedReleaseIdAction extends ActionBase {
  readonly type: ActionType.SET_selectedReleaseId
  readonly id: number
}

export interface SET_runningAssessmentAction extends ActionBase {
  readonly type: ActionType.SET_runningAssessment
  readonly runningAssessment: AssessmentFullBase | null
}

export interface SET_showLastEditingIpAddressAction extends ActionBase {
  readonly type: ActionType.SET_showLastEditingIpAddress
  readonly showLastEditingIpAddress: boolean
}

export interface SET_showPassedTestsColumnsAction extends ActionBase {
  readonly type: ActionType.SET_showPassedTestsColumns
  readonly showPassedTestsColumns: boolean
}

export interface ResetAction extends ActionBase {
  readonly type: ActionType.RESET
}


export type AllActions =
  ResetAction
  | SET_comesFromOwnExercisesSiteAction
  | SET_selectedReleaseIdAction
  | GetSubmissionsActions
  | RerunAutomaticAssessmentActions
  | RerunSingleAutomaticAssessmentActions
  | GetSingleReleaseActions
  | SET_selectedExerciseIdAction
  | GetOwnExercisesActions
  | GetGroupExercisesActions
  | SET_runningAssessmentAction
  | SET_sortByKeyAction
  | SET_isAutomaticRefreshViewEnabledAction
  | SET_automaticRefreshViewIntervalCounterAction
  | SET_isExportCsvDialogDisplayedAction
  | SET_showLastEditingIpAddressAction
  | SetSearchTextAction
  | SetActivePaginationPageAction
  | SetPaginationPageSizeAction
  | UploadCsvAssessmentFileActions
  | SET_showPassedTestsColumnsAction

  | RESET_GlobalAction


export function reducer(state: State = initial, action: AllActions): State {

  switch (action.type) {

    case ActionType.UPLOAD_csvAssessmentFile_PENDING:
    case ActionType.UPLOAD_csvAssessmentFile_FULFILLED:
    case ActionType.UPLOAD_csvAssessmentFile_REJECTED:
      return uploadCsvAssessmentFileReducer(state, action)

    case ActionType.RERUN_automaticAssessment_PENDING:
    case ActionType.RERUN_automaticAssessment_FULFILLED:
    case ActionType.RERUN_automaticAssessment_REJECTED:
      return rerunAutomaticAssessmentReducer(state, action)

    case ActionType.RERUN_singleAutomaticAssessment_PENDING:
    case ActionType.RERUN_singleAutomaticAssessment_FULFILLED:
    case ActionType.RERUN_singleAutomaticAssessment_REJECTED:
      return rerunSingleAutomaticAssessmentReducer(state, action)

    case ActionType.GET_submissions_PENDING:
    case ActionType.GET_submissions_FULFILLED:
    case ActionType.GET_submissions_REJECTED:
      return getSubmissionsReducer(state, action)

    case ActionType.GET_singleRelease_PENDING:
    case ActionType.GET_singleRelease_FULFILLED:
    case ActionType.GET_singleRelease_REJECTED:
      return getSingleReleaseCrudReducer(state, action)

    case ActionType.GET_singleOwnExercise_PENDING:
    case ActionType.GET_singleOwnExercise_FULFILLED:
    case ActionType.GET_singleOwnExercise_REJECTED:
      return getSingleOwnExerciseCrudReducer(state, action)

    case ActionType.GET_singleGroupExercise_PENDING:
    case ActionType.GET_singleGroupExercise_FULFILLED:
    case ActionType.GET_singleGroupExercise_REJECTED:
      return getSingleGroupExerciseCrudReducer(state, action)


    case ReleaseSiteActionType.SET_selectedExerciseId:
      return {
        ...state,
        selectedExerciseId: action.id
      }

    case ActionType.SET_selectedReleaseId:
      return {
        ...state,
        selectedReleaseId: action.id
      }

    case ActionType.SET_runningAssessment:

      //reset the points of the assessment that is running
      return {
        ...state,
        runningAssessment: action.runningAssessment,
        submissions: state.submissions.map<AssessmentFullBase>(p => {
          return action.runningAssessment !== null && p.pLangId === action.runningAssessment.pLangId && p.userId === action.runningAssessment.userId
                 ?
            {
              ...p,
              submitTestPoints: null,
              maxSubmitTestPoints: null,
              normalTestPoints: null,
              maxNormalTestPoints: null
            }
                 : p
        })
      }

    case ReleasesSiteActionTypes.SET_comesFromOwnExercisesSite:
      return {
        ...state,
        comesFromOwnExercisesSite: action.comesFromOwnExercisesSite
      }

    case ActionType.SET_isAutomaticRefreshViewEnabled:
      return {
        ...state,
        isAutomaticRefreshViewEnabled: action.isAutomaticRefreshViewEnabled
      }
    case ActionType.SET_automaticRefreshViewIntervalCounter:
      return {
        ...state,
        automaticRefreshViewIntervalCounter: action.automaticRefreshViewIntervalCounter
      }

    case ActionType.SET_isExportCsvDialogDisplayed:
      return {
        ...state,
        isExportCsvDialogDisplayed: action.isExportCsvDialogDisplayed
      }

    case ActionType.SET_sortByKey:

      FrontendSettingsManager.setSubmissionsSiteSorting({
                                                          sortByKey: action.sortByKey,
                                                          sortDirection: action.sortDirection
                                                        })

      return {
        ...state,
        pagination: {
          ...state.pagination,
          orderByProp: action.sortByKey,
          isOrderByDesc: action.sortDirection
        }
      }

    case ActionType.setSearchText:
      return {
        ...state,
        pagination: {
          ...state.pagination,
          searchText: action.searchText
        }
      }
    case ActionType.setPaginationActivePage:
      return {
        ...state,
        pagination: {
          ...state.pagination,
          currentPage: action.activePage
        }
      }

    case ActionType.setPaginationPageSize:

      FrontendSettingsManager.set_submissionsViewPageSize(action.pageSize)

      return {
        ...state,
        pagination: {
          ...state.pagination,
          pageSize: action.pageSize,
          currentPage: 1,
        }
      }
    case ActionType.SET_showLastEditingIpAddress:
      return {
        ...state,
        showLastEditingIpAddress: action.showLastEditingIpAddress
      }
    case ActionType.SET_showPassedTestsColumns: {

      FrontendSettingsManager.setShowPassedTestsColumns(action.showPassedTestsColumns)

      return {
        ...state,
        showPassedTestsColumns: action.showPassedTestsColumns
      }
    }


    case ActionType.RESET:
    case ActionTypeGlobal.RESET_Global:

      const sorting = FrontendSettingsManager.getFrontendSettings().submissionsSiteSorting
      return {
        ...initial,
        pagination: {
          ...initial.pagination,
          orderByProp: sorting === null
                       ? initial.pagination.orderByProp
                       : sorting.sortByKey,
          isOrderByDesc: sorting === null
                         ? initial.pagination.isOrderByDesc
                         : sorting.sortDirection,
          pageSize: FrontendSettingsManager.getFrontendSettings().submissionsViewPageSize,
        }
      }

    default:
      notExhaustive(action)
      return state
  }
}

