/**
 * Created by janis dähne.
 */
import {notExhaustive} from "../_notExhausiveHelper";
import {ReduxPromiseAction, ReduxPromiseFulfilledAction, ReduxPromiseRejectedAction} from "../index";
import {ActionBase, ActionType} from "./settingsSiteActionTypes";
import {ChangeState, initial, State} from "./settingsSiteReducers";
import Logger from "../../../helpers/logger";
import {BaseUserData} from "../../../types/userData";


export interface UPDATE_userDataAction
  extends ActionBase, ReduxPromiseAction<BaseUserData, UPDATE_userData_PENDINGAction, UPDATE_userData_FULFILLEDAction, UPDATE_userData_REJECTEDAction> {
  readonly type: ActionType.UPDATE_userData
  readonly payload: Promise<BaseUserData>
}

//automatically dispatched by redux promise middleware
export interface UPDATE_userData_PENDINGAction
  extends ActionBase {
  readonly type: ActionType.UPDATE_userData_PENDING
}

//automatically dispatched by redux promise middleware
export interface UPDATE_userData_FULFILLEDAction
  extends ActionBase, ReduxPromiseFulfilledAction<BaseUserData> {
  readonly type: ActionType.UPDATE_userData_FULFILLED
  readonly payload: BaseUserData
}

//automatically dispatched by redux promise middleware
export interface UPDATE_userData_REJECTEDAction
  extends ActionBase, ReduxPromiseRejectedAction<void> {
  readonly type: ActionType.UPDATE_userData_REJECTED
  readonly payload: any
}


export type AllActions =
  UPDATE_userData_PENDINGAction
  | UPDATE_userData_FULFILLEDAction
  | UPDATE_userData_REJECTEDAction


export function reducer(state: State = initial, action: AllActions): State {

  switch (action.type) {

    case ActionType.UPDATE_userData_PENDING:
      return {
        ...state,
        isLoadingUserData: true,
      }
    case ActionType.UPDATE_userData_FULFILLED:
      return {
        ...state,
        isLoadingUserData: false,
        userData: {
          ...state.userData,
          email: action.payload.email
        },
        userDataChangeState: ChangeState.notActive
      }
    case ActionType.UPDATE_userData_REJECTED:
      return {
        ...state,
        isLoadingUserData: false,
      }

    default:
      notExhaustive(action)
      return state
  }
}

