/**
 * Created by janis dähne.
 */
import {notExhaustive} from "../_notExhausiveHelper";
import {ReduxPromiseAction, ReduxPromiseFulfilledAction, ReduxPromiseRejectedAction} from "../index";
import {ActionBase, ActionType} from "./settingsSiteActionTypes";
import {ChangeState, initial, State} from "./settingsSiteReducers";
import Logger from "../../../helpers/logger";
import {UserSettingsFullBase} from "../../../types/userData";



export interface UPDATE_userSettingsAction
  extends ActionBase, ReduxPromiseAction<UserSettingsFullBase, UPDATE_userSettings_PENDINGAction, UPDATE_userSettings_FULFILLEDAction, UPDATE_userSettings_REJECTEDAction> {
  readonly type: ActionType.UPDATE_userSettings
  readonly payload: Promise<UserSettingsFullBase>
}

//automatically dispatched by redux promise middleware
export interface UPDATE_userSettings_PENDINGAction
  extends ActionBase {
  readonly type: ActionType.UPDATE_userSettings_PENDING
}

//automatically dispatched by redux promise middleware
export interface UPDATE_userSettings_FULFILLEDAction
  extends ActionBase, ReduxPromiseFulfilledAction<UserSettingsFullBase> {
  readonly type: ActionType.UPDATE_userSettings_FULFILLED
  readonly payload: UserSettingsFullBase
}

//automatically dispatched by redux promise middleware
export interface UPDATE_userSettings_REJECTEDAction
  extends ActionBase, ReduxPromiseRejectedAction<UserSettingsFullBase> {
  readonly type: ActionType.UPDATE_userSettings_REJECTED
  readonly payload: any
}


export type AllActions =
  UPDATE_userSettings_PENDINGAction
  | UPDATE_userSettings_FULFILLEDAction
  | UPDATE_userSettings_REJECTEDAction


export function reducer(state: State = initial, action: AllActions): State {

  switch (action.type) {

    case ActionType.UPDATE_userSettings_PENDING:
      return {
        ...state,
        isLoadingMiscSettings: true
      }
    case ActionType.UPDATE_userSettings_FULFILLED:
      return {
        ...state,
        isLoadingMiscSettings: false,
        userData: {
          ...state.userData,
          settings: action.payload
        },
        userSettingsChangeState: ChangeState.notActive
      }
    case ActionType.UPDATE_userSettings_REJECTED:
      return {
        ...state,
        isLoadingMiscSettings: false
      }

    default:
      notExhaustive(action)
      return state
  }
}

