/**
 * Created by janis dähne.
 */
import {notExhaustive} from "../_notExhausiveHelper";
import {EmailAndPasswordTuple} from "../../../types/userData";
import {ActionBase, ActionType} from "./settingsSiteActionTypes";
import {
  getValidationCollection, getValidationMessagesCollection, isEmailAndNotEmpty, isEmptyOrNotSpaces
} from "../../../validation/validations";
import {isValidDialog} from "../../../validation/dialogValidations";


export type State = EmailAndPasswordTuple & {
  readonly newPasswordCheck: string
  readonly isValid: boolean
}

export const initial: State = {
  email: '',
  newPassword: '',
  oldPassword: '',
  oldPasswordBase64: '',
  newPasswordCheck: '',
  newPasswordBase64: '',
  isValid: false
}

export const validationRules = getValidationCollection<EmailAndPasswordTuple>({
  email: [isEmailAndNotEmpty],
  newPassword: [isEmptyOrNotSpaces],
  oldPassword: [],
  oldPasswordBase64: [],
  newPasswordBase64: [],
})



export interface SET_emailAction extends ActionBase {
  readonly type: ActionType.SET_emailCopy
  readonly email: string
}

export interface SET_newPasswordCopyAction extends ActionBase {
  readonly type: ActionType.SET_newPasswordCopy
  readonly password: string
}

export interface SET_newPasswordCheckCopyAction extends ActionBase {
  readonly type: ActionType.SET_newPasswordCheckCopy
  readonly password: string
}

export interface SET_oldPasswordCopyAction extends ActionBase {
  readonly type: ActionType.SET_oldPasswordCopy
  readonly password: string
}

export interface RESET_userDataAction extends ActionBase {
  readonly type: ActionType.RESET_userData
  readonly resetEmail: string
}

export type AllActions =
  RESET_userDataAction
  | SET_emailAction
  | SET_newPasswordCopyAction
  | SET_oldPasswordCopyAction
  | SET_newPasswordCheckCopyAction


export function reducer(state: State = initial, action: AllActions): State {

  // noinspection TsLint
  let temp: State = null

  switch (action.type) {

    case ActionType.SET_newPasswordCopy:

      temp = {
        ...state,
        newPassword: action.password
      }

      return {
        ...temp,
        isValid: isValidDialog(temp, validationRules)
      }

    case ActionType.SET_newPasswordCheckCopy:
      temp = {
        ...state,
        newPasswordCheck: action.password
      }

      return {
        ...temp,
        isValid: isValidDialog(temp, validationRules)
      }

    case ActionType.SET_oldPasswordCopy:
      temp = {
        ...state,
        oldPassword: action.password
      }

      return {
        ...temp,
        isValid: isValidDialog(temp, validationRules)
      }
    case ActionType.SET_emailCopy:
      temp = {
        ...state,
        email: action.email,
      }

      return {
        ...temp,
        isValid: isValidDialog(temp, validationRules)
      }

    case ActionType.RESET_userData:
      return {
        ...initial,
        email: action.resetEmail
      }

    default:
      notExhaustive(action)
      return state
  }
}

