/**
 * Created by janis dähne.
 */
import {SystemRoleFullBase} from "../../../types/userData";
import {notExhaustive} from "../_notExhausiveHelper";

import {AllActions as GetActions, reducer as getReducer} from './getSystemRolesReducer'
import {AllActions as DeleteActions, reducer as deleteReducer} from './deleteSystemRoleReducer'
import {ActionBase, ActionType} from "./systemRolesSiteActionTypes";
import {ActionTypeGlobal, RESET_GlobalAction} from '../globalActions'
import {
  AllActions as GetLockedSystemRoleIdsActions,
  reducer as getLockedSystemRoleIdsReducer
} from './getLockedSystemRoleIdsReducer'


export type State = {
  readonly systemRoles: ReadonlyArray<SystemRoleFullBase>

  /**
   * the ids of system roles that cannot be changed/deleted
   */
  readonly lockedSystemRoleIds: ReadonlyArray<number>
  /**
   * true: display the dialog,
   * false: display main view
   */
  readonly isChangeDialogDisplayed: boolean

  /**
   * true: feting data, display spinner,
   * false: not (finished error/or success)
   */
  readonly isLoading: boolean
}

export const initial: State = {
  systemRoles: [],
  lockedSystemRoleIds: [],
  isChangeDialogDisplayed: false,
  isLoading: false
}

export interface SetListAction extends ActionBase {
  readonly type: ActionType.SET_systemRoles
  readonly systemRoles: ReadonlyArray<SystemRoleFullBase>
}

export interface SET_isChangeDialogDisplayedAction extends ActionBase {
  readonly type: ActionType.SET_isChangeDialogDisplayed
  readonly isDisplayed: boolean
}

export interface ResetAction extends ActionBase {
  readonly type: ActionType.RESET
}

export type AllActions =
  SetListAction
  | ResetAction
  | SET_isChangeDialogDisplayedAction
  | GetActions
  | DeleteActions
  | GetLockedSystemRoleIdsActions

  | RESET_GlobalAction


export function reducer(state: State = initial, action: AllActions): State {

  switch (action.type) {

    case ActionType.SET_systemRoles:
      return {
        ...state,
        systemRoles: action.systemRoles
      }

    case ActionType.SET_isChangeDialogDisplayed:
      return {
        ...state,
        isChangeDialogDisplayed: action.isDisplayed
      }

    case ActionType.GET_lockedSystemRoleIds_PENDING:
    case ActionType.GET_lockedSystemRoleIds_FULFILLED:
    case ActionType.GET_lockedSystemRoleIds_REJECTED:
      return getLockedSystemRoleIdsReducer(state, action)

    case ActionType.GET_systemRoles_PENDING:
    case ActionType.GET_systemRoles_FULFILLED:
    case ActionType.GET_systemRoles_REJECTED:
      return getReducer(state, action)


    case ActionType.DELETE_systemRole_PENDING:
    case ActionType.DELETE_systemRole_FULFILLED:
    case ActionType.DELETE_systemRole_REJECTED:
      return deleteReducer(state, action)

    case ActionType.RESET:
    case ActionTypeGlobal.RESET_Global:
      return {
        ...initial
      }

    default:
      notExhaustive(action)
      return state
  }
}

