import {ActionBase, ActionType} from '../systemSettingsSiteActionTypes'
import {SystemSettingsFromBackend} from '../../../../types/systemSettings'
import {State, initial} from '../systemSettingsSiteReducer'
import {ReduxPromiseAction, ReduxPromiseFulfilledAction,ReduxPromiseRejectedAction} from '../../index'
import {notExhaustive} from '../../_notExhausiveHelper'

export interface UPDATE_systemSettingsAction
  extends ActionBase, ReduxPromiseAction<SystemSettingsFromBackend, UPDATE_systemSettings_PENDINGAction, UPDATE_systemSettings_FULFILLEDAction, UPDATE_systemSettings_REJECTEDAction> {
  readonly type: ActionType.UPDATE_systemSettings
  readonly payload: Promise<SystemSettingsFromBackend>
}

//automatically dispatched by redux promise middleware
export interface UPDATE_systemSettings_PENDINGAction
  extends ActionBase {
  readonly type: ActionType.UPDATE_systemSettings_PENDING
}

//automatically dispatched by redux promise middleware
export interface UPDATE_systemSettings_FULFILLEDAction
  extends ActionBase, ReduxPromiseFulfilledAction<SystemSettingsFromBackend> {
  readonly type: ActionType.UPDATE_systemSettings_FULFILLED
  readonly payload: SystemSettingsFromBackend
}

//automatically dispatched by redux promise middleware
export interface UPDATE_systemSettings_REJECTEDAction
  extends ActionBase, ReduxPromiseRejectedAction<void> {
  readonly type: ActionType.UPDATE_systemSettings_REJECTED
  readonly payload: any
}


export type AllActions =
  UPDATE_systemSettings_PENDINGAction
  | UPDATE_systemSettings_FULFILLEDAction
  | UPDATE_systemSettings_REJECTEDAction


export function reducer(state: State = initial, action: AllActions): State {

  switch (action.type) {

    case ActionType.UPDATE_systemSettings_PENDING:
      return {
        ...state,
        isLoading: true,
      }
    case ActionType.UPDATE_systemSettings_FULFILLED:
      return {
        ...state,
        isLoading: false,
        systemSettings: action.payload
      }
    case ActionType.UPDATE_systemSettings_REJECTED:
      return {
        ...state,
        isLoading: false
      }

    default:
      notExhaustive(action)
      return state
  }
}

