/**
 * Created by janis dähne.
 */
import {Action} from "redux";
import {notExhaustive} from "./_notExhausiveHelper";
import {TestTypeFullBase} from "../../types/testTypes";
import {ReduxPromiseAction, ReduxPromiseFulfilledAction, ReduxPromiseRejectedAction} from "./index";
import Logger from "../../helpers/logger";
import {ActionTypeGlobal, RESET_GlobalAction} from './globalActions'

export type State = {

  /**
   * the known test types
   *
   * sometimes KnownInternalTestTypes.justRunTest is added the some specific components (e.g. console view)
   * so be able to display the test
   */
  readonly testTypes: ReadonlyArray<TestTypeFullBase>
  readonly isLoading: boolean
}


export const initial: State = {
  testTypes: [],
  isLoading: false
}

export interface ActionBase extends Action {
  readonly type: ActionType
}

export enum ActionType {
  GET_testTypes = 'testTypesReducer_GET_testTypes',
  GET_testTypes_PENDING = 'testTypesReducer_GET_testTypes_PENDING',
  GET_testTypes_FULFILLED = 'testTypesReducer_GET_testTypes_FULFILLED',
  GET_testTypes_REJECTED = 'testTypesReducer_GET_testTypes_REJECTED',
  RESET = 'testTypesReducer_RESET',
}


export interface GET_testTypesAction
  extends ActionBase, ReduxPromiseAction<ReadonlyArray<TestTypeFullBase>, GET_testTypes_PENDINGAction, GET_testTypes_FULFILLEDAction, GET_testTypes_REJECTEDAction> {
  readonly type: ActionType.GET_testTypes
  readonly payload: Promise<ReadonlyArray<TestTypeFullBase>>
}

//automatically dispatched by redux promise middleware
export interface GET_testTypes_PENDINGAction
  extends ActionBase {
  readonly type: ActionType.GET_testTypes_PENDING
}

//automatically dispatched by redux promise middleware
export interface GET_testTypes_FULFILLEDAction
  extends ActionBase, ReduxPromiseFulfilledAction<ReadonlyArray<TestTypeFullBase>> {
  readonly type: ActionType.GET_testTypes_FULFILLED
  readonly payload: ReadonlyArray<TestTypeFullBase>
}

//automatically dispatched by redux promise middleware
export interface GET_testTypes_REJECTEDAction
  extends ActionBase, ReduxPromiseRejectedAction<void> {
  readonly type: ActionType.GET_testTypes_REJECTED
  readonly payload: any
}

export interface ResetAction extends ActionBase {
  readonly type: ActionType.RESET
}

export type AllActions =
  ResetAction
  | GET_testTypes_PENDINGAction
  | GET_testTypes_FULFILLEDAction
  | GET_testTypes_REJECTEDAction

  | RESET_GlobalAction


export function reducer(state: State = initial, action: AllActions): State {

  switch (action.type) {

    case ActionType.GET_testTypes_PENDING:
      return {
        ...state,
        isLoading: true
      }
    case ActionType.GET_testTypes_FULFILLED:
      return {
        ...state,
        isLoading: false,
        testTypes: action.payload
      }
    case ActionType.GET_testTypes_REJECTED:
      return {
        ...state,
        isLoading: false,
        testTypes: []
      }

    case ActionType.RESET:
    case ActionTypeGlobal.RESET_Global:
      return {
        ...initial
      }

    default:
      notExhaustive(action)
      return state
  }
}

