import {ReduxPromiseAction, ReduxPromiseFulfilledAction, ReduxPromiseRejectedAction} from "../../../index";
import {Action} from 'redux'
import {ActionType} from '../../tutorViewSiteActionTypes'
import {initial, State} from '../../tutorViewSiteReducer'
import {notExhaustive} from '../../../_notExhausiveHelper'
import {StoredTestResultFromBackend} from '../../../../../types/testResults'
import {RunnableExerciseTestFrontendExclusive} from '../../../../../types/exerciseEditor'
import {_storedTestResultToTestResult} from '../../../../../helpers/convertersAndTransformers'

export interface ActionBase extends Action {
  readonly type: ActionType
}

export interface GET_singleAfterCustomTestResultAction
  extends ActionBase, ReduxPromiseAction<StoredTestResultFromBackend, GET_singleAfterCustomTestResult_PENDINGAction, GET_singleAfterCustomTestResult_FULFILLEDAction, GET_singleAfterCustomTestResult_REJECTEDAction> {
  readonly type: ActionType.GET_singleAfterCustomTestResult
  readonly payload: Promise<StoredTestResultFromBackend>
}

//automatically dispatched by redux promise middleware
export interface GET_singleAfterCustomTestResult_PENDINGAction
  extends ActionBase {
  readonly type: ActionType.GET_singleAfterCustomTestResult_PENDING
}

//automatically dispatched by redux promise middleware
export interface GET_singleAfterCustomTestResult_FULFILLEDAction
  extends ActionBase, ReduxPromiseFulfilledAction<StoredTestResultFromBackend> {
  readonly type: ActionType.GET_singleAfterCustomTestResult_FULFILLED
  readonly payload: StoredTestResultFromBackend
}

//automatically dispatched by redux promise middleware
export interface GET_singleAfterCustomTestResult_REJECTEDAction
  extends ActionBase, ReduxPromiseRejectedAction<void> {
  readonly type: ActionType.GET_singleAfterCustomTestResult_REJECTED
  readonly payload: any
}


export type AllActions =
  GET_singleAfterCustomTestResult_PENDINGAction
  | GET_singleAfterCustomTestResult_FULFILLEDAction
  | GET_singleAfterCustomTestResult_REJECTEDAction


export function reducer(state: State = initial, action: AllActions): State {

  switch (action.type) {

    case ActionType.GET_singleAfterCustomTestResult_PENDING:
      return {
        ...state,
        isLoading: true
      }
    case ActionType.GET_singleAfterCustomTestResult_FULFILLED:
      return {
        ...state,
        isLoading: false,
        //custom tests are currently not displayed... also not needed
      }
    case ActionType.GET_singleAfterCustomTestResult_REJECTED:
      return {
        ...state,
        isLoading: false
      }

    default:
      notExhaustive(action)
      return state
  }
}

