import {ReduxPromiseAction, ReduxPromiseFulfilledAction, ReduxPromiseRejectedAction} from "../../../index";
import {Action} from 'redux'
import {ActionType} from '../../tutorViewSiteActionTypes'
import {initial, State} from '../../tutorViewSiteReducer'
import {notExhaustive} from '../../../_notExhausiveHelper'
import {StoredTestResultFromBackend} from '../../../../../types/testResults'
import {RunnableExerciseTestFrontendExclusive} from '../../../../../types/exerciseEditor'
import {_storedTestResultToTestResult} from '../../../../../helpers/convertersAndTransformers'
import {upState} from '../../../../../helpers/typeHelpers'

export interface ActionBase extends Action {
  readonly type: ActionType
}

export interface GET_singleAfterNormalTestResultAction
  extends ActionBase, ReduxPromiseAction<StoredTestResultFromBackend, GET_singleAfterNormalTestResult_PENDINGAction, GET_singleAfterNormalTestResult_FULFILLEDAction, GET_singleAfterNormalTestResult_REJECTEDAction> {
  readonly type: ActionType.GET_singleAfterNormalTestResult
  readonly payload: Promise<StoredTestResultFromBackend>
}

//automatically dispatched by redux promise middleware
export interface GET_singleAfterNormalTestResult_PENDINGAction
  extends ActionBase {
  readonly type: ActionType.GET_singleAfterNormalTestResult_PENDING
}

//automatically dispatched by redux promise middleware
export interface GET_singleAfterNormalTestResult_FULFILLEDAction
  extends ActionBase, ReduxPromiseFulfilledAction<StoredTestResultFromBackend> {
  readonly type: ActionType.GET_singleAfterNormalTestResult_FULFILLED
  readonly payload: StoredTestResultFromBackend
}

//automatically dispatched by redux promise middleware
export interface GET_singleAfterNormalTestResult_REJECTEDAction
  extends ActionBase, ReduxPromiseRejectedAction<void> {
  readonly type: ActionType.GET_singleAfterNormalTestResult_REJECTED
  readonly payload: any
}


export type AllActions =
  GET_singleAfterNormalTestResult_PENDINGAction
  | GET_singleAfterNormalTestResult_FULFILLEDAction
  | GET_singleAfterNormalTestResult_REJECTEDAction


export function reducer(state: State = initial, action: AllActions): State {

  switch (action.type) {

    case ActionType.GET_singleAfterNormalTestResult_PENDING:
      return {
        ...state,
        isLoading: true
      }
    case ActionType.GET_singleAfterNormalTestResult_FULFILLED:
      return {
        ...state,
        isLoading: false,
        afterNormalTests: state.afterNormalTests.map(p =>
          p.id !== action.payload.testId
          ? p
          : upState(p, {
            ...p,
            result: _storedTestResultToTestResult(action.payload)
          }))
        //custom tests are currently not displayed... also not needed
      }
    case ActionType.GET_singleAfterNormalTestResult_REJECTED:
      return {
        ...state,
        isLoading: false
      }

    default:
      notExhaustive(action)
      return state
  }
}

