/**
 * Created by janis dähne.
 */
import {ActionBase, ActionType} from "../tutorViewSiteActionTypes";
import {initial, State} from "../tutorViewSiteReducer";
import {StoredTestResultFromBackend} from "../../../../types/testResults";
import {_storedTestResultToTestResult} from "../../../../helpers/convertersAndTransformers";
import {ReduxPromiseAction, ReduxPromiseFulfilledAction, ReduxPromiseRejectedAction} from "../../index";
import {notExhaustive} from "../../_notExhausiveHelper";
import {upState} from '../../../../helpers/typeHelpers'
import {RunnableCustomTestFrontendExclusive} from '../../../../types/customTests'

//this is actually only the normal tests, custom tests

export interface GET_singleUserCustomTestResultAction
  extends ActionBase, ReduxPromiseAction<StoredTestResultFromBackend, GET_singleUserCustomTestResult_PENDINGAction, GET_singleUserCustomTestResult_FULFILLEDAction, GET_singleUserCustomTestResult_REJECTEDAction> {
  readonly type: ActionType.GET_singleUserCustomTestResult
  readonly payload: Promise<StoredTestResultFromBackend>
}

//automatically dispatched by redux promise middleware
export interface GET_singleUserCustomTestResult_PENDINGAction
  extends ActionBase {
  readonly type: ActionType.GET_singleUserCustomTestResult_PENDING
}

//automatically dispatched by redux promise middleware
export interface GET_singleUserCustomTestResult_FULFILLEDAction
  extends ActionBase, ReduxPromiseFulfilledAction<StoredTestResultFromBackend> {
  readonly type: ActionType.GET_singleUserCustomTestResult_FULFILLED
  readonly payload: StoredTestResultFromBackend
}

//automatically dispatched by redux promise middleware
export interface GET_singleUserCustomTestResult_REJECTEDAction
  extends ActionBase, ReduxPromiseRejectedAction<void> {
  readonly type: ActionType.GET_singleUserCustomTestResult_REJECTED
  readonly payload: any
}


export type AllActions =
  GET_singleUserCustomTestResult_PENDINGAction
  | GET_singleUserCustomTestResult_FULFILLEDAction
  | GET_singleUserCustomTestResult_REJECTEDAction


export function reducer(state: State = initial, action: AllActions): State {

  switch (action.type) {

    case ActionType.GET_singleUserCustomTestResult_PENDING:
      return {
        ...state,
        isLoading: true
      }
    case ActionType.GET_singleUserCustomTestResult_FULFILLED:
      return {
        ...state,
        isLoading: false,
        exercise: {
          ...state.exercise,
          customTests: state.exercise.customTests.map<RunnableCustomTestFrontendExclusive>(p =>
            p.id !== action.payload.testId
            ? p
            : upState(p, {
              ...p,
              result: _storedTestResultToTestResult(action.payload)
            }))
        },
      }
    case ActionType.GET_singleUserCustomTestResult_REJECTED:
      return {
        ...state,
        isLoading: false
      }

    default:
      notExhaustive(action)
      return state
  }
}

