/**
 * Created by janis dähne.
 */
import {ActionBase, ActionType} from "../tutorViewSiteActionTypes";
import {initial, State} from "../tutorViewSiteReducer";
import {StoredTestResultFromBackend} from "../../../../types/testResults";
import {
  _storedTestResultToTestResult,
  mergeTestResultsIntoDoExercise
} from "../../../../helpers/convertersAndTransformers";
import {ReduxPromiseAction, ReduxPromiseFulfilledAction, ReduxPromiseRejectedAction} from "../../index";
import {notExhaustive} from "../../_notExhausiveHelper";
import {RunnableExerciseTestFrontendExclusive} from '../../../../types/exerciseEditor'
import {upState} from '../../../../helpers/typeHelpers'

//this is actually only the normal tests, custom tests

export interface GET_singleUserNormalTestResultAction
  extends ActionBase, ReduxPromiseAction<StoredTestResultFromBackend, GET_singleUserNormalTestResult_PENDINGAction, GET_singleUserNormalTestResult_FULFILLEDAction, GET_singleUserNormalTestResult_REJECTEDAction> {
  readonly type: ActionType.GET_singleUserNormalTestResult
  readonly payload: Promise<StoredTestResultFromBackend>
}

//automatically dispatched by redux promise middleware
export interface GET_singleUserNormalTestResult_PENDINGAction
  extends ActionBase {
  readonly type: ActionType.GET_singleUserNormalTestResult_PENDING
}

//automatically dispatched by redux promise middleware
export interface GET_singleUserNormalTestResult_FULFILLEDAction
  extends ActionBase, ReduxPromiseFulfilledAction<StoredTestResultFromBackend> {
  readonly type: ActionType.GET_singleUserNormalTestResult_FULFILLED
  readonly payload: StoredTestResultFromBackend
}

//automatically dispatched by redux promise middleware
export interface GET_singleUserNormalTestResult_REJECTEDAction
  extends ActionBase, ReduxPromiseRejectedAction<void> {
  readonly type: ActionType.GET_singleUserNormalTestResult_REJECTED
  readonly payload: any
}


export type AllActions =
  GET_singleUserNormalTestResult_PENDINGAction
  | GET_singleUserNormalTestResult_FULFILLEDAction
  | GET_singleUserNormalTestResult_REJECTEDAction


export function reducer(state: State = initial, action: AllActions): State {

  switch (action.type) {

    case ActionType.GET_singleUserNormalTestResult_PENDING:
      return {
        ...state,
        isLoading: true
      }
    case ActionType.GET_singleUserNormalTestResult_FULFILLED:
      return {
        ...state,
        isLoading: false,
        exercise: {
          ...state.exercise,
          tests: state.exercise.tests.map<RunnableExerciseTestFrontendExclusive>(p =>
            p.id !== action.payload.testId
            ? p
            : upState(p, {
              ...p,
              result: _storedTestResultToTestResult(action.payload)
            }))
        },
      }
    case ActionType.GET_singleUserNormalTestResult_REJECTED:
      return {
        ...state,
        isLoading: false
      }

    default:
      notExhaustive(action)
      return state
  }
}

