/**
 * Created by janis dähne.
 */
import {ActionBase, ActionType} from "../tutorViewSiteActionTypes";
import {initial, State} from "../tutorViewSiteReducer";
import {AssessedSubmitTestFrontendExclusive, StoredTestResultFromBackend} from "../../../../types/testResults";
import {mergeTestResultsIntoDoExercise} from "../../../../helpers/convertersAndTransformers";
import {ReduxPromiseAction, ReduxPromiseFulfilledAction, ReduxPromiseRejectedAction} from "../../index";
import {notExhaustive} from "../../_notExhausiveHelper";
import {upState} from '../../../../helpers/typeHelpers'

//this is actually only the normal tests, custom tests

export interface GET_singleUserSubmitTestResultAction
  extends ActionBase, ReduxPromiseAction<StoredTestResultFromBackend, GET_singleUserSubmitTestResult_PENDINGAction, GET_singleUserSubmitTestResult_FULFILLEDAction, GET_singleUserSubmitTestResult_REJECTEDAction> {
  readonly type: ActionType.GET_singleUserSubmitTestResult
  readonly payload: Promise<StoredTestResultFromBackend>
}

//automatically dispatched by redux promise middleware
export interface GET_singleUserSubmitTestResult_PENDINGAction
  extends ActionBase {
  readonly type: ActionType.GET_singleUserSubmitTestResult_PENDING
}

//automatically dispatched by redux promise middleware
export interface GET_singleUserSubmitTestResult_FULFILLEDAction
  extends ActionBase, ReduxPromiseFulfilledAction<StoredTestResultFromBackend> {
  readonly type: ActionType.GET_singleUserSubmitTestResult_FULFILLED
  readonly payload: StoredTestResultFromBackend
}

//automatically dispatched by redux promise middleware
export interface GET_singleUserSubmitTestResult_REJECTEDAction
  extends ActionBase, ReduxPromiseRejectedAction<void> {
  readonly type: ActionType.GET_singleUserSubmitTestResult_REJECTED
  readonly payload: any
}


export type AllActions =
  GET_singleUserSubmitTestResult_PENDINGAction
  | GET_singleUserSubmitTestResult_FULFILLEDAction
  | GET_singleUserSubmitTestResult_REJECTEDAction


export function reducer(state: State = initial, action: AllActions): State {

  switch (action.type) {

    case ActionType.GET_singleUserSubmitTestResult_PENDING:
      return {
        ...state,
        isLoading: true
      }
    case ActionType.GET_singleUserSubmitTestResult_FULFILLED:
      return {
        ...state,
        isLoading: false,
        submitTestResults: state.submitTestResults.map<AssessedSubmitTestFrontendExclusive>(p =>
          p.id !== action.payload.testId
          ? p
          : upState(p, {
            ...p,
            result: action.payload
          }))
      }
    case ActionType.GET_singleUserSubmitTestResult_REJECTED:
      return {
        ...state,
        isLoading: false
      }

    default:
      notExhaustive(action)
      return state
  }
}

