/**
 * Created by janis dähne.
 */
import {ActionBase, ActionType} from "../tutorViewSiteActionTypes";
import {initial, State} from "../tutorViewSiteReducer";
import Logger from "../../../../helpers/logger";
import {
  AssessedSubmitTestFromBackend, AssessedSubmitTestFrontendExclusive,
  AssessmentForUserWithTestResults
} from "../../../../types/testResults";
import {ReduxPromiseAction, ReduxPromiseFulfilledAction, ReduxPromiseRejectedAction} from "../../index";
import {notExhaustive} from "../../_notExhausiveHelper";
import {convertAssessmentFromBackendToFrontend} from "../../../../helpers/convertersAndTransformers";


export interface GET_userSubmitTestResultsAction
  extends ActionBase, ReduxPromiseAction<AssessmentForUserWithTestResults, GET_userSubmitTestResults_PENDINGAction, GET_userSubmitTestResults_FULFILLEDAction, GET_userSubmitTestResults_REJECTEDAction> {
  readonly type: ActionType.GET_userSubmitTestResults
  readonly payload: Promise<AssessmentForUserWithTestResults>
}

//automatically dispatched by redux promise middleware
export interface GET_userSubmitTestResults_PENDINGAction
  extends ActionBase {
  readonly type: ActionType.GET_userSubmitTestResults_PENDING
}

//automatically dispatched by redux promise middleware
export interface GET_userSubmitTestResults_FULFILLEDAction
  extends ActionBase, ReduxPromiseFulfilledAction<AssessmentForUserWithTestResults> {
  readonly type: ActionType.GET_userSubmitTestResults_FULFILLED
  readonly payload: AssessmentForUserWithTestResults
}

//automatically dispatched by redux promise middleware
export interface GET_userSubmitTestResults_REJECTEDAction
  extends ActionBase, ReduxPromiseRejectedAction<void> {
  readonly type: ActionType.GET_userSubmitTestResults_REJECTED
  readonly payload: any
}


export type AllActions =
  GET_userSubmitTestResults_PENDINGAction
  | GET_userSubmitTestResults_FULFILLEDAction
  | GET_userSubmitTestResults_REJECTEDAction


export function reducer(state: State = initial, action: AllActions): State {

  switch (action.type) {

    case ActionType.GET_userSubmitTestResults_PENDING:
      return {
        ...state,
        isLoading: true
      }
    case ActionType.GET_userSubmitTestResults_FULFILLED:
      return {
        ...state,
        isLoading: false,
        submitTestResults: convertAssessmentFromBackendToFrontend(action.payload.submitTestResults, false),
        submitTestsForTutor: convertAssessmentFromBackendToFrontend(action.payload.submitTestResults, false)
        //normal test results are set via getAllUserTestResultsReducer
      }
    case ActionType.GET_userSubmitTestResults_REJECTED:
      return {
        ...state,
        isLoading: false,
        submitTestResults: []
      }

    default:
      notExhaustive(action)
      return state
  }
}

