import {ActionBase, ActionType} from '../tutorViewSiteActionTypes'
import {initial, State} from '../tutorViewSiteReducer'
import {TestAnswerFromBackend} from '../../../../types/testsAndTestProtocol'
import {ReduxPromiseAction, ReduxPromiseFulfilledAction, ReduxPromiseRejectedAction} from '../../index'
import {notExhaustive} from '../../_notExhausiveHelper'


export interface RUN_compileSingleFileAction
  extends ActionBase, ReduxPromiseAction<TestAnswerFromBackend, RUN_compileSingleFile_PENDINGAction, RUN_compileSingleFile_FULFILLEDAction, RUN_compileSingleFile_REJECTEDAction> {
  readonly type: ActionType.RUN_compileSingleFile
  readonly payload: Promise<TestAnswerFromBackend>
}

//automatically dispatched by redux promise middleware
export interface RUN_compileSingleFile_PENDINGAction
  extends ActionBase {
  readonly type: ActionType.RUN_compileSingleFile_PENDING
}

//automatically dispatched by redux promise middleware
export interface RUN_compileSingleFile_FULFILLEDAction
  extends ActionBase, ReduxPromiseFulfilledAction<TestAnswerFromBackend> {
  readonly type: ActionType.RUN_compileSingleFile_FULFILLED
  readonly payload: TestAnswerFromBackend
}

//automatically dispatched by redux promise middleware
export interface RUN_compileSingleFile_REJECTEDAction
  extends ActionBase, ReduxPromiseRejectedAction<void> {
  readonly type: ActionType.RUN_compileSingleFile_REJECTED
  readonly payload: any
}


export type AllActions =
  RUN_compileSingleFile_PENDINGAction
  | RUN_compileSingleFile_FULFILLEDAction
  | RUN_compileSingleFile_REJECTEDAction


export function reducer(state: State = initial, action: AllActions): State {

  switch (action.type) {

    case ActionType.RUN_compileSingleFile_PENDING:
      return {
        ...state,
        isLoading: true
      }
    case ActionType.RUN_compileSingleFile_FULFILLED:
      return {
        ...state,
        isLoading: false,
        lastCompileSingleFileResult: action.payload
      }
    case ActionType.RUN_compileSingleFile_REJECTED:
      return {
        ...state,
        isLoading: false,
        lastCompileSingleFileResult: null
      }

    default:
      notExhaustive(action)
      return state
  }
}

