/**
 * Created by janis dähne.
 */
import {ActionBase, ActionType} from "../tutorViewSiteActionTypes";
import {initial, State} from "../tutorViewSiteReducer";
import Logger from "../../../../helpers/logger";
import {ReduxPromiseAction, ReduxPromiseFulfilledAction, ReduxPromiseRejectedAction} from "../../index";
import {notExhaustive} from "../../_notExhausiveHelper";
import {TestAnswerFromBackend} from "../../../../types/testsAndTestProtocol";
import {RunnableExerciseTestFrontendExclusive} from "../../../../types/exerciseEditor";
import {mergeNormalTestResults} from "../../../../helpers/convertersAndTransformers";


export interface RUN_normalTestAsTutorAction
  extends ActionBase, ReduxPromiseAction<ReadonlyArray<TestAnswerFromBackend>, RUN_normalTestAsTutor_PENDINGAction, RUN_normalTestAsTutor_FULFILLEDAction, RUN_normalTestAsTutor_REJECTEDAction> {
  readonly type: ActionType.RUN_normalTestAsTutor
  readonly payload: Promise<ReadonlyArray<TestAnswerFromBackend>>
}

//automatically dispatched by redux promise middleware
export interface RUN_normalTestAsTutor_PENDINGAction
  extends ActionBase {
  readonly type: ActionType.RUN_normalTestAsTutor_PENDING
}

//automatically dispatched by redux promise middleware
export interface RUN_normalTestAsTutor_FULFILLEDAction
  extends ActionBase, ReduxPromiseFulfilledAction<ReadonlyArray<TestAnswerFromBackend>> {
  readonly type: ActionType.RUN_normalTestAsTutor_FULFILLED
  readonly payload: ReadonlyArray<TestAnswerFromBackend>
}

//automatically dispatched by redux promise middleware
export interface RUN_normalTestAsTutor_REJECTEDAction
  extends ActionBase, ReduxPromiseRejectedAction<void> {
  readonly type: ActionType.RUN_normalTestAsTutor_REJECTED
  readonly payload: any
}


export type AllActions =
  RUN_normalTestAsTutor_PENDINGAction
  | RUN_normalTestAsTutor_FULFILLEDAction
  | RUN_normalTestAsTutor_REJECTEDAction


export function reducer(state: State = initial, action: AllActions): State {

  switch (action.type) {

    case ActionType.RUN_normalTestAsTutor_PENDING:
      return {
        ...state,
        isLoading: false
      }
    case ActionType.RUN_normalTestAsTutor_FULFILLED:
      return {
        ...state,
        isLoading: false,
        normalTestsForTutor: mergeNormalTestResults(state.normalTestsForTutor, action.payload)
      }
    case ActionType.RUN_normalTestAsTutor_REJECTED:
      return {
        ...state,
        isLoading: false,
        normalTestsForTutor: mergeNormalTestResults(state.normalTestsForTutor, [])
      }

    default:
      notExhaustive(action)
      return state
  }
}

