/**
 * Created by janis dähne.
 */
import {Moment} from "moment";
import {
  FileWithData, ExerciseCodeTemplateFileFullBase, ExerciseTestFromBackend, FilePreviewFromBackend,
  RunnableExerciseTestFrontendExclusive, TestAssetFromBackendWithData,
  TestSettingsFullBase
} from "./exerciseEditor";
import {RunnableCustomTestFrontendExclusive} from "./customTests";
import {TestAnswerFromBackend} from "./testsAndTestProtocol";
import {AceEditSession} from "../components/codeEditors/multiTabCodeEditorProps";
import {StoredTestResultFromBackend} from "./testResults";
import {FileIdAssociation} from './exerciseSolution'

/**
 * a preview for a custom project
 */
export interface CustomProjectPreviewFullBase {
  readonly id: number

  /**
   * the display name
   */
  readonly displayName: string

  /**
   * the last edited plang... set through the custom solution
   */
  readonly lastEditorPLangId: number


  readonly lastUpdatedAt: Moment
}


export interface CustomProjectPropertiesFromFrontend {
  readonly displayName: string
}

export interface CustomProjectBase {
  readonly id: number
  readonly displayName: string
  /**
   * the last edited plang... set through the custom solution
   */
  readonly lastEditorPLangId: number

  readonly description: CustomProjectDescriptionFullBase

  readonly solution: CustomProjectSolutionFullBase
}

export interface CustomProjectFullBase extends CustomProjectBase {
  readonly tests: ReadonlyArray<CustomProjectTestFullBase>
}

export interface CustomProjectFrontendExclusive extends CustomProjectBase {
  readonly tests: ReadonlyArray<RunnableCustomProjectTestFrontendExclusive>
}


export interface CustomProjectDescriptionFullBase {
  readonly id: number
  readonly content: string
  /**
   * FROM frontend to backend this should be empty!
   */
  readonly assets: ReadonlyArray<FileWithData>
}

export interface CustomProjectSolutionFullBase {
  readonly id: number
  readonly pLangId: number

  /**
   * readonly rom backend
   * can be null if the custom project was not yet created
   */
  readonly createdAt: Moment | null

  /**
   * should not be null...
   * but can be null if this is a new custom project
   */
  readonly mainFileId: number |null

  readonly solutionFiles: ReadonlyArray<CustomProjectSolutionFileFullBase>

}

export interface CustomSolutionTupleFromBackend {
  readonly solution: CustomProjectSolutionFullBase
  readonly fileIdAssociationMap: ReadonlyArray<FileIdAssociation>
}


export interface CustomProjectSolutionFileFullBase extends ExerciseCodeTemplateFileFullBase {
  /**
   * ui property, true: is displayed as tab, false: not (only displayed in project explorer)
   */
  readonly uiIsDisplayed: boolean
}

export interface CustomProjectTestFullBase {
  readonly id: number
  /**
   * the display name
   */
  readonly displayName: string
  /**
   * the test content
   */
  readonly content: string
  /**
   * the maxPoints of the test
   */
  readonly maxPoints: number
  /**
   * ui property, the display index for the sorting in the frontend
   */
  readonly displayIndex: number

  /**
   * the test type id
   */
  readonly testTypeId: number

  /**
   * the test settings
   */
  readonly testSettings: TestSettingsFullBase

  readonly files: ReadonlyArray<FilePreviewFromBackend>
}

export interface RunnableCustomProjectTestFrontendExclusive extends CustomProjectTestFullBase {
  readonly type: 'customProjectTest' | 'tmp'

  /**
   * true: test is running, false: not
   * used to display a spinner
   */
  readonly uiIsRunning: boolean
  /**
   * true: this is an after test, false: a normal (not after/subsequent solution) test
   */
  readonly uiIsAfterTest: boolean
  /**
   * the result for this test
   */
  readonly result: TestAnswerFromBackend | null
}
