/**
 * Created by janis dähne.
 */


import {
  ExerciseCodeTemplateFullBase, ExerciseDescriptionFullBase, ExerciseTestFromBackend,
  RunnableExerciseTestFrontendExclusive,
  TestSettingsFullBase
} from "./exerciseEditor";
import {CustomTestFromBackend, RunnableCustomTestFrontendExclusive} from "./customTests";
import {Moment} from "moment";
import {ReleaseDurationType} from "./ReleaseDurationType";
import {ReleaseStartType} from "./ReleaseStartType";


export interface DoExerciseBase {
  readonly id: number
  /**
   * the display name
   */
  readonly displayName: string
  /**
   * the short description
   */
  readonly shortDescription: string
  /**
   * true: user can create own tests, false: not
   */
  readonly canUserCreateCustomTests: boolean
  /**
   * true: user can create own files (tabs), false: not
   *
   * this is also used as: can user delete files & can user rename files?
   */
  readonly canUserCreateFiles: boolean

  /**
   * the last edited plang to pre select first when entering the do exercise page
   */
  readonly lastEditedPLangId: number

  /**
   * exercise and all dependent entities should be readonly (no edit/no delete)!
   */
  readonly isPermanentlyLocked: boolean
  /**
   * the code templates for every plang (used to reset the files)
   */
  readonly codeTemplates: ReadonlyArray<ExerciseCodeTemplateFullBase>

  /**
   * the exercise description
   */
  readonly exerciseDescription: ExerciseDescriptionFullBase

  /**
   * the settings for all custom tests created for this exercise
   */
  readonly defaultCustomTestSettings: TestSettingsFullBase

  /**
   * the date time when the user has accessed the exercise for the first time (not actually accessed but could have accessed e.g. wehn he/she enters the release code)
   * used to calculate the left working time...
   */
  readonly firstAccessedAt: Moment

  /**
   * the related exercise release
   */
  readonly exerciseRelease: ExerciseReleaseDoExerciseFromBackend

  /**
   * the lock solutions flag from the participation
   */
  readonly lockSolutionsFlag: boolean

}

export interface DoExerciseFrontendExclusive extends DoExerciseBase {

  /**
   * the custom tests (without results)
   * CURRENTLY we always return an empty array for tutor view!
   */
  readonly customTests: ReadonlyArray<RunnableCustomTestFrontendExclusive>

  /**
   * the tests without the submit tests
   */
  readonly tests: ReadonlyArray<RunnableExerciseTestFrontendExclusive>

}


/**
 * the exercise for the do exercise page
 */
export interface DoExerciseFromBackend extends DoExerciseBase {

  /**
   * the tests (without results)
   */
  readonly customTests: ReadonlyArray<CustomTestFromBackend>

  /**
   * the custom tests (without results)
   */
  readonly tests: ReadonlyArray<ExerciseTestFromBackend>

}

/**
 * the connected (to the exercise) exercise release, we need this because the working time
 * could be limited
 */
export interface ExerciseReleaseDoExerciseFromBackend {
  readonly id: number

  /**
   * true: exercise is still released,
   * false: not,
   * when the release ends the user cannot modify the solution but look at it (and the result of the submit test if any)
   */
  readonly isReleased: boolean

  /**
   * the release code
   */
  readonly releaseCode: string

  /**
   * the available working time for the exercise in minutes
   */
  readonly availableWorkingTimeInMinutes: number

  /**
   * the date when the exercise release ends
   * null if never ends or date time when the release ends (release could end before the AvailableWorkingTimeInMinutes is reached
   */
  readonly automaticEndAt: Moment | null

  /**
   * true: the working time is limited, false: ignore the AvailableWorkingTimeInMinutes
   */
  readonly hasLimitedWorkingTime: boolean

  /**
   * the release duration type (to know if we can ignore the AutomaticEndAt datetime)
   * 0 = Infinitely, 1 = Limited
   */
  readonly releaseDurationType: ReleaseDurationType


  readonly releaseStartType: ReleaseStartType
  /**
   * the p lang for the release or NULL when released for all p langs
   */
  readonly pLangId: number

  /**
   * true: assessment is finished, false: not
   */
  readonly hasAutomaticAssessmentFinished: boolean

  /**
   * e.g. in an electronic exam (e-exam) we might have multiple windows in e.g. the safe exam browser
   * the student could copy & past exercises from other systems into yapex and try to compile, execute code...
   *
   * true: try to clear the clipboard if the tab is changed...
   *     show an overlay every time the student tabs or clicks outside of the browser window
   *     and when he/she wants to resume he/she must click a button to proceed
   * false: normal
   *
   * frontend only option...
   */
  readonly shouldClearClipboard: boolean

  /**
   * in the do exercise site the link mode always overwrites this setting!
   *
   * true: hide the site header bar (where the menu, user menu is located)
   *    so that a student cannot navigate away from the do exercise site (e.g. in an exam)
   *
   * false: not hide the bar
   */
  readonly hideSiteHeaderBar: boolean

  /**
   * hides all print options (e.g. print task description, code editor) for the exercise
   */
  readonly hidePrintOptions: boolean

  /**
   * hides all actions and buttons the user could use to leave the exercise (and e.g. select another exercise).
   * NOTE The buttons in the site header bar are not affected (use the appropriate option for that).
   * @see hideSiteHeaderBar
   */
  readonly hideExerciseLeaveActions: boolean

  /**
   * some options might hide the user header bar, this will display an additional button to show (and download) the log
   */
  readonly showAdditionalLogButton: boolean

  /**
   * the max points for manual assessment
   * NULL for no maximum
   *
   * however, one can give more points than this (e.g. extra points)
   * or if this null manual points are still possible to give
   */
  readonly maxManualPoints: number | null

}

