/**
 * type from the server
 */
import {TagsConnectors} from '../state/reducers/tagsFilterReducer'

export interface PaginatedData<T> {
  readonly currentPage: number
  readonly pageSize: number
  readonly totalPages: number
  readonly totalItems: number

  readonly orderByProp: string | null
  readonly isOrderByDesc: boolean | null //false = ascending, true: descending, null: not sorted

  readonly searchText: string

  readonly items: ReadonlyArray<T>
}

export interface PaginatedDataTagsFilter<T> extends PaginatedData<T> {
  readonly selectedFilterTagIds: ReadonlyArray<number>
  readonly selectedNegativeFilterTagsIds: ReadonlyArray<number>
}


/**
 * add this to the state type via & PaginatedStateExtend
 */
export interface PaginationStateExtend<TSorting> {
  readonly pagination: PaginationStateData<TSorting>
}

/**
 * these are the props needed for pagination in a state
 */
export interface PaginationStateData<TSorting> {
  readonly currentPage: number
  readonly pageSize: number
  readonly totalPages: number
  readonly totalItems: number

  /**
   * this can also be lower case!!
   */
  readonly orderByProp: keyof TSorting | null
  readonly isOrderByDesc: boolean | null //false = ascending, true: descending, null: not sorted

  readonly searchText: string
}

/**
 * pagination data to post to the server to receive paginated data
 */
export interface PaginationPostData {
  readonly page: number
  readonly pageSize: number

  readonly orderByProp: string | null
  readonly isOrderByDesc: boolean | null //false = ascending, true: descending, null: not sorted

  readonly searchText: string

}


export interface PaginationPostDataWithTagsFilter extends PaginationPostData {
  readonly selectedFilterTagIds: ReadonlyArray<number>
  readonly selectedNegativeFilterTagsIds: ReadonlyArray<number>
  readonly logicalConnector: TagsConnectors
}