/**
 * Created by janis dähne.
 */


export interface UserDataFullBase {
  readonly id: number
  /**
   * the first name
   */
  readonly firstName: string
  /**
   * the last name
   */
  readonly lastName: string
  /**
   * the email
   */
  readonly email: string

  /**
   * the user settings or null
   */
  readonly settings: UserSettingsFullBase | null
  /**
   * the system role or null
   */
  readonly systemRole: SystemRoleFullBase | null

  /**
   * true: the user belongs to a group and has permission to create an exercise,
   * (no system permission, just group permission),
   * false: not
   */
  readonly tempCanCreateExercise: boolean

}

/**
 * the system role for an user
 * see backend type SystemRolePermission for permission description
 */
export interface SystemRoleFullBase {
  readonly id:number
  /**
   * the display name
   */
  readonly displayName:string
  /**
   * the email for the role or null
   *
   */
  readonly email: string | null

  //permissions


  readonly canCreateGroups:boolean
  readonly canDeleteGroups:boolean

  //TODO currently used for system roles AND group roles...
  //then change also translations... (labels)
  readonly canCreateRoles:boolean
  readonly canChangeRoles:boolean
  readonly canDeleteRoles:boolean

  readonly canManageNewUsers:boolean
  readonly canChangeOtherUsersSystemRole:boolean

  readonly canDeleteActivatedUsers:boolean //or view
  readonly canChangeUserData: boolean //other users data


  readonly canChangeSystemSettings: boolean
  readonly canManageTags: boolean

  readonly canViewDashboard: boolean

  /**
   * true: can download personal data from others
   */
  readonly canDownloadPersonalDataFromOthers: boolean


}

/**
 * the user settings
 */
export interface UserSettingsFullBase {
  readonly id: number
  /**
   * the language id
   */
  readonly langId: number
  /**
   * the theme name
   */
  readonly theme: string,
  /**
   * the editor settings
   */
  readonly codeEditorSettings?: CodeEditorSettingsFullBase
}

/**
 * code editor settings
 */
export interface CodeEditorSettingsFullBase {
  readonly id:number
  /**
   * see availableThemeNames (themes.ts)
   * convert this via getEditorInternalThemeName to the editor specific theme
   */
  readonly theme:string
  readonly fontSize:number
  readonly tabSize:number
  readonly showLineNumbers:boolean
  readonly highlightCurrentLine:boolean
  readonly showInvisibles:boolean
  readonly showLineIndentions:boolean
  readonly useWrapping: boolean
}



/**
 * the tuple with
 */
export interface IdAndPasswordTuple {
  /**
   * the user id to know whose password to reset
   */
  readonly targetUserId: number
  /**
   * the password
   */
  readonly password: string
}

/**
 * tuple to change the user data (changed by user)
 */
export interface EmailAndPasswordTuple {

  /**
   * the new email (empty to not change or old value...)
   */
  readonly email: string

  /**
   * the old password to verify that the user is allowed to change his/her password
   */
  readonly oldPassword: string
  readonly oldPasswordBase64: string
  /**
   * the new password
   */
  readonly newPassword: string
  /**
   * password base64 encoded
   */
  readonly newPasswordBase64: string
}
/**
 * the data to set the new user data (called by an admin)
 */
export interface BaseUserData {
  /**
   * the targetted user to set the new values
   */
  readonly targetUserId: number
  /**
   * the first name
   */
  readonly firstName: string
  /**
   * the last name
   */
  readonly lastName: string

  /**
   * the email
   */
  readonly email: string
}
