/**
 * Created by janis dähne.
 */


import {SystemRoleFullBase} from "./userData";
import {Moment} from "moment";

/**
 * a user for the frontend (to display in a list)
 */
export interface UserPreviewFromBackend {

  readonly id: number

  /**
   * the token (id)
   */
  readonly token: string

  /**
   * the first name
   */
  readonly firstName: string
  /**
   * the last name
   */
  readonly lastName: string
  /**
   * the email
   */
  readonly email: string

  /**
   * true: user is activated and can login, false: not
   */
  readonly isActivated: boolean

  /**
   * the system role id or null
   */
  readonly systemRoleId?: number

  /**
   * the date when the user did a login (only set when allowed to see)
   * (from the backend dwe receive a string not a datetime (because asp core))
   */
  readonly lastLoginAt?: Moment

  /**
   * the date time when the user was created (might be helpful for the new users list)
   */
  readonly createdAt?: Moment

  readonly isCurrentUser: boolean
}


export interface NewUserFrontendExclusive extends UserPreviewFromBackend {
  /**
   * true: user is selected, false: not
   */
  readonly uiIsSelected:boolean

  /**
   * the system role from the systemRoleId
   */
  readonly systemRole: SystemRoleFullBase

  readonly uiIsChangeSystemRolePopupDisplayed: boolean
}

/**
 * a user that can be selected and has other data important for the frontend
 */
export interface SelectableUserFrontendExclusive extends UserPreviewFromBackend {
  /**
   * true: this user preview is the current user, false: not
   * TODO this should be calced in backend!!
   */
  readonly isCurrentUser: boolean
  /**
   * true: user is selected, false: not
   */
  readonly uiIsSelected:boolean

  /**
   * true: display user (in some list), false: not
   */
  readonly isDisplayed: boolean

  /**
   * the system role from the systemRoleId
   */
  readonly systemRole: SystemRoleFullBase | undefined
}

export interface AddableUserFrontendExclusive extends UserPreviewFromBackend {
  /**
   * true: this user preview is the current user, false: not
   * TODO this should be calced in backend!!
   */
  readonly isCurrentUser: boolean
  /**
   * true: user is selected, false: not
   */
  readonly uiIsSelected:boolean

  /**
   * true: display user (in some list), false: not
   */
  readonly uiIsDisplayed: boolean

  /**
   * true: cannot be added, false: not
   * else the paging is wrong...
   */
  readonly isAlreadyMember: boolean
}



/**
 * a tuple to activate a user / or change a users system role
 */
export interface UserSystemRoleTupleForBackend {
  /**
   * the user id
   */
  readonly userId: number
  /**
   * the system role id or null
   */
  readonly systemRoleId: number | null
}